@@ -19,8 +19,6 @@ the Free Software Foundation, 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.  */
 
 
-#include "../config.h"
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
@@ -53,6 +51,8 @@ Boston, MA 02111-1307, USA.  */
 
 #endif
 
+#include "../config.h"
+
 #ifdef HAVE_GUI
 #include <glib.h>
 #include <gdk/gdktypes.h>
@@ -91,6 +91,7 @@ static GIOChannel *channel;
 
 #endif /* HAVE_READLINE */
 
+void simulation_cleanup();
 
 extern const char *get_dir_delim(const char *path);
 int parse_string(const char * str);
@@ -102,7 +103,6 @@ LIBGPSIM_EXPORT bool gUsingThreads(); // in ../src/interface.cc
 void initialize_readline (void);
 void clear_input_buffer(void);
 
-void exit_gpsim(void);
 #ifdef HAVE_GUI
 void quit_gui(void);
 #endif
@@ -123,10 +123,10 @@ extern void stop_server(void);
 #endif // HAVE_SOCKETS
 
 
-static Boolean  &s_bSTCEcho = *new Boolean("CliTrace", false,
-  "Enable echoing commands from STC files to the console.");
-void EnableSTCEcho(bool bEnable) {
-  s_bSTCEcho = bEnable;
+static Boolean  *s_bSTCEcho = 0;
+void EnableSTCEcho(bool bEnable)
+{
+  *s_bSTCEcho = bEnable;
 }
 
 //------------------------------------------------------------------------
@@ -137,7 +137,7 @@ void EnableSTCEcho(bool bEnable) {
 class CCliCommandHandler : public ICommandHandler
 {
 public:
-  virtual char *GetName();
+  virtual const char *GetName();
   virtual int Execute(const char * commandline, ISimConsole *out);
   virtual int ExecuteScript(list<string *> &script, ISimConsole *out);
 };
@@ -305,8 +305,12 @@ void initialize_CLI()
 
 void initialize_gpsim(void)
 {
-  s_bSTCEcho.setClearableSymbol(false);
-  get_symbol_table().add(&s_bSTCEcho);
+  s_bSTCEcho = new Boolean("CliTrace", false,
+                           "Enable echoing commands from STC files to the console.");
+
+
+  globalSymbolTable().addSymbol(s_bSTCEcho);
+
   initialize_CLI();
   if(gUsingThreads())
     initialize_threads();
@@ -329,9 +333,9 @@ void LLStack::print(void)
       int depth =0;
       while(h) {
 
-	cout << "   " <<stack_number <<':'<<depth << "  "<<  h->data;
-	depth++;
-	h = h->next_input;
+        cout << "   " <<stack_number <<':'<<depth << "  "<<  h->data;
+        depth++;
+        h = h->next_input;
       }
       stack_number++;
       s = s->next_stack;
@@ -453,7 +457,7 @@ int start_parse(void)
   int retval = init_parser();
 
   if(quit_parse)
-    exit_gpsim();
+    exit_gpsim(0);
 
   return retval;
 }
@@ -481,7 +485,7 @@ int parse_string_only(const char * str) {
   return iRet;
 }
 
-void process_command_file(const char * file_name)
+void process_command_file(const char * file_name, bool bCanChangeDirectory)
 {
 
   FILE *cmd_file;
@@ -492,7 +496,7 @@ void process_command_file(const char * file_name)
     cout << __FUNCTION__ <<"()\n";
 
   dir_path_end = get_dir_delim(file_name);
-  if(dir_path_end)
+  if(dir_path_end && bCanChangeDirectory)
   {
       strncpy(directory,file_name,dir_path_end-file_name);
       directory[dir_path_end-file_name]=0;
@@ -535,7 +539,7 @@ void process_command_file(const char * file_name)
             str[iLast-1] = '\n';
           }
 #endif
-	  add_string_to_input_buffer(s);
+          add_string_to_input_buffer(s);
       }
 
       fclose(cmd_file);
@@ -566,7 +570,7 @@ void process_command_file(const char * file_name)
 */
 
 int gpsim_open(Processor *cpu, const char *pFileName,
-	       const char * pProcessorType, const char *pProcessorName)
+               const char * pProcessorType, const char *pProcessorName)
 {
   if(!pFileName)
     return 0;
@@ -575,7 +579,7 @@ int gpsim_open(Processor *cpu, const char *pFileName,
     printf (" gpsim_open file:%s proc name:%s\n", pFileName, (pProcessorName ? pProcessorName : "nil"));
   // Check for the command file, file extension.
   if(IsFileExtension(pFileName,"stc") || IsFileExtension(pFileName,"STC")) {
-    process_command_file(pFileName);
+    process_command_file(pFileName,true);
     // A stc file could have any sequence of commands.
     // Just ignore the return value of parse_string().
     parse_string("\n");
@@ -623,10 +627,8 @@ gpsim_read (char *buf, unsigned max_size)
   strncpy(buf, cPstr, count);
   buf[count] = 0;
   SetLastFullCommand(buf);
-  if(s_bSTCEcho) {
+  if(*s_bSTCEcho)
     cout << cPstr;
-  }
-
 
   if(verbose&4) {
     cout <<"gpsim_read returning " << count << ":" << cPstr << endl;
@@ -701,8 +703,8 @@ command_generator (const char *text, int state)
   while( i<number_of_commands)
     {
 
-      if(strstr(command_list[i]->name, text) == command_list[i]->name)
-	return(g_strndup(command_list[i++]->name, cMaxStringLen));
+      if(strstr(command_list[i]->name(), text) == command_list[i]->name())
+        return(g_strndup(command_list[i++]->name(), cMaxStringLen));
 
       i++;
     }
@@ -742,12 +744,12 @@ gpsim_completion (const char *text, int start, int end)
 #endif
 
 #if defined(_WIN32) || defined(WIN32)
-	if (start) {
-		char *empty = strdup("");
-		matches = (char **) malloc(2 * (sizeof(&empty)));
-		matches[0] = empty;
-		matches[1] = 0;
-	}
+        if (start) {
+                char *empty = strdup("");
+                matches = (char **) malloc(2 * (sizeof(&empty)));
+                matches[0] = empty;
+                matches[1] = 0;
+        }
 #endif
 
   return (matches);
@@ -816,7 +818,7 @@ void have_line(char *s)
 
 /**********************************************************************
  **/
-void exit_gpsim(void)
+void exit_cli(void)
 {
   if(get_use_icd())
     icd_disconnect();
@@ -837,8 +839,10 @@ void exit_gpsim(void)
 #ifdef HAVE_SOCKETS
   stop_server();
 #endif
+  globalSymbolTable().deleteSymbol("CliTrace");
+  cout << "Exiting gpsim\n";
+  simulation_cleanup();
 
-  exit(0);
 }
 
 /* redisplay_prompt will redisplay the current data in the readline buffer.
@@ -945,7 +949,7 @@ void EnableKeypressHook(bool bEnable) {
 // used by symbol files that embed gpsim scripts. See src/modules.cc.
 //
 
-char *CCliCommandHandler::GetName()
+const char *CCliCommandHandler::GetName()
 {
   return "gpsimCLI";
 }
