@@ -44,20 +44,19 @@ Boston, MA 02111-1307, USA.  */
 #else
 #include <winsock2.h>
 #endif
+void exit_gpsim(int);
 
 
 //******** E X P E R I M E N T A L   C O D E !!! *************************
 // Here is an experimental attribute for testing the socket interface.
 // The purpose is to isolate socket testing from the rest of the simulator.
 //
-class TestInt32Array : public symbol
+class TestInt32Array : public Value
 {
 public:
   TestInt32Array(const char *_name, int _sz)
-    : symbol(_name)
+    : Value(_name, " test array for testing sockets")
   {
-    set_description(" test array for testing sockets");
-
     size = _sz;
     array = new int[size];
   }
@@ -154,7 +153,7 @@ bool winsockets_init(void)
   things like sockets so that they work nicely when compiled for other
   platforms. Now, whenever a client connects to gpsim, another GIOChannel
   is created. This new channel has the client socket associated with
-  it and is used to communicate with the client. 
+  it and is used to communicate with the client.
 
   Thus, like most socket server applications, gpsim creates a single
   socket to listen for clients. As clients come along, connections are
@@ -166,7 +165,7 @@ bool winsockets_init(void)
   command line commands that are directed to the command line parser.
   Although, at the moment, the output of these commands still is sent
   to the terminal from which the gpsim executable was spawned (instead
-  of being sent to the client socket...). Probably more powerful though 
+  of being sent to the client socket...). Probably more powerful though
   is the ability for a client to establish 'links' to gpsim internal
   objects. These links are provide a client with an efficient access
   to gpsim's symbol table.
@@ -183,7 +182,7 @@ bool winsockets_init(void)
   time except when the simulation is running). So the idea is that
   clients can send stuff to the simulator, and if the simulator is not
   running, the information is processed and a response is returned to
-  the client. 
+  the client.
 
   Simulator Sources are for sending unsolicited simulator data to a
   client. This is a blocking operation, i.e. it will not return until
@@ -199,7 +198,7 @@ bool winsockets_init(void)
 #define SIM_SINK_PORT   0x1234
 #define SIM_SOURCE_PORT 0x1235
 
-#define BUFSIZE 	8192
+#define BUFSIZE         8192
 //------------------------------------------------------------------------
 bool ParseSocketLink(Packet *buffer, SocketLink **);
 
@@ -226,9 +225,9 @@ public:
 
 //------------------------------------------------------------------------
 // Socket wrapper class
-// 
+//
 // This class is a simple wrapper around the standard BSD socket calls.
-// 
+//
 
 
 class SocketBase
@@ -239,7 +238,7 @@ public:
 
   SOCKET getSocket();
   void Close();
-  bool Send(char *);
+  bool Send(const char *);
   void Service();
   void ParseObject();
 
@@ -281,6 +280,10 @@ class SocketLink
 public:
   SocketLink(unsigned int _handle, SocketBase *);
 
+  virtual ~SocketLink()
+  {
+  }
+
   /// Send a response back to the link.
   /// if the bTimeStamp is true, then the cycle counter is sent too.
   bool Send(bool bTimeStamp=false);
@@ -313,6 +316,10 @@ class AttributeLink : public SocketLink
 public:
   AttributeLink(unsigned int _handle, SocketBase *, Value *);
 
+  virtual ~AttributeLink()
+  {
+  }
+
   void set(Packet &);
   void get(Packet &);
 
@@ -459,7 +466,7 @@ void Socket::init(int port)
 
   if ((new_socket = socket(AF_INET, SOCK_STREAM, 0)) == INVALID_SOCKET) {
     psocketerror("socket");
-    exit(1);
+    exit_gpsim(1);
   }
 
   my_socket = new SocketBase(new_socket);
@@ -467,14 +474,14 @@ void Socket::init(int port)
   int on = 1;
   if ( setsockopt ( new_socket, SOL_SOCKET, SO_REUSEADDR, ( const char* ) &on, sizeof ( on ) ) != 0 ) {
     psocketerror("setsockopt");
-    exit(1);
+    exit_gpsim(1);
   }
 
   memset(&addr, 0, sizeof(addr));
   addr.sin_family = AF_INET;
   addr.sin_addr.s_addr = INADDR_ANY;
   addr.sin_port = htons(port);
-  
+
   Bind();
   Listen();
 }
@@ -493,16 +500,16 @@ void Socket::AssignChannel(gboolean (*server_function)(GIOChannel *,GIOCondition
 
     stat = g_io_channel_set_encoding (channel, NULL, &err);
 #if !defined _WIN32 || defined _DEBUG
-	stat = g_io_channel_set_flags (channel, G_IO_FLAG_SET_MASK, &err);
+        stat = g_io_channel_set_flags (channel, G_IO_FLAG_SET_MASK, &err);
 #endif
 #endif
 
-    g_io_add_watch(channel, 
-		   condition,
-		   server_function,
-		   (void*)this);
+    g_io_add_watch(channel,
+                   condition,
+                   server_function,
+                   (void*)this);
   }
- 
+
 }
 
 void Socket::Close(SocketBase **sock)
@@ -544,14 +551,14 @@ SocketBase *Socket::Accept()
 
   if (client_socket == INVALID_SOCKET) {
     psocketerror("accept");
-    exit(1);
+    exit_gpsim(1);
   }
 
   return new SocketBase(client_socket);
 }
 
 
-bool SocketBase::Send(char *b)
+bool SocketBase::Send(const char *b)
 {
   if(!socket)
     return false;
@@ -594,11 +601,11 @@ void SocketBase::ParseObject()
       AttributeLink *sl = gCreateSocketLink(handle, *packet, this);
 
       if(sl) {
-	links[handle&0x0f] = sl;
-	packet->EncodeHeader();
-	packet->EncodeUInt32(handle);
-	packet->txTerminate();
-	Send(packet->txBuff());
+        links[handle&0x0f] = sl;
+        packet->EncodeHeader();
+        packet->EncodeUInt32(handle);
+        packet->txTerminate();
+        Send(packet->txBuff());
       }
     }
     break;
@@ -609,18 +616,18 @@ void SocketBase::ParseObject()
       AttributeLink *sl = gCreateSocketLink(handle, *packet, this);
 
       if(sl) {
-	unsigned int i=0;
-	
-	if(packet->DecodeUInt32(i) && i) 
-	  sl->setBlocking(true);
+        unsigned int i=0;
 
-	//NotifyLink *nl = new NotifyLink(sl);
+        if(packet->DecodeUInt32(i) && i)
+          sl->setBlocking(true);
 
-	links[handle&0x0f] = sl;
-	packet->EncodeHeader();
-	packet->EncodeUInt32(handle);
-	packet->txTerminate();
-      	Send(packet->txBuff());
+        //NotifyLink *nl = new NotifyLink(sl);
+
+        links[handle&0x0f] = sl;
+        packet->EncodeHeader();
+        packet->EncodeUInt32(handle);
+        packet->txTerminate();
+        Send(packet->txBuff());
       }
     }
     break;
@@ -632,15 +639,15 @@ void SocketBase::ParseObject()
       std::cout << "Creating callback link\n";
       if(packet->DecodeUInt64(interval) && interval) {
 
-	std::cout << "Creating callback link interval=" << interval << endl;
+        std::cout << "Creating callback link interval=" << interval << endl;
 
-	//CyclicCallBackLink *cl = new CyclicCallBackLink(interval,this);
+        //CyclicCallBackLink *cl = new CyclicCallBackLink(interval,this);
 
-	//links[handle&0x0f] = sl;
-	packet->EncodeHeader();
-	packet->EncodeUInt32(handle);
-	packet->txTerminate();
-      	Send(packet->txBuff());
+        //links[handle&0x0f] = sl;
+        packet->EncodeHeader();
+        packet->EncodeUInt32(handle);
+        packet->txTerminate();
+        Send(packet->txBuff());
       }
     }
     break;
@@ -652,7 +659,7 @@ void SocketBase::ParseObject()
       ParseSocketLink(packet, &sl);
 
       if(sl)
-	CloseSocketLink(sl);
+        CloseSocketLink(sl);
       Send("$");
     }
     break;
@@ -663,8 +670,8 @@ void SocketBase::ParseObject()
 
       ParseSocketLink(packet, &sl);
 
-      if(sl) 
-	sl->Send();
+      if(sl)
+        sl->Send();
     }
     break;
 
@@ -674,8 +681,8 @@ void SocketBase::ParseObject()
 
       ParseSocketLink(packet, &sl);
       if(sl) {
-	sl->set(*packet);
-	Send("$");
+        sl->set(*packet);
+        Send("$");
       }
 
     }
@@ -687,15 +694,15 @@ void SocketBase::ParseObject()
 
       if(packet->DecodeString(tmp,256)) {
 
-	Value *sym = get_symbol_table().find(tmp);
-	if(sym) {
-	  packet->EncodeHeader();
-	  sym->get(*packet);
-	  packet->txTerminate();
-	  Send(packet->txBuff());
+        Value *sym = globalSymbolTable().findValue(tmp);
+        if(sym) {
+          packet->EncodeHeader();
+          sym->get(*packet);
+          packet->txTerminate();
+          Send(packet->txBuff());
 
-	} else
-	  Send("-");
+        } else
+          Send("-");
       }
     }
     break;
@@ -707,29 +714,29 @@ void SocketBase::ParseObject()
       // The client has requested to write directly to a symbol
       // (without using a SocketLink). So, we'll extract the symbol
       // name from the packet and look it up in the symbol table.
-      // If the symbol is found then, it will get assigned the 
+      // If the symbol is found then, it will get assigned the
       // value that's stored in the packet.
 
       char tmp[256];
 
       if(packet->DecodeString(tmp,256)) {
 
-	Value *sym = get_symbol_table().find(tmp);
+        Value *sym = globalSymbolTable().findValue(tmp);
 
-	if(sym) {
+        if(sym) {
 
-	  // We'll define the response header before we decode
-	  // the rest of the packet. The reason for this is that
-	  // some symbols may wish to generate a response.
+          // We'll define the response header before we decode
+          // the rest of the packet. The reason for this is that
+          // some symbols may wish to generate a response.
 
-	  packet->EncodeHeader();
-	  sym->set(*packet);
-	  packet->txTerminate();
+          packet->EncodeHeader();
+          sym->set(*packet);
+          packet->txTerminate();
 
-	  Send(packet->txBuff());
+          Send(packet->txBuff());
 
-	} else
-	  Send("-");
+        } else
+          Send("-");
       }
     }
     break;
@@ -744,10 +751,10 @@ void SocketBase::ParseObject()
       // a cycle breakpoint; otherwise we'll run forever.
 
       if(packet->DecodeUInt64(nCycles)) {
-	guint64 fc = startCycle + nCycles;
-	if(nCycles) {
-	  get_bp().set_cycle_break(0,fc);
-	}
+        guint64 fc = startCycle + nCycles;
+        if(nCycles) {
+          get_bp().set_cycle_break(0,fc);
+        }
 
       }
 
@@ -772,7 +779,7 @@ void SocketBase::ParseObject()
     get_interface().reset();
     Send("-");
     break;
-    
+
   default:
     printf("Invalid object type: %d\n",ObjectType);
     Send("-");
@@ -802,9 +809,9 @@ void SocketBase::Service()
       ParseObject();
     } else {
       if(parse_string(packet->rxBuff()) >= 0)
-	Send("+ACK");
+        Send("+ACK");
       else
-	Send("+BUSY");
+        Send("+BUSY");
     }
   }
 
@@ -851,20 +858,20 @@ bool SocketLink::Send(bool bTimeStamp)
     parent->packet->EncodeHeader();
     get(*parent->packet);
     if(bTimeStamp)
-      parent->packet->EncodeUInt64(get_cycles().value);
+      parent->packet->EncodeUInt64(get_cycles().get());
     parent->packet->txTerminate();
 
     /*
     std::cout << "SocketLink::Send() "
-	      << " socket=" << parent->getSocket()
-	      << " sending " 
-	      << parent->packet->txBuff() << endl;
+              << " socket=" << parent->getSocket()
+              << " sending "
+              << parent->packet->txBuff() << endl;
     */
 
     if(bWaitForResponse) {
       //std::cout << "SocketLink::Send waiting for response\n";
       if(parent->Send(parent->packet->txBuff()))
-	return Receive();
+        return Receive();
     } else
       return parent->Send(parent->packet->txBuff());
   }
@@ -880,12 +887,12 @@ bool SocketLink::Receive()
     parent->packet->prepare();
 
     int bytes= recv(parent->getSocket(),
-		    parent->packet->rxBuff(), 
-		    parent->packet->rxSize(), 0);
+                    parent->packet->rxBuff(),
+                    parent->packet->rxSize(), 0);
 
     if (bytes  == -1) {
       perror("recv");
-      exit(1);
+      exit_gpsim(1);
     }
 
     parent->packet->rxTerminate(bytes);
@@ -925,7 +932,7 @@ NotifyLink::NotifyLink(AttributeLink *_sl)
   if(sl && sl->getValue()) {
     Value *v = sl->getValue();
     std::cout<< "Creating a notify link and asoc with "<< v->name()<<endl;
-    sl->getValue()->set_xref(this);
+    //FIXME: removed 27JAN06 sl->getValue()->set_xref(this);
   }
 }
 void NotifyLink::set(gint64 i)
@@ -942,7 +949,7 @@ CyclicCallBackLink::CyclicCallBackLink(guint64 i, SocketBase *_sb)
   : interval(i), sb(_sb)
 {
   std::cout << " cyclic callback object\n ";
-  get_cycles().set_break(get_cycles().value + interval, this);
+  get_cycles().set_break(get_cycles().get() + interval, this);
 }
 
 void CyclicCallBackLink::callback(void)
@@ -965,7 +972,7 @@ void CyclicCallBackLink::callback(void)
       st[3] = '0';
 
     if(sb->Send(st))
-      get_cycles().set_break(get_cycles().value + interval, this);
+      get_cycles().set_break(get_cycles().get() + interval, this);
     else
       std::cout << "socket callback failed seq:" << seq++ << endl;
   }
@@ -986,12 +993,12 @@ AttributeLink *gCreateSocketLink(unsigned int handle, Packet &p, SocketBase *sb)
 
   if(p.DecodeString(tmp,256)) {
 
-    Value *sym = get_symbol_table().find(tmp);
+    Value *sym = globalSymbolTable().findValue(tmp);
     if(sym)
       return new AttributeLink(handle,sb,sym);
   }
 
-  return 0; 
+  return 0;
 }
 
 //////////////////////////////////////////////////////////////////////////
@@ -1072,8 +1079,8 @@ static void debugPrintCondition(GIOCondition cond)
   server_callback ()
 
   This call back function is invoked from the GTK main loop whenever a client
-  socket has sent something to send to gpsim. This callback will only be 
-  invoked for those clients that have already connected to gpsim. Those 
+  socket has sent something to send to gpsim. This callback will only be
+  invoked for those clients that have already connected to gpsim. Those
   connected clients have associated with them a 'SocketBase' object. A pointer
   to that object is passed in the server_callback parameters.
 
@@ -1084,9 +1091,9 @@ static void debugPrintCondition(GIOCondition cond)
 
  */
 
-static gboolean server_callback(GIOChannel *channel, 
-				GIOCondition condition, 
-				void *pSocketBase )
+static gboolean server_callback(GIOChannel *channel,
+                                GIOCondition condition,
+                                void *pSocketBase )
 {
 
 
@@ -1121,14 +1128,14 @@ static gboolean server_callback(GIOChannel *channel,
 #if !defined _WIN32 || defined _DEBUG
     g_io_channel_set_flags (channel, G_IO_FLAG_NONBLOCK, &err);
 #endif
-    stat = g_io_channel_read_chars(channel, 
-				   s->packet->rxBuff(), 
-				   s->packet->rxSize(), 
-				   &b, 
-				   &err);
+    stat = g_io_channel_read_chars(channel,
+                                   s->packet->rxBuff(),
+                                   s->packet->rxSize(),
+                                   &b,
+                                   &err);
     bytes_read = b;
     s->packet->rxAdvance(bytes_read);
- 
+
     if(err) {
       std::cout << "GError:" << err->message << endl;
     }
@@ -1140,10 +1147,10 @@ static gboolean server_callback(GIOChannel *channel,
 
     if(bytes_read) {
       if(get_interface().bSimulating()) {
-	std::cout << "setting a socket break point because sim is running \n";
-	get_bp().set_socket_break();
+        std::cout << "setting a socket break point because sim is running \n";
+        get_bp().set_socket_break();
       } else
-	s->Service();
+        s->Service();
     } else
       return FALSE;
 
@@ -1193,10 +1200,10 @@ static gboolean sink_server_accept(GIOChannel *channel, GIOCondition condition,
 #endif
 #endif
 
-  g_io_add_watch(new_channel, 
-		 new_condition,
-		 server_callback,
-		 (void*)client);
+  g_io_add_watch(new_channel,
+                 new_condition,
+                 server_callback,
+                 (void*)client);
 
   return TRUE;
 }
@@ -1214,16 +1221,16 @@ static gboolean source_server_accept(GIOChannel *channel, GIOCondition condition
   if(!client)
     return FALSE;
 
-  int bytes= recv(client->getSocket(), 
-		  client->packet->rxBuff(), 
-		  client->packet->rxSize(), 0);
+  int bytes= recv(client->getSocket(),
+                  client->packet->rxBuff(),
+                  client->packet->rxSize(), 0);
 
-  std::cout << " SourceServer received data" << client->packet->rxBuff() 
-	    << endl;
+  std::cout << " SourceServer received data" << client->packet->rxBuff()
+            << endl;
 
   if (bytes  == -1) {
     perror("recv");
-    exit(1);
+    exit_gpsim(1);
   }
 
   client->packet->rxAdvance(bytes);
