@@ -67,15 +67,15 @@ GtkItemFactory *item_factory=0;
 
 extern GUI_Processor *gpGuiProcessor;
 
-static void 
-do_quit_app(GtkWidget *widget) 
+static void
+do_quit_app(GtkWidget *widget)
 {
-	exit_gpsim();
+        exit_gpsim(0);
 }
 
 
 static void
-show_message (char *title, char *message)
+show_message (const char *title, const char *message)
 {
   GtkWidget *window;
   GtkWidget *label;
@@ -90,8 +90,8 @@ show_message (char *title, char *message)
   button = gtk_button_new_with_label ("close");
   gtk_container_set_border_width (GTK_CONTAINER (button), 10);
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			     GTK_SIGNAL_FUNC(gtk_widget_destroy),
-			     GTK_OBJECT (window));
+                             GTK_SIGNAL_FUNC(gtk_widget_destroy),
+                             GTK_OBJECT (window));
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window)->action_area), button, TRUE, TRUE, 0);
   gtk_widget_grab_default(button);
@@ -109,18 +109,18 @@ show_message (char *title, char *message)
 
 }
 //========================================================================
-static void 
+static void
 about_cb (gpointer             callback_data,
-	  guint                callback_action,
-	  GtkWidget           *widget)
+          guint                callback_action,
+          GtkWidget           *widget)
 {
 
   show_message(  "The GNUPIC Simulator - " VERSION, "A simulator for Microchip PIC microcontrollers.\n"
-		 "by T. Scott Dattalo - mailto:scott@dattalo.com\n"
-		 "   Ralf Forsberg - mailto:rfg@home.se\n"
-		 "   Borut Ra" "\xc5\xbe" "em - mailto:borut.razem@siol.net\n\n"
-		 "gpsim homepage: http://www.dattalo.com/gnupic/gpsim.html\n"
-		 "gpsimWin32: http://gpsim.sourceforge.net/gpsimWin32/gpsimWin32.html\n");
+                 "by T. Scott Dattalo - mailto:scott@dattalo.com\n"
+                 "   Ralf Forsberg - mailto:rfg@home.se\n"
+                 "   Borut Ra" "\xc5\xbe" "em - mailto:borut.razem@siol.net\n\n"
+                 "gpsim homepage: http://gpsim.sourceforge.net/gpsim.html\n"
+                 "gpsimWin32: http://gpsim.sourceforge.net/gpsimWin32/gpsimWin32.html\n");
 
 }
 
@@ -139,19 +139,19 @@ about_cb (gpointer             callback_data,
 //
 // Creates a GtkColorButton and places it into a parent widget.
 // When the color button is clicked and changed, the signal will
-// call back into this class and keep track of the selected 
+// call back into this class and keep track of the selected
 // color state.
 class SourceBrowserPreferences;
 class ColorButton
 {
 public:
-  ColorButton (GtkWidget *pParent, 
-	       TextStyle *pStyle,
-	       const char *label,
-	       SourceBrowserPreferences *
-	       );
+  ColorButton (GtkWidget *pParent,
+               TextStyle *pStyle,
+               const char *label,
+               SourceBrowserPreferences *
+               );
   static void setColor_cb(GtkColorButton *widget,
-			  ColorButton    *This);
+                          ColorButton    *This);
   void apply();
   void cancel();
   TextStyle *m_pStyle;
@@ -176,12 +176,12 @@ public:
     eOpcodes
   };
   MarginButton (GtkWidget *pParent,
-		const char *pName,
-		eMarginType id,
-		SourceBrowserPreferences *
-	       );
+                const char *pName,
+                eMarginType id,
+                SourceBrowserPreferences *
+               );
   static void toggle_cb(GtkToggleButton *widget,
-			MarginButton    *This);
+                        MarginButton    *This);
   void set_active();
 private:
   GtkWidget *m_button;
@@ -200,11 +200,11 @@ class TabButton
 {
 public:
   TabButton (GtkWidget *pParent, GtkWidget *pButton,
-	     int id,
-	     SourceBrowserPreferences *
-	     );
+             int id,
+             SourceBrowserPreferences *
+             );
   static void toggle_cb(GtkToggleButton *widget,
-			TabButton    *This);
+                        TabButton    *This);
   void set_active();
 private:
   GtkWidget *m_button;
@@ -220,11 +220,11 @@ private:
 class FontSelection
 {
 public:
-  FontSelection (GtkWidget *pParent, 
-		 SourceBrowserPreferences *
-		 );
+  FontSelection (GtkWidget *pParent,
+                 SourceBrowserPreferences *
+                 );
   static void setFont_cb(GtkFontButton *widget,
-			FontSelection  *This);
+                        FontSelection  *This);
   void setFont();
 private:
   SourceBrowserPreferences *m_prefs;
@@ -282,8 +282,8 @@ public:
   ~gpsimGuiPreferences();
 
   static void setup (gpointer             callback_data,
-		     guint                callback_action,
-		     GtkWidget           *widget);
+                     guint                callback_action,
+                     GtkWidget           *widget);
 
 
 private:
@@ -299,8 +299,8 @@ static GtkWidget *LocalWindow=0;
 
 
 void gpsimGuiPreferences::setup (gpointer             callback_data,
-				 guint                callback_action,
-				 GtkWidget           *widget)
+                                 guint                callback_action,
+                                 GtkWidget           *widget)
 {
   new gpsimGuiPreferences();
 }
@@ -324,7 +324,7 @@ gint gpsimGuiPreferences::apply_cb  (gpsimGuiPreferences *Self)
 //------------------------------------------------------------------------
 // ColorButton Constructor
 ColorButton::ColorButton(GtkWidget *pParent, TextStyle *pStyle,
-			 const char *colorName,SourceBrowserPreferences *prefs)
+                         const char *colorName,SourceBrowserPreferences *prefs)
   : m_pStyle(pStyle),m_prefs(prefs), m_label(colorName)
 {
   GtkWidget *hbox        = gtk_hbox_new(0,0);
@@ -335,10 +335,10 @@ ColorButton::ColorButton(GtkWidget *pParent, TextStyle *pStyle,
   gtk_box_pack_start (GTK_BOX(hbox),colorButton,FALSE, FALSE, 0);
   gtk_widget_show(colorButton);
 
-  gtk_signal_connect (GTK_OBJECT(colorButton), 
-		      "color-set", 
-		      GTK_SIGNAL_FUNC(setColor_cb),
-		      this);
+  gtk_signal_connect (GTK_OBJECT(colorButton),
+                      "color-set",
+                      GTK_SIGNAL_FUNC(setColor_cb),
+                      this);
   const int cBORDER = 10; // pixels
   GtkWidget *label       = gtk_label_new(colorName);
   gtk_box_pack_start (GTK_BOX(hbox),label,FALSE,FALSE, cBORDER);
@@ -349,7 +349,7 @@ ColorButton::ColorButton(GtkWidget *pParent, TextStyle *pStyle,
 
 //------------------------------------------------------------------------
 void ColorButton::setColor_cb(GtkColorButton *widget,
-			      ColorButton    *This)
+                              ColorButton    *This)
 {
   GdkColor newColor;
   gtk_color_button_get_color (widget, &newColor);
@@ -367,8 +367,8 @@ void ColorButton::cancel()
 
 //------------------------------------------------------------------------
 MarginButton::MarginButton(GtkWidget *pParent, const char *pName,
-			   eMarginType id,
-			   SourceBrowserPreferences *prefs)
+                           eMarginType id,
+                           SourceBrowserPreferences *prefs)
   : m_prefs(prefs), m_id(id)
 {
   m_button = gtk_check_button_new_with_label (pName);
@@ -385,17 +385,17 @@ MarginButton::MarginButton(GtkWidget *pParent, const char *pName,
     break;
   }
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(m_button),
-				bState);
+                                bState);
   gtk_box_pack_start (GTK_BOX (pParent), m_button, FALSE, TRUE, 10);
 
   gtk_signal_connect (GTK_OBJECT(m_button),
-		      "toggled", 
-		      GTK_SIGNAL_FUNC(toggle_cb),
-		      this);
+                      "toggled",
+                      GTK_SIGNAL_FUNC(toggle_cb),
+                      this);
 }
 //------------------------------------------------------------------------
 void MarginButton::toggle_cb(GtkToggleButton *widget,
-			     MarginButton    *This)
+                             MarginButton    *This)
 {
   //  This->m_pStyle->setFG(&newColor);
   This->set_active();
@@ -419,20 +419,20 @@ void MarginButton::set_active()
 
 //------------------------------------------------------------------------
 TabButton::TabButton(GtkWidget *pParent, GtkWidget *pButton,
-		     int id,
-		     SourceBrowserPreferences *prefs)
+                     int id,
+                     SourceBrowserPreferences *prefs)
   : m_button(pButton), m_prefs(prefs), m_id(id)
 {
   gtk_box_pack_start (GTK_BOX (pParent), m_button, FALSE, TRUE, 5);
 
   gtk_signal_connect (GTK_OBJECT(m_button),
-		      "toggled", 
-		      GTK_SIGNAL_FUNC(toggle_cb),
-		      this);
+                      "toggled",
+                      GTK_SIGNAL_FUNC(toggle_cb),
+                      this);
 }
 //------------------------------------------------------------------------
 void TabButton::toggle_cb(GtkToggleButton *widget,
-			  TabButton    *This)
+                          TabButton    *This)
 {
   //  This->m_pStyle->setFG(&newColor);
   This->set_active();
@@ -445,7 +445,7 @@ void TabButton::set_active()
 
 //------------------------------------------------------------------------
 FontSelection::FontSelection (GtkWidget *pParent,
-			      SourceBrowserPreferences *pPrefs)
+                              SourceBrowserPreferences *pPrefs)
   : m_prefs(pPrefs)
 {
   GtkWidget *frame = gtk_frame_new ("Font");
@@ -460,10 +460,10 @@ FontSelection::FontSelection (GtkWidget *pParent,
   gtk_box_pack_start (GTK_BOX(hbox),m_fontButton,FALSE, FALSE, 0);
   gtk_widget_show(m_fontButton);
 
-  gtk_signal_connect (GTK_OBJECT(m_fontButton), 
-		      "font-set", 
-		      GTK_SIGNAL_FUNC(setFont_cb),
-		      this);
+  gtk_signal_connect (GTK_OBJECT(m_fontButton),
+                      "font-set",
+                      GTK_SIGNAL_FUNC(setFont_cb),
+                      this);
 
   const char *fontName = "font";
   const int cBORDER = 10; // pixels
@@ -475,7 +475,7 @@ FontSelection::FontSelection (GtkWidget *pParent,
 }
 //------------------------------------------------------------------------
 void FontSelection::setFont_cb (GtkFontButton *pFontButton,
-				FontSelection *This)
+                                FontSelection *This)
 {
   This->setFont();
 }
@@ -488,7 +488,7 @@ void FontSelection::setFont()
 #if 0 // defined but not used
 static bool isButtonEvent (GdkEventType type)
 {
-  return 
+  return
     type == GDK_BUTTON_PRESS ||
     type == GDK_2BUTTON_PRESS ||
     type == GDK_3BUTTON_PRESS ||
@@ -497,19 +497,19 @@ static bool isButtonEvent (GdkEventType type)
 }
 
 static gboolean    TagEvent  (GtkTextTag *texttag,
-		       GObject *arg1,
-		       GdkEvent *event,
-		       GtkTextIter *arg2,
-		       gpointer user_data)
+                       GObject *arg1,
+                       GdkEvent *event,
+                       GtkTextIter *arg2,
+                       gpointer user_data)
 {
   printf("Received tag event signal Tag:%p arg1:%p Event:%p iter:%p user:%p %08X Line:%d\n",
-	 texttag, arg1,event,arg2, user_data, event->type, gtk_text_iter_get_line(arg2));
+         texttag, arg1,event,arg2, user_data, event->type, gtk_text_iter_get_line(arg2));
 
   if (isButtonEvent(event->type)) {
     GdkEventButton *evtButton = (GdkEventButton *) event;
 
     printf("Button Event: button:%d  modifier:%d coords(%g,%g)\n",
-	   evtButton->button, evtButton->state, evtButton->x,evtButton->y);
+           evtButton->button, evtButton->state, evtButton->x,evtButton->y);
     // If the right mouse button is pressed then suppress the GTK pop up menu.
     if (evtButton->button == 3)
       return TRUE;
@@ -519,7 +519,7 @@ static gboolean    TagEvent  (GtkTextTag *texttag,
 //------------------------------------------------------------------------
 
 static void preferences_AddFontSelect(GtkWidget *pParent, const char *fontDescription,
-				      const char *fontName  )
+                                      const char *fontName  )
 {
 }
 #endif
@@ -563,21 +563,21 @@ SourceBrowserPreferences::SourceBrowserPreferences(GtkWidget *pParent)
     GtkWidget *colorVbox = gtk_vbox_new(0,0);
     gtk_container_add (GTK_CONTAINER (colorFrame), colorVbox);
 
-    m_LabelColor    = new ColorButton(colorVbox, 
-				      m_pParent->mLabel,
-				      "Label", this);
-    m_MnemonicColor = new ColorButton(colorVbox, 
-				      m_pParent->mMnemonic,
-				      "Mnemonic", this);
-    m_SymbolColor   = new ColorButton(colorVbox, 
-				      m_pParent->mSymbol,
-				      "Symbols", this);
+    m_LabelColor    = new ColorButton(colorVbox,
+                                      m_pParent->mLabel,
+                                      "Label", this);
+    m_MnemonicColor = new ColorButton(colorVbox,
+                                      m_pParent->mMnemonic,
+                                      "Mnemonic", this);
+    m_SymbolColor   = new ColorButton(colorVbox,
+                                      m_pParent->mSymbol,
+                                      "Symbols", this);
     m_ConstantColor = new ColorButton(colorVbox,
-				      m_pParent->mConstant,
-				      "Constants", this);
+                                      m_pParent->mConstant,
+                                      "Constants", this);
     m_CommentColor  = new ColorButton(colorVbox,
-				      m_pParent->mComment,
-				      "Comments", this);
+                                      m_pParent->mComment,
+                                      "Comments", this);
 
     // Font selector
     //preferences_AddFontSelect(GTK_WIDGET(vbox), "Font Selector", "font");
@@ -607,27 +607,27 @@ SourceBrowserPreferences::SourceBrowserPreferences(GtkWidget *pParent)
 
     m_Up    = new TabButton(tabVbox, radioUp, GTK_POS_TOP, this);
     m_Left  = new TabButton(tabVbox, gtk_radio_button_new_with_label_from_widget (rb,"left"),
-			   GTK_POS_LEFT, this);
+                           GTK_POS_LEFT, this);
     m_Down  = new TabButton(tabVbox, gtk_radio_button_new_with_label_from_widget (rb,"down"),
-			   GTK_POS_BOTTOM, this);
+                           GTK_POS_BOTTOM, this);
     m_Right = new TabButton(tabVbox, gtk_radio_button_new_with_label_from_widget (rb,"right"),
-			   GTK_POS_RIGHT, this);
+                           GTK_POS_RIGHT, this);
     m_None  = new TabButton(tabVbox, gtk_radio_button_new_with_label_from_widget (rb,"none"),
-			   -1, this);
+                           -1, this);
 
 
-    // Source browser margin 
+    // Source browser margin
     GtkWidget *marginFrame = gtk_frame_new ("Margin");
     gtk_box_pack_start (GTK_BOX (hbox), marginFrame, FALSE, TRUE, 0);
     GtkWidget *marginVbox = gtk_vbox_new(0,0);
     gtk_container_add (GTK_CONTAINER (marginFrame), marginVbox);
 
-    m_LineNumbers = new MarginButton(marginVbox, "Line Numbers", 
-				     MarginButton::eLineNumbers, this);
-    m_Addresses   = new MarginButton(marginVbox, "Addresses", 
-				     MarginButton::eAddresses, this);
-    m_Opcodes     = new MarginButton(marginVbox, "Opcodes", 
-				     MarginButton::eOpcodes, this);
+    m_LineNumbers = new MarginButton(marginVbox, "Line Numbers",
+                                     MarginButton::eLineNumbers, this);
+    m_Addresses   = new MarginButton(marginVbox, "Addresses",
+                                     MarginButton::eAddresses, this);
+    m_Opcodes     = new MarginButton(marginVbox, "Opcodes",
+                                     MarginButton::eOpcodes, this);
 
     label = gtk_label_new("Margins");
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook),hbox,label);
@@ -640,7 +640,7 @@ SourceBrowserPreferences::SourceBrowserPreferences(GtkWidget *pParent)
 
     GtkWidget *frame = gtk_frame_new ("Sample");
     gtk_box_pack_start (GTK_BOX (pParent), frame, FALSE, TRUE, 0);
-    
+
     m_Notebook = gtk_notebook_new();
     //m_currentTabPosition = m_pParent->getTabPosition();
     //gtk_notebook_set_tab_pos((GtkNotebook*)m_Notebook,m_TabPosition);
@@ -656,7 +656,8 @@ SourceBrowserPreferences::SourceBrowserPreferences(GtkWidget *pParent)
     pages[id]->m_pBuffer->parseLine( "; Comment only",1);
     pages[id]->m_pBuffer->parseLine( "Label:  ADDWF  Variable,F  ; Comment",1);
 
-#if 0
+    gtk_widget_show_all(frame);
+#if 1
     label = gtk_label_new("file2.asm");
     GtkWidget *emptyBox = gtk_hbox_new(0,0);
     gtk_notebook_append_page(GTK_NOTEBOOK(m_Notebook),emptyBox,label);
@@ -760,8 +761,8 @@ gpsimGuiPreferences::gpsimGuiPreferences()
   button = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
   gtk_container_set_border_width (GTK_CONTAINER (button), 10);
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			     GTK_SIGNAL_FUNC(gpsimGuiPreferences::cancel_cb),
-			     this);
+                             GTK_SIGNAL_FUNC(gpsimGuiPreferences::cancel_cb),
+                             this);
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
 
   gtk_box_pack_start (GTK_BOX (buttonBox), button, TRUE, TRUE, 0);
@@ -770,8 +771,8 @@ gpsimGuiPreferences::gpsimGuiPreferences()
   button = gtk_button_new_from_stock (GTK_STOCK_APPLY);
   gtk_container_set_border_width (GTK_CONTAINER (button), 10);
   gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
-			     GTK_SIGNAL_FUNC(gpsimGuiPreferences::apply_cb),
-			     this);
+                             GTK_SIGNAL_FUNC(gpsimGuiPreferences::apply_cb),
+                             this);
 
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
   gtk_box_pack_start (GTK_BOX (buttonBox), button, TRUE, TRUE, 0);
@@ -779,7 +780,6 @@ gpsimGuiPreferences::gpsimGuiPreferences()
 
 
   gtk_widget_show (window);
-  gtk_grab_add (window);
 
 }
 
@@ -803,16 +803,16 @@ gpsimGuiPreferences::~gpsimGuiPreferences()
 //========================================================================
 void
 file_selection_hide_fileops (GtkWidget *widget,
-			     GtkFileSelection *fs)
+                             GtkFileSelection *fs)
 {
   gtk_file_selection_hide_fileop_buttons (fs);
 }
 
-extern int gui_message(char *message);
+extern int gui_message(const char *message);
 
 void
 file_selection_ok (GtkWidget        *w,
-		   GtkFileSelection *fs)
+                   GtkFileSelection *fs)
 {
 
   const char *file;
@@ -832,10 +832,10 @@ file_selection_ok (GtkWidget        *w,
 
 extern int gui_question(char *question, char *a, char *b);
 
-static GtkItemFactoryCallback 
+static GtkItemFactoryCallback
 fileopen_dialog(gpointer             callback_data,
-	      guint                callback_action,
-	      GtkWidget           *widget)
+              guint                callback_action,
+              GtkWidget           *widget)
 {
   static GtkWidget *window = 0;
   GtkWidget *button;
@@ -850,26 +850,26 @@ fileopen_dialog(gpointer             callback_data,
     gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_MOUSE);
 
     gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (window)->ok_button),
-			"clicked", GTK_SIGNAL_FUNC(file_selection_ok),
-			window);
+                        "clicked", GTK_SIGNAL_FUNC(file_selection_ok),
+                        window);
     gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (window)->cancel_button),
-			       "clicked", GTK_SIGNAL_FUNC(gtk_widget_hide),
-			       GTK_OBJECT (window));
-      
+                               "clicked", GTK_SIGNAL_FUNC(gtk_widget_hide),
+                               GTK_OBJECT (window));
+
     button = gtk_button_new_with_label ("Hide Fileops");
     gtk_signal_connect (GTK_OBJECT (button), "clicked",
-			(GtkSignalFunc) file_selection_hide_fileops, 
-			(gpointer) window);
-    gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (window)->action_area), 
-			button, FALSE, FALSE, 0);
+                        (GtkSignalFunc) file_selection_hide_fileops,
+                        (gpointer) window);
+    gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (window)->action_area),
+                        button, FALSE, FALSE, 0);
     gtk_widget_show (button);
 
     button = gtk_button_new_with_label ("Show Fileops");
     gtk_signal_connect (GTK_OBJECT (button), "clicked",
-			(GtkSignalFunc) gtk_file_selection_show_fileop_buttons,
-			(gpointer) window);
-    gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (window)->action_area), 
-			button, FALSE, FALSE, 0);
+                        (GtkSignalFunc) gtk_file_selection_show_fileop_buttons,
+                        (gpointer) window);
+    gtk_box_pack_start (GTK_BOX (GTK_FILE_SELECTION (window)->action_area),
+                        button, FALSE, FALSE, 0);
     gtk_widget_show (button);
   }
     gtk_widget_show (window);
@@ -880,19 +880,19 @@ fileopen_dialog(gpointer             callback_data,
 
 
 // Menuhandler for Windows menu buttons
-static GtkItemFactoryCallback 
+static GtkItemFactoryCallback
 toggle_window (gpointer             callback_data,
-	      guint                callback_action,
-	      GtkWidget           *widget)
+              guint                callback_action,
+              GtkWidget           *widget)
 {
   GtkWidget *menu_item = 0;
 
   menu_item = gtk_item_factory_get_item (item_factory,
-					 gtk_item_factory_path_from_widget (widget));
+                                         gtk_item_factory_path_from_widget (widget));
   if(gpGuiProcessor && menu_item) {
-    
+
     int view_state =  GTK_CHECK_MENU_ITEM(menu_item)->active ? VIEW_SHOW : VIEW_HIDE;
-			
+
     switch(callback_action) {
     case WT_opcode_source_window:
       gpGuiProcessor->program_memory->ChangeView(view_state);
@@ -941,43 +941,42 @@ toggle_window (gpointer             callback_data,
 
 //========================================================================
 // Button callbacks
-static void 
+static void
 runbutton_cb(GtkWidget *widget)
 {
-  if(gpGuiProcessor && gpGuiProcessor->cpu)
-    gpGuiProcessor->cpu->pma->run();
+  get_interface().start_simulation();
 }
 
-static void 
+static void
 stopbutton_cb(GtkWidget *widget)
 {
   if(gpGuiProcessor && gpGuiProcessor->cpu)
     gpGuiProcessor->cpu->pma->stop();
 }
-    
-static void 
+
+static void
 stepbutton_cb(GtkWidget *widget)
 {
-  if(gpGuiProcessor && gpGuiProcessor->cpu) 
+  if(gpGuiProcessor && gpGuiProcessor->cpu)
     gpGuiProcessor->cpu->pma->step(1);
 }
-    
-static void 
+
+static void
 overbutton_cb(GtkWidget *widget)
 {
-  if(gpGuiProcessor && gpGuiProcessor->cpu) 
+  if(gpGuiProcessor && gpGuiProcessor->cpu)
     gpGuiProcessor->cpu->pma->step_over();
 
 }
-    
-static void 
+
+static void
 finishbutton_cb(GtkWidget *widget)
 {
-  if(gpGuiProcessor && gpGuiProcessor->cpu) 
+  if(gpGuiProcessor && gpGuiProcessor->cpu)
     gpGuiProcessor->cpu->pma->finish();
 }
 
-static void 
+static void
 resetbutton_cb(GtkWidget *widget)
 {
   if(gpGuiProcessor && gpGuiProcessor->cpu)
@@ -1033,16 +1032,16 @@ gui_update_cb(GtkWidget *widget, gpointer data)
   UpdateRateMenuItem *umi = UpdateRateMenuItemIndexed[index];
   if (umi)
     umi->Select();
-  else 
+  else
     cout << "Error UpdateRateMenuItem bad index:" << index << endl;
 }
 
 UpdateRateMenuItem::UpdateRateMenuItem(GtkWidget *parent,
-				       char _id, 
-				       const char *label,
-				       int _update_rate,
-				       bool _bRealTime,
-				       bool _bWithGui)
+                                       char _id,
+                                       const char *label,
+                                       int _update_rate,
+                                       bool _bRealTime,
+                                       bool _bWithGui)
   : id(_id), bRealTime(_bRealTime), bWithGui(_bWithGui), update_rate(_update_rate)
 {
 
@@ -1084,9 +1083,9 @@ void UpdateRateMenuItem::Select()
   config_set_variable("dispatcher", "SimulationMode", id);
 
   if (0)
-    cout << "Update gui refresh: " << hex << update_rate 
-	 << " ID:" << id << "Seq no:" << menu_index
-	 << endl;
+    cout << "Update gui refresh: " << hex << update_rate
+         << " ID:" << id << "Seq no:" << menu_index
+         << endl;
 }
 
 
@@ -1096,7 +1095,6 @@ class TimeWidget;
 class TimeFormatter
 {
 public:
-
   enum eMenuID {
     eCyclesHex=0,
     eCyclesDec,
@@ -1105,12 +1103,17 @@ public:
     eSeconds,
     eHHMMSS
   } time_format;
+
   TimeFormatter(TimeWidget *_tw,GtkWidget *menu, const char*menu_text)
    : tw(_tw)
   {
     AddToMenu(menu,menu_text);
   }
 
+  virtual ~TimeFormatter()
+  {
+  }
+
   void ChangeFormat();
   void AddToMenu(GtkWidget *menu, const char*menu_text);
   virtual void Format(char *, int)=0;
@@ -1133,11 +1136,16 @@ public:
 class TimeMicroSeconds : public TimeFormatter
 {
 public:
-  TimeMicroSeconds(TimeWidget *tw, GtkWidget *menu) 
+  TimeMicroSeconds(TimeWidget *tw, GtkWidget *menu)
     : TimeFormatter(tw,menu,"MicroSeconds") {}
+
+  virtual ~TimeMicroSeconds()
+  {
+  }
+
   void Format(char *buf, int size)
   {
-    double time_db = gpGuiProcessor->cpu->get_InstPeriod() * get_cycles().value * 1e6;
+    double time_db = gpGuiProcessor->cpu->get_InstPeriod() * get_cycles().get() * 1e6;
     snprintf(buf,size, "%19.2f us",time_db);
   }
 };
@@ -1145,11 +1153,16 @@ public:
 class TimeMilliSeconds : public TimeFormatter
 {
 public:
-  TimeMilliSeconds(TimeWidget *tw, GtkWidget *menu) 
+  TimeMilliSeconds(TimeWidget *tw, GtkWidget *menu)
     : TimeFormatter(tw,menu,"MilliSeconds") {}
+
+  virtual ~TimeMilliSeconds()
+  {
+  }
+
   void Format(char *buf, int size)
   {
-    double time_db = gpGuiProcessor->cpu->get_InstPeriod() * get_cycles().value * 1e3;
+    double time_db = gpGuiProcessor->cpu->get_InstPeriod() * get_cycles().get() * 1e3;
     snprintf(buf,size, "%19.3f ms",time_db);
   }
 };
@@ -1157,11 +1170,16 @@ public:
 class TimeSeconds : public TimeFormatter
 {
 public:
-  TimeSeconds(TimeWidget *tw, GtkWidget *menu) 
+  TimeSeconds(TimeWidget *tw, GtkWidget *menu)
     : TimeFormatter(tw,menu,"Seconds") {}
+
+  virtual ~TimeSeconds()
+  {
+  }
+
   void Format(char *buf, int size)
   {
-    double time_db = gpGuiProcessor->cpu->get_InstPeriod() * get_cycles().value;
+    double time_db = gpGuiProcessor->cpu->get_InstPeriod() * get_cycles().get();
     snprintf(buf,size, "%19.3f Sec",time_db);
   }
 };
@@ -1169,11 +1187,16 @@ public:
 class TimeHHMMSS : public TimeFormatter
 {
 public:
-  TimeHHMMSS(TimeWidget *tw, GtkWidget *menu) 
+  TimeHHMMSS(TimeWidget *tw, GtkWidget *menu)
     : TimeFormatter(tw,menu,"HH:MM:SS.mmm") {}
+
+  virtual ~TimeHHMMSS()
+  {
+  }
+
   void Format(char *buf, int size)
   {
-    double time_db = gpGuiProcessor->cpu->get_InstPeriod() * get_cycles().value;
+    double time_db = gpGuiProcessor->cpu->get_InstPeriod() * get_cycles().get();
     double v=time_db;
     int hh=(int)(v/3600),mm,ss,cc;
     v-=hh*3600.0;
@@ -1188,22 +1211,32 @@ public:
 class TimeCyclesHex : public TimeFormatter
 {
 public:
-  TimeCyclesHex(TimeWidget *tw, GtkWidget *menu) 
+  TimeCyclesHex(TimeWidget *tw, GtkWidget *menu)
     : TimeFormatter(tw,menu,"Cycles (Hex)") {}
+
+  virtual ~TimeCyclesHex()
+  {
+  }
+
   void Format(char *buf, int size)
   {
-    snprintf(buf,size,"0x%016" PRINTF_INT64_MODIFIER "x",get_cycles().value);
+    snprintf(buf,size,"0x%016" PRINTF_GINT64_MODIFIER "x",get_cycles().get());
   }
 };
 
 class TimeCyclesDec : public TimeFormatter
 {
 public:
-  TimeCyclesDec(TimeWidget *tw, GtkWidget *menu) 
+  TimeCyclesDec(TimeWidget *tw, GtkWidget *menu)
     : TimeFormatter(tw,menu,"Cycles (Dec)") {}
+
+  virtual ~TimeCyclesDec()
+  {
+  }
+
   void Format(char *buf, int size)
   {
-    snprintf(buf,size,"%016" PRINTF_INT64_MODIFIER "d",get_cycles().value);
+    snprintf(buf,size,"%016" PRINTF_GINT64_MODIFIER "d",get_cycles().get());
   }
 };
 
@@ -1214,7 +1247,7 @@ cbTimeFormatActivated(GtkWidget *widget, gpointer data)
 {
   if(!widget || !data)
     return;
-    
+
   TimeFormatter *tf = static_cast<TimeFormatter *>(data);
   tf->ChangeFormat();
 }
@@ -1224,11 +1257,11 @@ cbTimeFormatPopup(GtkWidget *widget, GdkEventButton *event, TimeWidget *tw)
 {
   if(!widget || !event || !tw)
     return 0;
-  
+
   if( (event->type == GDK_BUTTON_PRESS) ) {// &&  (event->button == 3) ) {
 
     gtk_menu_popup(GTK_MENU(tw->menu), 0, 0, 0, 0,
-		   3, event->time);
+                   3, event->time);
     // It looks like we need it to avoid a selection in the entry.
     // For this we tell the entry to stop reporting this event.
     gtk_signal_emit_stop_by_name(GTK_OBJECT(tw->entry),"button_press_event");
@@ -1243,13 +1276,13 @@ void TimeFormatter::ChangeFormat()
     tw->NewFormat(this);
 }
 
-void TimeFormatter::AddToMenu(GtkWidget *menu, 
-			      const char*menu_text)
+void TimeFormatter::AddToMenu(GtkWidget *menu,
+                              const char*menu_text)
 {
   GtkWidget *item = gtk_menu_item_new_with_label(menu_text);
   gtk_signal_connect(GTK_OBJECT(item),"activate",
-		     (GtkSignalFunc) cbTimeFormatActivated,
-		     this);
+                     (GtkSignalFunc) cbTimeFormatActivated,
+                     this);
   gtk_widget_show(item);
   gtk_menu_append(GTK_MENU(menu),item);
 }
@@ -1279,9 +1312,9 @@ void TimeWidget::Create(GtkWidget *container)
 
   // Associate a callback with the user button-click actions
   gtk_signal_connect(GTK_OBJECT(entry),
-		     "button_press_event",
-		     (GtkSignalFunc) cbTimeFormatPopup,
-		     this);
+                     "button_press_event",
+                     (GtkSignalFunc) cbTimeFormatPopup,
+                     this);
 }
 
 
@@ -1314,8 +1347,8 @@ TimeWidget::TimeWidget()
 
 //========================================================================
 static int dispatcher_delete_event(GtkWidget *widget,
-				   GdkEvent  *event,
-				   gpointer data)
+                                   GdkEvent  *event,
+                                   gpointer data)
 {
     do_quit_app(0);
 
@@ -1324,8 +1357,8 @@ static int dispatcher_delete_event(GtkWidget *widget,
 
 static GtkItemFactoryCallback
 gtk_ifactory_cb (gpointer             callback_data,
-		 guint                callback_action,
-		 GtkWidget           *widget)
+                 guint                callback_action,
+                 GtkWidget           *widget)
 {
   g_message ("\"%s\" is not supported yet.", gtk_item_factory_path_from_widget (widget));
     return 0;
@@ -1344,13 +1377,13 @@ static GtkItemFactoryEntry menu_items[] =
   { "/File/sep1",        0,         (GtkItemFactoryCallback)gtk_ifactory_cb,       0, "<Separator>" },
   { "/File/_Quit",       "<control>Q", (GtkItemFactoryCallback)do_quit_app,         0 },
 
-  //  { "/_Processor",     	 0,       0,               0, "<Branch>" },
+  //  { "/_Processor",           0,       0,               0, "<Branch>" },
   //  { "/_Processor/New",   0,       (GtkItemFactoryCallback)new_processor_dialog,       0 },
   //  { "/_Processor/Delete",0,       (GtkItemFactoryCallback)gtk_ifactory_cb,       0 },
   //  { "/_Processor/Switch",0,       (GtkItemFactoryCallback)gtk_ifactory_cb,       0 },
 
 
-  //  { "/_Break",     	 0, 0,               0, "<Branch>" },
+  //  { "/_Break",       0, 0,               0, "<Branch>" },
   //  { "/_Break/Set",       0, (GtkItemFactoryCallback)gtk_ifactory_cb,       0 },
   //  { "/_Break/Clear",     0, (GtkItemFactoryCallback)gtk_ifactory_cb,       0 },
 
@@ -1388,7 +1421,7 @@ GtkWidget *dispatcher_window = 0;
 //========================================================================
 //========================================================================
 
-class MainWindow 
+class MainWindow
 {
 public:
 
@@ -1442,13 +1475,13 @@ void MainWindow::Create ()
   GtkWidget *button;
   GtkWidget *frame;
   GtkAccelGroup *accel_group;
-      
+
   int x,y,width,height;
 
   GtkWidget *update_rate_menu;
 
   int SimulationMode;
-      
+
   dispatcher_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
   if(!config_get_variable("dispatcher", "x", &x))
@@ -1461,12 +1494,12 @@ void MainWindow::Create ()
     height=1;
   gtk_window_set_default_size(GTK_WINDOW(dispatcher_window), width,height);
   gtk_widget_set_uposition(GTK_WIDGET(dispatcher_window),x,y);
-      
-      
+
+
   gtk_signal_connect (GTK_OBJECT (dispatcher_window), "delete-event",
-		      GTK_SIGNAL_FUNC (dispatcher_delete_event),
-		      0);
-      
+                      GTK_SIGNAL_FUNC (dispatcher_delete_event),
+                      0);
+
 #if GTK_MAJOR_VERSION >= 2
   accel_group = gtk_accel_group_new();
 #else
@@ -1474,59 +1507,59 @@ void MainWindow::Create ()
 #endif
   item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>", accel_group);
   gtk_object_set_data_full (GTK_OBJECT (dispatcher_window),
-			    "<main>",
-			    item_factory,
-			    (GtkDestroyNotify) gtk_object_unref);
+                            "<main>",
+                            item_factory,
+                            (GtkDestroyNotify) gtk_object_unref);
   //      gtk_accel_group_attach (accel_group, GTK_OBJECT (dispatcher_window));
   gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, 0);
-  gtk_window_set_title (GTK_WINDOW (dispatcher_window), 
-			VERSION);
+  gtk_window_set_title (GTK_WINDOW (dispatcher_window),
+                        VERSION);
   gtk_container_set_border_width (GTK_CONTAINER (dispatcher_window), 0);
-      
+
   box1 = gtk_vbox_new (FALSE, 0);
   gtk_container_add (GTK_CONTAINER (dispatcher_window), box1);
-      
+
   gtk_box_pack_start (GTK_BOX (box1),
-		      gtk_item_factory_get_widget (item_factory, "<main>"),
-		      FALSE, FALSE, 0);
+                      gtk_item_factory_get_widget (item_factory, "<main>"),
+                      FALSE, FALSE, 0);
+
 
 
-      
   buttonbox = gtk_hbox_new(FALSE,0);
   gtk_container_set_border_width (GTK_CONTAINER (buttonbox), 1);
   gtk_box_pack_start (GTK_BOX (box1), buttonbox, TRUE, TRUE, 0);
 
-      
-      
+
+
   // Buttons
   button = gtk_button_new_with_label ("step");
   gtk_signal_connect(GTK_OBJECT(button), "clicked",
-		     (GtkSignalFunc) stepbutton_cb, 0);
+                     (GtkSignalFunc) stepbutton_cb, 0);
   gtk_box_pack_start (GTK_BOX (buttonbox), button, TRUE, TRUE, 0);
-      
+
   button = gtk_button_new_with_label ("over");
   gtk_signal_connect(GTK_OBJECT(button), "clicked",
-		     (GtkSignalFunc) overbutton_cb, 0);
+                     (GtkSignalFunc) overbutton_cb, 0);
   gtk_box_pack_start (GTK_BOX (buttonbox), button, TRUE, TRUE, 0);
 
   button = gtk_button_new_with_label ("finish");
   gtk_signal_connect(GTK_OBJECT(button), "clicked",
-		     (GtkSignalFunc) finishbutton_cb, 0);
+                     (GtkSignalFunc) finishbutton_cb, 0);
   gtk_box_pack_start (GTK_BOX (buttonbox), button, TRUE, TRUE, 0);
 
   button = gtk_button_new_with_label ("run");
   gtk_signal_connect(GTK_OBJECT(button), "clicked",
-		     (GtkSignalFunc) runbutton_cb, 0);
+                     (GtkSignalFunc) runbutton_cb, 0);
   gtk_box_pack_start (GTK_BOX (buttonbox), button, TRUE, TRUE, 0);
 
   button = gtk_button_new_with_label ("stop");
   gtk_signal_connect(GTK_OBJECT(button), "clicked",
-		     (GtkSignalFunc) stopbutton_cb, 0);
+                     (GtkSignalFunc) stopbutton_cb, 0);
   gtk_box_pack_start (GTK_BOX (buttonbox), button, TRUE, TRUE, 0);
 
   button = gtk_button_new_with_label ("reset");
   gtk_signal_connect(GTK_OBJECT(button), "clicked",
-		     (GtkSignalFunc) resetbutton_cb, 0);
+                     (GtkSignalFunc) resetbutton_cb, 0);
   gtk_box_pack_start (GTK_BOX (buttonbox), button, TRUE, TRUE, 0);
 
 
@@ -1569,8 +1602,8 @@ void MainWindow::Create ()
   gtk_combo_box_set_active(GTK_COMBO_BOX(update_rate_menu), umi->menu_index);
 
   gtk_signal_connect(GTK_OBJECT(update_rate_menu),"changed",
-		     (GtkSignalFunc) gui_update_cb,
-		     (gpointer)update_rate_menu);
+                     (GtkSignalFunc) gui_update_cb,
+                     (gpointer)update_rate_menu);
 
   gtk_box_pack_start (GTK_BOX (buttonbox), frame, FALSE, FALSE, 5);
 
@@ -1591,11 +1624,11 @@ void MainWindow::Create ()
   gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 5);
   button = gtk_button_new_with_label ("Quit gpsim");
   gtk_signal_connect(GTK_OBJECT(button), "clicked",
-		     (GtkSignalFunc) do_quit_app, 0);
+                     (GtkSignalFunc) do_quit_app, 0);
 
   gtk_box_pack_start (GTK_BOX (box1), button, FALSE, TRUE, 5);
   gtk_widget_show_all (dispatcher_window);
-      
+
 
 }
 
