@@ -20,7 +20,7 @@ Boston, MA 02111-1307, USA.  */
 
 
 /*
-           S C O P E 
+           S C O P E
 
 A simple waveform viewer for gpsim. Inspired from Nuno Sucena Almeida's
 gpsim_la - plug in.
@@ -68,10 +68,10 @@ static GdkColor highDensity_line_color;
 class WaveformSink;
 class Waveform;
 /***********************************************************************
-  timeMap - a structure that's used to map horizontal screen coordinates 
+  timeMap - a structure that's used to map horizontal screen coordinates
   into simulation time.
 */
-struct timeMap 
+struct timeMap
 {
   // simulation time
   double time;
@@ -89,7 +89,7 @@ struct timeMap
 /***********************************************************************
   WaveSource
 
-  This is an attribute that provides the interface between the WaveForm 
+  This is an attribute that provides the interface between the WaveForm
   class (below) and the names of nodes or stimuli that source information.
 
 */
@@ -142,7 +142,7 @@ public:
   GdkPixmap *pixmap() { return m_pixmap; }
   gint width;
   gint height;
-  gint yoffset;  // 
+  gint yoffset;  //
 protected:
   GdkPixmap *m_pixmap;
 };
@@ -196,6 +196,10 @@ class WaveBase
 public:
   WaveBase(Scope_Window *parent, const char *name);
 
+  virtual ~WaveBase()
+  {
+  }
+
   virtual void Update(guint64 start=0, guint64 stop=0)=0;
   virtual void Build(PixMap *pWavePixmap, PixMap *pSignalPixmap);
 
@@ -204,7 +208,7 @@ public:
   PixMap *signalPixmap() { return m_signalPixmap;}
   const char *get_text();
   virtual void setSource(const char *) { }
-  
+
 protected:
 
   Scope_Window *sw;          // Parent
@@ -219,12 +223,17 @@ protected:
 
 };
 /***********************************************************************
-  TimeAxis - A special 'Waveform' for plotting the time 
+  TimeAxis - A special 'Waveform' for plotting the time
  */
 class TimeAxis : public WaveBase
 {
 public:
   TimeAxis(Scope_Window *parent, const char *name);
+
+  virtual ~TimeAxis()
+  {
+  }
+
   virtual void Build(PixMap *pWavePixmap, PixMap *pSignalPixmap);
   virtual void Update(guint64 start=0, guint64 stop=0);
 
@@ -242,10 +251,12 @@ TimeAxis *m_TimeAxis;
 class Waveform : public WaveBase
 {
 public:
-
-
   Waveform(Scope_Window *parent, const char *name);
 
+  virtual ~Waveform()
+  {
+  }
+
   virtual void Update(guint64 start=0, guint64 stop=0);
   virtual void Build(PixMap *pWavePixmap, PixMap *pSignalPixmap);
 
@@ -291,6 +302,10 @@ class WaveformSink : public SignalSink
 public:
   WaveformSink(Waveform *pParent);
   virtual void setSinkState(char);
+  virtual void release()
+  {
+    cout << "Fixme - WaveformSink::release need to notify scope\n";
+  }
 protected:
   Waveform *m_pWaveform;
 };
@@ -300,6 +315,10 @@ protected:
 class ZoomInEvent : public KeyEvent
 {
 public:
+  virtual ~ZoomInEvent()
+  {
+  }
+
   void press(gpointer data)
   {
     Scope_Window *sw = (Scope_Window *)(data);
@@ -312,6 +331,10 @@ public:
 class ZoomOutEvent : public KeyEvent
 {
 public:
+  virtual ~ZoomOutEvent()
+  {
+  }
+
   void press(gpointer data)
   {
     Scope_Window *sw = (Scope_Window *)(data);
@@ -325,6 +348,10 @@ public:
 class PanLeftEvent : public KeyEvent
 {
 public:
+  virtual ~PanLeftEvent()
+  {
+  }
+
   void press(gpointer data)
   {
     Scope_Window *sw = (Scope_Window *)(data);
@@ -337,6 +364,10 @@ public:
 class PanRightEvent : public KeyEvent
 {
 public:
+  virtual ~PanRightEvent()
+  {
+  }
+
   void press(gpointer data)
   {
     Scope_Window *sw = (Scope_Window *)(data);
@@ -352,13 +383,13 @@ static map<guint, KeyEvent *> KeyMap;
 
 static gint
 key_press(GtkWidget *widget,
-	  GdkEventKey *key, 
-	  gpointer data)
+          GdkEventKey *key,
+          gpointer data)
 {
   Dprintf (("press 0x%x\n",key->keyval));
 
   KeyEvent *pKE = KeyMap[key->keyval];
-  if(pKE) 
+  if(pKE)
     {
       pKE->press(data);
       return TRUE;
@@ -369,16 +400,16 @@ key_press(GtkWidget *widget,
 
 static gint
 key_release(GtkWidget *widget,
-	  GdkEventKey *key, 
-	  gpointer data)
+          GdkEventKey *key,
+          gpointer data)
 {
   return TRUE;
 }
 
 static gint
 scroll_event(GtkWidget *widget,
-	     GdkEventScroll *scroll, 
-	     gpointer data)
+             GdkEventScroll *scroll,
+             gpointer data)
 {
   //printf ("scroll: x:%g y:%g dir:%d keymod:0x%x\n",scroll->x,scroll->y,scroll->direction,scroll->state);
 
@@ -395,7 +426,6 @@ WaveformSource::WaveformSource(Waveform *pParent, const char *_name)
   assert(m_pParent);
   // Prevent removal from the symbol table (all clearable symbols are
   // removed from the symbol table when a new processor is loaded).
-  m_bClearableSymbol = false;
 }
 
 void WaveformSource::set(const char *cp, int len)
@@ -411,12 +441,10 @@ void WaveformSource::set(const char *cp, int len)
 //========================================================================
 
 ZoomAttribute::ZoomAttribute(Scope_Window *pSW)
-  : Integer("scope.zoom",0,"Scope Zoom; positive values zoom in, negative values zoom out"), 
+  : Integer("scope.zoom",0,"Scope Zoom; positive values zoom in, negative values zoom out"),
     m_pSW(pSW)
 {
   assert(m_pSW);
-  m_bClearableSymbol = false;
-
 }
 void ZoomAttribute::set(gint64 i)
 {
@@ -427,12 +455,10 @@ void ZoomAttribute::set(gint64 i)
 //========================================================================
 
 PanAttribute::PanAttribute(Scope_Window *pSW)
-  : Integer("scope.pan",0,"Scope Pan; positive values pan right, negative values pan left"), 
+  : Integer("scope.pan",0,"Scope Pan; positive values pan right, negative values pan left"),
     m_pSW(pSW)
 {
   assert(m_pSW);
-  m_bClearableSymbol = false;
-
 }
 void PanAttribute::set(gint64 i)
 {
@@ -469,12 +495,12 @@ void WaveBase::Build(PixMap *pWavePixmap, PixMap *pSignalPixmap)
 
   if (m_wavePixmap && m_wavePixmap->pixmap())
     gdk_pixmap_unref(m_wavePixmap->pixmap());
-  
+
   m_wavePixmap = pWavePixmap;
 
   if (m_signalPixmap && m_signalPixmap->pixmap())
     gdk_pixmap_unref(m_signalPixmap->pixmap());
-  
+
   m_signalPixmap = pSignalPixmap;
 
   m_layout = gtk_widget_create_pango_layout (GTK_WIDGET (signalDrawingArea), "");
@@ -496,7 +522,7 @@ Waveform::Waveform(Scope_Window *parent, const char *name)
   m_pSink = new WaveformSink(this);
   m_logger = new ThreeStateEventLogger();
   m_pSourceName = new WaveformSource(this,name);
-  get_symbol_table().add(m_pSourceName);
+  globalSymbolTable().addSymbol(m_pSourceName);
 
   m_logger->event('0');
 }
@@ -522,7 +548,8 @@ void Waveform::updateLayout()
 
 void Waveform::setSource(const char *sourceName)
 {
-  IOPIN *ppin = dynamic_cast<IOPIN*>(get_symbol_table().findStimulus(sourceName));
+  //IOPIN *ppin = dynamic_cast<IOPIN*>(get_symbol_table().findStimulus(sourceName));
+  IOPIN *ppin = dynamic_cast<IOPIN*>(globalSymbolTable().find(string(sourceName)));
   if (ppin) {
     if (m_ppm)
       m_ppm->removeSink(m_pSink);
@@ -556,13 +583,13 @@ void Waveform::PlotTo(timeMap &left, timeMap &right)
 {
   // Event(s) has(have) been found.
   // The plotting region has been subdivided as finely as possible
-  // and there are one or more events. 
+  // and there are one or more events.
   // First draw a horizontal line from the last known event to here:
 
 
   gdk_draw_line(m_wavePixmap->pixmap(),drawing_gc,
-		m_last.pos, m_last.event,   // last point drawn
-		right.pos,  m_last.event);  // right most point of this region.
+                m_last.pos, m_last.event,   // last point drawn
+                right.pos,  m_last.event);  // right most point of this region.
 
   // Now draw a vertical line for the event
 
@@ -576,8 +603,8 @@ void Waveform::PlotTo(timeMap &left, timeMap &right)
 
     if (c != highDensity_line_color.blue) {
       gdk_colormap_free_colors(gdk_colormap_get_system(),
-			       &highDensity_line_color,
-			       1);
+                               &highDensity_line_color,
+                               1);
       // variations of yellow
       highDensity_line_color.green = 0xffff;
       highDensity_line_color.red = 0xffff;
@@ -590,16 +617,16 @@ void Waveform::PlotTo(timeMap &left, timeMap &right)
     }
 
     gdk_draw_line(m_wavePixmap->pixmap(),highDensity_gc,
-		  right.pos, 1,
-		  right.pos, m_wavePixmap->height-3);
+                  right.pos, 1,
+                  right.pos, m_wavePixmap->height-3);
     if (left.pos != right.pos)
       gdk_draw_line(m_wavePixmap->pixmap(),highDensity_gc,
-		    left.pos, 1,
-		    left.pos, m_wavePixmap->height-3);
-  } else 
+                    left.pos, 1,
+                    left.pos, m_wavePixmap->height-3);
+  } else
     gdk_draw_line(m_wavePixmap->pixmap(),drawing_gc,
-		  right.pos, m_last.event,    // last point drawn
-		  right.pos, nextEvent); // next event
+                  right.pos, m_last.event,    // last point drawn
+                  right.pos, nextEvent); // next event
 
   if (plotDebug)
     printf("pos=%d time=%g\n",right.pos,right.time);
@@ -613,7 +640,7 @@ void Waveform::PlotTo(timeMap &left, timeMap &right)
 // Waveform SearchAndPlot
 //
 //  Recursively divide the plotting area into smaller and smaller
-// regions until either only one event exists or the region can 
+// regions until either only one event exists or the region can
 // be divided no smaller.
 //
 
@@ -621,16 +648,16 @@ void Waveform::SearchAndPlot(timeMap &left, timeMap &right)
 {
   if (right.eventIndex == left.eventIndex)
     // The time span cannot be divided any smaller.
-    // If there are no events in this subdivided region 
+    // If there are no events in this subdivided region
     // So just return.
     // m_last = left;
-    ; 
+    ;
   else if (left.pos+1 >= right.pos)
     PlotTo(left,right);
   else {
     // the subdivided region is larger than 1-pixel wide
     // and there is at least one event. So subdivide even smaller
-    // and recursively call 
+    // and recursively call
 
     timeMap mid;
 
@@ -640,12 +667,12 @@ void Waveform::SearchAndPlot(timeMap &left, timeMap &right)
 
     if (plotDebug)
       cout << " Mid pos="<<mid.pos
-	   << " Mid.time=" << mid.time
-	   << " left.time=" << left.time
-	   << " right.time=" << right.time
-	   << " evt idx=" << mid.eventIndex
-	   << " evt time=" << m_logger->get_time(mid.eventIndex)
-	   << endl;
+           << " Mid.time=" << mid.time
+           << " left.time=" << left.time
+           << " right.time=" << right.time
+           << " evt idx=" << mid.eventIndex
+           << " evt time=" << m_logger->get_time(mid.eventIndex)
+           << endl;
 
     SearchAndPlot(left, mid);
     SearchAndPlot(mid, right);
@@ -678,8 +705,8 @@ void Waveform::Update(guint64 uiStart, guint64 uiEnd)
     return;
   }
 
-  if (uiEnd == 0) 
-    uiEnd = get_cycles().value;
+  if (uiEnd == 0)
+    uiEnd = get_cycles().get();
 
   if (m_start == uiStart && m_stop == uiEnd)
     return;
@@ -688,32 +715,32 @@ void Waveform::Update(guint64 uiStart, guint64 uiEnd)
   m_stop  = uiEnd;
 
   gdk_draw_rectangle (m_wavePixmap->pixmap(),
-		      waveDrawingArea->style->black_gc,
-		      TRUE,
-		      0, 0,
-		      m_wavePixmap->width,
-		      m_wavePixmap->height);
+                      waveDrawingArea->style->black_gc,
+                      TRUE,
+                      0, 0,
+                      m_wavePixmap->width,
+                      m_wavePixmap->height);
 
 
   gdk_draw_rectangle (m_signalPixmap->pixmap(),
-		      signalDrawingArea->style->bg_gc[GTK_STATE_NORMAL],
-		      //signalDrawingArea->style->black_gc,
-		      TRUE,
-		      0, 0,
-		      m_signalPixmap->width,
-		      m_signalPixmap->height);
+                      signalDrawingArea->style->bg_gc[GTK_STATE_NORMAL],
+                      //signalDrawingArea->style->black_gc,
+                      TRUE,
+                      0, 0,
+                      m_signalPixmap->width,
+                      m_signalPixmap->height);
   if (m_layout) {
     updateLayout();
     int text_height=0;
     pango_layout_get_pixel_size(m_layout,
-				NULL,
-				&text_height);
+                                NULL,
+                                &text_height);
     Dprintf(("signal name update:%s\n",pango_layout_get_text(m_layout)));
     gdk_draw_layout (GDK_DRAWABLE(m_signalPixmap->pixmap()),
-		     signalDrawingArea->style->fg_gc[GTK_STATE_NORMAL],
-		     0,
-		     (m_signalPixmap->height-text_height)/2,
-		     m_layout);
+                     signalDrawingArea->style->fg_gc[GTK_STATE_NORMAL],
+                     0,
+                     (m_signalPixmap->height-text_height)/2,
+                     m_layout);
   }
 
   //
@@ -726,19 +753,19 @@ void Waveform::Update(guint64 uiStart, guint64 uiEnd)
     int x = sw->MajorTicks().pixel(i);
 
     gdk_draw_line(m_wavePixmap->pixmap(),grid_gc,
-		  x,1,
-		  x,m_wavePixmap->height-1);
+                  x,1,
+                  x,m_wavePixmap->height-1);
   }
 
   //
   // Draw Horizontal Grid Lines:
   //
   gdk_draw_line(m_wavePixmap->pixmap(),grid_gc,
-		0,m_wavePixmap->height-1,
-		m_wavePixmap->width,m_wavePixmap->height-1);
+                0,m_wavePixmap->height-1,
+                m_wavePixmap->width,m_wavePixmap->height-1);
 
   if (m_stop == 0)
-    return; 
+    return;
 
   // Draw Signals:
 
@@ -760,8 +787,8 @@ void Waveform::Update(guint64 uiStart, guint64 uiEnd)
   SearchAndPlot(left,right);
   if (right.pos > m_last.pos)
     gdk_draw_line(m_wavePixmap->pixmap(),drawing_gc,
-		  m_last.pos, m_last.event,   // last point drawn
-		  right.pos,  m_last.event);  // right most point
+                  m_last.pos, m_last.event,   // last point drawn
+                  right.pos,  m_last.event);  // right most point
 
 
 }
@@ -789,8 +816,8 @@ void TimeAxis::Update(guint64 uiStart, guint64 uiEnd)
     return;
   }
 
-  if (uiEnd == 0) 
-    uiEnd = get_cycles().value;
+  if (uiEnd == 0)
+    uiEnd = get_cycles().get();
 
   if (m_start == uiStart && m_stop == uiEnd)
     return;
@@ -799,18 +826,18 @@ void TimeAxis::Update(guint64 uiStart, guint64 uiEnd)
   m_stop  = uiEnd;
 
   gdk_draw_rectangle (m_wavePixmap->pixmap(),
-		      waveDrawingArea->style->bg_gc[GTK_STATE_NORMAL],
-		      TRUE,
-		      0, 0,
-		      m_wavePixmap->width,
-		      m_wavePixmap->height);
+                      waveDrawingArea->style->bg_gc[GTK_STATE_NORMAL],
+                      TRUE,
+                      0, 0,
+                      m_wavePixmap->width,
+                      m_wavePixmap->height);
 
   gdk_draw_rectangle (m_signalPixmap->pixmap(),
-		      signalDrawingArea->style->bg_gc[GTK_STATE_NORMAL],
-		      TRUE,
-		      0, 0,
-		      m_signalPixmap->width,
-		      m_signalPixmap->height);
+                      signalDrawingArea->style->bg_gc[GTK_STATE_NORMAL],
+                      TRUE,
+                      0, 0,
+                      m_signalPixmap->width,
+                      m_signalPixmap->height);
 
   //
   // Draw Major Ticks:
@@ -821,29 +848,29 @@ void TimeAxis::Update(guint64 uiStart, guint64 uiEnd)
     int x = sw->MajorTicks().pixel(i);
 
     gdk_draw_line(m_wavePixmap->pixmap(),grid_gc,
-		  x,m_wavePixmap->height-3,
-		  x,m_wavePixmap->height-1);
+                  x,m_wavePixmap->height-3,
+                  x,m_wavePixmap->height-1);
 
     if (m_TicText) {
       char buff[100];
-      snprintf(buff, sizeof(buff),"%" PRINTF_INT64_MODIFIER "d", sw->MajorTicks().cycle(i));
+      snprintf(buff, sizeof(buff),"%" PRINTF_GINT64_MODIFIER "d", sw->MajorTicks().cycle(i));
       pango_layout_set_text(m_TicText, buff, -1);
 
       int text_height=0;
       int text_width=0;
 
       pango_layout_get_pixel_size(m_TicText,
-				  &text_width,
-				  &text_height);
+                                  &text_width,
+                                  &text_height);
       text_width /= 2;
       x = ((x-text_width) < 0) ? 0 : (x-text_width);
       x = ((x+text_width) > m_wavePixmap->width) ? (x-text_width) : x;
-      
+
       gdk_draw_layout (GDK_DRAWABLE(m_wavePixmap->pixmap()),
-		       waveDrawingArea->style->fg_gc[GTK_STATE_NORMAL],
-		       x,
-		       (m_wavePixmap->height-text_height)/2,
-		       m_TicText);
+                       waveDrawingArea->style->fg_gc[GTK_STATE_NORMAL],
+                       x,
+                       (m_wavePixmap->height-text_height)/2,
+                       m_TicText);
 
       }
 
@@ -857,15 +884,15 @@ void TimeAxis::Update(guint64 uiStart, guint64 uiEnd)
     int x = sw->MinorTicks().pixel(i);
 
     gdk_draw_line(m_wavePixmap->pixmap(),grid_gc,
-		  x,m_wavePixmap->height-3,
-		  x,m_wavePixmap->height-1);
+                  x,m_wavePixmap->height-3,
+                  x,m_wavePixmap->height-1);
   }
   //
   // Draw Horizontal Grid Lines:
   //
   gdk_draw_line(m_wavePixmap->pixmap(),grid_gc,
-		0,m_wavePixmap->height-1,
-		m_wavePixmap->width,m_wavePixmap->height-1);
+                0,m_wavePixmap->height-1,
+                m_wavePixmap->width,m_wavePixmap->height-1);
 
 }
 
@@ -897,7 +924,7 @@ void TimeAxis::Build(PixMap *pWavePixmap, PixMap *pSignalPixmap)
 // scope window, we'll capture that event and hide the scope.
 //
 static int delete_event(GtkWidget *widget,
-			GdkEvent  *event,
+                        GdkEvent  *event,
                         Scope_Window *sw)
 {
 
@@ -906,15 +933,15 @@ static int delete_event(GtkWidget *widget,
 }
 
 static gint Scope_Window_expose_event (GtkWidget *widget,
-				       GdkEventExpose  *event,
-				      Scope_Window   *sw)
+                                       GdkEventExpose  *event,
+                                      Scope_Window   *sw)
 {
   Dprintf(( " %s\n",__FUNCTION__));
 
   g_return_val_if_fail (widget != NULL, TRUE);
   //  g_return_val_if_fail (GTK_IS_DRAWING_AREA (widget), TRUE);
 
-  
+
   if(sw) {
     sw->refreshSignalNameGraphics();
 
@@ -923,8 +950,8 @@ static gint Scope_Window_expose_event (GtkWidget *widget,
   return FALSE;
 }
 static gint DrawingArea_expose_event (GtkWidget *widget,
-				      GdkEventExpose *event,
-				      Scope_Window   *sw)
+                                      GdkEventExpose *event,
+                                      Scope_Window   *sw)
 {
   Dprintf(( " %s\n",__FUNCTION__));
 
@@ -949,7 +976,6 @@ TimeMarker::TimeMarker(Scope_Window *parent, const char *_name, const char *desc
     m_pParent(parent)
 {
   assert(m_pParent);
-  m_bClearableSymbol = false;
 }
 void TimeMarker::set(gint64 i)
 {
@@ -977,14 +1003,14 @@ bool SignalNameEntry::Select(WaveBase *pWave)
 {
 
   if (pWave) {
-    gtk_entry_set_text (GTK_ENTRY(m_entry), 
-			pWave->get_text());
+    gtk_entry_set_text (GTK_ENTRY(m_entry),
+                        pWave->get_text());
     gtk_widget_show (GTK_WIDGET(m_entry));
     gtk_widget_grab_focus (GTK_WIDGET(m_entry));
 
     m_selectedWave = pWave;
     return true;
-  } 
+  }
 
   return unSelect();
 }
@@ -1027,7 +1053,7 @@ void Scope_Window::gridPoints(guint64 *uiStart, guint64 *uiEnd)
   guint64 start = m_Markers[eStart]->getVal();
   guint64 stop  = m_Markers[eStop]->getVal();
   if (!stop)
-    stop = get_cycles().value;
+    stop = get_cycles().get();
 
   if (uiStart)
     *uiStart = start;
@@ -1071,21 +1097,21 @@ void Scope_Window::gridPoints(guint64 *uiStart, guint64 *uiEnd)
 
       /*
       printf ("t=%g dt_tic=%g tmajor=%g %d %lld\n",
-	      t,dt_tic,ttic,m_MajorTicks.m_pixel[iMajor],uiTime);
+              t,dt_tic,ttic,m_MajorTicks.m_pixel[iMajor],uiTime);
       */
       iMajor++;
 
       ttic+=dt_minor;
 
       for (int it=1; it<nMinorTics; it++,ttic+=dt_minor) {
-	uiTime = (guint64)(ttic);
-	m_MinorTicks.m_pixel[iMinor] = mapTimeToPixel(uiTime);
-	m_MinorTicks.m_cycle[iMinor] = uiTime;
-	/*
-	printf ("     tminor=%g %d %lld\n",
-		ttic,m_MinorTicks.m_pixel[iMinor],uiTime);
-	*/
-	iMinor++;
+        uiTime = (guint64)(ttic);
+        m_MinorTicks.m_pixel[iMinor] = mapTimeToPixel(uiTime);
+        m_MinorTicks.m_cycle[iMinor] = uiTime;
+        /*
+        printf ("     tminor=%g %d %lld\n",
+                ttic,m_MinorTicks.m_pixel[iMinor],uiTime);
+        */
+        iMinor++;
 
       }
     }
@@ -1103,9 +1129,9 @@ GtkObject *m_hAdj=0;
 
 int Scope_Window::waveXoffset()
 {
-  return (int)((m_PixmapWidth - 
-		(m_pHpaned->allocation.width - gtk_paned_get_position(GTK_PANED(m_pHpaned))))
-	       *gNormalizedHorizontalPosition);
+  return (int)((m_PixmapWidth -
+                (m_pHpaned->allocation.width - gtk_paned_get_position(GTK_PANED(m_pHpaned))))
+               *gNormalizedHorizontalPosition);
 }
 
 void Scope_Window::Expose(WaveBase *wf)
@@ -1118,11 +1144,11 @@ void Scope_Window::Expose(WaveBase *wf)
   PixMap *pm = wf->wavePixmap();
 
   gdk_draw_pixmap(waveDrawingArea->window,
-		  waveDrawingArea->style->fg_gc[GTK_STATE_NORMAL],
-		  pm->pixmap(),
-		  xoffset,0,      // source
-		  0,pm->yoffset,  // destination
-		  pm->width,pm->height);
+                  waveDrawingArea->style->fg_gc[GTK_STATE_NORMAL],
+                  pm->pixmap(),
+                  xoffset,0,      // source
+                  0,pm->yoffset,  // destination
+                  pm->width,pm->height);
 
   pm = wf->signalPixmap();
 
@@ -1132,12 +1158,12 @@ void Scope_Window::Expose(WaveBase *wf)
 
   if (!m_entry->isSelected(wf)) {
 
-    gdk_draw_pixmap(SignalWindow, 
-		    signalDrawingArea->style->fg_gc[GTK_STATE_NORMAL],
-		    pm->pixmap(),
-		    0,0,            // source
-		    0,pm->yoffset,  // destination
-		    pm->width,pm->height);
+    gdk_draw_pixmap(SignalWindow,
+                    signalDrawingArea->style->fg_gc[GTK_STATE_NORMAL],
+                    pm->pixmap(),
+                    0,0,            // source
+                    0,pm->yoffset,  // destination
+                    pm->width,pm->height);
   }
 }
 
@@ -1145,24 +1171,24 @@ void Scope_Window::Expose(WaveBase *wf)
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 #if 0 // defined but not used
 static void ScrollAdjustments(GtkViewport   *viewport,
-				GtkAdjustment *arg1,
-				GtkAdjustment *arg2,
-				gpointer       user_data)
+                                GtkAdjustment *arg1,
+                                GtkAdjustment *arg2,
+                                gpointer       user_data)
 {
   printf("%s\n",__FUNCTION__);
 }
 
 static void ScrollChildren(GtkScrolledWindow *scrolledwindow,
-			    GtkScrollType     *arg1,
-			    gboolean           arg2,
-			    gpointer           user_data)
+                            GtkScrollType     *arg1,
+                            gboolean           arg2,
+                            gpointer           user_data)
 {
   printf("%s\n",__FUNCTION__);
 }
 #endif
 
 static void hAdjVChange(GtkAdjustment *pAdj,
-			gpointer       user_data)
+                        gpointer       user_data)
 {
 
   gdouble width = pAdj->upper - pAdj->lower - pAdj->page_size;
@@ -1171,15 +1197,15 @@ static void hAdjVChange(GtkAdjustment *pAdj,
 }
 static gint
 button_press(GtkWidget *widget,
-	     GdkEventButton *pEventButton, 
-	     Scope_Window *sw)
+             GdkEventButton *pEventButton,
+             Scope_Window *sw)
 {
-  
+
   if (pEventButton) {
 
     Dprintf (("button: button:%d x=%g y=%g evt=%d modifier=0x%x\n",
-	      pEventButton->button,
-	      pEventButton->x,pEventButton->y,pEventButton->type,pEventButton->state));
+              pEventButton->button,
+              pEventButton->x,pEventButton->y,pEventButton->type,pEventButton->state));
     if (sw) {
       sw->UpdateMarker(pEventButton->x,pEventButton->y, pEventButton->button, pEventButton->state);
     }
@@ -1228,8 +1254,8 @@ bool Scope_Window::selectSignalName(int y)
 
     if (pm)
       gtk_layout_move(GTK_LAYOUT(signalDrawingArea),
-		      GTK_WIDGET(m_entry->m_entry),
-		      0,pm->yoffset-2);
+                      GTK_WIDGET(m_entry->m_entry),
+                      0,pm->yoffset-2);
     bRet = m_entry->Select(signals[y]);
 
   } else
@@ -1243,12 +1269,12 @@ bool Scope_Window::selectSignalName(int y)
 
 
 //************************************************************************
-//           S I G N A L S  
+//           S I G N A L S
 //************************************************************************
 
 gint Scope_Window::signalEntryKeyPress(GtkEntry *widget,
-				       GdkEventKey *key, 
-				       Scope_Window *sw)
+                                       GdkEventKey *key,
+                                       Scope_Window *sw)
 {
   Dprintf (("Entry keypress 0x%x\n",key->keyval));
 
@@ -1256,7 +1282,7 @@ gint Scope_Window::signalEntryKeyPress(GtkEntry *widget,
 
     if ( key->keyval==GDK_Return)
       sw->endSignalNameSelection(true);
-    
+
     if ( key->keyval==GDK_Escape)
       sw->endSignalNameSelection(false);
   }
@@ -1266,15 +1292,15 @@ gint Scope_Window::signalEntryKeyPress(GtkEntry *widget,
 
 
 gint Scope_Window::signalButtonPress(GtkWidget *widget,
-				     GdkEventButton *pEventButton, 
-				     Scope_Window *sw)
+                                     GdkEventButton *pEventButton,
+                                     Scope_Window *sw)
 {
-  
+
   if (pEventButton) {
 
     Dprintf ((" Signal: button:%d x=%g y=%g evt=%d modifier=0x%x\n",
-	      pEventButton->button,
-	      pEventButton->x,pEventButton->y,pEventButton->type,pEventButton->state));
+              pEventButton->button,
+              pEventButton->x,pEventButton->y,pEventButton->type,pEventButton->state));
 
     sw->selectSignalName((int)(pEventButton->y));
 
@@ -1290,25 +1316,25 @@ gint Scope_Window::signalButtonPress(GtkWidget *widget,
 //         ------------------------------------------
 //         |                  Window
 //         |   --------------------------------------
-//         |   |            Horizontal pane  
+//         |   |            Horizontal pane
 //         |   |  ---------------+  +------------------------+
 //         |   |  | signal name  |  |    Wave layout
 //         |   |  | drawing area |  |  +---------------------
-//         |   |  | 
+//         |   |  |
 //
 //------------------------------------------------------------------------
 
 void Scope_Window::Build()
 {
 
-  GtkTooltips *tooltips;    
+  GtkTooltips *tooltips;
 
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   if (!window)
     return;
 
 
-  // The "realize" operation creates basic resources like GC's etc. that 
+  // The "realize" operation creates basic resources like GC's etc. that
   // are referenced below. Without this call, those resources will be NULL.
   gtk_widget_realize (window);
 
@@ -1320,27 +1346,27 @@ void Scope_Window::Build()
 
 
   gtk_signal_connect(GTK_OBJECT (window), "delete_event",
-		     GTK_SIGNAL_FUNC(delete_event), this);
+                     GTK_SIGNAL_FUNC(delete_event), this);
 
   gtk_signal_connect (GTK_OBJECT (window),
-		      "expose_event",
-		      GTK_SIGNAL_FUNC (Scope_Window_expose_event),
-		      this);
+                      "expose_event",
+                      GTK_SIGNAL_FUNC (Scope_Window_expose_event),
+                      this);
 
 #if 0
   GtkWidget *spin_button;
   button = gtk_button_new_with_label ("Zoom In");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
-		      GTK_SIGNAL_FUNC (analyzer_zoom_in_callback),this);    
-  gtk_table_attach_defaults (GTK_TABLE(table),button,2,4,9,10);    
+                      GTK_SIGNAL_FUNC (analyzer_zoom_in_callback),this);
+  gtk_table_attach_defaults (GTK_TABLE(table),button,2,4,9,10);
   button = gtk_button_new_with_label ("Zoom Out");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
-		      GTK_SIGNAL_FUNC (analyzer_zoom_out_callback),this);    
+                      GTK_SIGNAL_FUNC (analyzer_zoom_out_callback),this);
   gtk_table_attach_defaults (GTK_TABLE(table),button,4,6,9,10);
   button = gtk_button_new_with_label ("About");
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
-		      GTK_SIGNAL_FUNC (analyzer_screen_about),this);    
-  gtk_table_attach_defaults (GTK_TABLE(table),button,6,8,9,10);    
+                      GTK_SIGNAL_FUNC (analyzer_screen_about),this);
+  gtk_table_attach_defaults (GTK_TABLE(table),button,6,8,9,10);
 #endif
 
 
@@ -1376,24 +1402,24 @@ void Scope_Window::Build()
 
   gtk_widget_set_usize (waveDrawingArea,dawidth,daheight);
   gtk_widget_set_events (waveDrawingArea,
-			 GDK_EXPOSURE_MASK | 
-			 GDK_BUTTON_PRESS_MASK | 
-			 GDK_KEY_PRESS_MASK | 
-			 GDK_KEY_RELEASE_MASK  );
+                         GDK_EXPOSURE_MASK |
+                         GDK_BUTTON_PRESS_MASK |
+                         GDK_KEY_PRESS_MASK |
+                         GDK_KEY_RELEASE_MASK  );
 
   signalDrawingArea = gtk_layout_new (NULL, NULL);
   gtk_widget_set_usize (signalDrawingArea,100,daheight);
   gtk_widget_set_events (signalDrawingArea,
-			 GDK_EXPOSURE_MASK | 
-			 GDK_BUTTON_PRESS_MASK | 
-			 GDK_KEY_PRESS_MASK | 
-			 GDK_KEY_RELEASE_MASK  );
+                         GDK_EXPOSURE_MASK |
+                         GDK_BUTTON_PRESS_MASK |
+                         GDK_KEY_PRESS_MASK |
+                         GDK_KEY_RELEASE_MASK  );
 
 
 
   pvbox = gtk_vbox_new(FALSE,0);
   gtk_container_add (GTK_CONTAINER (window), pvbox);
-  
+
 
 
   m_pHpaned = gtk_hpaned_new ();
@@ -1402,7 +1428,7 @@ void Scope_Window::Build()
   gtk_box_pack_start_defaults (GTK_BOX (pvbox), m_pHpaned);
 
 
-  m_hAdj = gtk_adjustment_new 
+  m_hAdj = gtk_adjustment_new
     (0.0,    // value
      0.0,    // lower
      m_PixmapWidth,      // upper
@@ -1413,7 +1439,7 @@ void Scope_Window::Build()
   m_phScrollBar = gtk_hscrollbar_new(GTK_ADJUSTMENT(m_hAdj));
   gtk_box_pack_start_defaults(GTK_BOX(pvbox),m_phScrollBar);
   gtk_signal_connect(m_hAdj,"value-changed",
-		     (GtkSignalFunc) hAdjVChange, this);
+                     (GtkSignalFunc) hAdjVChange, this);
 
   // Add the drawing areas to the panes
   gtk_paned_add1(GTK_PANED(m_pHpaned), signalDrawingArea);
@@ -1429,8 +1455,8 @@ void Scope_Window::Build()
   highDensity_gc = gdk_gc_new(waveDrawingArea->window);
   leftMarker_gc = gdk_gc_new(waveDrawingArea->window);
 
-  gdk_gc_set_foreground(grid_gc,&grid_line_color);    
-  gdk_gc_set_foreground(drawing_gc,&signal_line_color);    
+  gdk_gc_set_foreground(grid_gc,&grid_line_color);
+  gdk_gc_set_foreground(drawing_gc,&signal_line_color);
   gdk_gc_set_foreground(highDensity_gc,&highDensity_line_color);
   gdk_gc_set_foreground(leftMarker_gc,&highDensity_line_color);
   gdk_gc_set_function(leftMarker_gc,GDK_XOR);
@@ -1441,20 +1467,20 @@ void Scope_Window::Build()
 
   int timeHeight = 15;
   m_TimeAxis->Build(new PixMap(waveDrawingArea->window, m_PixmapWidth, timeHeight, 0),
-		    new PixMap(waveDrawingArea->window, 100, timeHeight, 0));
+                    new PixMap(waveDrawingArea->window, 100, timeHeight, 0));
   m_TimeAxis->Update(start,stop);
 
   for(int i=0; i<nSignals; i++) {
     int waveHeight=20;
     int yoffset = timeHeight +i*waveHeight;
     signals[i]->Build(new PixMap(waveDrawingArea->window, m_PixmapWidth, waveHeight, yoffset),
-		      new PixMap(waveDrawingArea->window, 100, waveHeight, yoffset));
+                      new PixMap(waveDrawingArea->window, 100, waveHeight, yoffset));
   }
 
   gtk_signal_connect (GTK_OBJECT (waveDrawingArea),
-		      "expose_event",
-		      GTK_SIGNAL_FUNC (DrawingArea_expose_event),
-		      this);
+                      "expose_event",
+                      GTK_SIGNAL_FUNC (DrawingArea_expose_event),
+                      this);
 
   KeyMap['z'] = new ZoomInEvent();
   KeyMap['Z'] = new ZoomOutEvent();
@@ -1466,34 +1492,34 @@ void Scope_Window::Build()
    * key commands for single stepping, running, etc.
    */
   gtk_signal_connect(GTK_OBJECT(waveDrawingArea),
-		     "key_press_event",
-		     (GtkSignalFunc) key_press,
-		     (gpointer) this);
+                     "key_press_event",
+                     (GtkSignalFunc) key_press,
+                     (gpointer) this);
 
   gtk_signal_connect(GTK_OBJECT(waveDrawingArea),
-		     "button_press_event",
-		     (GtkSignalFunc) button_press,
-		     (gpointer) this);
+                     "button_press_event",
+                     (GtkSignalFunc) button_press,
+                     (gpointer) this);
 
   gtk_signal_connect(GTK_OBJECT(waveDrawingArea),
-		     "key_release_event",
-		     (GtkSignalFunc) key_release,
-		     (gpointer) this);
+                     "key_release_event",
+                     (GtkSignalFunc) key_release,
+                     (gpointer) this);
   gtk_signal_connect(GTK_OBJECT(waveDrawingArea),
-		     "scroll-event",
-		     (GtkSignalFunc) scroll_event,
-		     (gpointer) this);
+                     "scroll-event",
+                     (GtkSignalFunc) scroll_event,
+                     (gpointer) this);
   GTK_WIDGET_SET_FLAGS( waveDrawingArea,
-			GTK_CAN_FOCUS );
+                        GTK_CAN_FOCUS );
 
 
   gtk_signal_connect(GTK_OBJECT(signalDrawingArea),
-		     "button_press_event",
-		     (GtkSignalFunc) signalButtonPress,
-		     (gpointer) this);
+                     "button_press_event",
+                     (GtkSignalFunc) signalButtonPress,
+                     (gpointer) this);
 
   gtk_widget_show_all (window);
-    
+
   bIsBuilt = true;
 
   UpdateMenuItem();
@@ -1504,13 +1530,13 @@ void Scope_Window::Build()
   m_entry = new SignalNameEntry(this, GTK_ENTRY(gtk_entry_new ()));
 
   gtk_layout_put(GTK_LAYOUT(signalDrawingArea),
-		 GTK_WIDGET(m_entry->m_entry),
-		 0,0);
+                 GTK_WIDGET(m_entry->m_entry),
+                 0,0);
 
   gtk_signal_connect(GTK_OBJECT(m_entry->m_entry),
-		     "key_press_event",
-		     (GtkSignalFunc) signalEntryKeyPress,
-		     (gpointer) this);
+                     "key_press_event",
+                     (GtkSignalFunc) signalEntryKeyPress,
+                     (gpointer) this);
 
 }
 
@@ -1521,7 +1547,7 @@ void Scope_Window::Update()
 
   if(!enabled)
     return;
-    
+
   if(!bIsBuilt)
     Build();
 
@@ -1548,15 +1574,15 @@ void Scope_Window::Update()
   // the scroll bar maps pixel positions into time.
   // the span of the scroll bar is the span of time that is currently
   //  cached (or soon to be cached) in the waveform pixmaps.
-  // the thumb position is the current view into the cache. 
+  // the thumb position is the current view into the cache.
   // the page-size property is 20% of the span
 
 
   double dspan = stop-start;
   dspan = (dspan<m_PixmapWidth) ? m_PixmapWidth : dspan;
   g_object_set(G_OBJECT(m_hAdj),
-	       "page-size", m_PixmapWidth * 200.0/dspan,
-	       NULL);
+               "page-size", m_PixmapWidth * 200.0/dspan,
+               NULL);
   gtk_widget_queue_draw (m_phScrollBar);
 
 
@@ -1576,10 +1602,10 @@ void Scope_Window::Update()
   int xpos = mapTimeToPixel(m_Markers[eLeftButton]->getVal()) + waveXoffset();
   if (xpos)
     gdk_draw_line(waveDrawingArea->window,leftMarker_gc,
-		  xpos, 0,
-		  xpos, 1000);
+                  xpos, 0,
+                  xpos, 1000);
   /*
-  cout << "Left marker pos="<<dec<<xpos 
+  cout << "Left marker pos="<<dec<<xpos
        <<" time=" <<m_Markers[eLeftButton]->getVal()
        <<" start=" << start
        <<" stop=" << stop
@@ -1598,7 +1624,7 @@ void Scope_Window::UpdateMarker(gdouble x, gdouble y, guint button, guint state)
   switch (button) {
 
   case 1: // Left Button
-    
+
   case 2: // Right Button
     ;
   }
@@ -1629,12 +1655,12 @@ Scope_Window::Scope_Window(GUI_Processor *_gp)
   m_zoom = new ZoomAttribute(this);
   m_pan  = new PanAttribute(this);
 
-  get_symbol_table().add(m_Markers[eStart]);
-  get_symbol_table().add(m_Markers[eStop]);
-  get_symbol_table().add(m_Markers[eLeftButton]);
-  get_symbol_table().add(m_Markers[eRightButton]);
-  get_symbol_table().add(m_zoom);
-  get_symbol_table().add(m_pan);
+  globalSymbolTable().addSymbol(m_Markers[eStart]);
+  globalSymbolTable().addSymbol(m_Markers[eStop]);
+  globalSymbolTable().addSymbol(m_Markers[eLeftButton]);
+  globalSymbolTable().addSymbol(m_Markers[eRightButton]);
+  globalSymbolTable().addSymbol(m_zoom);
+  globalSymbolTable().addSymbol(m_pan);
 
   m_bFrozen = false;
 
@@ -1659,14 +1685,14 @@ Scope_Window::Scope_Window(GUI_Processor *_gp)
 }
 
 // zoom(i)
-// 
+//
   const int minSpan = 10;
 void Scope_Window::zoom(int i)
 {
   m_bFrozen = true;
   gint64 start = (gint64) m_Markers[eStart]->getVal();
   gint64 stop  = (gint64) m_Markers[eStop]->getVal();
-  gint64 now  = (gint64) get_cycles().value;
+  gint64 now  = (gint64) get_cycles().get();
 
   if (!stop)
     stop = now;
@@ -1689,7 +1715,7 @@ void Scope_Window::zoom(int i)
     start = mid - 1;
     stop = mid + 1;
   }
-  
+
   start = (start < 0) ? 0 : start;
   stop  = (stop >= now) ? 0 : stop;
 
@@ -1706,7 +1732,7 @@ void Scope_Window::pan(int i)
 
   gint64 start = i+(gint64) m_Markers[eStart]->getVal();
   gint64 stop  = (gint64) m_Markers[eStop]->getVal();
-  gint64 now  = (gint64) get_cycles().value;
+  gint64 now  = (gint64) get_cycles().get();
 
   if (start < 0)
     return;
@@ -1726,11 +1752,11 @@ gdouble Scope_Window::getSpan()
 {
   guint64 start = m_Markers[eStart]->getVal();
   guint64 stop  = m_Markers[eStop]->getVal();
-  stop = stop ? stop : get_cycles().value;
+  stop = stop ? stop : get_cycles().get();
   return start > stop ? 0.0 : ((gdouble)(stop-start));
 }
 
-guint64 Scope_Window::mapPixelToTime(int pixel)
+guint64 Scope_Window::mapPixelToTime(unsigned int pixel)
 {
   gdouble x = (pixel>=0 && pixel<m_PixmapWidth) ? pixel : 0;
 
