@@ -22,6 +22,7 @@ Boston, MA 02111-1307, USA.  */
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
+#include <typeinfo>
 
 #include "../config.h"
 #ifdef HAVE_GUI
@@ -56,7 +57,7 @@ typedef enum {
 
 
 typedef struct _menu_item {
-    char *name;
+    const char *name;
     menu_id id;
     GtkWidget *item;
 } menu_item;
@@ -71,6 +72,10 @@ class GUISymbol
 public:
   symbol *s;
 
+  virtual ~GUISymbol()
+  {
+  }
+
   virtual void select(void);
 
 };
@@ -84,20 +89,20 @@ static void update_menus(Symbol_Window *sw)
     unsigned int i;
 
     for (i=0; i < (sizeof(menu_items)/sizeof(menu_items[0])) ; i++){
-	item=menu_items[i].item;
-	if(sw)
-	{
+        item=menu_items[i].item;
+        if(sw)
+        {
             Value *entry;
-	    entry = (Value*) gtk_clist_get_row_data(GTK_CLIST(sw->symbol_clist),sw->current_row);
-	    if(entry==0)
-		gtk_widget_set_sensitive (item, FALSE);
-	    else
-		gtk_widget_set_sensitive (item, TRUE);
-	}
-	else
-	{
-	    gtk_widget_set_sensitive (item, FALSE);
-	}
+            entry = (Value*) gtk_clist_get_row_data(GTK_CLIST(sw->symbol_clist),sw->current_row);
+            if(entry==0)
+                gtk_widget_set_sensitive (item, FALSE);
+            else
+                gtk_widget_set_sensitive (item, TRUE);
+        }
+        else
+        {
+            gtk_widget_set_sensitive (item, FALSE);
+        }
     }
 }
 
@@ -112,10 +117,10 @@ popup_activated(GtkWidget *widget, gpointer data)
       printf("Warning popup_activated(%p,%p)\n",widget,data);
       return;
     }
-    
+
   item = (menu_item *)data;
 
-  Value *entry = 
+  Value *entry =
     (Value*) gtk_clist_get_row_data(GTK_CLIST(popup_sw->symbol_clist),popup_sw->current_row);
 
   if(!entry)
@@ -149,28 +154,28 @@ build_menu(GtkWidget *sheet, Symbol_Window *sw)
       printf("Warning build_menu(%p,%p)\n",sheet,sw);
       return 0;
   }
-    
+
   popup_sw = sw;
-  
+
   menu=gtk_menu_new();
 
   item = gtk_tearoff_menu_item_new ();
   gtk_menu_append (GTK_MENU (menu), item);
   gtk_widget_show (item);
-  
+
   for (i=0; i < (sizeof(menu_items)/sizeof(menu_items[0])) ; i++){
       menu_items[i].item=item=gtk_menu_item_new_with_label(menu_items[i].name);
 
       gtk_signal_connect(GTK_OBJECT(item),"activate",
-			 (GtkSignalFunc) popup_activated,
-			 &menu_items[i]);
+                         (GtkSignalFunc) popup_activated,
+                         &menu_items[i]);
 
       gtk_widget_show(item);
       gtk_menu_append(GTK_MENU(menu),item);
   }
 
   update_menus(sw);
-  
+
   return menu;
 }
 
@@ -180,7 +185,7 @@ do_popup(GtkWidget *widget, GdkEventButton *event, Symbol_Window *sw)
 {
 
     GtkWidget *popup;
-//	GdkModifierType mods;
+//      GdkModifierType mods;
 
   if(widget==0 || event==0 || sw==0)
   {
@@ -192,20 +197,76 @@ do_popup(GtkWidget *widget, GdkEventButton *event, Symbol_Window *sw)
     {
 
       gtk_menu_popup(GTK_MENU(popup), 0, 0, 0, 0,
-		     3, event->time);
+                     3, event->time);
     }
     return FALSE;
 }
 
 static void unselect_row(GtkCList *clist,
-			 gint row,
-			 gint column,
-			 GdkEvent *event,
-			 Symbol_Window *sw)
+                         gint row,
+                         gint column,
+                         GdkEvent *event,
+                         Symbol_Window *sw)
 {
     update_menus(sw);
 }
+#if 1
+static Symbol_Window *lpSW=0;
+static string table;
 
+void updateOneSymbol(const SymbolEntry_t &sym)
+{
+
+  Value *pVal = dynamic_cast<Value *>(sym.second);
+  if (lpSW && pVal) {
+
+    Register *pReg = dynamic_cast<Register *>(pVal);
+    if((typeid(*pVal) == typeid(LineNumberSymbol) ) |
+       (lpSW->filter_addresses && (typeid(*pVal) == typeid(AddressSymbol)))  ||
+       (lpSW->filter_constants && (typeid(*pVal) == typeid(Integer))) ||
+       (lpSW->filter_constants && (typeid(*pVal) == typeid(Boolean))) ||
+       (lpSW->filter_registers && (pReg)))
+        return;
+
+#define SYM_LEN 32
+    const char *entry[3];
+    string symbol_name;
+    char type[SYM_LEN];
+    char value[SYM_LEN];
+
+    if (table != "__global__")
+        symbol_name = table + "." + pVal->name();
+    else
+        symbol_name = pVal->name();
+
+    entry[0] = symbol_name.c_str();
+    strncpy(type, pVal->showType().c_str(), sizeof(type));
+    type[SYM_LEN-1] = 0;
+    entry[1] = type;
+    entry[2] = value;
+    if (pReg)
+      snprintf(value, sizeof(value), "%02x / %d (0x%02x)", pReg->getAddress(), pReg->get_value(), pReg->get_value());
+    else
+      pVal->get(value,sizeof(value));
+
+    char *pLF = strchr(value, '\n');
+    if(pLF)
+      *pLF = 0;
+
+    lpSW->symbols=g_list_append(lpSW->symbols, pVal);
+
+    int row = gtk_clist_append(GTK_CLIST(lpSW->symbol_clist),(gchar **)entry);
+    gtk_clist_set_row_data(GTK_CLIST(lpSW->symbol_clist),row,pVal);
+  }
+}
+
+static void updateSymbolTables(const SymbolTableEntry_t &st)
+{
+  if(verbose)cout << " gui Symbol Window: " << st.first << endl;
+  table = st.first;
+  (st.second)->ForEachSymbolTable(updateOneSymbol);
+}
+#endif
 
 void Symbol_Window::Update(void)
 {
@@ -213,28 +274,23 @@ void Symbol_Window::Update(void)
 
 
   load_symbols=1;
-    
+
   if(!enabled)
     return;
-    
+
 
   gtk_clist_freeze(GTK_CLIST(symbol_clist));
-    
+
   gtk_clist_clear(GTK_CLIST(symbol_clist));
 
-  // free all old allocations
-  /*
-  GList *iter;
-  for(iter=symbols;iter!=0;)
-    {
-      GList *next=iter->next;
-      g_list_remove(iter,iter->data);
-      iter=next;
-    }
-  */
   while (symbols)
     symbols = g_list_remove(symbols,symbols->data);
+#if 1
 
+  lpSW = this;
+  globalSymbolTable().ForEachModule(updateSymbolTables);
+  lpSW = 0;
+  /*
   Symbol_Table &st = CSimulationContext::GetContext()->GetSymbolTable();
   Symbol_Table::iterator symIt;
   Symbol_Table::iterator symItEnd = st.end();
@@ -242,15 +298,15 @@ void Symbol_Window::Update(void)
   for(symIt=st.begin(); symIt != symItEnd; symIt++) {
     Value *sym = *symIt;
     // ignore line numbers
-    if((typeid(*sym) == typeid(line_number_symbol) )            ||
-       (filter_addresses && (typeid(*sym) == typeid(address_symbol)))  ||
+    if((typeid(*sym) == typeid(LineNumberSymbol) )            ||
+       (filter_addresses && (typeid(*sym) == typeid(AddressSymbol)))  ||
        (filter_constants && (typeid(*sym) == typeid(Integer))) ||
        (filter_registers && (typeid(*sym) == typeid(register_symbol)))) {
 
       continue;
     }
 
-    
+
     char **entry = (char**)malloc(3*sizeof(char*));
     const int cMaxLength = 32;
 
@@ -277,7 +333,8 @@ void Symbol_Window::Update(void)
     gtk_clist_set_row_data(GTK_CLIST(symbol_clist),row,sym);
 
   }
-
+  */
+#endif
   gtk_clist_thaw(GTK_CLIST(symbol_clist));
 
 }
@@ -291,16 +348,16 @@ static void do_symbol_select(Symbol_Window *sw, Value *e)
   // Do what is to be done when a symbol is selected.
   // Except for selecting the symbol row in the symbol_clist
 
-  if(typeid(*e) == typeid(line_number_symbol) ||
-     typeid(*e) == typeid(address_symbol)) {
+  if(typeid(*e) == typeid(LineNumberSymbol) ||
+     typeid(*e) == typeid(AddressSymbol)) {
     if(sw->gp->source_browser)
       sw->gp->source_browser->SelectAddress(e);
     if(sw->gp->program_memory)
       sw->gp->program_memory->SelectAddress(e);
-  } else 
-    if(typeid(*e) == typeid(register_symbol))
+  } else
+    if(typeid(*e) == typeid(Register))
       if(sw->gp->regwin_ram)
-	sw->gp->regwin_ram->SelectRegister(e);
+        sw->gp->regwin_ram->SelectRegister(e);
 }
 
 static gint symbol_list_row_selected(GtkCList *symlist,gint row, gint column,GdkEvent *event, Symbol_Window *sw)
@@ -324,35 +381,35 @@ void SymbolWindow_select_symbol_regnumber(Symbol_Window *sw, int regnumber)
     if(!sw)
       return;
     if(!sw->enabled)
-	return;
-    
+        return;
+
     p=sw->symbols;
     while(p)
     {
       Value *e = (Value*)p->data;
 
-      if(typeid(*e) == typeid(register_symbol)) {
+      if(typeid(*e) == typeid(Register)) {
 
-	int i;
+        int i;
 
-	e->get(i);
+        e->get(i);
 
-	if(i == regnumber) {
-	  int row;
-	  row=gtk_clist_find_row_from_data(GTK_CLIST(sw->symbol_clist),e);
+        if(i == regnumber) {
+          int row;
+          row=gtk_clist_find_row_from_data(GTK_CLIST(sw->symbol_clist),e);
 
-	  if(row!=-1) {
-	    gtk_clist_select_row(GTK_CLIST(sw->symbol_clist),row,0);
-	    gtk_clist_moveto(GTK_CLIST(sw->symbol_clist),row,0,0.5,0.5);
+          if(row!=-1) {
+            gtk_clist_select_row(GTK_CLIST(sw->symbol_clist),row,0);
+            gtk_clist_moveto(GTK_CLIST(sw->symbol_clist),row,0,0.5,0.5);
 
-	    do_symbol_select(sw,e);
-	  }
-	  break;
-	}
-	p=p->next;
+            do_symbol_select(sw,e);
+          }
+          break;
+        }
+        p=p->next;
       }
     }
-       
+
 }
 
 void Symbol_Window::SelectSymbolName(char *symbol_name)
@@ -361,7 +418,7 @@ void Symbol_Window::SelectSymbolName(char *symbol_name)
 #if 0
   GList *p;
   //sym *s;
-    
+
   if(!symbol_name)
     return;
 
@@ -379,8 +436,8 @@ void Symbol_Window::SelectSymbolName(char *symbol_name)
 
     // ignore line numbers
     if(typeid(*sym) == typeid(line_number_symbol))
-		  continue;
-    
+                  continue;
+
     if(!strcasecmp((*sti)->name().data(),symbol_name)) {
 
        if(filter_addresses && (typeid(*sym) == typeid(address_symbol)))
@@ -391,39 +448,39 @@ void Symbol_Window::SelectSymbolName(char *symbol_name)
          gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (registersbutton), TRUE);
     }
 
-    
+
 /*
   list <symbol *>::iterator sti = st.begin();
 
   for(sti = st.begin(); sti != st.end(); sti++) {
-    
+
     if(!strcasecmp((*sti)->name().data(),symbol_name)) {
 
       switch((*sti)->isa()) {
-	    
+
       case SYMBOL_ADDRESS:
-	if(filter_addresses) {
-	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (addressesbutton), TRUE);
-	  while(gtk_events_pending()) // FIXME. Not so nice...
-	    gtk_main_iteration();
-	}
-	break;
+        if(filter_addresses) {
+          gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (addressesbutton), TRUE);
+          while(gtk_events_pending()) // FIXME. Not so nice...
+            gtk_main_iteration();
+        }
+        break;
       case SYMBOL_CONSTANT:
-	if(filter_constants) {
-	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (constantsbutton), TRUE);
-	  while(gtk_events_pending()) // FIXME. Not so nice...
-	    gtk_main_iteration();
-	}
-	break;
+        if(filter_constants) {
+          gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (constantsbutton), TRUE);
+          while(gtk_events_pending()) // FIXME. Not so nice...
+            gtk_main_iteration();
+        }
+        break;
       case SYMBOL_REGISTER:
-	if(filter_registers) {
-	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (registersbutton), TRUE);
-	  while(gtk_events_pending()) // FIXME. Not so nice...
-	    gtk_main_iteration();
-	}
-	break;
+        if(filter_registers) {
+          gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (registersbutton), TRUE);
+          while(gtk_events_pending()) // FIXME. Not so nice...
+            gtk_main_iteration();
+        }
+        break;
       default:
-	break;
+        break;
       }
       break;
     }
@@ -433,20 +490,20 @@ void Symbol_Window::SelectSymbolName(char *symbol_name)
   // Find the symbol and select it from the clist
   p=symbols;
   while(p) {
-    
+
     Value *e;
     e=(sym*)p->data;
     if(!strcasecmp(e->name,symbol_name)) {
-      
+
       int row;
       row=gtk_clist_find_row_from_data(GTK_CLIST(symbol_clist),e);
       if(row!=-1) {
-        
-				gtk_clist_select_row(GTK_CLIST(symbol_clist),row,0);
+
+                                gtk_clist_select_row(GTK_CLIST(symbol_clist),row,0);
         gtk_clist_moveto(GTK_CLIST(symbol_clist),row,0,0.5,0.5);
-        
-				do_symbol_select(this,e);
-		
+
+                                do_symbol_select(this,e);
+
       }
     }
     p=p->next;
@@ -476,39 +533,39 @@ symbol_compare_func(GtkCList *clist, gconstpointer ptr1,gconstpointer ptr2)
     switch (row1->cell[clist->sort_column].type)
     {
     case GTK_CELL_TEXT:
-	text1 = GTK_CELL_TEXT (row1->cell[clist->sort_column])->text;
-	break;
+        text1 = GTK_CELL_TEXT (row1->cell[clist->sort_column])->text;
+        break;
     case GTK_CELL_PIXTEXT:
-	text1 = GTK_CELL_PIXTEXT (row1->cell[clist->sort_column])->text;
-	break;
+        text1 = GTK_CELL_PIXTEXT (row1->cell[clist->sort_column])->text;
+        break;
     default:
-	assert(0);
-	break;
+        assert(0);
+        break;
     }
 
     switch (row2->cell[clist->sort_column].type)
     {
     case GTK_CELL_TEXT:
-	text2 = GTK_CELL_TEXT (row2->cell[clist->sort_column])->text;
-	break;
+        text2 = GTK_CELL_TEXT (row2->cell[clist->sort_column])->text;
+        break;
     case GTK_CELL_PIXTEXT:
-	text2 = GTK_CELL_PIXTEXT (row2->cell[clist->sort_column])->text;
-	break;
+        text2 = GTK_CELL_PIXTEXT (row2->cell[clist->sort_column])->text;
+        break;
     default:
-	assert(0);
-	break;
+        assert(0);
+        break;
     }
 
     if (!text2)
-	assert(0);
+        assert(0);
 
     if (!text1)
-	assert(0);
+        assert(0);
 
     if(1==sscanf(text1,"%li",&val1))
     {
       if(1==sscanf(text2,"%li",&val2))
-	return val1-val2;
+        return val1-val2;
 
     }
     return strcmp(text1,text2);
@@ -518,22 +575,22 @@ static void symbol_click_column(GtkCList *clist, int column)
 {
     static int last_col=-1;
     static GtkSortType last_sort_type=GTK_SORT_DESCENDING;
-    
+
     if(last_col==-1)
-	last_col=column;
+        last_col=column;
 
     if(last_col == column)
     {
-	if(last_sort_type==GTK_SORT_DESCENDING)
-	{
-	    gtk_clist_set_sort_type(clist,GTK_SORT_ASCENDING);
-	    last_sort_type=GTK_SORT_ASCENDING;
-	}
-	else
-	{
-	    gtk_clist_set_sort_type(clist,GTK_SORT_DESCENDING);
-	    last_sort_type=GTK_SORT_DESCENDING;
-	}
+        if(last_sort_type==GTK_SORT_DESCENDING)
+        {
+            gtk_clist_set_sort_type(clist,GTK_SORT_ASCENDING);
+            last_sort_type=GTK_SORT_ASCENDING;
+        }
+        else
+        {
+            gtk_clist_set_sort_type(clist,GTK_SORT_DESCENDING);
+            last_sort_type=GTK_SORT_DESCENDING;
+        }
     }
 
     gtk_clist_set_sort_column(clist,column);
@@ -542,7 +599,7 @@ static void symbol_click_column(GtkCList *clist, int column)
 }
 
 static int delete_event(GtkWidget *widget,
-			GdkEvent  *event,
+                        GdkEvent  *event,
                         Symbol_Window *sw)
 {
     sw->ChangeView(VIEW_HIDE);
@@ -573,7 +630,7 @@ toggle_registers (GtkToggleButton *button, Symbol_Window *sw)
     sw->Update();
 }
 
-static char *symbol_titles[3]={"Name","Type","Address/Value"};
+static gchar *symbol_titles[3]={"Name","Type","Address/Value"};
 
 //------------------------------------------------------------------------
 // Build
@@ -591,13 +648,13 @@ void Symbol_Window::Build(void)
   window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
 
   gtk_window_set_title(GTK_WINDOW(window), "Symbol Viewer");
-  
+
   gtk_window_set_default_size(GTK_WINDOW(window), width,height);
   gtk_widget_set_uposition(GTK_WIDGET(window),x,y);
   gtk_window_set_wmclass(GTK_WINDOW(window),name(),"Gpsim");
-  
+
   gtk_signal_connect (GTK_OBJECT (window), "delete_event",
-		      GTK_SIGNAL_FUNC(delete_event), (gpointer)this);
+                      GTK_SIGNAL_FUNC(delete_event), (gpointer)this);
 
   symbol_clist=gtk_clist_new_with_titles(3,symbol_titles);
   gtk_widget_show(symbol_clist);
@@ -606,26 +663,26 @@ void Symbol_Window::Build(void)
   gtk_clist_set_column_auto_resize(GTK_CLIST(symbol_clist),2,TRUE);
   gtk_clist_set_auto_sort(GTK_CLIST(symbol_clist),TRUE);
   gtk_clist_set_compare_func(GTK_CLIST(symbol_clist),
-			     (GtkCListCompareFunc)symbol_compare_func);
+                             (GtkCListCompareFunc)symbol_compare_func);
 
   gtk_signal_connect(GTK_OBJECT(symbol_clist),"click_column",
-		     (GtkSignalFunc)symbol_click_column,0);
+                     (GtkSignalFunc)symbol_click_column,0);
   gtk_signal_connect(GTK_OBJECT(symbol_clist),"select_row",
-		     (GtkSignalFunc)symbol_list_row_selected,this);
+                     (GtkSignalFunc)symbol_list_row_selected,this);
   gtk_signal_connect(GTK_OBJECT(symbol_clist),"unselect_row",
-		     (GtkSignalFunc)unselect_row,this);
+                     (GtkSignalFunc)unselect_row,this);
   gtk_signal_connect(GTK_OBJECT(symbol_clist),
-		     "button_press_event",
-		     (GtkSignalFunc) do_popup,
-		     this);
+                     "button_press_event",
+                     (GtkSignalFunc) do_popup,
+                     this);
 
   scrolled_window=gtk_scrolled_window_new(0, 0);
   gtk_widget_show(scrolled_window);
 
   vbox = gtk_vbox_new(FALSE,1);
-  
+
   gtk_container_add(GTK_CONTAINER(scrolled_window), symbol_clist);
-  
+
   gtk_container_add(GTK_CONTAINER(window),vbox);
 
 
@@ -641,9 +698,9 @@ void Symbol_Window::Build(void)
   else
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (addressesbutton), TRUE);
   gtk_signal_connect (GTK_OBJECT (addressesbutton), "toggled",
-		      GTK_SIGNAL_FUNC (toggle_addresses), (gpointer)this);
+                      GTK_SIGNAL_FUNC (toggle_addresses), (gpointer)this);
+
 
-  
   constantsbutton = gtk_check_button_new_with_label ("constants");
   gtk_box_pack_start (GTK_BOX (hbox), constantsbutton, TRUE, TRUE, 5);
   if(filter_constants)
@@ -651,9 +708,9 @@ void Symbol_Window::Build(void)
   else
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (constantsbutton), TRUE);
   gtk_signal_connect (GTK_OBJECT (constantsbutton), "toggled",
-		      GTK_SIGNAL_FUNC (toggle_constants), (gpointer)this);
+                      GTK_SIGNAL_FUNC (toggle_constants), (gpointer)this);
+
 
-  
   registersbutton = gtk_check_button_new_with_label ("registers");
   gtk_box_pack_start (GTK_BOX (hbox), registersbutton, TRUE, TRUE, 5);
   if(filter_registers)
@@ -661,14 +718,14 @@ void Symbol_Window::Build(void)
   else
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (registersbutton), TRUE);
   gtk_signal_connect (GTK_OBJECT (registersbutton), "toggled",
-		      GTK_SIGNAL_FUNC (toggle_registers), (gpointer)this);
+                      GTK_SIGNAL_FUNC (toggle_registers), (gpointer)this);
 
   gtk_signal_connect_after(GTK_OBJECT(window), "configure_event",
-			   GTK_SIGNAL_FUNC(gui_object_configure_event),this);
+                           GTK_SIGNAL_FUNC(gui_object_configure_event),this);
+
 
-  
   gtk_widget_show_all (window);
-  
+
 
   bIsBuilt = true;
 
@@ -676,9 +733,9 @@ void Symbol_Window::Build(void)
     NewSymbols();
 
   UpdateMenuItem();
-  
+
   popup_menu=build_menu(window,this);
-  
+
 }
 
 
@@ -704,7 +761,7 @@ Symbol_Window::Symbol_Window(GUI_Processor *_gp)
   filter_registers=0;
 
   load_symbols=0;
-  
+
   get_config();
 
   config_get_variable(name(),"filter_addresses",&filter_addresses);
@@ -713,7 +770,7 @@ Symbol_Window::Symbol_Window(GUI_Processor *_gp)
 
   if(enabled)
     Build();
-  
+
 }
 
 #endif // HAVE_GUI
