@@ -67,6 +67,7 @@ support:
 #include "stimuli.h"
 #include "ttl.h"
 #include "i2c-eeprom.h"
+#include "i2c.h"
 
 /*
 class Module_Types
@@ -105,7 +106,9 @@ Module_Types available_modules[] =
   { {"led", "led"}, Leds::Led::construct},
   { {"push_button",      "pb"},   PushButton::construct },
 #endif
-  { {"PortStimulus",     "ps"},   ExtendedStimuli::PortStimulus::construct },
+  { {"PortStimulus",     "ps"},   ExtendedStimuli::PortStimulus::construct8 },
+  { {"PortStimulus16",   "ps"},   ExtendedStimuli::PortStimulus::construct16 },
+  { {"PortStimulus32",   "ps"},   ExtendedStimuli::PortStimulus::construct32 },
   { {"pullup",           "pu"},   PullupResistor::pu_construct },
   { {"pulldown",         "pd"},   PullupResistor::pd_construct },
   { {"pulsegen",         "pg"},   ExtendedStimuli::PulseGen::construct },
@@ -129,6 +132,7 @@ Module_Types available_modules[] =
   { {"I2C-EEPROM2k", "e24xx024"}, I2C_EEPROM_Modules::I2C_EE_Module::construct_2k},
   { {"I2C-EEPROM16k", "e24xx16b"}, I2C_EEPROM_Modules::I2C_EE_Module::construct_16k},
   { {"I2C-EEPROM256k", "e24xx256"}, I2C_EEPROM_Modules::I2C_EE_Module::construct_256k},
+  { {"i2cmaster", "I2CMaster"},   I2C_Module::I2CMaster::construct },
 
   // No more modules
   { {0,0},0}
