@@ -27,6 +27,7 @@ Boston, MA 02111-1307, USA.  */
 #include <gtk/gtk.h>
 #endif
 
+class Processor;
 #include "../src/i2c-ee.h"
 #include "i2c-eeprom.h"
 #include "../src/stimuli.h"
@@ -34,6 +35,7 @@ Boston, MA 02111-1307, USA.  */
 #include "../src/symbol.h"
 #include "../src/value.h"
 #include "../src/packages.h"
+#include "../src/gpsim_interface.h"
 
 namespace I2C_EEPROM_Modules {
 
@@ -61,57 +63,65 @@ void I2C_ENABLE::setDrivenState(bool bNewState)
     m_pParent->setEnable(bNewState, m_bit);
 }
 
-  I2C_EE_Module::I2C_EE_Module(const char *_name)
+  I2C_EE_Module::I2C_EE_Module(const char *_name) : Module(_name, "EEProm")
   {
-    // Set module name
-    if (_name)
-	new_name(_name);
-    initializeAttributes();
+    //initializeAttributes();
     chip_select = 0;
   }
 
 
   I2C_EE_Module::~I2C_EE_Module()
   {
+    delete att_eeprom;
     delete m_eeprom;
   }
 
+
+
   Module *I2C_EE_Module::construct_2k(const char *_new_name)
   {
+     string att_name = _new_name;
 
     I2C_EE_Module *pEE = new I2C_EE_Module(_new_name);
     // I2C_EE size in bytes prom size in bits
-    (pEE->m_eeprom) = new I2C_EE(256, 16, 1, 0xe, 0, 0);
+    (pEE->m_eeprom) = new I2C_EE((Processor *)pEE,256, 16, 1, 0xe, 0, 0);
     pEE->create_iopin_map();
+    att_name += ".eeprom";
+    pEE->att_eeprom = new PromAddress(pEE->m_eeprom, "eeprom", "Address I2C_EE");
+    pEE->addSymbol(pEE->att_eeprom);
 
-    //if(get_interface().bUsingGUI()) 
-    //  pEE->create_widget(pEe);
    return(pEE);
 
   }
   Module *I2C_EE_Module::construct_16k(const char *_new_name)
   {
 
+     string att_name = _new_name;
+
     I2C_EE_Module *pEE = new I2C_EE_Module(_new_name);
     // I2C_EE size in bytes prom size in bits
-    (pEE->m_eeprom) = new I2C_EE(2048, 16, 1, 0, 0xe, 1);
+    (pEE->m_eeprom) = new I2C_EE((Processor *)pEE,2048, 16, 1, 0, 0xe, 1);
     pEE->create_iopin_map();
+    att_name += ".eeprom";
+    pEE->att_eeprom = new PromAddress(pEE->m_eeprom, att_name.c_str(), "Address I2C_EE");
+    pEE->addSymbol(pEE->att_eeprom);
 
-    //if(get_interface().bUsingGUI()) 
-    //  pEE->create_widget(pEe);
    return(pEE);
 
   }
   Module *I2C_EE_Module::construct_256k(const char *_new_name)
   {
 
+     string att_name = _new_name;
+
     I2C_EE_Module *pEE = new I2C_EE_Module(_new_name);
     // I2C_EE size in bytes prom size in bits
-    (pEE->m_eeprom) = new I2C_EE(32768, 64, 2, 0xe, 0, 0);
+    (pEE->m_eeprom) = new I2C_EE((Processor *)pEE,32768, 64, 2, 0xe, 0, 0);
     pEE->create_iopin_map();
 
-    //if(get_interface().bUsingGUI()) 
-    //  pEE->create_widget(pEe);
+    att_name += ".eeprom";
+    pEE->att_eeprom = new PromAddress(pEE->m_eeprom, att_name.c_str(), "Address I2C_EE");
+    pEE->addSymbol(pEE->att_eeprom);
    return(pEE);
 
   }
