@@ -31,7 +31,7 @@ Boston, MA 02111-1307, USA.  */
   --------------------
        a
       ---
-   f | g | b    
+   f | g | b
       ---
    e |   | c
       ---
@@ -68,6 +68,7 @@ Boston, MA 02111-1307, USA.  */
 #include <stdlib.h>
 #include <string>
 #include <iostream>
+#include <typeinfo>
 
 #include "../config.h"    // get the definition for HAVE_GUI
 
@@ -103,14 +104,14 @@ namespace Leds {
     }
     virtual void GuiUpdate  (gpointer object)
     {
-      if(led) 
-	led->update();
+      if(led)
+        led->update();
       /* {
-	int portval = led->port->get_value();
-	if(lastport != portval) {
-	  lastport=portval;
-	  led->update();
-	}
+        int portval = led->port->get_value();
+        if(lastport != portval) {
+          lastport=portval;
+          led->update();
+        }
 
       }
       */
@@ -158,19 +159,22 @@ namespace Leds {
 
   void Led_7Segments::callback()
   {
-                                                                                
+
     get_cycles().set_break_delta( get_interface().get_update_rate()+1, this);
     update();
-                                                                                
+
   }
 
   void Led_7Segments::update(  GtkWidget *widget,
-			       guint new_width,
-			       guint new_height)
+                               guint new_width,
+                               guint new_height)
   {
 
     guint i;
 
+    if(!get_interface().bUsingGUI())
+      return;
+
     w_width = new_width;
     w_height = new_height;
     GdkDrawable *drawable = widget->window;
@@ -180,13 +184,13 @@ namespace Leds {
 
     if(segment_gc==NULL)
       {
-	segment_gc = gdk_gc_new(darea->window);
-	gdk_gc_set_line_attributes(segment_gc,
-				   5,
-				   GDK_LINE_SOLID,
-				   GDK_CAP_ROUND,
-				   GDK_JOIN_ROUND);
-	g_assert(segment_gc!=NULL);
+        segment_gc = gdk_gc_new(darea->window);
+        gdk_gc_set_line_attributes(segment_gc,
+                                   5,
+                                   GDK_LINE_SOLID,
+                                   GDK_CAP_ROUND,
+                                   GDK_JOIN_ROUND);
+        g_assert(segment_gc!=NULL);
       }
 
 
@@ -197,14 +201,14 @@ namespace Leds {
     GdkGC *gc = segment_gc;
 
     gdk_gc_set_foreground(gc,
-			  &led_background_color);
+                          &led_background_color);
 
     gdk_draw_rectangle (drawable, gc,
-			TRUE,
-			0,
-			0,
-			w_width,
-			w_height);
+                        TRUE,
+                        0,
+                        0,
+                        w_width,
+                        w_height);
 
 
     // cout << "expose led, segment states = " << segment_states << '\n';
@@ -216,12 +220,12 @@ namespace Leds {
       gdk_gc_set_foreground(gc,&led_segment_on_color);
 
       for(i=0; i<7; i++) {
-	if(segment_states & (2<<i))
-	  gdk_draw_polygon ( drawable,
-			     gc,
-			     TRUE,
-			     segments[i].p,
-			     6);
+        if(segment_states & (2<<i))
+          gdk_draw_polygon ( drawable,
+                             gc,
+                             TRUE,
+                             segments[i].p,
+                             6);
 
 
       }
@@ -233,11 +237,11 @@ namespace Leds {
 
     for(i=0; i<7; i++) {
       if((segment_states & (2<<i)) == 0)
-	gdk_draw_polygon ( drawable,
-			   gc,
-			   TRUE,
-			   segments[i].p,
-			   6);
+        gdk_draw_polygon ( drawable,
+                           gc,
+                           TRUE,
+                           segments[i].p,
+                           6);
 
     }
   }
@@ -245,8 +249,8 @@ namespace Leds {
 
   static gint
   led7_expose_event (GtkWidget *widget,
-		     GdkEvent  *event,
-		     gpointer   user_data)
+                     GdkEvent  *event,
+                     gpointer   user_data)
   {
 
 
@@ -270,14 +274,14 @@ namespace Leds {
 
   static gint
   cursor_event (GtkWidget          *widget,
-		GdkEvent           *event,
-		gpointer  *user_data)
+                GdkEvent           *event,
+                gpointer  *user_data)
   {
     if ((event->type == GDK_BUTTON_PRESS) &&
-	((event->button.button == 1) ||
-	 (event->button.button == 3)))
+        ((event->button.button == 1) ||
+         (event->button.button == 3)))
       {
-	return TRUE;
+        return TRUE;
       }
 
     return FALSE;
@@ -293,7 +297,7 @@ namespace Leds {
   //
   // Each segment on the LED is comprised of a 6 point polygon.
   // This routine will calculate what those points should be and
-  // store them an arrary. 
+  // store them an arrary.
 
   void Led_7Segments::build_segments( int w, int h)
   {
@@ -315,7 +319,7 @@ namespace Leds {
     angle = 6;
 
     /* define various useful constants */
- 
+
     segxw = sxw * w;
     slope = angle;
     seg_width = width_factor * w;
@@ -326,8 +330,8 @@ namespace Leds {
     midpt = (float)h / 2;
 
     /* define some trigonometric values */
-    /*  phi is the forward angle separating two segments; 
-	psi is the reverse angle separating two segments. */
+    /*  phi is the forward angle separating two segments;
+        psi is the reverse angle separating two segments. */
 
     invsinphi = sqrt(1 + fslope * fslope) / fslope;
     invcosphi = sqrt(1 + 1/(fslope * fslope)) * fslope;
@@ -373,7 +377,7 @@ namespace Leds {
     pts[2].x = pts[5].x + xfactor;
     pts[3].y = pts[4].y = seg_width;
     pts[4].x = temp_xpts[3] + dx4;
-    pts[3].x = temp_xpts[0] + xfactor - dx3; 
+    pts[3].x = temp_xpts[0] + xfactor - dx3;
 
     pts = &(seg_pts[MIDDLE][0]);
     pts[0].y = pts[1].y = midpt - seg_width/2;
@@ -390,7 +394,7 @@ namespace Leds {
     pts[3].y = pts[4].y = (float)h;
     pts[2].y = pts[5].y = h - (seg_width / 2) + dy5 + dy6;
     pts[0].y = pts[1].y = h - seg_width;
-    pts[0].x = spacer + segxw + seg_width/slope + dx3;  
+    pts[0].x = spacer + segxw + seg_width/slope + dx3;
     pts[1].x = spacer + (h - pts[1].y)/slope + xfactor - dx4;
     pts[4].x = spacer + segxw - dx4;
     pts[5].x = spacer + segxw/2 + (h - pts[5].y)/slope + dx6 - dx5;
@@ -406,7 +410,7 @@ namespace Leds {
     pts[5].y = seg_width - dy1;
     pts[0].x = temp_xpts[1] - dx5 - dx6;
     pts[1].x = temp_xpts[3] - dx2;
-    pts[2].x = seg_pts[MIDDLE][0].x + 2 * dx1; 
+    pts[2].x = seg_pts[MIDDLE][0].x + 2 * dx1;
     pts[3].x = seg_pts[MIDDLE][5].x - 2 * dx6;
     pts[4].x = spacer + (h - pts[4].y)/slope;
     pts[5].x = temp_xpts[0] + dx1;
@@ -459,8 +463,8 @@ namespace Leds {
 
       for(j=0; j<MAX_PTS; j++) {
 
-	segments[i].p[j].x = (int)seg_pts[i][j].x;
-	segments[i].p[j].y = (int)seg_pts[i][j].y;
+        segments[i].p[j].x = (int)seg_pts[i][j].x;
+        segments[i].p[j].y = (int)seg_pts[i][j].y;
       }
     }
 
@@ -476,31 +480,31 @@ namespace Leds {
 
     vbox =
       gtk_widget_new (gtk_vbox_get_type (),
-		      "GtkBox::homogeneous", FALSE,
-		      //"GtkBox::spacing", 5,
-		      //"GtkContainer::border_width", 10,
-		      "GtkWidget::parent", main_vbox,
-		      "GtkWidget::visible", TRUE,
-		      NULL);
+                      "GtkBox::homogeneous", FALSE,
+                      //"GtkBox::spacing", 5,
+                      //"GtkContainer::border_width", 10,
+                      "GtkWidget::parent", main_vbox,
+                      "GtkWidget::visible", TRUE,
+                      NULL);
     gtk_widget_show(vbox);
 
 
     darea = gtk_drawing_area_new ();
 
-    gtk_widget_set_usize (darea, 
-			  100, 
-			  100);
+    gtk_widget_set_usize (darea,
+                          100,
+                          110);
     gtk_container_add (GTK_CONTAINER (vbox), darea);
 
     gtk_signal_connect (GTK_OBJECT (darea),
-			"expose_event",
-			GTK_SIGNAL_FUNC (led7_expose_event),
-			this);
+                        "expose_event",
+                        GTK_SIGNAL_FUNC (led7_expose_event),
+                        this);
     gtk_widget_set_events (darea, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK);
     gtk_signal_connect (GTK_OBJECT (darea),
-			"button_press_event",
-			GTK_SIGNAL_FUNC (cursor_event),
-			NULL);
+                        "button_press_event",
+                        GTK_SIGNAL_FUNC (cursor_event),
+                        NULL);
 
     gtk_widget_show (darea);
 
@@ -513,21 +517,21 @@ namespace Leds {
     led_segment_on_color.green = 0x0000;
     led_segment_on_color.blue = 0x0000;
 
-    gdk_color_alloc(gdk_colormap_get_system(), &led_segment_on_color);
+    gdk_colormap_alloc_color(gdk_colormap_get_system(), &led_segment_on_color, FALSE, TRUE);
 
     // The `off' color is dark red
     led_segment_off_color.red = 0x4000;
     led_segment_off_color.green = 0x0000;
     led_segment_off_color.blue = 0x0000;
 
-    gdk_color_alloc(gdk_colormap_get_system(), &led_segment_off_color);
+    gdk_colormap_alloc_color(gdk_colormap_get_system(), &led_segment_off_color, FALSE, TRUE);
 
     // The background is black like my coffee
     led_background_color.red = 0x0000;
     led_background_color.green = 0x0000;
     led_background_color.blue = 0x0000;
 
-    gdk_color_alloc(gdk_colormap_get_system(), &led_background_color);
+    gdk_colormap_alloc_color(gdk_colormap_get_system(), &led_background_color, FALSE, TRUE);
 
     //  }
 
@@ -536,31 +540,31 @@ namespace Leds {
 
   //--------------------------------------------------------------
 
-  Led_7Segments::Led_7Segments()
+  Led_7Segments::Led_7Segments(const char *name) : Module(name, "7 Segment LED")
   {
 
-    //cout << "7-segment led constructor\n";
-    new_name("LED7SEG");
-
     if(get_interface().bUsingGUI()) {
-      build_segments(100, 100);
+      build_segments(100, 110);
       build_window();
     }
 
-    interface = new LED_Interface(this);
-    get_interface().add_interface(interface);
+    led_interface = new LED_Interface(this);
+    get_interface().add_interface(led_interface);
     callback();
+    create_iopin_map();
   }
 
   Led_7Segments::~Led_7Segments()
   {
+    /* Deleted elsewhere RRR
     for (int i=0; i<8; i++)
       delete m_pins[i];
     delete [] m_pins;
+   */
   }
 
   //--------------------------------------------------------------
-  // create_iopin_map 
+  // create_iopin_map
   //
   //  This is where the information for the Module's package is defined.
   // Specifically, the I/O pins of the module are created.
@@ -577,18 +581,12 @@ namespace Leds {
     create_pkg(8);
     m_pins = new Led_Input *[8];
 
-    // Position pins on left side of package
-    float pos=0.0;
-    const float dp = (float)(0.9999/7.0);
-    package->set_pin_position(1,pos);pos+=dp;
-    package->set_pin_position(2,pos);pos+=dp;
-    package->set_pin_position(3,pos);pos+=dp;
-    package->set_pin_position(4,pos);pos+=dp;
-    package->set_pin_position(5,pos);pos+=dp;
-    package->set_pin_position(6,pos);pos+=dp;
-    package->set_pin_position(7,pos);pos+=dp;
-    package->set_pin_position(8,pos);pos+=dp;
-
+    float ypos = 6.0;
+    for (int i = 1; i <= 8; i++)
+    {
+        package->setPinGeometry(i, 0.0, ypos, 0, false);
+        ypos += 12.;
+    }
 
     // Here, we create and name the I/O pins. In gpsim, we will reference
     //   the bit positions as LED.seg0, LED.seg1, ..., where LED is the
@@ -603,7 +601,7 @@ namespace Leds {
     for (i=0; i<8; i++)
       assign_pin(i+1,m_pins[i]);
 
-    initializeAttributes();
+    //initializeAttributes();
 
   }
 
@@ -623,16 +621,119 @@ namespace Leds {
   Module * Led_7Segments::construct(const char *_new_name=0)
   {
 
-    Led_7Segments *l7sP = new Led_7Segments ;
-    l7sP->new_name(_new_name);
-    l7sP->create_iopin_map();
+    Led_7Segments *l7sP = new Led_7Segments(_new_name) ;
 
     return l7sP;
 
   }
 
 
+class ColorAttribute : public Value
+{
+  public:
+        ColorAttribute(Led *_led) :
+                Value("color", "On color of LED"), m_led(_led)
+        {
+        }
 
+        virtual void get(char *return_str, int len);
+        virtual void set(const char *buffer, int buf_size = 0);
+        virtual void set(Value *v);
+        virtual bool Parse(const char *pValue, Colors &bValue);
+
+  private:
+        Led *m_led;
+};
+
+void ColorAttribute::set(Value *v)
+{
+  if ( typeid(*v) == typeid(String))
+  {
+     char buff[20];
+
+     v->get((char *)buff, sizeof(buff));
+     set(buff);
+  }
+  else
+    throw new TypeMismatch(string("set "), "ColorAttribute", v->showType());
+}
+
+void ColorAttribute::set(const char *buffer, int len)
+{
+  if(buffer) {
+    Colors color;
+
+    if(Parse(buffer, color)) {
+      m_led->set_on_color(color);
+    }
+    else
+    {
+        cout << "ColorAttribute::set " << buffer << " unknown color\n";
+   }
+  }
+
+}
+void ColorAttribute::get(char *return_str, int len)
+{
+  if(return_str) {
+
+    switch(m_led->get_on_color())
+    {
+    case RED:
+        strncpy(return_str, "red", len);
+        break;
+
+    case ORANGE:
+        strncpy(return_str, "orange", len);
+        break;
+
+    case GREEN:
+        strncpy(return_str, "green", len);
+        break;
+
+    case YELLOW:
+        strncpy(return_str, "yellow", len);
+        break;
+
+    case BLUE:
+        strncpy(return_str, "blue", len);
+        break;
+
+    }
+  }
+}
+
+bool ColorAttribute::Parse(const char *pValue, Colors &bValue)
+{
+
+
+    if(strncmp("red", pValue, sizeof("red")) == 0)
+    {
+        bValue = RED;
+        return true;
+    }
+    else if (strncmp("orange", pValue, sizeof("orange")) == 0)
+    {
+        bValue = ORANGE;
+        return true;
+    }
+    else if (strncmp("green", pValue, sizeof("green")) == 0)
+    {
+        bValue = GREEN;
+        return true;
+    }
+    else if (strncmp("yellow", pValue, sizeof("yellow")) == 0)
+    {
+        bValue = YELLOW;
+        return true;
+    }
+    else if (strncmp("blue", pValue, sizeof("blue")) == 0)
+    {
+        bValue = BLUE;
+        return true;
+    }
+    return false;
+}
 
   //-------------------------------------------------------------
   // Led (simple)
@@ -644,14 +745,14 @@ namespace Leds {
 
   void Led::callback()
   {
-                                                                                
+
     get_cycles().set_break_delta( get_interface().get_update_rate()+1, this);
     update();
-                                                                                
+
   }
   void Led::update(  GtkWidget *widget,
-		     guint new_width,
-		     guint new_height)
+                     guint new_width,
+                     guint new_height)
   {
     if(!get_interface().bUsingGUI())
       return;
@@ -665,41 +766,56 @@ namespace Leds {
 
     if(gc==NULL)
       {
-	gc = gdk_gc_new(darea->window);
-	gdk_gc_set_line_attributes(gc,
-				   5,
-				   GDK_LINE_SOLID,
-				   GDK_CAP_ROUND,
-				   GDK_JOIN_ROUND);
-	g_assert(gc!=NULL);
+        gc = gdk_gc_new(darea->window);
+        gdk_gc_set_line_attributes(gc,
+                                   5,
+                                   GDK_LINE_SOLID,
+                                   GDK_CAP_ROUND,
+                                   GDK_JOIN_ROUND);
+        g_assert(gc!=NULL);
       }
 
 
     gdk_gc_set_foreground(gc,&led_segment_off_color);
     gdk_draw_rectangle (drawable, gc,
-			TRUE,
-			0,
-			0,
-			w_width,
-			w_height);
+                        TRUE,
+                        0,
+                        0,
+                        w_width,
+                        w_height);
 
     if(m_pin->getDrivenState()) {
-      gdk_gc_set_foreground(gc,&led_segment_on_color);
-      gdk_draw_arc(drawable, gc,
-		   TRUE,
-		   0,
-		   0,
-		   w_width,
-		   w_height,
-		   0,64*360);
+        gdk_gc_set_foreground(gc,&led_on_color[on_color]);
+        gdk_draw_arc(drawable, gc,
+                   TRUE,
+                   0,
+                   0,
+                   w_width,
+                   w_height,
+                   0,64*360);
     }
   }
 
+  void Led::set_on_color(Colors color)
+  {
+        if (color != on_color)
+        {
+            on_color = color;
+            if (get_interface().bUsingGUI() )
+            {
+                if (!led_on_color[color].pixel) // color not allocated
+                        gdk_colormap_alloc_color(gdk_colormap_get_system(),
+                                            &led_on_color[color], FALSE, TRUE);
+                update();
+            }
+        }
+  }
+
 
   static gint
   led_expose_event (GtkWidget *widget,
-		    GdkEvent  *event,
-		    gpointer   user_data)
+                    GdkEvent  *event,
+                    gpointer   user_data)
   {
 
 
@@ -733,13 +849,13 @@ namespace Leds {
     w_height=20;
     w_width=20;
 
-    gtk_widget_set_usize (darea, 
-			  w_height,
-			  w_width);
+    gtk_widget_set_usize (darea,
+                          w_height,
+                          w_width);
     gtk_signal_connect (GTK_OBJECT (darea),
-			"expose_event",
-			GTK_SIGNAL_FUNC (led_expose_event),
-			this);
+                        "expose_event",
+                        GTK_SIGNAL_FUNC (led_expose_event),
+                        this);
     gtk_widget_set_events (darea, GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK);
     gtk_widget_show (darea);
 
@@ -747,43 +863,59 @@ namespace Leds {
 
     gc=NULL;
 
-    // The 'on' color is bright red
-    led_segment_on_color.red = 0xc000;
-    led_segment_on_color.green = 0x0000;
-    led_segment_on_color.blue = 0x0000;
+    // The default 'on' color is bright red
 
-    gdk_color_alloc(gdk_colormap_get_system(), &led_segment_on_color);
+    for(int i = RED; i <= BLUE; i++ )
+        led_on_color[i].pixel = 0;
+
+    gdk_color_parse("red3", &led_on_color[RED]);
+    gdk_color_parse("orange", &led_on_color[ORANGE]);
+    gdk_color_parse("green", &led_on_color[GREEN]);
+    gdk_color_parse("yellow", &led_on_color[YELLOW]);
+    gdk_color_parse("blue", &led_on_color[BLUE]);
+
+    GdkColormap *colormap = gdk_colormap_get_system();
+
+    for(int i = RED; i <= BLUE; i++ )
+        gdk_colormap_alloc_color(colormap, &led_on_color[i], FALSE, TRUE);
 
     // The `off' color is dark red
     led_segment_off_color.red = 0x4000;
     led_segment_off_color.green = 0x0000;
     led_segment_off_color.blue = 0x0000;
 
-    gdk_color_alloc(gdk_colormap_get_system(), &led_segment_off_color);
-
+    gdk_colormap_alloc_color(colormap, &led_segment_off_color, FALSE, TRUE);
   }
 
   //--------------------------------------------------------------
 
-  Led::Led()
+  Led::Led(const char *name) : Module(name, "Simple LED")
   {
-    new_name("LED");
 
+    create_iopin_map();
     if(get_interface().bUsingGUI())
       build_window();
 
-    interface = new LED_Interface(this);
-    get_interface().add_interface(interface);
+
+    on_color = RED;
+    m_colorAttribute = new ColorAttribute(this);
+    addSymbol(m_colorAttribute);
+    led_interface = new LED_Interface(this);
+    get_interface().add_interface(led_interface);
     callback();
   }
 
   Led::~Led()
   {
+
+    delete m_colorAttribute;
+    /* deleted elsewhere RRR
     delete m_pin;
+    */
   }
 
   //--------------------------------------------------------------
-  // create_iopin_map 
+  // create_iopin_map
   //
   //  This is where the information for the Module's package is defined.
   // Specifically, the I/O pins of the module are created.
@@ -801,7 +933,7 @@ namespace Leds {
 
     m_pin = new Led_Input((name() + ".in").c_str(), this);
     assign_pin(1, m_pin);
-    initializeAttributes();
+    //initializeAttributes();
   }
 
   //--------------------------------------------------------------
@@ -810,9 +942,8 @@ namespace Leds {
   Module * Led::construct(const char *_new_name=0)
   {
 
-    Led *ledP = new Led;
-    ledP->new_name(_new_name);
-    ledP->create_iopin_map();
+    Led *ledP = new Led(_new_name);
+    //ledP->new_name(_new_name);
 
     return ledP;
 
