@@ -39,19 +39,32 @@ namespace Leds {
   class LED_Interface;  // Defined in led.cc
   class Led_Input;
 
+  enum Colors
+  {
+	RED = 0,
+	ORANGE,
+	GREEN,
+	YELLOW,
+	BLUE		// Must be last
+  };
+	
+
   //------------------------------------------------------------------------
   // LED base class
 
-  class Led_base : public Module
+  class Led_base //: public Module
   {
   public:
+    virtual ~Led_base()
+    {
+    }
 
     virtual void build_window() = 0;
     virtual void update() = 0;
     virtual void update( GtkWidget *drawable,   
-			 guint max_width,
-			 guint max_height) = 0;
-    LED_Interface *interface;
+                         guint max_width,
+                         guint max_height) = 0;
+    LED_Interface *led_interface;
   };
 
 
@@ -64,24 +77,24 @@ namespace Leds {
     float y;
   } XfPoint;
 
-#define MAX_PTS		6	/* max # of pts per segment polygon */
-#define NUM_SEGS	7	/* number of segments in a digit */
+#define MAX_PTS         6       /* max # of pts per segment polygon */
+#define NUM_SEGS        7       /* number of segments in a digit */
   /*
    * These constants give the bit positions for the segmask[]
    * digit masks.
    */
-#define TOP		0
-#define TOP_RIGHT	1
-#define BOT_RIGHT	2
-#define BOTTOM		3
-#define BOT_LEFT	4
-#define TOP_LEFT	5
-#define MIDDLE		6
+#define TOP             0
+#define TOP_RIGHT       1
+#define BOT_RIGHT       2
+#define BOTTOM          3
+#define BOT_LEFT        4
+#define TOP_LEFT        5
+#define MIDDLE          6
   //#define DECIMAL_POINT   7
 
   typedef XfPoint segment_pts[NUM_SEGS][MAX_PTS];
 
-  class Led_7Segments : public Led_base, public TriggerObject
+  class Led_7Segments : public Module, public Led_base, public TriggerObject
   {
   public:
 
@@ -96,12 +109,12 @@ namespace Leds {
 
     float sxw;
 
-    float		angle;		// rise over run 
-    float		width_factor;   // ratio of digit to segment width 
-    float		small_ratio;	// ratio of small to large digits
-    float		sec_gap;	// gap between normal digits and 
+    float               angle;          // rise over run 
+    float               width_factor;   // ratio of digit to segment width 
+    float               small_ratio;    // ratio of small to large digits
+    float               sec_gap;        // gap between normal digits and 
     // seconds, as ratio to digit width 
-    float		space_factor;   /* ratio of digit width to border sp.*/
+    float               space_factor;   /* ratio of digit width to border sp.*/
 
 
     guint w_width;
@@ -116,7 +129,7 @@ namespace Leds {
     GdkColor led_segment_off_color;
     GdkColor led_background_color;
 
-    Led_7Segments();
+    Led_7Segments(const char *);
     ~Led_7Segments();  
 
 
@@ -126,8 +139,8 @@ namespace Leds {
     virtual void build_window();
     virtual void update();
     virtual void update( GtkWidget *drawable,   
-			 guint max_width,
-			 guint max_height);
+                         guint max_width,
+                         guint max_height);
     unsigned int getPinState();
 
     // Inheritances from the Package class
@@ -136,7 +149,6 @@ namespace Leds {
     // Inheritance from Module class
     const virtual char *type() { return ("led_7segments"); };
     static Module *construct(const char *new_name);
-
   private:
     //unsigned int m_segmentStates;
     Led_Input **m_pins;
@@ -147,28 +159,29 @@ namespace Leds {
   //------------------------------------------------------------------------
   // Simple LED
   //
+  class ColorAttribute;
 
-  class Led: public Led_base, public TriggerObject
+  class Led: public Module, public Led_base, public TriggerObject
   {
   public:
 
     GtkWidget *darea;
     GdkGC *gc;
-    GdkColor led_segment_on_color;
+    GdkColor led_on_color[BLUE+1];
     GdkColor led_segment_off_color;
     int w_width, w_height;
 
     gpointer cbp;  // cycle break point pointer (need to delete in destructor)
 
-    Led();
+    Led(const char *);
     ~Led();
 
     virtual void callback();
     virtual void build_window();
     virtual void update();
     virtual void update( GtkWidget *drawable,   
-			 guint max_width,
-			 guint max_height);
+                         guint max_width,
+                         guint max_height);
 
     // Inheritances from the Package class
     virtual void create_iopin_map();
@@ -176,9 +189,14 @@ namespace Leds {
     // Inheritance from Module class
     const virtual char *type() { return ("led"); };
     static Module *construct(const char *new_name);
+    Colors get_on_color() { return on_color; }
+    void set_on_color(Colors color);
+    
 
   private:
     Led_Input *m_pin;
+    Colors on_color;
+    ColorAttribute *m_colorAttribute;
   };
 
 } // end of namespace Led
