@@ -36,13 +36,14 @@ In here you'll find some simple logic devices:
 #include "../config.h"    // get the definition for HAVE_GUI
 
 #ifdef HAVE_GUI
+#include <gtk/gtk.h>
 
 /* XPM */
-static char * and2_pixmap[] = {
+static gchar * and2_pixmap[] = {
 "32 32 3 1",
-" 	c black",
-".	c None",
-"X	c white",
+"       c black",
+".      c None",
+"X      c white",
 "                    ............",
 "                        ........",
 "  XXXXXXXXXXXXXXXXXX    ........",
@@ -77,11 +78,11 @@ static char * and2_pixmap[] = {
 "                    ............"};
 
 /* XPM */
-static char * or2_pixmap[] = {
+static gchar * or2_pixmap[] = {
 "32 32 3 1",
-" 	c black",
-".	c None",
-"X	c white",
+"       c black",
+".      c None",
+"X      c white",
 "                    ............",
 "                        ........",
 "  XXXXXXXXXXXXXXXXXX    ........",
@@ -115,11 +116,11 @@ static char * or2_pixmap[] = {
 "                        ........",
 "                    ............"};
 
-static char * xor2_pixmap[] = {
+static gchar * xor2_pixmap[] = {
 "40 32 3 1",
-" 	c None",
-".	c black",
-"X	c white",
+"       c None",
+".      c black",
+"X      c white",
 "        ....................            ",
 "  ..    ....................            ",
 " ....   ........................        ",
@@ -153,11 +154,11 @@ static char * xor2_pixmap[] = {
 " ....   ..XXXXXXXXXXXXXXXXXX....        ",
 "  ..    ........................        "};
 
-static char * not_pixmap[] = {
+static gchar * not_pixmap[] = {
 "32 32 3 1",
-" 	c black",
-".	c None",
-"X	c white",
+"       c black",
+".      c None",
+"X      c white",
 " ...............................",
 "   .............................",
 "     ...........................",
@@ -190,9 +191,6 @@ static char * not_pixmap[] = {
 "     ...........................",
 "   .............................",
 " ..............................."};
-
-#include <gtk/gtk.h>
-
 #endif
 
 #include <errno.h>
@@ -210,14 +208,14 @@ static char * not_pixmap[] = {
 //   This class is a minor extension of a normal IO_input. I may
 // remove it later, but for now it does serve a simple purpose.
 // Specifically, this derivation will intercept when a stimulus
-// is being changed. 
+// is being changed.
 
 void Logic_Input::setDrivenState( bool new_state)
 {
 
-  if(0)
-    cout << name()<< " setDrivenState= " 
-	 << (new_state ? "high" : "low") << endl;
+  if(1)
+    cout << name()<< " setDrivenState= "
+         << (new_state ? "high" : "low") << endl;
 
   if(new_state != getDrivenState()) {
 
@@ -232,19 +230,46 @@ void Logic_Input::setDrivenState( bool new_state)
 }
 
 
+ANDGate::ANDGate(const char *name, const char *desc)
+  : LogicGate(name, desc)
+{
+}
+ANDGate::~ANDGate()
+{
+}
+ORGate::ORGate(const char *name, const char *desc)
+  : LogicGate(name, desc)
+{
+}
+ORGate::~ORGate()
+{
+}
+XORGate::XORGate(const char *name, const char *desc)
+  : LogicGate(name, desc)
+{
+}
+XORGate::~XORGate()
+{
+}
 /*************************************************************
 *
 *  LogicGate class
 */
 
-LogicGate::LogicGate(void)
+LogicGate::LogicGate(const char *name, const char *desc)
+  : Module(name, desc),
+    number_of_pins(0),
+    input_bit_mask(0),
+    input_state(0),
+    pInputPins(0),
+    pOutputPin(0)
 {
 #ifdef HAVE_GUI
   pixmap=0;
 #endif
 }
 
-LogicGate::~LogicGate(void)
+LogicGate::~LogicGate()
 {
 }
 
@@ -257,12 +282,12 @@ void LogicGate::update_input_pin(unsigned int pin, bool bValue)
 }
 
 //--------------------------------------------------------------
-// create_iopin_map 
+// create_iopin_map
 //
 //  This is where the information for the Module's package is defined.
 // Specifically, the I/O pins of the module are created.
 
-void LogicGate::create_iopin_map(void)
+void LogicGate::create_iopin_map()
 {
   int i;
 
@@ -276,7 +301,7 @@ void LogicGate::create_iopin_map(void)
 
   // Define the I/O pins and assign them to the package.
   //   There are two things happening here. First, there is
-  //   a new I/O pin that is being created. For the binary 
+  //   a new I/O pin that is being created. For the binary
   //   indicator, both pins are inputs. The second thing is
   //   that the pins are "assigned" to the package. If we
   //   need to reference these newly created I/O pins (like
@@ -329,32 +354,32 @@ void LogicGate::create_iopin_map(void)
   // Form the logic gate bit masks
   input_bit_mask = (1<< (number_of_pins-1)) - 1;
 
-  initializeAttributes();
+  //initializeAttributes();
 
 }
 
 #ifdef HAVE_GUI
 static gboolean expose(GtkWidget *widget,
-		GdkEventExpose *event,
-		LogicGate *lg)
+                GdkEventExpose *event,
+                LogicGate *lg)
 {
-	if(lg->pixmap==0)
-	{
-		puts("LogicGate has no pixmap");
-		return 0;
-	}
+        if(lg->pixmap==0)
+        {
+                puts("LogicGate has no pixmap");
+                return 0;
+        }
 
-	gdk_draw_pixmap(widget->window,
-			widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
-			lg->pixmap,
-			event->area.x, event->area.y,
-			event->area.x, event->area.y,
-			event->area.width, event->area.height);
-	
-	return 0;
+        gdk_draw_pixmap(widget->window,
+                        widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
+                        lg->pixmap,
+                        event->area.x, event->area.y,
+                        event->area.x, event->area.y,
+                        event->area.width, event->area.height);
+
+        return 0;
 }
 
-GtkWidget *LogicGate::create_pixmap(char **pixmap_data)
+GtkWidget *LogicGate::create_pixmap(gchar **pixmap_data)
 {
     GtkStyle *style;
     GdkBitmap *mask;
@@ -362,12 +387,12 @@ GtkWidget *LogicGate::create_pixmap(char **pixmap_data)
     int width,height;
 
     style = gtk_style_new();
-    
+
     pixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL,
-						   gdk_colormap_get_system(),
-						   &mask,
-						   &style->bg[GTK_STATE_NORMAL],
-						   pixmap_data);
+                                                   gdk_colormap_get_system(),
+                                                   &mask,
+                                                   &style->bg[GTK_STATE_NORMAL],
+                                                   pixmap_data);
 #if GTK_MAJOR_VERSION >= 2
     gdk_drawable_get_size(pixmap,&width,&height);
 #else
@@ -376,7 +401,7 @@ GtkWidget *LogicGate::create_pixmap(char **pixmap_data)
     da = gtk_drawing_area_new();
     gtk_drawing_area_size(GTK_DRAWING_AREA(da),width,height);
     gtk_signal_connect(GTK_OBJECT(da),"expose_event",
-    		(GtkSignalFunc) expose,this);
+                (GtkSignalFunc) expose,this);
     return da;
 }
 #endif
@@ -388,9 +413,8 @@ Module * AND2Gate::construct(const char *_new_name)
 {
 
 
-  AND2Gate *a2gP = new AND2Gate ;
+  AND2Gate *a2gP = new AND2Gate(_new_name) ;
 
-  a2gP->new_name(_new_name);
   a2gP->set_number_of_pins(3);
   a2gP->create_iopin_map();
 
@@ -398,7 +422,7 @@ Module * AND2Gate::construct(const char *_new_name)
 
 }
 
-AND2Gate::AND2Gate(void)
+AND2Gate::AND2Gate(const char *name) : ANDGate(name, "And2Gate")
 {
 #ifdef HAVE_GUI
   if(get_interface().bUsingGUI())
@@ -406,12 +430,12 @@ AND2Gate::AND2Gate(void)
 #endif
 
 }
-AND2Gate::~AND2Gate(void)
+AND2Gate::~AND2Gate()
 {
 }
 
 
-void ANDGate::update_state(void)
+void ANDGate::update_state()
 {
   pOutputPin->putState((input_state & input_bit_mask) == input_bit_mask);
   if (pOutputPin->snode)
@@ -422,7 +446,7 @@ void ANDGate::update_state(void)
 // construct
 
 
-OR2Gate::OR2Gate(void)
+OR2Gate::OR2Gate(const char *name) : ORGate(name, "OR2Gate")
 {
 #ifdef HAVE_GUI
 
@@ -430,7 +454,7 @@ OR2Gate::OR2Gate(void)
     set_widget(create_pixmap(or2_pixmap));
 #endif
 }
-OR2Gate::~OR2Gate(void)
+OR2Gate::~OR2Gate()
 {
 
 }
@@ -439,9 +463,8 @@ Module * OR2Gate::construct(const char *_new_name)
 {
 
 
-  OR2Gate *o2gP = new OR2Gate ;
+  OR2Gate *o2gP = new OR2Gate(_new_name) ;
 
-  o2gP->new_name(_new_name);
   o2gP->set_number_of_pins(3);
   o2gP->create_iopin_map();
 
@@ -449,7 +472,7 @@ Module * OR2Gate::construct(const char *_new_name)
 
 }
 
-void ORGate::update_state(void)
+void ORGate::update_state()
 {
   pOutputPin->putState((input_state & input_bit_mask) != 0);
 
@@ -461,17 +484,17 @@ void ORGate::update_state(void)
 Module * NOTGate::construct(const char *_new_name)
 {
 
-  NOTGate *a2gP = new NOTGate ;
+  NOTGate *a2gP = new NOTGate(_new_name) ;
 
-  a2gP->new_name(_new_name);
   a2gP->set_number_of_pins(2);
   a2gP->create_iopin_map();
+  a2gP->update_state();
 
   return a2gP;
 
 }
 
-NOTGate::NOTGate(void)
+NOTGate::NOTGate(const char *name) : LogicGate(name, "NOTGate")
 {
 #ifdef HAVE_GUI
 
@@ -479,22 +502,22 @@ NOTGate::NOTGate(void)
     set_widget(create_pixmap(not_pixmap));
 #endif
 }
-NOTGate::~NOTGate(void)
+NOTGate::~NOTGate()
 {
 
 }
 
-void NOTGate::update_state(void)
+void NOTGate::update_state()
 {
+  cout << name() << " update_state\n";
   pOutputPin->putState((input_state & input_bit_mask) == 0);
-
 }
 
 //--------------------------------------------------------------
 // construct
 
 
-XOR2Gate::XOR2Gate(void)
+XOR2Gate::XOR2Gate(const char *name) : XORGate(name, "XOR2Gate")
 {
 #ifdef HAVE_GUI
 
@@ -502,16 +525,15 @@ XOR2Gate::XOR2Gate(void)
     set_widget(create_pixmap(xor2_pixmap));
 #endif
 }
-XOR2Gate::~XOR2Gate(void)
+XOR2Gate::~XOR2Gate()
 {
 }
 
 Module * XOR2Gate::construct(const char *_new_name)
 {
 
-  XOR2Gate *o2gP = new XOR2Gate ;
+  XOR2Gate *o2gP = new XOR2Gate(_new_name);
 
-  o2gP->new_name(_new_name);
   o2gP->set_number_of_pins(3);
   o2gP->create_iopin_map();
 
@@ -519,7 +541,7 @@ Module * XOR2Gate::construct(const char *_new_name)
 
 }
 
-void XORGate::update_state(void)
+void XORGate::update_state()
 {
   bool bNewOutputState=false;
 
