@@ -58,9 +58,9 @@ public:
 
   virtual void setDrivenState( bool new_state);
 
-  Logic_Input (LogicGate *parent, unsigned int b, const char *opt_name=0) 
+  Logic_Input (LogicGate *parent, unsigned int b, const char *opt_name=0)
     : IOPIN(opt_name), LGParent(parent), m_iobit(b)
-    { 
+    {
     }
 
 };
@@ -72,9 +72,9 @@ private:
   unsigned int m_iobit;
 public:
 
-  Logic_Output (LogicGate *parent, unsigned int b,const char *opt_name=0) 
+  Logic_Output (LogicGate *parent, unsigned int b,const char *opt_name=0)
     : IO_bi_directional(opt_name), LGParent(parent), m_iobit(b)
-    { 
+    {
     }
 
 };
@@ -112,7 +112,7 @@ public:
 #ifdef HAVE_GUI
   GdkPixmap *pixmap;
 #endif
-  LogicGate(void);
+  LogicGate(const char *name, const char * desc);
   ~LogicGate(void);
 
   // Inheritances from the Package class
@@ -124,7 +124,7 @@ public:
   virtual int get_num_of_pins(void) {return number_of_pins;};
   void set_number_of_pins(int npins){number_of_pins=npins;};
 #ifdef HAVE_GUI
-  GtkWidget *create_pixmap(char **pixmap_data);
+  GtkWidget *create_pixmap(gchar **pixmap_data);
 #endif
 };
 
@@ -135,6 +135,8 @@ class ANDGate: public LogicGate
 public:
 
   virtual void update_state(void);
+  ANDGate(const char *name, const char * desc);
+  ~ANDGate(void);
 
 };
 
@@ -145,7 +147,7 @@ public:
   static Module *construct(const char *new_name);
   const virtual char *type(void) { return ("and2"); };
   // virtual void update_state(void);
-  AND2Gate(void);
+  AND2Gate(const char *);
   ~AND2Gate(void);
 
 };
@@ -154,6 +156,8 @@ class ORGate: public LogicGate
 {
 public:
 
+  ORGate(const char *name, const char * desc);
+  ~ORGate(void);
   virtual void update_state(void);
 
 };
@@ -167,15 +171,17 @@ public:
   //virtual void update_state(void);
   static Module *construct(const char *new_name);
   const virtual char *type(void) { return ("or2"); };
-  OR2Gate(void);
+  OR2Gate(const char *name);
   ~OR2Gate(void);
-  
+
 };
 
 class XORGate: public LogicGate
 {
 public:
 
+  XORGate(const char *name, const char * desc);
+  ~XORGate(void);
   virtual void update_state(void);
 
 };
@@ -189,9 +195,9 @@ public:
   //virtual void update_state(void);
   static Module *construct(const char *new_name);
   const virtual char *type(void) { return ("xor2"); };
-  XOR2Gate(void);
+  XOR2Gate(const char *name);
   ~XOR2Gate(void);
-  
+
 };
 
 class NOTGate: public LogicGate
@@ -202,9 +208,9 @@ public:
   static Module *construct(const char *new_name);
   const virtual char *type(void) { return ("not"); };
   virtual void update_state(void);
-  NOTGate(void);
+  NOTGate(const char *name);
   ~NOTGate(void);
-  
+
 };
 #endif //  __LOGIC_H__
 
