@@ -30,7 +30,7 @@ class _12bit_processor;
 class IOPIN;
 class OptionTraceType;
 
-extern instruction *disasm12 (pic_processor *cpu,unsigned int inst);
+extern instruction *disasm12 (pic_processor *cpu,unsigned int address,unsigned int inst);
 
 
 class _12bit_processor : public pic_processor
@@ -53,11 +53,13 @@ enum _12BIT_DEFINITIONS
 
   unsigned int pa_bits;   /* a CPU dependent bit-mask defining which of the program
                            * page bits in the status register are significant. */
+  OPTION_REG   *option_reg;
+
 
   virtual void reset(RESET_TYPE r);
+  virtual void save_state();
 
   virtual void create_symbols();
-  virtual void por();
   #define FILE_REGISTERS  0x100
   virtual unsigned int register_memory_size () const { return FILE_REGISTERS;};
   virtual void dump_registers();
@@ -68,7 +70,7 @@ enum _12BIT_DEFINITIONS
   virtual PROCESSOR_TYPE base_isa(){return _12BIT_PROCESSOR_;};
   virtual instruction * disasm (unsigned int address, unsigned int inst)
     {
-      return disasm12(this, inst);
+      return disasm12(this, address, inst);
     }
   void interrupt() { return; };
 
@@ -109,16 +111,19 @@ enum _12BIT_DEFINITIONS
       return 0;     // Assume only one register page.
     }
 
+  virtual void put_option_reg(unsigned int);
   virtual void option_new_bits_6_7(unsigned int);
 
   virtual unsigned int config_word_address() const {return 0xfff;};
   virtual bool set_config_word(unsigned int address, unsigned int cfg_word);
+  virtual void enter_sleep();
+  virtual void exit_sleep();
 
   _12bit_processor(const char *_name=0, const char *desc=0);
   virtual ~_12bit_processor();
 
 protected:
-  OptionTraceType *mOptionTT;  
+  OptionTraceType *mOptionTT;
 };
 
 #define cpu12 ( (_12bit_processor *)cpu)
