@@ -1,5 +1,7 @@
 /*
    Copyright (C) 1998 T. Scott Dattalo
+   Copyright (C) 2009 Roy R. Rankin
+
 
 This file is part of gpsim.
 
@@ -23,23 +25,24 @@ Boston, MA 02111-1307, USA.  */
 #include <iomanip>
 
 #include "../config.h"
-//#include "14bit-registers.h"
-//#include "14bit-instructions.h"
 #include "14bit-processors.h"
 #include "pic-ioports.h"
-
+#include "pic-registers.h"
 #include <string>
 #include "stimuli.h"
+#include "packages.h"
 
 //========================================================================
 // Generic Configuration word for the midrange family.
 
-class Generic14bitConfigWord : public ConfigMemory 
+class Generic14bitConfigWord : public ConfigWord
 {
 public:
   Generic14bitConfigWord(_14bit_processor *pCpu)
-    : ConfigMemory("CONFIG", 0x3fff, "Configuration Word", pCpu, 0x2007)
+    : ConfigWord("CONFIG", 0x3fff, "Configuration Word", pCpu, 0x2007)
   {
+    assert(pCpu);
+    pCpu->wdt.initialize(true);
   }
 
   enum {
@@ -59,12 +62,37 @@ public:
       gint64 diff = oldV ^ v;
 
       if (diff & WDTEN)
-	m_pCpu->wdt.initialize((v & WDTEN) == WDTEN);
+        m_pCpu->wdt.initialize((v & WDTEN) == WDTEN);
+
+      m_pCpu->config_modes->set_fosc01(v & (FOSC0 | FOSC1));
+      m_pCpu->config_modes->set_wdte((v&WDTEN)==WDTEN);
+      m_pCpu->config_modes->set_pwrte((v&PWRTEN)==PWRTEN);
 
     }
 
   }
 
+  virtual string toString()
+  {
+    gint64 i64;
+    get(i64);
+    int i = i64 &0xfff;
+
+    char buff[256];
+
+    snprintf(buff,sizeof(buff),
+             "$%3x\n"
+             " FOSC=%d - Clk source = %s\n"
+             " WDTEN=%d - WDT is %s\n"
+             " PWRTEN=%d - Power up timer is %s\n",
+             i,
+             i&(FOSC0|FOSC1), (i&FOSC0 ? (i&FOSC1 ? "EXTRC":"XT"):(i&FOSC1 ? "INTRC":"LP")),
+             (i&WDTEN?1:0), ((i&WDTEN) ? "enabled" : "disabled"),
+             (i&PWRTEN?1:0), ((i&PWRTEN) ? "disabled" : "enabled"));
+
+    return string(buff);
+  }
+
 };
 
 
@@ -72,42 +100,43 @@ public:
 _14bit_processor::_14bit_processor(const char *_name, const char *_desc)
   : pic_processor(_name,_desc), intcon(0)
 {
-  pc = new Program_Counter();
-  pc->set_trace_command(trace.allocateTraceType(new PCTraceType(this,1)));
-
+  pc = new Program_Counter("pc", "Program Counter", this);
+  pc->set_trace_command(); //trace.allocateTraceType(new PCTraceType(this,1)));
+  option_reg = new OPTION_REG(this,"option_reg");
+  stack = new Stack();
 }
 
 _14bit_processor::~_14bit_processor()
 {
-  //  delete pc;
+  delete_sfr_register(fsr);
+  delete_sfr_register(option_reg);
+  delete pc; pc=0;
 }
 
 //-------------------------------------------------------------------
 //
-// 
+//
 //    create
 //
 //  The purpose of this member function is to 'create' those things
 // that are unique to the 14-bit core processors.
 
-void _14bit_processor :: create (void)
+void _14bit_processor :: create ()
 {
 
   if(verbose)
     cout << "_14bit_processor create, type = " << isa() << '\n';
 
   pic_processor::create();
-  fsr = new FSR;
-  fsr->new_name("fsr");
-
+  fsr = new FSR(this, "fsr", "File Select Register for indirect addressing");
 }
 
 
 
 //-------------------------------------------------------------------
-void _14bit_processor::interrupt (void)
+void _14bit_processor::interrupt ()
 {
-  
+
   bp.clear_interrupt();
 
   stack->push(pc->value);
@@ -118,9 +147,11 @@ void _14bit_processor::interrupt (void)
 }
 
 //-------------------------------------------------------------------
-void _14bit_processor::por(void)
+void _14bit_processor::save_state()
 {
-  pic_processor::por();
+  pic_processor::save_state();
+
+  option_reg->put_trace_state(option_reg->value);
 }
 
 //-------------------------------------------------------------------
@@ -128,6 +159,11 @@ void _14bit_processor::option_new_bits_6_7(unsigned int bits)
 {
   cout << "14bit, option bits 6 and/or 7 changed\n";
 }
+//-------------------------------------------------------------------
+void _14bit_processor::put_option_reg(unsigned int val)
+{
+  option_reg->put(val);
+}
 
 
 //------------------------------------------------------------------
@@ -136,7 +172,7 @@ unsigned int _14bit_processor::get_program_memory_at_address(unsigned int addres
 {
   unsigned int uIndex = map_pm_address2index(address);
 
-  
+
   if (uIndex < program_memory_size())
     return  program_memory[uIndex] ? program_memory[uIndex]->get_opcode() : 0xffffffff;
 
@@ -146,112 +182,93 @@ unsigned int _14bit_processor::get_program_memory_at_address(unsigned int addres
 //-------------------------------------------------------------------
 void _14bit_processor::create_config_memory()
 {
-  m_configMemory = new ConfigMemory *[1];
-  *m_configMemory = new Generic14bitConfigWord(this);
+  m_configMemory = new ConfigMemory(this,1);
+  m_configMemory->addConfigWord(0,new Generic14bitConfigWord(this));
 }
 
-#if 0
 //-------------------------------------------------------------------
-class PortBSink;
 
-class PortBIntEdgeSink :: public SignalSink
+bool _14bit_processor::set_config_word(unsigned int address,unsigned int cfg_word)
 {
-public:
-  PortBIntEdgeSink(PortBSink *, unsigned int iobit);
-  virtual void setSinkState(bool);
-private:
-  PortBSink    *m_PortBSink;
-  unsigned int  m_bitMask;
-};
-class PortBSink
-{
-public:
-  PortBSink(PortRegister *portReg);
-  void setSink(unsigned int, bool);
-  void setPullups(bool);
-private:
-  PortRegister *m_port;
-  bool m_bPullupState;
-};
 
-//------------------------------------------------------------------------
-PortBIntEdgeSink::PortBIntEdgeSink(PortBSink *_PortBSink, unsigned int iobit)
-  : m_PortBSink(_PortBSink), m_bitMask(1<<iobit)
-{
-}
+  if((address == config_word_address()) && config_modes) {
 
-void PortBIntEdgeSink::setSinkState(bool bMewState)
-{
-  m_PortBSink->setSink(m_bitBask, bNewState);
-}
+    config_word = cfg_word;
 
-//------------------------------------------------------------------------
-PortBSink::PortBSink(PicPortRegister *portReg)
-  : m_port(portReg),
-    m_bPullupState(false)
-{
-  assert (portReg);
+    if (m_configMemory && m_configMemory->getConfigWord(0))
+      m_configMemory->getConfigWord(0)->set((int)cfg_word);
 
-  portReg->addSink(new PortBIntEdgeSink(this, 0), 0);
+    return true;
+  }
 
-  unsigned int mask = portReg->getEnableMask();
-  for (unsigned int i=0, m=1; mask; i++, m<<= 1)
-    if (mask & m) {
-      mask ^= m;
-      portReg->addSink(new PortBPinSink(this, i), i);
-    }
+  return false;
 
 }
 
-void PortBSink::setPullups(bool new_pullupState)
+//-------------------------------------------------------------------
+void _14bit_processor::enter_sleep()
 {
-  unsigned int mask = portReg->getEnableMask();
-  for (unsigned int i=0, m=1; mask; i++, m<<= 1)
-    if (mask & m) {
-      mask ^= m;
-      (*portReg)[i].update_pullup(new_pullupState);
-    }
+    tmr0.sleep();
+    pic_processor::enter_sleep();
+}
+
+ //-------------------------------------------------------------------
+void _14bit_processor::exit_sleep()
+{
+  if (m_ActivityState == ePASleeping)
+  {
+    tmr0.wake();
+    pic_processor::exit_sleep();
+  }
+
 }
-#endif
 
 //-------------------------------------------------------------------
 Pic14Bit::Pic14Bit(const char *_name, const char *_desc)
-  : _14bit_processor(_name,_desc)
+  : _14bit_processor(_name,_desc),
+    intcon_reg(this,"intcon","Interrupt Control")
 {
-  m_porta = new PicPortRegister("porta",8,0x1f);
-  m_trisa = new PicTrisRegister("trisa",m_porta);
+  m_porta = new PicPortRegister(this,"porta","", 8,0x1f);
+  m_trisa = new PicTrisRegister(this,"trisa","", m_porta, false);
 
-  tmr0.set_cpu(this, m_porta, 4);
+  tmr0.set_cpu(this, m_porta, 4, option_reg);
   tmr0.start(0);
 
-  m_portb = new PicPortBRegister("portb",8,0xff);
-  m_trisb = new PicTrisRegister("trisb",m_portb);
+  m_portb = new PicPortBRegister(this,"portb","",&intcon_reg,8,0xff);
+  m_trisb = new PicTrisRegister(this,"trisb","", m_portb, false);
 }
 
 //-------------------------------------------------------------------
 Pic14Bit::~Pic14Bit()
 {
+  //delete m_MCLR; <-- this is a package pin
+  delete m_MCLRMonitor;
 
+  delete_sfr_register(m_portb);
+  delete_sfr_register(m_trisb);
+
+  delete_sfr_register(m_porta);
+  delete_sfr_register(m_trisa);
 }
 //-------------------------------------------------------------------
-void Pic14Bit::create_symbols(void)
+void Pic14Bit::create_symbols()
 {
   pic_processor::create_symbols();
 
-  // add a special symbol for W
-  symbol_table.add_w(W);
+  addSymbol(W);
 
 }
 
 //-------------------------------------------------------------------
-void Pic14Bit::create_sfr_map(void)
+void Pic14Bit::create_sfr_map()
 {
- 
-  add_sfr_register(indf,    0x80);
+
   add_sfr_register(indf,    0x00);
+  alias_file_registers(0x00,0x00,0x80);
+  //add_sfr_register(indf,    0x00);
 
   add_sfr_register(&tmr0,   0x01);
-  add_sfr_register(&option_reg,  0x81, RegisterValue(0xff,0));
+  add_sfr_register(option_reg,  0x81, RegisterValue(0xff,0));
 
   add_sfr_register(pcl,     0x02, RegisterValue(0,0));
   add_sfr_register(status,  0x03, RegisterValue(0x18,0));
@@ -264,11 +281,12 @@ void Pic14Bit::create_sfr_map(void)
   add_sfr_register(m_portb, 0x06);
   add_sfr_register(m_trisb, 0x86, RegisterValue(0xff,0));
 
-  add_sfr_register(pclath,  0x8a, RegisterValue(0,0));
   add_sfr_register(pclath,  0x0a, RegisterValue(0,0));
+  //add_sfr_register(pclath,  0x8a, RegisterValue(0,0));
 
-  add_sfr_register(&intcon_reg, 0x8b, RegisterValue(0,0));
   add_sfr_register(&intcon_reg, 0x0b, RegisterValue(0,0));
+  //add_sfr_register(&intcon_reg, 0x8b, RegisterValue(0,0));
+  alias_file_registers(0x0a,0x0b,0x80);
 
   intcon = &intcon_reg;
 
@@ -281,3 +299,4 @@ void Pic14Bit::option_new_bits_6_7(unsigned int bits)
   m_portb->setRBPU( (bits & (1<<7)) == (1<<7));
   m_portb->setIntEdge((bits & (1<<6)) == (1<<6));
 }
+
