@@ -34,6 +34,12 @@ Boston, MA 02111-1307, USA.  */
 #include "xref.h"
 #define PCLATH_MASK              0x1f
 
+//#define DEBUG
+#if defined(DEBUG)
+#define Dprintf(arg) {printf("0x%06X %s() ",cycles.get(),__FUNCTION__); printf arg; }
+#else
+#define Dprintf(arg) {}
+#endif
 
 pic_processor *temp_cpu;
 // FIXME file_register::put_value has a useful feature...
@@ -95,6 +101,33 @@ void file_register::put_value(unsigned int new_value)
 
 #endif
 
+// Adjust internal RC oscillator frequency as per 12f675/629
+// Spec sheet does not give range so assume +/- 12.5% as per 16f88
+// The fact that base_freq is not 0. indicates the RC oscillator is being used
+// and thus an adjustment should be made.
+//
+// This will work for any number of adjustment bits in byte but must be left justified
+// and 1000000 centre frequency and 11111111 highest frequency
+void  OSCCAL::put(unsigned int new_value)
+{
+  int   adj = new_value & bit_mask;
+  float tune;
+  trace.raw(write_trace.get() | value.get());
+  value.put(adj);
+  if (base_freq > 0.)
+  {
+    adj  = adj -  0x80;
+    tune = (1. + 0.125 * adj / 0x80) * base_freq;
+    cpu_pic->set_frequency(tune);
+  }
+}
+
+void OSCCAL::set_freq(float new_base_freq)
+{
+  base_freq = new_base_freq;
+  put(value.get());
+}
+
 void  OSCTUNE::put(unsigned int new_value)
 {
   trace.raw(write_trace.get() | value.get());
@@ -108,17 +141,10 @@ void OSCCON::callback()
 }
 bool OSCCON::set_rc_frequency()
 {
-  unsigned int cfg_word = cpu_pic->get_config_word(cpu_pic->config_word_address());
   double base_frequency = 31.25e3;
-  enum {
-    CFG_FOSC0 = 1<<0,
-    CFG_FOSC1 = 1<<1,
-    CFG_FOSC2 = 1<<4,
-    CFG_MCLRE = 1<<5
-  };
 
-  if(   !((cfg_word & ( CFG_FOSC0 | CFG_FOSC1 | CFG_FOSC2)) == 0x10) && 
-	!((cfg_word & ( CFG_FOSC0 | CFG_FOSC1 | CFG_FOSC2)) == 0x11))
+
+  if (!cpu_pic->get_int_osc())
      return false;
 
     unsigned int new_IRCF = (value.get() & ( IRCF0 | IRCF1 | IRCF2)) >> 4;
@@ -189,46 +215,62 @@ void  OSCCON::put(unsigned int new_value)
 	    guint64 settle;
 
 	    settle = (guint64) (get_cycles().instruction_cps() * 4e-3);
-	    get_cycles().set_break(get_cycles().value + settle, this);
+	    get_cycles().set_break(get_cycles().get() + settle, this);
 	    new_value &= ~ IOFS;
 	}
 	else
 	    new_value |= IOFS;
   }
 }
+
+void WDTCON::put(unsigned int new_value)
+{
+  unsigned int masked_value = new_value & valid_bits;
+
+  trace.raw(write_trace.get() | value.get());
+
+  value.put(masked_value);
+
+  if (valid_bits > 1)
+      cpu_pic->wdt.set_prescale(masked_value >> 1);
+  cpu_pic->wdt.swdten((masked_value & SWDTEN) == SWDTEN);
+}
+void WDTCON::reset(RESET_TYPE r)
+{
+   putRV(por_value);
+}
 //
 //--------------------------------------------------
 // member functions for the FSR class
 //--------------------------------------------------
 //
+FSR::FSR(Processor *pCpu, const char *pName, const char *pDesc)
+  : sfr_register(pCpu, pName, pDesc)
+{}
+
 void  FSR::put(unsigned int new_value)
 {
   trace.raw(write_trace.get() | value.get());
-  //trace.register_write(address,value.get());
+
   value.put(new_value);
 }
 
 void  FSR::put_value(unsigned int new_value)
 {
-
   put(new_value);
 
   update();
   cpu_pic->indf->update();
-
-
 }
 
 
-unsigned int FSR::get(void)
+unsigned int FSR::get()
 {
-
   trace.raw(read_trace.get() | value.get());
-  //trace.register_read(address,value.get());
   return(value.get());
 }
 
-unsigned int FSR::get_value(void)
+unsigned int FSR::get_value()
 {
   return(value.get());
 }
@@ -239,16 +281,15 @@ unsigned int FSR::get_value(void)
 // member functions for the FSR_12 class
 //--------------------------------------------------
 //
-FSR_12::FSR_12(unsigned int _rpb, unsigned int _valid_bits)
-{
-  register_page_bits = _rpb;
-  valid_bits = _valid_bits;
-}
+FSR_12::FSR_12(Processor *pCpu, const char *pName, unsigned int _rpb, unsigned int _valid_bits)
+  : FSR(pCpu, pName, ""),
+    valid_bits(_valid_bits),
+    register_page_bits(_rpb)
+{}
 
 void  FSR_12::put(unsigned int new_value)
 {
   trace.raw(write_trace.get() | value.get());
-  //trace.register_write(address,value.get());
   value.put(new_value);
 
   /* The 12-bit core selects the register page using the fsr */
@@ -266,15 +307,14 @@ void  FSR_12::put_value(unsigned int new_value)
 }
 
 
-unsigned int FSR_12::get(void)
+unsigned int FSR_12::get()
 {
   unsigned int v = get_value();
   trace.raw(read_trace.get() | value.get());
-  //trace.register_read(address,v);
   return(v);
 }
 
-unsigned int FSR_12::get_value(void)
+unsigned int FSR_12::get_value()
 {
   // adjust for missing bits
   //cout << "FSR_12:get_value - valid_bits 0x" << hex << valid_bits << endl;
@@ -291,8 +331,10 @@ unsigned int FSR_12::get_value(void)
 
 //--------------------------------------------------
 
-Status_register::Status_register(void)
+Status_register::Status_register(Processor *pCpu, const char *pName, const char *pDesc)
+  : sfr_register(pCpu, pName, pDesc)
 {
+  rcon = NULL;
   break_point = 0;
   break_on_z =0;
   break_on_c =0;
@@ -301,6 +343,28 @@ Status_register::Status_register(void)
   write_mask = 0xff & ~STATUS_TO & ~STATUS_PD;
   new_name("status");
 }
+
+//--------------------------------------------------
+void Status_register::reset(RESET_TYPE r)
+{
+  switch (r) {
+
+  case POR_RESET:
+    putRV(por_value);
+    put_TO(1);
+    put_PD(1);
+    break;
+
+  case WDT_RESET:
+    put_TO(0);
+    break;
+
+  default:
+    break;
+  }
+
+}
+
 //--------------------------------------------------
 // put
 
@@ -321,7 +385,7 @@ void Status_register::put(unsigned int new_value)
 
 //--------------------------------------------------
 // get
-//unsigned int Status_register::get(void)
+//unsigned int Status_register::get()
 
 //--------------------------------------------------
 // put_Z
@@ -330,7 +394,7 @@ void Status_register::put(unsigned int new_value)
 
 //--------------------------------------------------
 // get_Z
-//unsigned int Status_register::get_Z(void)
+//unsigned int Status_register::get_Z()
 //--------------------------------------------------
 // put_C
 
@@ -338,7 +402,7 @@ void Status_register::put(unsigned int new_value)
 
 //--------------------------------------------------
 // get_C
-//unsigned int Status_register::get_C(void)
+//unsigned int Status_register::get_C()
 
 //--------------------------------------------------
 // put_Z_C_DC
@@ -346,15 +410,15 @@ void Status_register::put(unsigned int new_value)
 //--------------------------------------------------
 // member functions for the INDF class
 //--------------------------------------------------
-INDF::INDF(void)
+INDF::INDF(Processor *pCpu, const char *pName, const char *pDesc)
+  : sfr_register(pCpu, pName, pDesc)
 {
   fsr_mask = 0x7f;           // assume a 14bit core
   base_address_mask1 = 0;    //   "          "
   base_address_mask2 = 0xff; //   "          "
-  new_name("indf");
 }
 
-void INDF::initialize(void)
+void INDF::initialize()
 {
 
   switch(cpu_pic->base_isa()) {
@@ -370,7 +434,8 @@ void INDF::initialize(void)
     fsr_mask = 0x7f;
     break;
 
-  case _16BIT_PROCESSOR_:
+  case _PIC17_PROCESSOR_:
+  case _PIC18_PROCESSOR_:
     cout << "BUG: INDF::"<<__FUNCTION__<<". 16bit core uses a different class for indf.";
     break;
   default:
@@ -410,14 +475,14 @@ void INDF::put_value(unsigned int new_value)
 
   update();
   int r = (cpu_pic->fsr->get_value() + //cpu_pic->fsr->value + 
-	   ((cpu_pic->status->value.get() & base_address_mask1)<<1)& base_address_mask2);
+	   (((cpu_pic->status->value.get() & base_address_mask1)<<1)& base_address_mask2));
   if(r & fsr_mask) 
     cpu_pic->registers[r]->update();
 
 }
 
 
-unsigned int INDF::get(void)
+unsigned int INDF::get()
 {
 
   trace.raw(read_trace.get() | value.get());
@@ -430,7 +495,7 @@ unsigned int INDF::get(void)
     return(0);   // avoid infinite loop if fsr points to the indf
 }
 
-unsigned int INDF::get_value(void)
+unsigned int INDF::get_value()
 {
   int reg = (cpu_pic->fsr->get_value() +
 	       ((cpu_pic->status->value.get() & base_address_mask1)<<1) ) &  base_address_mask2;
@@ -440,57 +505,15 @@ unsigned int INDF::get_value(void)
     return(0);   // avoid infinite loop if fsr points to the indf
 }
 
-//--------------------------------------------------
-// member functions for the OPTION base class
-//--------------------------------------------------
-OPTION_REG::OPTION_REG(void)
-{
-  por_value = RegisterValue(0xff,0);
-  wdtr_value = RegisterValue(0xff,0);
-  value = por_value;
-  new_name("option");
-}
-
-void OPTION_REG::put(unsigned int new_value)
-{
-
-  //FIXME trace OPTION for 12bit processors is broken.
-  trace.raw(write_trace.get() | value.get());
-
-  unsigned int old_value = value.get();
-  value.put(new_value);
-
-  // First, check the tmr0 clock source bit to see if we are  changing from
-  // internal to external (or vice versa) clocks.
-  //if( (value ^ old_value) & T0CS)
-  //    cpu_pic->tmr0.new_clock_source();
-
-  // %%%FIX ME%%% - can changing the state of TOSE cause the timer to
-  // increment if tmr0 is being clocked by an external clock?
-
-  // Now check the rest of the tmr0 bits.
-  if( (value.get() ^ old_value) & (T0CS | T0SE | PSA | PS2 | PS1 | PS0))
-    cpu_pic->tmr0.new_prescale();
-
-  if( (value.get() ^ old_value) & (PSA | PS2 | PS1 | PS0))
-    cpu_pic->wdt.new_prescale();
-
-  if( (value.get() ^ old_value) & (BIT6 | BIT7))
-    cpu_pic->option_new_bits_6_7(value.get() & (BIT6 | BIT7));
-
-
-}
-
-
 
 
 //--------------------------------------------------
 // member functions for the PCL base class
 //--------------------------------------------------
-PCL::PCL() 
+PCL::PCL(Processor *pCpu, const char *pName, const char *pDesc)
+  : sfr_register(pCpu, pName, pDesc)
 {
   por_value = RegisterValue(0,0);
-  new_name("pcl");
 }
 
 // %%% FIX ME %%% breaks are different
@@ -511,12 +534,12 @@ void PCL::put_value(unsigned int new_value)
   // The gui (if present) will be updated in the pc->put_value call.
 }
 
-unsigned int PCL::get(void)
+unsigned int PCL::get()
 {
   return((value.get()+1) & 0xff);
 }
 
-unsigned int PCL::get_value(void)
+unsigned int PCL::get_value()
 {
   value.put(cpu_pic->pc->get_value() & 0xff);
   return(value.get());
@@ -534,9 +557,9 @@ void PCL::reset(RESET_TYPE r)
 // member functions for the PCLATH base class
 //--------------------------------------------------
 
-PCLATH::PCLATH(void)
+PCLATH::PCLATH(Processor *pCpu, const char *pName, const char *pDesc)
+  : sfr_register(pCpu, pName, pDesc)
 {
-  new_name("pclath");
 }
 
 void PCLATH::put(unsigned int new_value)
@@ -548,14 +571,16 @@ void PCLATH::put(unsigned int new_value)
 
 void PCLATH::put_value(unsigned int new_value)
 {
-
+  cout << "PCLATH::put_value(" << new_value << ")\n";
   value.put(new_value & PCLATH_MASK);
-  cpu_pic->pc->put_value( (cpu_pic->pc->get_value() & 0xffff00ff) | (value.get()<<8) );
+
+  // RP - I cannot think of a single possible reason I'd want to affect the real PC here!
+  //  cpu_pic->pc->put_value( (cpu_pic->pc->get_value() & 0xffff00ff) | (value.get()<<8) );
 
   // The gui (if present) will be updated in the pc->put_value call.
 }
 
-unsigned int PCLATH::get(void)
+unsigned int PCLATH::get()
 {
   trace.raw(read_trace.get() | value.get());
   //trace.register_read(address,value.get());
@@ -566,7 +591,8 @@ unsigned int PCLATH::get(void)
 // member functions for the PCON base class
 //--------------------------------------------------
 //
-PCON::PCON(void)
+PCON::PCON(Processor *pCpu, const char *pName, const char *pDesc)
+  : sfr_register(pCpu, pName, pDesc)
 {
   valid_bits = BOR | POR;
 }
@@ -580,7 +606,7 @@ void PCON::put(unsigned int new_value)
 
 
 //--------------------------------------------------
-Stack::Stack(void)
+Stack::Stack()
 {
 
   stack_warnings_flag = 0;   // Do not display over/under flow stack warnings
@@ -602,7 +628,7 @@ Stack::Stack(void)
 
 void Stack::push(unsigned int address)
 {
-
+  Dprintf(("pointer=%d address0x%x\n",pointer,address));
   // Write the address at the current point location. Note that the '& stack_mask'
   // implicitly handles the stack wrap around.
 
@@ -625,7 +651,7 @@ void Stack::push(unsigned int address)
 // Stack::pop
 //
 
-unsigned int Stack::pop(void)
+unsigned int Stack::pop()
 {
 
   // First decrement the stack pointer.
@@ -638,7 +664,8 @@ unsigned int Stack::pop(void)
   }
 
 
-  
+  Dprintf(("pointer=%d address0x%x\n",pointer,contents[pointer & stack_mask]));
+
   return(contents[pointer & stack_mask]);
 }
 
@@ -677,29 +704,6 @@ bool Stack::set_break_on_underflow(bool clear_or_set)
 }
 
 
-
-//--------------------------------------------------
-// WREG
-//
-/*
-unsigned int WREG::get(void)
-{
-  //  trace.read_W(value.get());
-  trace.raw(read_trace.get() | value.get());
-
-  return(value.get());
-}
-
-void WREG::put(unsigned int new_value)
-{
-
-  trace.raw(write_trace.get() | value.get());
-  value.put(new_value);
-
-}
-*/
-
-
 //========================================================================
 class WReadTraceObject : public RegisterReadTraceObject
 {
@@ -721,7 +725,7 @@ class WTraceType : public ProcessorTraceType
 public:
   WTraceType(Processor *_cpu, 
 	     unsigned int s)
-    : ProcessorTraceType(_cpu,s)
+    : ProcessorTraceType(_cpu,s,"W reg")
   {}
 
   TraceObject *decode(unsigned int tbi);
@@ -793,17 +797,11 @@ TraceObject * WTraceType::decode(unsigned int tbi)
 }
 
 
-WREG::WREG(void)
+WREG::WREG(Processor *pCpu, const char *pName, const char *pDesc)
+  : sfr_register(pCpu,pName,pDesc)
 {
-  new_name("W");
-}
-
-WREG::WREG(Processor *_cpu) 
-  : sfr_register(_cpu)
-{
-  new_name("W");
   if(cpu) {
-    unsigned int trace_command = trace.allocateTraceType(new WTraceType(get_cpu(),1));
+    unsigned int trace_command = trace.allocateTraceType(m_tt = new WTraceType(get_cpu(),1));
     RegisterValue rv(trace_command+(0<<22), trace_command+(2<<22));
     set_write_trace(rv);
     rv = RegisterValue(trace_command+(1<<22), trace_command+(3<<22));
@@ -812,3 +810,37 @@ WREG::WREG(Processor *_cpu)
 
 }
 
+WREG::~WREG()
+{
+  delete m_tt;
+}
+
+
+void WPU::put(unsigned int new_value)
+{
+    unsigned int masked_value = new_value & bit_mask;
+    int i;
+
+    trace.raw(write_trace.get() | value.get());
+
+    value.put(masked_value);
+    for(i = 0; i < 8; i++)
+    {
+	if((1<<i) & bit_mask)
+	{
+	    (&(*wpu_gpio)[i])->getPin().update_pullup((((1<<i) & masked_value) && wpu_pu )? '1' : '0', true);
+	}
+    }
+}
+/*
+ * enable/disable all WPU  pullups
+ */
+void WPU::set_wpu_pu(bool pullup_enable)
+{
+    if (pullup_enable != wpu_pu)
+    {
+	wpu_pu = pullup_enable;
+        put(value.get());	// change pull-ups based on value of WPU and gpio_pu
+    }
+}
+
