@@ -33,6 +33,7 @@ class _14bit_processor;
 
 #include "breakpoints.h"
 
+#include "rcon.h"
 class stimulus;  // forward reference
 class IOPIN;
 class source_stimulus;
@@ -49,11 +50,11 @@ class pic_processor;
 class FSR : public sfr_register
 {
 public:
-
+  FSR(Processor *, const char *pName, const char *pDesc=0);
   virtual void put(unsigned int new_value);
   virtual void put_value(unsigned int new_value);
-  virtual unsigned int get(void);
-  virtual unsigned int get_value(void);
+  virtual unsigned int get();
+  virtual unsigned int get_value();
 
 };
 
@@ -68,12 +69,13 @@ public:
   unsigned int valid_bits;
   unsigned int register_page_bits;   /* Only used by the 12-bit core to define
                                         the valid paging bits in the FSR. */
-  FSR_12(unsigned int _register_page_bits, unsigned int _valid_bits);
+  FSR_12(Processor *, const char *pName, 
+         unsigned int _register_page_bits, unsigned int _valid_bits);
 
   virtual void put(unsigned int new_value);
   virtual void put_value(unsigned int new_value);
-  virtual unsigned int get(void);
-  virtual unsigned int get_value(void);
+  virtual unsigned int get();
+  virtual unsigned int get_value();
 
 };
 
@@ -81,6 +83,7 @@ public:
 //---------------------------------------------------------
 // Status register
 //
+class RCON;
 
 class Status_register : public sfr_register
 {
@@ -112,12 +115,16 @@ public:
   unsigned int break_on_z,break_on_c;
   unsigned int rp_mask;
   unsigned int write_mask;    // Bits that instructions can modify
+  RCON *rcon;
 
-  Status_register(void);
+  Status_register(Processor *, const char *pName, const char *pDesc=0);
+  void reset(RESET_TYPE r);
+
+  void set_rcon(RCON *p_rcon) { rcon = p_rcon;}
 
   virtual void put(unsigned int new_value);
 
-  inline unsigned int get(void)
+  inline unsigned int get()
   {
     get_trace().raw(read_trace.get() | value.get());
     return(value.get());
@@ -131,7 +138,7 @@ public:
     value.put((value.get() & ~STATUS_Z) | ((new_z) ? STATUS_Z : 0));
   }
 
-  inline unsigned int get_Z(void)
+  inline unsigned int get_Z()
   {
     get_trace().raw(read_trace.get() | value.get());
     return( ( (value.get() & STATUS_Z) == 0) ? 0 : 1);
@@ -145,7 +152,7 @@ public:
     value.put((value.get() & ~STATUS_C) | ((new_c) ? STATUS_C : 0));
   }
 
-  unsigned int get_C(void)
+  unsigned int get_C()
   {
     get_trace().raw(read_trace.get() | value.get());
     return( ( (value.get() & STATUS_C) == 0) ? 0 : 1);
@@ -178,28 +185,46 @@ public:
 
   inline void put_PD(unsigned int new_pd)
   {
+    if (rcon)
+        rcon->put_PD(new_pd);
+    else
+    {
+        get_trace().raw(write_trace.get() | value.get());
+        value.put((value.get() & ~STATUS_PD) | ((new_pd) ? STATUS_PD : 0));
+    }  
+}
 
-    get_trace().raw(write_trace.get() | value.get());
-    value.put((value.get() & ~STATUS_PD) | ((new_pd) ? STATUS_PD : 0));
-  }
-
-  inline unsigned int get_PD(void)
+  inline unsigned int get_PD()
   {
-
-    get_trace().raw(read_trace.get() | value.get());
-    return( ( (value.get() & STATUS_PD) == 0) ? 0 : 1);
+    if (rcon)
+	return (rcon->get_PD());
+    else
+    {
+        get_trace().raw(read_trace.get() | value.get());
+        return( ( (value.get() & STATUS_PD) == 0) ? 0 : 1);
+    }
   }
 
   inline void put_TO(unsigned int new_to)
   {
-    get_trace().raw(write_trace.get() | value.get());
-    value.put((value.get() & ~STATUS_TO) | ((new_to) ? STATUS_TO : 0));
+    if (rcon)
+	rcon->put_TO(new_to);
+    else
+    {
+        get_trace().raw(write_trace.get() | value.get());
+        value.put((value.get() & ~STATUS_TO) | ((new_to) ? STATUS_TO : 0));
+    }
   }
 
-  inline unsigned int get_TO(void)
+  inline unsigned int get_TO()
   {
-    get_trace().raw(read_trace.get() | value.get());
-    return( ( (value.get() & STATUS_TO) == 0) ? 0 : 1);
+    if (rcon)
+	return(rcon->get_TO());
+    else
+    {
+        get_trace().raw(read_trace.get() | value.get());
+        return( ( (value.get() & STATUS_TO) == 0) ? 0 : 1);
+    }
   }
 
   // Special member function to set Z, C, DC, OV, and N for the 18cxxx family
@@ -240,7 +265,7 @@ public:
 	      ((new_value & 0xff)   ? 0 : STATUS_Z)   |
 	      ((new_value & 0x100)  ? 0 : STATUS_C)   |
 	      (((new_value ^ src1 ^ src2)&0x10) ? 0 : STATUS_DC) |
-	      (((src1 & ~src2 & ~new_value | new_value & ~src1 & src2) & 0x80) ? STATUS_OV : 0) |
+	      ((((src1 & ~src2 & ~new_value) | (new_value & ~src1 & src2)) & 0x80) ? STATUS_OV : 0) |
 	      ((new_value & 0x80)   ? STATUS_N : 0));
   }
 
@@ -271,7 +296,6 @@ public:
     return( (value.get()>>STATUS_FSR1_BIT) & 0x03);
   }
 
-  
 
 };
 
@@ -292,11 +316,11 @@ public:
   bool break_on_overflow;          /* Should over flow cause a break? */
   bool break_on_underflow;         /* Should under flow cause a break? */
 
-  Stack(void);
+  Stack();
   virtual ~Stack() {}
   virtual void push(unsigned int);
-  virtual unsigned int pop(void);
-  virtual void reset(void) {pointer = 0;};  // %%% FIX ME %%% reset may need to change 
+  virtual unsigned int pop();
+  virtual void reset() {pointer = 0;};  // %%% FIX ME %%% reset may need to change 
   // because I'm not sure how the stack is affected by a reset.
   virtual bool set_break_on_overflow(bool clear_or_set);
   virtual bool set_break_on_underflow(bool clear_or_set);
@@ -306,15 +330,16 @@ public:
 
 //---------------------------------------------------------
 // W register
+class WTraceType;
 
 class WREG : public sfr_register
 {
 public:
 
-  //void put(unsigned int new_value);
-  //unsigned int get(void);
-  WREG(void);
-  WREG(Processor *);
+  WREG(Processor *, const char *pName, const char *pDesc=0);
+  ~WREG();
+protected:
+  WTraceType *m_tt;
 };
 
 #include "tmr0.h"
@@ -329,61 +354,14 @@ public:
   unsigned int base_address_mask1;
   unsigned int base_address_mask2;
 
-  INDF(void);
+  INDF(Processor *, const char *pName, const char *pDesc=0);
   void put(unsigned int new_value);
   virtual void put_value(unsigned int new_value);
-  unsigned int get(void);
-  unsigned int get_value(void);
-  virtual void initialize(void);
+  unsigned int get();
+  unsigned int get_value();
+  virtual void initialize();
 };
 
-//---------------------------------------------------------
-// OPTION_REG - 
-
-class OPTION_REG : public sfr_register
-{
-public:
-
-enum
-  {
-    PS0    = 1<<0,
-    PS1    = 1<<1,
-    PS2    = 1<<2,
-    PSA    = 1<<3,
-    T0SE   = 1<<4,
-    T0CS   = 1<<5,
-    BIT6   = 1<<6,
-    BIT7   = 1<<7
-  };
-
-  unsigned int prescale;
-
-
-  OPTION_REG(void);
-
-  inline unsigned int get_prescale(void)
-    {
-      return value.get() & (PS0 | PS1 | PS2);
-    }
-
-  inline unsigned int get_psa(void)
-    {
-      return value.get() & PSA;
-    }
-
-  inline unsigned int get_t0cs(void)
-    {
-      return value.get() & T0CS;
-    }
-
-  inline unsigned int get_t0se(void)
-    {
-      return value.get() & T0SE;
-    }
-
-  void put(unsigned int new_value);
-
-};
 
 
 //---------------------------------------------------------
@@ -396,11 +374,11 @@ public:
 
   virtual void put(unsigned int new_value);
   virtual void put_value(unsigned int new_value);
-  virtual unsigned int get(void);
-  virtual unsigned int get_value(void);
+  virtual unsigned int get();
+  virtual unsigned int get_value();
   virtual void reset(RESET_TYPE r);
 
-  PCL(void);
+  PCL(Processor *, const char *pName, const char *pDesc=0);
 };
 
 //---------------------------------------------------------
@@ -412,9 +390,9 @@ class PCLATH : public sfr_register
 public:
   void put(unsigned int new_value);
   void put_value(unsigned int new_value);
-  unsigned int get(void);
+  unsigned int get();
 
-  PCLATH(void);
+  PCLATH(Processor *, const char *pName, const char *pDesc=0);
 };
 
 //---------------------------------------------------------
@@ -433,7 +411,7 @@ class PCON : public sfr_register
 
   void put(unsigned int new_value);
 
-  PCON(void);
+  PCON(Processor *, const char *pName, const char *pDesc=0);
 };
 
 class OSCCON;
@@ -455,8 +433,29 @@ class OSCTUNE : public  sfr_register
   };
   OSCCON *osccon;
                                                                                 
-  OSCTUNE(void) {
-    valid_bits = 6;
+  OSCTUNE(Processor *pCpu, const char *pName, const char *pDesc)
+    : sfr_register(pCpu,pName,pDesc), valid_bits(6)
+  {
+  }
+};
+
+
+// This class is used to trim the frequency of the internal RC clock
+//  111111 - Max freq
+//  100000 - no adjustment
+//  000000 - mix freq
+class OSCCAL : public  sfr_register
+{
+ public:
+
+  void put(unsigned int new_value);
+  void set_freq(float base_freq);
+  unsigned int bit_mask;
+  float base_freq;
+                                                                                
+  OSCCAL(Processor *pCpu, const char *pName, const char *pDesc, unsigned int bitMask)
+    : sfr_register(pCpu,pName,pDesc), bit_mask(bitMask), base_freq(0.)
+  {
   }
 };
 
@@ -480,8 +479,9 @@ class OSCCON : public  sfr_register,  public TriggerObject
     IRCF2 = 1<<6
   };
                                                                                 
-  OSCCON(void) {
-    valid_bits = 7;
+  OSCCON(Processor *pCpu, const char *pName, const char *pDesc)
+    : sfr_register(pCpu,pName,pDesc), valid_bits(7)
+  {
   }
 };
 
@@ -492,14 +492,63 @@ class WDTCON : public  sfr_register
   unsigned int valid_bits;
                                                                                 
   enum {
+    WDTPS3 = 1<<4,
+    WDTPS2 = 1<<3,
+    WDTPS1 = 1<<2,
+    WDTPS0 = 1<<1,
     SWDTEN = 1<<0
   };
                                                                                 
-  WDTCON(void) {
-    valid_bits = 1;
+  WDTCON(Processor *pCpu, const char *pName, const char *pDesc, unsigned int bits)
+    : sfr_register(pCpu,pName,pDesc), valid_bits(bits)
+  {
   }
+  virtual void put(unsigned int new_value);
+  virtual void reset(RESET_TYPE r);
                                                                                 
 };
 
 
+
+
+// Interrupt-On-Change GPIO Register
+class IOC :  public sfr_register
+{
+public:
+ unsigned int bit_mask;
+
+ IOC(Processor *pCpu, const char *pName, const char *pDesc)
+    : sfr_register(pCpu,pName,pDesc), bit_mask(0x3f)
+  {
+  }
+
+  void put(unsigned int new_value)
+  {
+    unsigned int masked_value = new_value & bit_mask;
+
+    get_trace().raw(write_trace.get() | value.get());
+    value.put(masked_value);
+  }
+
+};
+
+class PicPortRegister;
+// WPU set weak pullups on pin by pin basis
+//
+class WPU  : public  sfr_register
+{
+
+public:
+  unsigned int bit_mask;
+  PicPortRegister *wpu_gpio;
+  bool wpu_pu;
+
+  void put(unsigned int new_value);
+  void set_wpu_pu(bool pullup_enable);
+
+  WPU(Processor *pCpu, const char *pName, const char *pDesc, PicPortRegister* gpio, unsigned int mask=0x37)
+    : sfr_register(pCpu,pName,pDesc), bit_mask(mask), wpu_gpio(gpio), wpu_pu(false)
+  {
+  }
+};
 #endif
