@@ -1,6 +1,6 @@
 /*
    Copyright (C) 1998 T. Scott Dattalo
-   Copyright (C) 2006 Roy R Rankin
+   Copyright (C) 2006,2009 Roy R Rankin
 
 This file is part of gpsim.
 
@@ -46,23 +46,24 @@ Boston, MA 02111-1307, USA.  */
 // CCPRL
 //--------------------------------------------------
 
-CCPRL::CCPRL()
+CCPRL::CCPRL(Processor *pCpu, const char *pName, const char *pDesc)
+  : sfr_register(pCpu, pName, pDesc),
+    ccprh(0), ccpcon(0), tmrl(0)
 {
+}
 
-  ccprh = 0;
-  tmrl = 0;
-
+bool CCPRL::test_compare_mode()
+{
+    return tmrl && ccpcon && ccpcon->test_compare_mode();
 }
 
 void CCPRL::put(unsigned int new_value)
 {
 
   trace.raw(write_trace.get() | value.get());
-  //trace.register_write(address,value.get());
-
   value.put(new_value);
 
-  if(tmrl && tmrl->compare_mode)
+  if(test_compare_mode())
     start_compare_mode();   // Actually, re-start with new capture value.
 
 }
@@ -85,27 +86,32 @@ void CCPRL::capture_tmr()
     cout << "CCPRL captured: " << c << '\n';
 }
 
-void CCPRL::start_compare_mode()
+void CCPRL::start_compare_mode(CCPCON *ref)
 {
-
-  tmrl->compare_mode = 1;
-
   int capture_value = value.get() + 256*ccprh->value.get();
-  //cout << "start compare mode with capture value = " << capture_value << '\n';
-  tmrl->compare_value = capture_value;
-  tmrl->update();
+  if(verbose & 4)
+    cout << "start compare mode with capture value = " << capture_value << '\n';
+
+  if ( ref )
+  {
+    ccpcon = ref;
+  }
+  if ( ccpcon )
+    tmrl->set_compare_event ( capture_value, ccpcon );
+  else
+    cout << "CPRL: Attempting to set a compare callback with no CCPCON\n";
 }
 
 void CCPRL::stop_compare_mode()
 {
-  // If tmr1 is in the compare mode, then change to non-compare and update
+  // If this CCP is in the compare mode, then change to non-compare and cancel
   // the tmr breakpoint.
 
-  if(tmrl && tmrl->compare_mode)
-    {
-      tmrl->compare_mode = 0;
-      tmrl->update();
-    }
+  if(test_compare_mode())
+  {
+    tmrl->clear_compare_event ( ccpcon );
+  }
+  ccpcon = 0;
 }
 
 void CCPRL::start_pwm_mode()
@@ -141,22 +147,19 @@ void CCPRL::assign_tmr(TMRL *ptmr)
 // CCPRH
 //--------------------------------------------------
 
-CCPRH::CCPRH()
+CCPRH::CCPRH(Processor *pCpu, const char *pName, const char *pDesc)
+  : sfr_register(pCpu, pName, pDesc),
+    ccprl(0),pwm_mode(0),pwm_value(0)
 {
-
-  ccprl = 0;
-  pwm_mode = 0;
-  pwm_value = 0;
 }
 
 // put_value allows PWM code to put data
 void CCPRH::put_value(unsigned int new_value)
 {
-      trace.raw(write_trace.get() | value.get());
-      //trace.register_write(address,value.get());
-
-      value.put(new_value);
+  trace.raw(write_trace.get() | value.get());
+  value.put(new_value);
 }
+
 void CCPRH::put(unsigned int new_value)
 {
 
@@ -166,7 +169,7 @@ void CCPRH::put(unsigned int new_value)
     {
       put_value(new_value);
 
-      if(ccprl && ccprl->tmrl && ccprl->tmrl->compare_mode)
+      if(ccprl && ccprl->test_compare_mode())
 	ccprl->start_compare_mode();   // Actually, re-start with new capture value.
 
     }
@@ -192,10 +195,18 @@ public:
   {
     assert(m_ccp);
   }
-  char getState()
+  ~CCPSignalSource()
+  {
+  }
+
+  virtual char getState()
   {
     return m_ccp->getState();
   }
+  virtual void release() 
+  {
+    delete this;
+  }
 private:
   CCPCON *m_ccp;
 };
@@ -213,6 +224,10 @@ public:
     assert(_ccp);
   }
 
+  void release() 
+  {
+    delete this;
+  }
   void setSinkState(char new3State)
   {
     m_ccp->new_edge( new3State=='1' || new3State=='W');
@@ -224,17 +239,24 @@ private:
 //--------------------------------------------------
 // CCPCON
 //--------------------------------------------------
-CCPCON::CCPCON()
-  : m_PinModule(0),
+CCPCON::CCPCON(Processor *pCpu, const char *pName, const char *pDesc)
+  : sfr_register(pCpu, pName, pDesc),
+    m_PinModule(0),
     m_source(0),
     m_bInputEnabled(false),
     m_bOutputEnabled(false),
     m_cOutputState('?'),
     edges(0),
-    ccprl(0), pir_set(0), tmr2(0), adcon0(0)
-
+    ccprl(0), pir(0), tmr2(0), adcon0(0)
 {
 }
+
+CCPCON::~CCPCON()
+{
+  //delete m_sink;
+  //delete m_source;
+}
+
 void CCPCON::setIOpin(PinModule *new_PinModule)
 {
   Dprintf(("CCPCON::setIOpin\n"));
@@ -247,16 +269,19 @@ void CCPCON::setIOpin(PinModule *new_PinModule)
   m_source = new CCPSignalSource(this);
 
 }
-void CCPCON::setCrosslinks(CCPRL *_ccprl, PIR_SET *_pir_set, TMR2 *_tmr2)
+
+void CCPCON::setCrosslinks(CCPRL *_ccprl, PIR *_pir, TMR2 *_tmr2)
 {
   ccprl = _ccprl;
-  pir_set = _pir_set;
+  pir = _pir;
   tmr2 = _tmr2;
 }
+
 void CCPCON::setADCON(ADCON0 *_adcon0)
 {
   adcon0 = _adcon0;
 }
+
 char CCPCON::getState()
 {
   return m_bOutputEnabled ?  m_cOutputState : '?';
@@ -278,7 +303,7 @@ void CCPCON::new_edge(unsigned int level)
     case CAP_FALLING_EDGE:
       if (level == 0 && ccprl) {
 	ccprl->capture_tmr();
-	pir_set->set_ccpif();
+	pir->set_ccpif();
 	Dprintf(("--CCPCON caught falling edge\n"));
       }
       break;
@@ -286,7 +311,7 @@ void CCPCON::new_edge(unsigned int level)
     case CAP_RISING_EDGE:
       if (level && ccprl) {
 	ccprl->capture_tmr();
-	pir_set->set_ccpif();
+	pir->set_ccpif();
 	Dprintf(("--CCPCON caught rising edge\n"));
       }
       break;
@@ -294,7 +319,7 @@ void CCPCON::new_edge(unsigned int level)
     case CAP_RISING_EDGE4:
       if (level && --edges <= 0) {
 	ccprl->capture_tmr();
-	pir_set->set_ccpif();
+	pir->set_ccpif();
 	edges = 4;
 	Dprintf(("--CCPCON caught 4th rising edge\n"));
       }
@@ -305,7 +330,7 @@ void CCPCON::new_edge(unsigned int level)
     case CAP_RISING_EDGE16:
       if (level && --edges <= 0) {
 	ccprl->capture_tmr();
-	pir_set->set_ccpif();
+	pir->set_ccpif();
 	edges = 16;
 	Dprintf(("--CCPCON caught 4th rising edge\n"));
       }
@@ -349,30 +374,30 @@ void CCPCON::compare_match()
     case COM_SET_OUT:
       m_cOutputState = '1';
       m_PinModule->updatePinModule();
-      if (pir_set)
-	pir_set->set_ccpif();
+      if (pir)
+	pir->set_ccpif();
       Dprintf(("-- CCPCON setting compare output to 1\n"));
       break;
 
     case COM_CLEAR_OUT:
       m_cOutputState = '0';
       m_PinModule->updatePinModule();
-      if (pir_set)
-	pir_set->set_ccpif();
+      if (pir)
+	pir->set_ccpif();
       Dprintf(("-- CCPCON setting compare output to 0\n"));
       break;
 
     case COM_INTERRUPT:
-      if (pir_set)
-	pir_set->set_ccpif();
+      if (pir)
+	pir->set_ccpif();
       Dprintf(("-- CCPCON setting interrupt\n"));
       break;
 
     case COM_TRIGGER:
       if (ccprl)
 	ccprl->tmrl->clear_timer();
-      if (pir_set)
-	pir_set->set_ccpif();
+      if (pir)
+	pir->set_ccpif();
       if(adcon0)
 	adcon0->start_conversion();
 
@@ -443,7 +468,11 @@ void CCPCON::put(unsigned int new_value)
 	tmr2->stop_pwm(address);
       m_bInputEnabled = false;
       m_bOutputEnabled = false;
+
+      // RP - According to 16F87x data sheet section 8.2.1 clearing CCP1CON also clears the latch
+      m_cOutputState = '0';
       break;
+
     case CAP_FALLING_EDGE:
     case CAP_RISING_EDGE:
       edges = 0;
@@ -473,21 +502,22 @@ void CCPCON::put(unsigned int new_value)
 
     case COM_SET_OUT:
     case COM_CLEAR_OUT:
+      m_bOutputEnabled = true;
     case COM_INTERRUPT:
     case COM_TRIGGER:
-      ccprl->tmrl->ccpcon = this;
-      ccprl->start_compare_mode();
+      ccprl->start_compare_mode(this);
       ccprl->stop_pwm_mode();
       tmr2->stop_pwm(address);
 
-      if(adcon0)
-	adcon0->start_conversion();
+      // RP - just writing CCP2CON doesn't trigger the ADC; that only happens on a match
+      //if(adcon0)
+      //  adcon0->start_conversion();
 
-      m_bInputEnabled = true;
-      m_bOutputEnabled = false;
+      m_bInputEnabled = false;
       //if(adcon0) cout << "CCP triggering an A/D\n";
 
       break;
+
     case PWM0:
     case PWM1:
     case PWM2:
@@ -514,13 +544,76 @@ void CCPCON::put(unsigned int new_value)
 
 }
 
+bool CCPCON::test_compare_mode()
+{
+  switch(value.get() & (CCPM3 | CCPM2 | CCPM1 | CCPM0))
+  {
+    case ALL_OFF0:
+    case ALL_OFF1:
+    case ALL_OFF2:
+    case ALL_OFF3:
+    case CAP_FALLING_EDGE:
+    case CAP_RISING_EDGE:
+    case CAP_RISING_EDGE4:
+    case CAP_RISING_EDGE16:
+    case PWM0:
+    case PWM1:
+    case PWM2:
+    case PWM3:
+      return false;  
+      break;
+
+    case COM_SET_OUT:
+    case COM_CLEAR_OUT:
+    case COM_INTERRUPT:
+    case COM_TRIGGER:
+      return true;  
+      break;
+  }
+  return false;
+}
+
+
+
+// Attribute for frequency of external Timer1 oscillator
+class TMR1_Freq_Attribute : public Float
+{
+public:
+  TMR1_Freq_Attribute(Processor * _cpu, double freq); 
+
+  virtual void set(double d);
+  double get_freq();
+
+private:
+  Processor * cpu;
+};
+
+TMR1_Freq_Attribute::TMR1_Freq_Attribute(Processor * _cpu, double freq)
+  : Float("tmr1_freq",freq, " Tmr1 oscillator frequency."),
+    cpu(_cpu)
+{
+}
+
+double TMR1_Freq_Attribute::get_freq()
+{
+	double d;
+	Float::get(d);
+	return(d);
+}
+void TMR1_Freq_Attribute::set(double d)
+{
+    Float::set ( d );
+}
+
 //--------------------------------------------------
 // T1CON
 //--------------------------------------------------
-T1CON::T1CON()
-  : tmrl(0)
+T1CON::T1CON(Processor *pCpu, const char *pName, const char *pDesc)
+  : sfr_register(pCpu, pName, pDesc),
+    tmrl(0)
 {
 
+  pCpu->addSymbol(freq_attribute = new TMR1_Freq_Attribute(pCpu, 32768.));
   new_name("T1CON");
 
 }
@@ -537,12 +630,12 @@ void T1CON::put(unsigned int new_value)
     return;
   // First, check the tmr1 clock source bit to see if we are  changing from
   // internal to external (or vice versa) clocks.
-  if( diff & TMR1CS)
+  if( diff & (TMR1CS | T1OSCEN))
     tmrl->new_clock_source();
 
   if( diff & TMR1ON)
     tmrl->on_or_off(value.get() & TMR1ON);
-  else  if( diff & (T1CKPS0 | T1CKPS1))
+  else  if( diff & (T1CKPS0 | T1CKPS1 | TMR1GE | T1GINV))
     tmrl->update();
 
 }
@@ -556,7 +649,7 @@ unsigned int T1CON::get()
 
 unsigned int T1CON::get_prescale()
 {
-  return( ((value.get() &(T1CKPS0 | T1CKPS1)) >> 4) + cpu_pic->pll_factor);
+  return( ((value.get() &(T1CKPS0 | T1CKPS1)) >> 4) );
 }
 
 
@@ -564,8 +657,9 @@ unsigned int T1CON::get_prescale()
 //--------------------------------------------------
 // member functions for the TMRH base class
 //--------------------------------------------------
-TMRH::TMRH()
-  : tmrl(0)
+TMRH::TMRH(Processor *pCpu, const char *pName, const char *pDesc)
+  : sfr_register(pCpu, pName, pDesc),
+    tmrl(0)
 {
 
   value.put(0);
@@ -575,14 +669,21 @@ TMRH::TMRH()
 
 void TMRH::put(unsigned int new_value)
 {
-
   trace.raw(write_trace.get() | value.get());
   //trace.register_write(address,value.get());
-  value.put(new_value & 0xff);
   if(!tmrl)
+  {
+    value.put(new_value & 0xff);
     return;
-  tmrl->synchronized_cycle = get_cycles().value;
-  tmrl->last_cycle = tmrl->synchronized_cycle - (tmrl->value.get() + (value.get()<<8))*tmrl->prescale;
+  }
+
+  tmrl->set_ext_scale();
+  value.put(new_value & 0xff);
+  tmrl->synchronized_cycle = get_cycles().get();
+  tmrl->last_cycle = tmrl->synchronized_cycle 
+	- (gint64)((tmrl->value.get() + (value.get()<<8) 
+	* tmrl->prescale * tmrl->ext_scale) +0.5);
+
 
   if(tmrl->t1con->get_tmr1on())
     tmrl->update();
@@ -602,41 +703,112 @@ unsigned int TMRH::get_value()
   // If the TMR1 is being read immediately after being written, then
   // it hasn't had enough time to synchronize with the PIC's clock.
 
-  if(get_cycles().value <= tmrl->synchronized_cycle)
+  if(get_cycles().get() <= tmrl->synchronized_cycle)
     return value.get();
 
-  // If he TMR is not running then return.
+  // If the TMR is not running then return.
   if(!tmrl->t1con->get_tmr1on())
     return value.get();
 
   tmrl->current_value();
 
-  value.put(((tmrl->value_16bit)>>8) & 0xff);
-
   return(value.get());
   
 }
 
 
+//--------------------------------------------------
+//
+//--------------------------------------------------
+
+class TMRl_GateSignalSink : public SignalSink
+{
+public:
+  TMRl_GateSignalSink(TMRL *_tmr1l)
+    : m_tmr1l(_tmr1l)
+  {
+    assert(_tmr1l);
+  }
+
+  void release() 
+  {
+    delete this;
+  }
+  void setSinkState(char new3State)
+  {
+    m_tmr1l->IO_gate( new3State=='1' || new3State=='W');
+  }
+private:
+  TMRL *m_tmr1l;
+};
+
+//--------------------------------------------------
+// trivial class to represent a compare event reference
+//--------------------------------------------------
+
+class TMR1CapComRef
+{
+public:
+    TMR1CapComRef * next;
+
+    CCPCON * ccpcon;
+    unsigned int value;
+
+    TMR1CapComRef ( CCPCON * c, unsigned int v ) : ccpcon(c), value(v) {};
+};
+
 //--------------------------------------------------
 // member functions for the TMRL base class
 //--------------------------------------------------
-TMRL::TMRL()
-  : m_cState('?'), m_bExtClkEnabled(false), m_Interrupt(0)
+TMRL::TMRL(Processor *pCpu, const char *pName, const char *pDesc)
+  : sfr_register(pCpu, pName, pDesc),
+    m_cState('?'), m_GateState(false), m_bExtClkEnabled(false), 
+    m_sleeping(false), m_t1gss(true), m_Interrupt(0)
 {
 
   value.put(0);
   synchronized_cycle=0;
   prescale_counter=prescale=1;
   break_value = 0x10000;
-  compare_value = 0;
-  compare_mode = 0;
   last_cycle = 0;
+  future_cycle = 0;
 
+  ext_scale = 1.;
   tmrh    = 0;
   t1con   = 0;
-  ccpcon  = 0;
-  new_name("TMRL");
+  compare_queue = 0;
+}
+
+TMRL::~TMRL()
+{
+  if (m_Interrupt)
+    m_Interrupt->release();
+}
+/*
+ * If we are similating an external RTC crystal for timer1,
+ * compute scale factor between crsytal speed and processor
+ * instruction cycle rate
+ */ 
+void TMRL::set_ext_scale() 
+{ 
+    current_value();
+    if (t1con->get_t1oscen() && t1con->get_tmr1cs())
+    {
+    	ext_scale = get_cycles().instruction_cps()/
+	            t1con->freq_attribute->get_freq();
+    }
+    else
+	ext_scale = 1.;
+
+    if (future_cycle)
+    {
+      last_cycle = get_cycles().get()
+			- (gint64)(value_16bit *( prescale * ext_scale) + 0.5);
+    }
+}
+
+void TMRL::release()
+{
 
 }
 
@@ -648,6 +820,7 @@ void TMRL::setIOpin(PinModule *extClkSource)
     extClkSource->addSink(this);
 }
 
+
 void TMRL::setSinkState(char new3State)
 {
   if (new3State != m_cState) {
@@ -658,6 +831,98 @@ void TMRL::setSinkState(char new3State)
   }
 }
 
+void TMRL::set_compare_event ( unsigned int value, CCPCON *host )
+{
+  TMR1CapComRef * event = compare_queue;
+
+  if ( host )
+  {
+    while ( event )
+    {
+      if ( event->ccpcon == host )
+      {
+        event->value = value;
+        update();
+        return;
+      }
+      event = event->next;
+    }
+    event = new TMR1CapComRef ( host, value );
+    event->next = compare_queue;
+    compare_queue = event;
+    update();
+  }
+  else
+      cout << "TMRL::set_compare_event called with no CAPCOM\n";
+}
+
+void TMRL::clear_compare_event ( CCPCON *host )
+{
+  TMR1CapComRef * event = compare_queue;
+  TMR1CapComRef * * eptr = &compare_queue;
+
+  while ( event )
+  {
+    if ( event->ccpcon == host )
+    {
+      *eptr = event->next;
+      delete event;
+      update();
+      return;
+    }
+    eptr = &event->next;
+    event = event->next;
+  }
+}
+
+void TMRL::setGatepin(PinModule *extGateSource)
+{
+  Dprintf(("TMRL::setGatepin\n"));
+
+  if (extGateSource)
+    extGateSource->addSink(new TMRl_GateSignalSink(this));
+}
+
+void TMRL::set_T1GSS(bool arg)
+{
+
+    m_t1gss = arg;
+    if (m_t1gss)
+	IO_gate(m_io_GateState);
+    else
+	compare_gate(m_compare_GateState);
+}
+void TMRL::compare_gate(bool state)
+{
+  m_compare_GateState = state;
+  if (!m_t1gss && m_GateState != state)
+  {
+    m_GateState = state;
+    
+    Dprintf(("TMRL::compare_gate state %d \n", state));
+
+    if (t1con->get_tmr1GE())
+    {
+	update();
+    }
+  }
+}
+void TMRL::IO_gate(bool state)
+{
+  m_io_GateState = state;
+  if (m_t1gss && m_GateState != state)
+  {
+    m_GateState = state;
+    
+    Dprintf(("TMRL::IO_gate state %d \n", state));
+
+    if (t1con->get_tmr1GE())
+    {
+	update();
+    }
+  }
+}
+
 //------------------------------------------------------------
 // setInterruptSource()
 //
@@ -678,6 +943,10 @@ void TMRL::increment()
   if(--prescale_counter == 0) {
     prescale_counter = prescale;
 
+  // In synchronous counter mode prescaler works but rest of tmr1 does not
+  if (t1con->get_t1sync() == 0 && m_sleeping)
+    return;
+
     // If TMRH/TMRL have been manually changed, we'll want to 
     // get the up-to-date values;
 
@@ -690,7 +959,11 @@ void TMRL::increment()
     tmrh->value.put((value_16bit >> 8) & 0xff);
     value.put(value_16bit & 0xff);
     if(value_16bit == 0 && m_Interrupt)
+    {
+      if (verbose & 4)
+          cout << "TMRL:increment interrupt now=" << dec << get_cycles().get() << " value_16bit "  << value_16bit << endl;
       m_Interrupt->Trigger();
+    }
   }
 
 }
@@ -708,7 +981,8 @@ void TMRL::on_or_off(int new_state)
     // Compute the "effective last cycle", i.e. the cycle
     // at which TMR1 was last 0 had it always been counting.
 
-    last_cycle = get_cycles().value - value_16bit*prescale;
+    last_cycle = (gint64)(get_cycles().get() -
+	(value.get() + (tmrh->value.get()<<8)) * prescale * ext_scale + 0.5);
     update();
   }
   else {
@@ -717,8 +991,11 @@ void TMRL::on_or_off(int new_state)
 
     // turn off the timer and save the current value
     current_value();
-    value.put(value_16bit & 0xff);
-    tmrh->value.put((value_16bit>>8) & 0xff);
+    if (future_cycle)
+    {
+        get_cycles().clear_break(this);
+	future_cycle = 0;
+    }
   }
 
 }
@@ -731,58 +1008,96 @@ void TMRL::on_or_off(int new_state)
 void TMRL::update()
 {
 
-  Dprintf(("TMR1 update 0x%llx\n",get_cycles().value));
+  Dprintf(("TMR1 update 0x%llx\n",get_cycles().get()));
 
-  if(t1con->get_tmr1on()) {
-
-    if(t1con->get_tmr1cs()) {
-      cout << "TMRl::update - external clock is not fully implemented\n";
-    } else {
-	
+  // The second part of the if will always be true unless TMR1 Gate enable
+  // pin has been defined by a call to TMRL::setGatepin()
+  //
+  bool gate = t1con->get_t1GINV() ? m_GateState : !m_GateState;
+  if(t1con->get_tmr1on() && (t1con->get_tmr1GE() ? gate : true)) 
+  {
+    if(t1con->get_tmr1cs() && t1con->get_t1oscen()) 
+    {
+	/*
+	 external timer1 clock runs off a crystal which is typically
+	 32768 Hz and is independant on the instruction clock, but
+	 gpsim runs on the instruction clock. Ext_scale is the ratio 
+	 of these two clocks so the breakpoint can be adjusted to be
+	 triggered at the correct time. 
+	*/
       if(verbose & 0x4)
-	cout << "Internal clock\n";
+	cout << "Tmr1 External clock\n";
+      
+    }
+    else if(t1con->get_tmr1cs() && !  t1con->get_t1oscen()) 
+    {
+      prescale = 1 << t1con->get_prescale();
+      prescale_counter = prescale;
+      set_ext_scale();
+      return;
+    }
+    else
+    {
+      if(verbose & 0x4)
+	cout << "Tmr1 Internal clock\n";
+    }
 
-      current_value();
+    set_ext_scale();
 
 
       // Note, unlike TMR0, anytime something is written to TMRL, the 
-      // prescaler is unaffected.
+      // prescaler is unaffected on the P18 processors. However, it is
+      // reset on the p16f88 processor, which is how the current code
+      // works. This only effects the external drive mode.
 
       prescale = 1 << t1con->get_prescale();
       prescale_counter = prescale;
 
-      //cout << "TMRL: Current prescale " << prescale << '\n';
-      //  synchronized_cycle = cycles.value + 2;
-      synchronized_cycle = get_cycles().value;
+      if(verbose & 0x4)
+        cout << "TMRL: Current prescale " << prescale << ", ext scale " << ext_scale << '\n';
+      //  synchronized_cycle = cycles.get() + 2;
+      synchronized_cycle = get_cycles().get();
+
+
+      last_cycle = synchronized_cycle  
+			- (gint64)(value_16bit *( prescale * ext_scale) + 0.5);
 
-      last_cycle = synchronized_cycle - value_16bit * prescale;
 
       break_value = 0x10000;  // Assume that a rollover will happen first.
 
-      if(compare_mode)
+      for ( TMR1CapComRef * event = compare_queue; event; event = event->next )
+      {
+        if(verbose & 0x4)
+	    cout << "compare mode on " << event->ccpcon << ", value = " << event->value << '\n';
+	if ( event->value > value_16bit && event->value < break_value )
 	{
-	  //cout << "compare mode. compare_value = " << compare_value << '\n';
-	  if(compare_value > value_16bit)
-	    {
-	      // A compare interrupt is going to happen before the timer
-	      // will rollover.
-	      break_value = compare_value - value_16bit;
-	    }
+	    // A compare interrupt is going to happen before the timer
+	    // will rollover.
+	    break_value = event->value;
 	}
+      }
+      if(verbose & 0x4)
+        cout << "TMR1 now at " << value_16bit << ", next event at " << break_value << '\n';
 
-      guint64 fc = get_cycles().value + (break_value - value_16bit) * prescale;
+      guint64 fc = get_cycles().get() 
+		+ (guint64)((break_value - value_16bit) * prescale * ext_scale);
 
       if(future_cycle)
 	get_cycles().reassign_break(future_cycle, fc, this);
       else
 	get_cycles().set_break(fc, this);
 
-      //cout << "TMR1: update; new break cycle = " << fc << '\n';
       future_cycle = fc;
     }
-    }
   else
     {
+      // turn off the timer and save the current value
+      if (future_cycle)
+      {
+        current_value();
+        get_cycles().clear_break(this);
+	future_cycle = 0;
+      } 
       //cout << "TMR1: not running\n";
     }
 }
@@ -790,6 +1105,7 @@ void TMRL::update()
 void TMRL::put(unsigned int new_value)
 {
 
+  set_ext_scale();
   trace.raw(write_trace.get() | value.get());
   //trace.register_write(address,value.get());
   value.put(new_value & 0xff);
@@ -797,8 +1113,11 @@ void TMRL::put(unsigned int new_value)
   if (!tmrh || !t1con)
     return;
 
-  synchronized_cycle = get_cycles().value;
-  last_cycle = synchronized_cycle - ( value.get() + (tmrh->value.get()<<8)) * prescale;
+  synchronized_cycle = get_cycles().get();
+  last_cycle = synchronized_cycle - (gint64)(( value.get() 
+	+ (tmrh->value.get()<<8)) * prescale * ext_scale + 0.5);
+
+  current_value();
 
   if(t1con->get_tmr1on())
     update();
@@ -816,7 +1135,7 @@ unsigned int TMRL::get_value()
 {
   // If the TMRL is being read immediately after being written, then
   // it hasn't had enough time to synchronize with the PIC's clock.
-  if(get_cycles().value <= synchronized_cycle)
+  if(get_cycles().get() <= synchronized_cycle)
     return value.get();
 
   // If TMRL is not on, then return the current value
@@ -825,51 +1144,83 @@ unsigned int TMRL::get_value()
 
   current_value();
 
-  value.put(value_16bit & 0xff);
-
   return(value.get());
 }
 
 //%%%FIXME%%% inline this
+// if break inactive (future_cycle == 0), just read the TMR1H and TMR1L 
+// registers otherwise compute what the register should be and then
+// update TMR1H and TMR1L.
+// RP: Using future_cycle here is not strictly right. What we really want is
+// the condition "TMR1 is running on a GPSIM-generated clock" (as opposed
+// to being off, or externally clocked by a stimulus). The presence of a
+// breakpoint is _usually_ a good indication of this, but not while we're
+// actually processing that breakpoint. For the time being, we work around
+// this by calling current_value "redundantly" in callback()
+//
 void TMRL::current_value()
 {
-  if(t1con->get_tmr1cs())
+  if(future_cycle == 0)
     value_16bit = tmrh->value.get() * 256 + value.get();
   else
-    value_16bit = (unsigned int)((get_cycles().value - last_cycle)/ prescale) & 0xffff;
+  {
+    value_16bit = (guint64)((get_cycles().get() - last_cycle)/ 
+		(prescale* ext_scale)) & 0xffff;
+    value.put(value_16bit & 0xff);
+    tmrh->value.put((value_16bit>>8) & 0xff);
+  }
 }
 
 unsigned int TMRL::get_low_and_high()
 {
-
   // If the TMRL is being read immediately after being written, then
   // it hasn't had enough time to synchronize with the PIC's clock.
-  if(get_cycles().value <= synchronized_cycle)
+  if(get_cycles().get() <= synchronized_cycle)
     return value.get();
 
-  //  value_16bit = (cycles.value.lo - last_cycle)/ prescale;
-
   current_value();
 
-  value.put(value_16bit & 0xff);
   trace.raw(read_trace.get() | value.get());
-  //trace.register_read(address, value.get());
 
-  tmrh->value.put((value_16bit>>8) & 0xff);
   trace.raw(tmrh->read_trace.get() | tmrh->value.get());
-  //trace.register_read(tmrh->address, tmrh->value.get());
 
   return(value_16bit);
   
 }
 
+// set m_bExtClkEnable is tmr1 is being clocked by an external stimulus
 void TMRL::new_clock_source()
 {
 
-  if(t1con->get_tmr1cs())
-    m_bExtClkEnabled = true;
+  m_bExtClkEnabled = false;
+
+  current_value();
+
+  if(t1con->get_tmr1cs() && ! t1con->get_t1oscen()) // external stimuli
+  {
+      if(verbose & 0x4)
+	cout << "Tmr1 External Stimuli\n";
+      if (future_cycle)
+      {
+        // Compute value_16bit with old prescale and ext_scale
+      	current_value();
+        get_cycles().clear_break(this);
+        future_cycle = 0;
+      }
+      prescale = 1 << t1con->get_prescale();
+      prescale_counter = prescale;
+      set_ext_scale();
+      m_bExtClkEnabled = true;
+  }
+  else if(! t1con->get_tmr1cs() && ! t1con->get_t1oscen()) // Fosc/4
+  {
+      if(verbose & 0x4)
+	cout << "Tmr1 Fosc/4 \n";
+      put(value.get());
+  }
   else {
-    m_bExtClkEnabled = false;
+      if(verbose & 0x4)
+	cout << "Tmr1 External Crystal\n";
     put(value.get());    // let TMRL::put() set a cycle counter break point
   }
 }
@@ -883,8 +1234,9 @@ void TMRL::new_clock_source()
 void TMRL::clear_timer()
 {
 
-  last_cycle = get_cycles().value;
-  //cout << "TMR1 has been cleared\n";
+  last_cycle = get_cycles().get();
+  if(verbose & 0x4)
+    cout << "TMR1 has been cleared\n";
 }
 
 // TMRL callback is called when the cycle counter hits the break point that
@@ -894,44 +1246,61 @@ void TMRL::clear_timer()
 void TMRL::callback()
 {
 
+
   if(verbose & 4)
     cout << "TMRL::callback\n";
 
   // If TMRL is being clocked by the external clock, then at some point
   // the simulate code must have switched from the internal clock to
   // external clock. The cycle break point was still set, so just ignore it.
-  if(t1con->get_tmr1cs())
+  if(t1con->get_tmr1cs() && ! t1con->get_t1oscen())
     {
+      if(verbose & 4)
+        cout << "TMRL:callback No oscillator\n";
+      value.put(0);
+      tmrh->value.put(0);
       future_cycle = 0;  // indicates that TMRL no longer has a break point
       return;
     }
 
+  current_value();      // Because this relies on future_cycle, we must call it before clearing that
+
   future_cycle = 0;     // indicate that there's no break currently set
-  //cout << "in tmrl callback break_value = " << break_value << '\n';
 
   if(break_value < 0x10000)
-    {
-
+  {
       // The break was due to a "compare"
 
-      //cout << "TMR1 break due to compare "  << hex << cycles.value.lo << '\n';
-      ccpcon->compare_match();
+      if ( value_16bit != break_value )
+          cout << "TMR1 compare break: value=" << value_16bit << " but break_value=" << break_value << '\n';
 
+      if(verbose & 4)
+        cout << "TMR1 break due to compare "  << hex << get_cycles().get() << '\n';
+
+      for ( TMR1CapComRef * event = compare_queue; event; event = event->next )
+      {
+	if ( event->value == break_value )
+	{
+	    // This CCP channel has a compare at this time
+          event->ccpcon->compare_match();
+	}
+      }
     }
   else
     {
 
       // The break was due to a roll-over
 
-      //cout<<"TMRL rollover: " << hex << cycles.value.lo << '\n';
+      //cout<<"TMRL rollover: " << hex << cycles.get() << '\n';
       if (m_Interrupt)
 	m_Interrupt->Trigger();
 
       // Reset the timer to 0.
 
-      synchronized_cycle = get_cycles().value;
+      synchronized_cycle = get_cycles().get();
       last_cycle = synchronized_cycle;
-
+      value.put(0);
+      tmrh->value.put(0);
     }
 
   update();
@@ -947,16 +1316,44 @@ void TMRL::callback_print()
 }
 
 
+//---------------------------
+
+void TMRL::sleep()
+{
+    m_sleeping = true;
+    Dprintf(("TMRL::sleep t1sysc %d\n", t1con->get_t1sync()));
+    // If tmr1 is running off Fosc/4 this assumes Fosc stops during sleep
+    if (  t1con->get_tmr1on() && t1con->get_tmr1cs() == 0)
+    {
+      if (future_cycle)
+      {
+        current_value();
+        get_cycles().clear_break(this);
+	future_cycle = 0;
+      } 
+    }
+}
+
+//---------------------------
+
+void TMRL::wake()
+{
+    m_sleeping = false;
+    Dprintf(("TMRL::wake\n"));
+    if (  t1con->get_tmr1on() && t1con->get_tmr1cs() == 0)
+    {
+	update();
+    }
+}
+
 //--------------------------------------------------
 // member functions for the PR2 base class
 //--------------------------------------------------
 
-PR2::PR2()
-  :  tmr2(0)
+PR2::PR2(Processor *pCpu, const char *pName, const char *pDesc)
+  : sfr_register(pCpu, pName, pDesc),
+    tmr2(0)
 {
-
-  new_name("PR2");
-
 }
 
 void PR2::put(unsigned int new_value)
@@ -980,12 +1377,10 @@ void PR2::put(unsigned int new_value)
 // member functions for the T2CON base class
 //--------------------------------------------------
 
-T2CON::T2CON()
-  : tmr2(0)
+T2CON::T2CON(Processor *pCpu, const char *pName, const char *pDesc)
+  : sfr_register(pCpu, pName, pDesc),
+    tmr2(0)
 {
-
-  new_name("T2CON");
-
 }
 
 void T2CON::put(unsigned int new_value)
@@ -995,7 +1390,6 @@ void T2CON::put(unsigned int new_value)
   unsigned int diff = value.get() ^ new_value;
   value.put(new_value);
 
-
   if (tmr2) {
     
     tmr2->new_pre_post_scale();
@@ -1011,15 +1405,18 @@ void T2CON::put(unsigned int new_value)
 //--------------------------------------------------
 // member functions for the TMR2 base class
 //--------------------------------------------------
-TMR2::TMR2()
-  : pr2(0), pir_set(0), t2con(0), ccp1con(0), ccp2con(0)
+TMR2::TMR2(Processor *pCpu, const char *pName, const char *pDesc)
+  : sfr_register(pCpu, pName, pDesc),
+    pwm_mode(0),
+    update_state(TMR2_PWM1_UPDATE | TMR2_PWM2_UPDATE | TMR2_PR2_UPDATE),
+    last_update(0),
+    prescale(1),
+    prescale_counter(0),
+    last_cycle(0),
+    pr2(0), pir_set(0), t2con(0), ccp1con(0), ccp2con(0)
 {
-  update_state = TMR2_PWM1_UPDATE | TMR2_PWM2_UPDATE | TMR2_PR2_UPDATE;
-  pwm_mode = 0;
   value.put(0);
   future_cycle = 0;
-  prescale=1;
-  new_name("TMR2");
 }
 
 void TMR2::callback_print() 
@@ -1051,7 +1448,7 @@ void TMR2::on_or_off(int new_state)
     // Compute the "effective last cycle", i.e. the cycle
     // at which TMR2 was last 0 had it always been counting.
 
-    last_cycle = get_cycles().value - value.get()*prescale;
+    last_cycle = get_cycles().get() - value.get()*prescale;
     update();
   }
   else {
@@ -1151,7 +1548,7 @@ void TMR2::stop_pwm(unsigned int ccp_address)
 void TMR2::update(int ut)
 {
 
-  //cout << "TMR2 update. cpu cycle " << hex << cycles.value <<'\n';
+  //cout << "TMR2 update. cpu cycle " << hex << cycles.get() <<'\n';
 
   if(t2con->get_tmr2on()) {
 
@@ -1167,7 +1564,7 @@ void TMR2::update(int ut)
       // be due to tmr2 matching pr2)
 
       break_value = 1 + pr2->value.get();
-      guint64 fc = get_cycles().value + (break_value - value.get()) * prescale;
+      guint64 fc = get_cycles().get() + (break_value - value.get()) * prescale;
 
       last_update = TMR2_PR2_UPDATE;
 
@@ -1245,9 +1642,8 @@ void TMR2::update(int ut)
 }
 void TMR2::put(unsigned int new_value)
 {
-  current_value();
 
-  unsigned int old_value = value.get();
+  unsigned int old_value = get_value();
 
 
   trace.raw(write_trace.get() | value.get());
@@ -1261,7 +1657,7 @@ void TMR2::put(unsigned int new_value)
       // which means there's a cycle break point set on TMR2 that needs to
       // be moved to a new cycle.
 
-      guint64 current_cycle = get_cycles().value;
+      guint64 current_cycle = get_cycles().get();
       unsigned int delta = (future_cycle - last_cycle);
       int shift = (new_value - old_value) * prescale;
 
@@ -1386,13 +1782,13 @@ void TMR2::new_pre_post_scale()
       if (prescale != old_prescale)	// prescaler value change
       {
 	// togo is number of cycles to next callback based on new prescaler.
-	guint64 togo = (future_cycle - get_cycles().value) * prescale / old_prescale;
+	guint64 togo = (future_cycle - get_cycles().get()) * prescale / old_prescale;
 
 	if (!togo)	// I am not sure this can happen RRR
 	    callback();
 	else
 	{
-	   guint64 fc = togo + get_cycles().value;
+	  guint64 fc = togo + get_cycles().get();
 
           get_cycles().reassign_break(future_cycle, fc, this);
           future_cycle = fc;
@@ -1405,7 +1801,7 @@ void TMR2::new_pre_post_scale()
 
       if (value.get() == pr2->value.get()) // TMR2 == PR2
       {
-        future_cycle = get_cycles().value;
+        future_cycle = get_cycles().get();
         get_cycles().set_break(future_cycle, this);
 	callback();
       }
@@ -1413,15 +1809,15 @@ void TMR2::new_pre_post_scale()
       {
 	cout << "Warning TMR2 turned on with TMR2 greater than PR2\n";
 	// this will cause TMR2 to wrap
-        future_cycle  = get_cycles().value + 
+        future_cycle  = get_cycles().get() + 
 		(1 + pr2->value.get() + (0x100 -  value.get())) * prescale;
         get_cycles().set_break(future_cycle, this);
       }
       else
       {
-          future_cycle = get_cycles().value + 1;
+	future_cycle = get_cycles().get() + 1;
           get_cycles().set_break(future_cycle, this);
-	  last_cycle = get_cycles().value - value.get();
+	  last_cycle = get_cycles().get() - value.get();
           update(update_state);
       }
   }
@@ -1436,7 +1832,7 @@ void TMR2::new_pr2(unsigned int new_value)
 
       unsigned int cur_break = (future_cycle - last_cycle)/prescale;
       unsigned int new_break = (pwm_mode)? (1 + new_value) << 2 : 1 + new_value;
-      unsigned int now_cycle = (get_cycles().value - last_cycle) / prescale;
+      unsigned int now_cycle = (get_cycles().get() - last_cycle) / prescale;
 
       guint64 fc = last_cycle;
 
@@ -1476,7 +1872,7 @@ void TMR2::new_pr2(unsigned int new_value)
 
 void TMR2::current_value()
 {
-  unsigned int tmr2_val = (get_cycles().value - last_cycle)/ prescale;
+  unsigned int tmr2_val = (get_cycles().get() - last_cycle)/ prescale;
 
   if (pwm_mode)
        tmr2_val >>= 2;
@@ -1484,7 +1880,9 @@ void TMR2::current_value()
   value.put(tmr2_val & 0xff);
 
   if(tmr2_val > 0x100)	// Can get to 0x100 during transition
-   cout << "TMR2 BUG!! value = " << value.get() << " which is greater than 0xff\n";
+  {
+   cout << "TMR2 BUG!! value = " << tmr2_val << " which is greater than 0xff\n";
+  }
 }
 
 // TMR2 callback is called when the cycle counter hits the break point that
@@ -1507,7 +1905,7 @@ void TMR2::callback()
       {
 	last_update &= ~TMR2_WRAP;
 	// This (implicitly) resets the timer to zero:
-	last_cycle = get_cycles().value;
+	last_cycle = get_cycles().get();
       }
       else if( last_update & (TMR2_PWM1_UPDATE | TMR2_PWM2_UPDATE))
       {
@@ -1534,7 +1932,7 @@ void TMR2::callback()
 	  //cout << "TMR2: PR2 match. pwm_mode is " << pwm_mode <<'\n';
 
 	  // This (implicitly) resets the timer to zero:
-	  last_cycle = get_cycles().value;
+	  last_cycle = get_cycles().get();
 
          if (ssp_module)
                ssp_module->tmr2_clock();
