@@ -41,29 +41,77 @@ Boston, MA 02111-1307, USA.  */
 //
 
 
-//------------------------------------------------------------------------
-// Config1H - default 
 
-class Config1H : public ConfigMemory 
-{
-#define FOSC0   (1<<0)
-#define FOSC1   (1<<1)
-#define FOSC2   (1<<2)
-#define OSCEN   (1<<5)
-
-#define CONFIG1H_default (OSCEN | FOSC2 | FOSC1 | FOSC0)
-public:
-  Config1H(_16bit_processor *pCpu, unsigned int addr)
-    : ConfigMemory("CONFIG1H", CONFIG1H_default, "Oscillator configuration", pCpu, addr)
+  string Config1H::toString()
   {
+    gint64 i64;
+    get(i64);
+    int i = i64 &0xfff;
+
+    char buff[256];
+
+    const char *OSCdesc[8] = {
+      "LP oscillator",
+      "XT oscillator",
+      "HS oscillator",
+      "RC oscillator",
+      "EC oscillator w/ OSC2 configured as divide-by-4 clock output",
+      "EC oscillator w/ OSC2 configured as RA6",
+      "HS oscillator with PLL enabled/Clock frequency = (4 x FOSC)",
+      "RC oscillator w/ OSC2 configured as RA6"
+    };
+    snprintf(buff,sizeof(buff),
+             "$%04x\n"
+             " FOSC=%d - Clk source = %s\n"
+             " OSCEN=%d - Oscillator switching is %s\n",
+             i,
+             i&(FOSC0|FOSC1|FOSC2), OSCdesc[i&(FOSC0|FOSC1|FOSC2)],
+             (i&OSCEN?1:0), ((i&OSCEN) ? "disabled" : "enabled"));
+
+    return string(buff);
   }
-};
 
 
+  string Config1H_4bits::toString()
+  {
+    gint64 i64;
+    get(i64);
+    int i = i64 &0xfff;
+
+    char buff[256];
+
+    const char *OSCdesc[] = {
+      "LP oscillator",
+      "XT oscillator",
+      "HS oscillator",
+      "RC oscillator",
+      "EC oscillator w/ OSC2 configured as divide-by-4 clock output",
+      "EC oscillator w/ OSC2 configured as RA6",
+      "HS oscillator with PLL enabled/Clock frequency = (4 x FOSC)",
+      "RC oscillator w/ OSC2 configured as RA6",
+      "Internal oscillator block, port function on RA6 and RA7",
+      "Internal oscillator block, CLKO function on RA6, port function on RA7",
+      "External RC oscillator, CLKO function on RA6",
+      "External RC oscillator, CLKO function on RA6",
+      "External RC oscillator, CLKO function on RA6",
+      "External RC oscillator, CLKO function on RA6",
+      "External RC oscillator, CLKO function on RA6",
+      "External RC oscillator, CLKO function on RA6"
+    };
+    snprintf(buff,sizeof(buff),
+             "$%04x\n"
+             " FOSC=%d - Clk source = %s\n"
+             " OSCEN=%d - Oscillator switching is %s\n",
+             i,
+             i&(FOSC0|FOSC1|FOSC2|FOSC3), OSCdesc[i&(FOSC0|FOSC1|FOSC2|FOSC3)],
+             (i&OSCEN?1:0), ((i&OSCEN) ? "disabled" : "enabled"));
+
+    return string(buff);
+  }
 //------------------------------------------------------------------------
 // Config2H - default 
 //  The default Config2H register controls the 18F series WDT. 
-class Config2H : public ConfigMemory 
+class Config2H : public ConfigWord 
 {
 #define WDTEN   (1<<0)
 #define WDTPS0  (1<<1)
@@ -73,14 +121,36 @@ class Config2H : public ConfigMemory
 #define CONFIG2H_default (WDTEN | WDTPS0 | WDTPS1 | WDTPS2)
 public:
   Config2H(_16bit_processor *pCpu, unsigned int addr)
-    : ConfigMemory("CONFIG2H", CONFIG2H_default, "WatchDog configuration", pCpu, addr)
+    : ConfigWord("CONFIG2H", CONFIG2H_default, "WatchDog configuration", pCpu, addr)
   {
+	set(CONFIG2H_default);
   }
   virtual void set(gint64 v)
   {
     Integer::set(v);
     if (m_pCpu)
+    {
+      m_pCpu->wdt.set_postscale((v & (WDTPS0|WDTPS1|WDTPS2)) >> 1);
       m_pCpu->wdt.initialize((v & WDTEN) == WDTEN);
+    }
+  }
+
+  virtual string toString()
+  {
+    gint64 i64;
+    get(i64);
+    int i = i64 &0xfff;
+
+    char buff[256];
+
+    snprintf(buff,sizeof(buff),
+             "$%04x\n"
+             " WDTEN=%d - WDT is %s, prescale=1:%d\n",
+             i,
+             (i&WDTEN?1:0), ((i&WDTEN) ? "enabled" : "disabled"),
+             1 << (i & (WDTPS0 | WDTPS1 | WDTPS2)>>1));
+
+    return string(buff);
   }
 };
 
@@ -97,29 +167,85 @@ public:
 //-------------------------------------------------------------------
 _16bit_processor::_16bit_processor(const char *_name, const char *desc)
   : pic_processor(_name,desc),
-    pir1(0,0), pir2(0,0)
+/*
+    adcon0(this, "adcon0", "A2D control register"),
+    adcon1(this, "adcon1", "A2D control register"),
+*/
+    adresl(this, "adresl", "A2D result low"),
+    adresh(this, "adresh", "A2D result high"),
+    intcon(this, "intcon", "Interrupt control"),
+    intcon2(this, "intcon2", "Interrupt control"),
+    intcon3(this, "intcon3", "Interrupt control"),
+    bsr(this, "bsr", "Bank Select Register"),
+    tmr0l(this, "tmr0l", "TMR0 Low"),
+    tmr0h(this, "tmr0h", "TMR0 High"),
+    t0con(this, "t0con", "TMR0 Control"),
+    rcon(this, "rcon", "Reset Control"),
+    pir1(this,"pir1","Peripheral Interrupt Register",0,0),
+    ipr1(this, "ipr1", "Interrupt Priorities"),
+    ipr2(this, "ipr2", "Interrupt Priorities"),
+    t1con(this, "t1con", "TMR1 Control"),
+    pie1(this, "pie1", "Peripheral Interrupt Enable"),
+    pir2(this,"pir2","Peripheral Interrupt Register",0,0),
+    pie2(this, "pie2", "Peripheral Interrupt Enable"),
+    t2con(this, "t2con", "TMR2 Control"),
+    pr2(this, "pr2", "TMR2 Period Register"),
+    tmr2(this, "tmr2", "TMR2 Register"),
+    tmr1l(this, "tmr1l", "TMR1 Low"),
+    tmr1h(this, "tmr1h", "TMR1 High"),
+    ccp1con(this, "ccp1con", "Capture Compare Control"),
+    ccpr1l(this, "ccpr1l", "Capture Compare 1 Low"),
+    ccpr1h(this, "ccpr1h", "Capture Compare 1 High"),
+    ccp2con(this, "ccp2con", "Capture Compare Control"),
+    ccpr2l(this, "ccpr2l", "Capture Compare 2 Low"),
+    ccpr2h(this, "ccpr2h", "Capture Compare 2 High"),
+    tmr3l(this, "tmr3l", "TMR3 Low"),
+    tmr3h(this, "tmr3h", "TMR3 High"),
+    t3con(this, "t3con", "TMR3 Control"),
+
+    osccon(this, "osccon", "OSC Control"),
+    lvdcon(this, "lvdcon", "LVD Control"),
+    wdtcon(this, "wdtcon", "WDT Control", 1),
+    prodh(this, "prodh", "Product High"),
+    prodl(this, "prodl", "Product Low"),
+    pclatu(this, "pclatu", "Program Counter Latch upper byte"),
+
+    ind0(this,string("0")),
+    ind1(this,string("1")),
+    ind2(this,string("2")),
+    usart(this),
+
+    tbl(this),
+    ssp(this)
 {
 
+  set_osc_pin_Number(0, 253, NULL);
+  set_osc_pin_Number(1, 253, NULL);
   package = 0;
-  cout << "FIXME: 16bit processor is assuming that PLL is on - should check config bits\n";
-  pll_factor = 2;
+  pll_factor = 0;
 
-  pc = new Program_Counter16();
-  pc->set_trace_command(trace.allocateTraceType(new PCTraceType(this,1)));
+  pc = new Program_Counter16(this);
+  pc->set_trace_command(); //trace.allocateTraceType(new PCTraceType(this,1)));
 
-  m_porta = new PicPortRegister("porta",8,0x7f);
-  m_trisa = new PicTrisRegister("trisa", m_porta);
-  m_lata  = new PicLatchRegister("lata", m_porta);
+  m_porta = new PicPortRegister(this,"porta","",8,0x7f);
+  m_trisa = new PicTrisRegister(this,"trisa","", m_porta, false);
+  m_lata  = new PicLatchRegister(this,"lata","", m_porta);
   m_lata->setEnableMask(0x7f);
 
-  m_portb = new PicPortRegister("portb",8,0xff);
-  m_trisb = new PicTrisRegister("trisb", m_portb);
-  m_latb  = new PicLatchRegister("latb", m_portb);
+  m_portb = new PicPortBRegister(this,"portb","", &intcon, 8,0xff);
+  m_portb->assignRBPUSink(7,&intcon2);
+  m_trisb = new PicTrisRegister(this,"trisb","", m_portb, false);
+  m_latb  = new PicLatchRegister(this,"latb","", m_portb);
 
-  m_portc = new PicPortRegister("portc",8,0xff);
-  m_trisc = new PicTrisRegister("trisc", m_portc);
-  m_latc  = new PicLatchRegister("latc", m_portc);
+  m_portc = new PicPortRegister(this,"portc","",8,0xff);
+  m_trisc = new PicTrisRegister(this,"trisc","", m_portc, false);
+  m_latc  = new PicLatchRegister(this,"latc","", m_portc);
 
+  //tmr0l.set_cpu(this, m_porta, 4, option_reg);
+  //tmr0l.start(0);
+  m_porta->addSink(&tmr0l,4);
+
+  stack = new Stack16(this);
 }
 
 //-------------------------------------------------------------------
@@ -142,7 +268,7 @@ pic_processor *_16bit_processor::construct()
   p->create_invalid_registers();
   p->create_symbols();
   p->name_str = "generic 16bit processor";
-  symbol_table.add_module(p,p->name_str.c_str());
+  globalSymbolTable().addModule(p);
 
   return p;
 
@@ -155,9 +281,10 @@ void _16bit_processor :: create_sfr_map()
   if(verbose)
     cout << "creating 18cxxx common registers\n";
 
-  add_file_registers(0x0, 0xf7f, 0);
+  add_file_registers(0x0, last_actual_register(), 0);
 
   RegisterValue porv(0,0);
+  RegisterValue porv2(0,0);
 
   add_sfr_register(m_porta,       0xf80,porv);
   add_sfr_register(m_portb,       0xf81,porv);
@@ -181,7 +308,8 @@ void _16bit_processor :: create_sfr_map()
 
 
   usart.initialize(&pir_set_def,&(*m_portc)[6], &(*m_portc)[7],
-		   new _TXREG(&usart), new _RCREG(&usart));
+		   new _TXREG(this,"txreg", "USART Transmit Register", &usart), 
+                   new _RCREG(this,"rcreg", "USART Receiver Register", &usart));
 
   add_sfr_register(&usart.rcsta,    0xfab,porv,"rcsta");
   add_sfr_register(&usart.txsta,    0xfac,RegisterValue(0x02,0),"txsta");
@@ -200,8 +328,6 @@ void _16bit_processor :: create_sfr_map()
   add_sfr_register(&ccpr1l,	  0xfbe,porv,"ccpr1l");
   add_sfr_register(&ccpr1h,	  0xfbf,porv,"ccpr1h");
 
-  add_sfr_register(&adcon1,	  0xfc1,porv,"adcon1");
-  add_sfr_register(&adcon0,	  0xfc2,porv,"adcon0");
   add_sfr_register(&adresl,	  0xfc3,porv,"adresl");
   add_sfr_register(&adresh,	  0xfc4,porv,"adresh");
 
@@ -219,7 +345,7 @@ void _16bit_processor :: create_sfr_map()
   add_sfr_register(&tmr1l,	  0xfce,porv,"tmr1l");
   add_sfr_register(&tmr1h,	  0xfcf,porv,"tmr1h");
 
-  add_sfr_register(&rcon,	  0xfd0,porv,"rcon");
+  add_sfr_register(&rcon,	  0xfd0,RegisterValue(0x1c,0),"rcon");
   add_sfr_register(&wdtcon,	  0xfd1,porv,"wdtcon");
   add_sfr_register(&lvdcon,	  0xfd2,porv,"lvdcon");
   add_sfr_register(&osccon,	  0xfd3,porv,"osccon");
@@ -230,6 +356,7 @@ void _16bit_processor :: create_sfr_map()
   t0con.put(0xff);  /**FIXME - need a way to set this to 0xff at reset*/
 
   add_sfr_register(status,       0xfd8);
+  status->set_rcon(&rcon);
 
   add_sfr_register(&ind2.fsrl,	  0xfd9,porv,"fsr2l");
   add_sfr_register(&ind2.fsrh,    0xfda,porv,"fsr2h");
@@ -260,7 +387,8 @@ void _16bit_processor :: create_sfr_map()
   add_sfr_register(&ind0.indf,    0xfef,porv,"indf0");
 
   add_sfr_register(&intcon3, 0xff0, porv,"intcon3");
-  add_sfr_register(&intcon2, 0xff1, porv,"intcon2");
+  porv2.data = 0xF5;
+  add_sfr_register(&intcon2, 0xff1, porv2,"intcon2");
   add_sfr_register(&intcon,  0xff2, porv,"intcon");
 
   add_sfr_register(&prodl, 0xff3, porv,"prodl");
@@ -273,17 +401,17 @@ void _16bit_processor :: create_sfr_map()
 
   if(pcl)
     delete pcl;
-  pcl = new PCL16();
+  pcl = new PCL16(this,"pcl", "Program Counter Low byte");
 
   add_sfr_register(pcl,     0xff9);
   add_sfr_register(pclath,  0xffa);
   add_sfr_register(&pclatu, 0xffb, porv, "pclatu");
 
-  stack = &stack16;
-  add_sfr_register(&stack16.stkptr,  0xffc,porv,"stkptr");
-  add_sfr_register(&stack16.tosl,    0xffd,porv,"tosl");
-  add_sfr_register(&stack16.tosh,    0xffe,porv,"tosh");
-  add_sfr_register(&stack16.tosu,    0xfff,porv,"tosu");
+  Stack16 *stack16 = static_cast<Stack16 *>(stack);
+  add_sfr_register(&stack16->stkptr,  0xffc,porv,"stkptr");
+  add_sfr_register(&stack16->tosl,    0xffd,porv,"tosl");
+  add_sfr_register(&stack16->tosh,    0xffe,porv,"tosh");
+  add_sfr_register(&stack16->tosu,    0xfff,porv,"tosu");
 
 
   EEPROM *e = get_eeprom();
@@ -304,7 +432,7 @@ void _16bit_processor :: create_sfr_map()
   tmr1l.tmrh   = &tmr1h;
   tmr1l.t1con  = &t1con;
   tmr1l.setInterruptSource(new InterruptSource(&pir1, PIR1v1::TMR1IF));
-  tmr1l.ccpcon = &ccp1con;
+//  tmr1l.ccpcon = &ccp1con;
 
   tmr1h.tmrl  = &tmr1l;
 
@@ -322,7 +450,7 @@ void _16bit_processor :: create_sfr_map()
   tmr3l.tmrh  = &tmr3h;
   tmr3l.t1con = &t3con;
   tmr3l.setInterruptSource(new InterruptSource(&pir2, PIR2v2::TMR3IF));
-  tmr3l.ccpcon = &ccp1con;
+//  tmr3l.ccpcon = &ccp1con;
 
   tmr3h.tmrl  = &tmr3l;
 
@@ -331,7 +459,7 @@ void _16bit_processor :: create_sfr_map()
   t3con.ccpr1l = &ccpr1l;
   t3con.ccpr2l = &ccpr2l;
 
-  ccp1con.setCrosslinks(&ccpr1l, &pir_set_def, &tmr2);
+  ccp1con.setCrosslinks(&ccpr1l, &pir1, &tmr2);
   ccp1con.setIOpin(&((*m_portc)[2]));
   ccpr1l.ccprh  = &ccpr1h;
   ccpr1l.tmrl   = &tmr1l;
@@ -339,73 +467,25 @@ void _16bit_processor :: create_sfr_map()
 
   pir1.set_intcon(&intcon);
   pir1.set_pie(&pie1);
-  pie1.pir = &pir1;
-  pie1.new_name("pie1");
+  pir1.set_ipr(&ipr1);
+  pie1.setPir(&pir1);
+  //pie1.new_name("pie1");
 
   pir2.set_intcon(&intcon);
   pir2.set_pie(&pie2);
-  
-  pie2.pir    = &pir2;
-  pie2.new_name("pie2");
+  pir2.set_ipr(&ipr2);
+  pie2.setPir(&pir2);
+  //pie2.new_name("pie2");
 
   // All of the status bits on the 16bit core are writable
   status->write_mask = 0xff;
 
 
-  adcon0.setAdresLow(&adresl);
-  adcon0.setAdres(&adresh);
-  adcon0.setAdcon1(&adcon1);
-  adcon0.setIntcon(&intcon);
-  adcon0.setPir(&pir1);
-  adcon0.setChannel_Mask(7); // Greater than 4 channels
-  adcon0.setA2DBits(10);
-
-  adcon1.setValidCfgBits(ADCON1::PCFG0 | ADCON1::PCFG1 | 
-			 ADCON1::PCFG2 | ADCON1::PCFG3,0);
-
-  adcon1.setNumberOfChannels(8);	// Allow 8 channel configuration
-  adcon1.setChannelConfiguration(0, 0xff);
-  adcon1.setChannelConfiguration(1, 0xff);
-  adcon1.setChannelConfiguration(2, 0x1f);
-  adcon1.setChannelConfiguration(3, 0x1f);
-  adcon1.setChannelConfiguration(4, 0x0b);
-  adcon1.setChannelConfiguration(5, 0x0b);
-  adcon1.setChannelConfiguration(6, 0x00);
-  adcon1.setChannelConfiguration(7, 0x00);
-  adcon1.setChannelConfiguration(8, 0xff);
-  adcon1.setChannelConfiguration(9, 0x3f);
-  adcon1.setChannelConfiguration(10, 0x3f);
-  adcon1.setChannelConfiguration(11, 0x3f);
-  adcon1.setChannelConfiguration(12, 0x1f);
-  adcon1.setChannelConfiguration(13, 0x0f);
-  adcon1.setChannelConfiguration(14, 0x01);
-  adcon1.setChannelConfiguration(15, 0x0d);
-
-  adcon1.setVrefHiConfiguration(1, 3);
-  adcon1.setVrefHiConfiguration(3, 3);
-  adcon1.setVrefHiConfiguration(5, 3);
-  adcon1.setVrefHiConfiguration(8, 3);
-  adcon1.setVrefHiConfiguration(10, 3);
-  adcon1.setVrefHiConfiguration(11, 3);
-  adcon1.setVrefHiConfiguration(12, 3);
-  adcon1.setVrefHiConfiguration(13, 3);
-  adcon1.setVrefHiConfiguration(15, 3);
-
-  adcon1.setVrefLoConfiguration(8, 2);
-  adcon1.setVrefLoConfiguration(11, 2);
-  adcon1.setVrefLoConfiguration(12, 2);
-  adcon1.setVrefLoConfiguration(13, 2);
-  adcon1.setVrefLoConfiguration(15, 2);
-
-  adcon1.setNumberOfChannels(5);
-  adcon1.setIOPin(0, &(*m_porta)[0]);
-  adcon1.setIOPin(1, &(*m_porta)[1]);
-  adcon1.setIOPin(2, &(*m_porta)[2]);
-  adcon1.setIOPin(3, &(*m_porta)[3]);
-  adcon1.setIOPin(4, &(*m_porta)[5]);
   // AN5,AN6 and AN7 exist only on devices with a PORTE.
 }
 
+//-------------------------------------------------------------------
+
 //-------------------------------------------------------------------
 //
 // 
@@ -421,10 +501,11 @@ void _16bit_processor :: create ()
     cout << " _16bit_processor :: create\n" ;
 
   fast_stack.init(this);
+  /*
   ind0.init(this);
   ind1.init(this);
   ind2.init(this);
-
+  */
   pic_processor::create();
   create_sfr_map();
 
@@ -432,9 +513,9 @@ void _16bit_processor :: create ()
 
   intcon.set_rcon(&rcon);
   intcon.set_intcon2(&intcon2);
-  intcon.set_cpu(this);
+  intcon.set_pir_set(&pir_set_def);
 
-  tbl.initialize(this);
+  //tbl.initialize(this);
   tmr0l.start(0);
 
   if(pma) {
@@ -472,8 +553,7 @@ void _16bit_processor::create_symbols ()
 // in 'interrupt_vector'.
 //
 //-------------------------------------------------------------------
-void _16bit_processor::
-interrupt ()
+void _16bit_processor::interrupt ()
 {
   
   bp.clear_interrupt();
@@ -486,14 +566,14 @@ interrupt ()
 
   intcon.clear_gies();  // The appropriate gie bits get cleared (not just gieh)
 
-  pc->jump(intcon.get_interrupt_vector());
+#if defined(CLOCK_EXPERIMENTS)
+  pc->interrupt(intcon.get_interrupt_vector());
+#else
+  pc->interrupt(intcon.get_interrupt_vector());
+  //pc->jump(intcon.get_interrupt_vector());
+#endif
 
 }
-//-------------------------------------------------------------------
-void _16bit_processor::por()
-{
-  pic_processor::por();
-}
 
 //-------------------------------------------------------------------
 void _16bit_processor::option_new_bits_6_7(unsigned int bits)
@@ -504,6 +584,26 @@ void _16bit_processor::option_new_bits_6_7(unsigned int bits)
 
 }
 
+//-------------------------------------------------------------------
+void _16bit_processor::enter_sleep()
+{
+    if (verbose)
+	cout << "_16bit_processor::enter_sleep() \n";
+    tmr0l.sleep();
+    pic_processor::enter_sleep();
+}
+
+//-------------------------------------------------------------------
+void _16bit_processor::exit_sleep()
+{
+  if (verbose)
+	cout << "_16bit_processor::exit_sleep() \n";
+  if (m_ActivityState == ePASleeping)
+  {
+      tmr0l.wake();
+      pic_processor::exit_sleep();
+  }
+}
 
 //-------------------------------------------------------------------
 // Fetch the rom contents at a particular address.
@@ -540,13 +640,13 @@ unsigned int _16bit_processor::get_config_word(unsigned int address)
     address &= 0xfffe; // Clear LSB
     unsigned int ret = 0xffff;
 
-    if (m_configMemory[address])
-      ret = (ret & 0xff00) |  (((unsigned int )(m_configMemory[address]->getVal())) & 0x00ff);
+    if (m_configMemory->getConfigWord(address))
+      ret = (ret & 0xff00) |  (((unsigned int )(m_configMemory->getConfigWord(address)->getVal())) & 0x00ff);
 
     address++;
 
-    if (m_configMemory[address])
-      ret = (ret & 0x00ff) |  ((((unsigned int )(m_configMemory[address]->getVal()))<<8) & 0xff00);
+    if (m_configMemory->getConfigWord(address))
+      ret = (ret & 0x00ff) |  ((((unsigned int )(m_configMemory->getConfigWord(address)->getVal()))<<8) & 0xff00);
 
     return ret;
   }
@@ -564,16 +664,18 @@ bool  _16bit_processor::set_config_word(unsigned int address, unsigned int cfg_w
 
       address &= 0xfffe;
 
-      if(m_configMemory[address])
-	m_configMemory[address]->set((int)(cfg_word&0xff));
+      if (m_configMemory->getConfigWord(address))
+	m_configMemory->getConfigWord(address)->set((int)(cfg_word&0xff));
 
       address++;
 
-      if(m_configMemory[address])
-	m_configMemory[address]->set((int)((cfg_word>>8)&0xff));
+      if (m_configMemory->getConfigWord(address))
+	m_configMemory->getConfigWord(address)->set((int)((cfg_word>>8)&0xff));
 
       return true;
     }
+    else
+	cout << "Setting config word no m_configMemory\n";
   }
 
   return false;
@@ -582,14 +684,9 @@ bool  _16bit_processor::set_config_word(unsigned int address, unsigned int cfg_w
 //-------------------------------------------------------------------
 void _16bit_processor::create_config_memory()
 {
-  m_configMemory = new ConfigMemory *[configMemorySize()];
-
-  for (unsigned int i=0; i<configMemorySize(); i++)
-    m_configMemory[i] = 0;
-
-  m_configMemory[CONFIG1H-CONFIG1L] = new Config1H(this, CONFIG1H);
-  m_configMemory[CONFIG2H-CONFIG1L] = new Config2H(this, CONFIG2H);
-
+  m_configMemory = new ConfigMemory(this,configMemorySize());
+  m_configMemory->addConfigWord(CONFIG1H-CONFIG1L,new Config1H(this, CONFIG1H));
+  m_configMemory->addConfigWord(CONFIG2H-CONFIG1L,new Config2H(this, CONFIG2H));
 }
 
 //-------------------------------------------------------------------
@@ -601,3 +698,138 @@ void _16bit_processor::create_iopin_map()
   cout << "_16bit_processor::create_iopin_map WARNING --- not creating package \n";
 }
 
+
+void _16bit_compat_adc::create()
+{
+
+    adcon0 = new ADCON0(this, "adcon0", "A2D control register"),
+    adcon1 = new ADCON1(this, "adcon1", "A2D control register"),
+
+    _16bit_processor::create();
+
+    a2d_compat();
+}
+
+void _16bit_compat_adc::create_sfr_map()
+{
+	_16bit_processor::create_sfr_map();
+}
+
+void _16bit_compat_adc::create_symbols()
+{
+	_16bit_processor::create_symbols();
+}
+
+void _16bit_compat_adc :: a2d_compat()
+{
+  if(verbose)
+      cout << "creating old (compatible) A2D\n";
+
+  RegisterValue porv(0,0);
+
+  add_sfr_register(adcon1,	  0xfc1,porv,"adcon1");
+  add_sfr_register(adcon0,	  0xfc2,porv,"adcon0");
+
+
+  adcon0->setAdresLow(&adresl);
+  adcon0->setAdres(&adresh);
+  adcon0->setAdcon1(adcon1);
+  adcon0->setIntcon(&intcon);
+  adcon0->setPir(&pir1);
+  adcon0->setChannel_Mask(7); // Greater than 4 channels
+  adcon0->setA2DBits(10);
+
+  adcon1->setValidCfgBits(ADCON1::PCFG0 | ADCON1::PCFG1 | 
+			 ADCON1::PCFG2 | ADCON1::PCFG3,0);
+
+  adcon1->setChannelConfiguration(0, 0xff);
+  adcon1->setChannelConfiguration(1, 0xff);
+  adcon1->setChannelConfiguration(2, 0x1f);
+  adcon1->setChannelConfiguration(3, 0x1f);
+  adcon1->setChannelConfiguration(4, 0x0b);
+  adcon1->setChannelConfiguration(5, 0x0b);
+  adcon1->setChannelConfiguration(6, 0x00);
+  adcon1->setChannelConfiguration(7, 0x00);
+  adcon1->setChannelConfiguration(8, 0xff);
+  adcon1->setChannelConfiguration(9, 0x3f);
+  adcon1->setChannelConfiguration(10, 0x3f);
+  adcon1->setChannelConfiguration(11, 0x3f);
+  adcon1->setChannelConfiguration(12, 0x1f);
+  adcon1->setChannelConfiguration(13, 0x0f);
+  adcon1->setChannelConfiguration(14, 0x01);
+  adcon1->setChannelConfiguration(15, 0x0d);
+
+  adcon1->setVrefHiConfiguration(1, 3);
+  adcon1->setVrefHiConfiguration(3, 3);
+  adcon1->setVrefHiConfiguration(5, 3);
+  adcon1->setVrefHiConfiguration(8, 3);
+  adcon1->setVrefHiConfiguration(10, 3);
+  adcon1->setVrefHiConfiguration(11, 3);
+  adcon1->setVrefHiConfiguration(12, 3);
+  adcon1->setVrefHiConfiguration(13, 3);
+  adcon1->setVrefHiConfiguration(15, 3);
+
+  adcon1->setVrefLoConfiguration(8, 2);
+  adcon1->setVrefLoConfiguration(11, 2);
+  adcon1->setVrefLoConfiguration(12, 2);
+  adcon1->setVrefLoConfiguration(13, 2);
+  adcon1->setVrefLoConfiguration(15, 2);
+
+  adcon1->setNumberOfChannels(5);
+  adcon1->setIOPin(0, &(*m_porta)[0]);
+  adcon1->setIOPin(1, &(*m_porta)[1]);
+  adcon1->setIOPin(2, &(*m_porta)[2]);
+  adcon1->setIOPin(3, &(*m_porta)[3]);
+  adcon1->setIOPin(4, &(*m_porta)[5]);
+
+}
+_16bit_compat_adc::_16bit_compat_adc(const char *_name, const char *desc) 
+	: _16bit_processor(_name, desc)
+{
+}
+void _16bit_v2_adc::create(int nChannels)
+{
+
+    adcon0 = new ADCON0_V2(this, "adcon0", "A2D control register");
+    adcon1 = new ADCON1_V2(this, "adcon1", "A2D control register");
+    adcon2 = new ADCON2_V2(this, "adcon2", "A2D control register");
+
+
+    RegisterValue porv(0,0);
+
+    add_sfr_register(adcon2,	  0xfc0,porv,"adcon2");
+    add_sfr_register(adcon1,	  0xfc1,porv,"adcon1");
+    add_sfr_register(adcon0,	  0xfc2,porv,"adcon0");
+
+
+    adcon0->setAdresLow(&adresl);
+    adcon0->setAdres(&adresh);
+    adcon0->setAdcon1(adcon1);
+    adcon0->setAdcon2(adcon2);
+    adcon0->setIntcon(&intcon);
+    adcon0->setPir(&pir1);
+    adcon0->setChannel_Mask(0xf); // upto 16 channels
+    adcon0->setA2DBits(10);
+
+    adcon1->setValidCfgBits(ADCON1::PCFG0 | ADCON1::PCFG1 | 
+			 ADCON1::PCFG2 | ADCON1::PCFG3,0);
+
+    adcon1->setNumberOfChannels(nChannels);	
+    adcon1->setChanTable(0x1fff, 0x1fff, 0x1fff, 0x0fff,
+        0x07ff, 0x03ff, 0x01ff, 0x00ff, 0x007f, 0x003f,
+        0x001f, 0x000f, 0x0007, 0x0003, 0x0001, 0x0000);
+    adcon1->setVrefHiChannel(3);
+    adcon1->setVrefLoChannel(2);
+
+
+    adcon1->setIOPin(0, &(*m_porta)[0]);
+    adcon1->setIOPin(1, &(*m_porta)[1]);
+    adcon1->setIOPin(2, &(*m_porta)[2]);
+    adcon1->setIOPin(3, &(*m_porta)[3]);
+
+
+}
+_16bit_v2_adc::_16bit_v2_adc(const char *_name, const char *desc) 
+	: _16bit_processor(_name, desc)
+{
+}
