@@ -31,6 +31,7 @@ Boston, MA 02111-1307, USA.  */
 #include "pir.h"
 #include "uart.h"
 #include "a2dconverter.h"
+#include "a2d_v2.h"
 #include "value.h"
 
 // forward references
@@ -73,7 +74,7 @@ public:
   PicTrisRegister  *m_trisa;
   PicLatchRegister *m_lata;
 
-  PicPortRegister  *m_portb;
+  PicPortBRegister *m_portb;
   PicTrisRegister  *m_trisb;
   PicLatchRegister *m_latb;
 
@@ -82,10 +83,8 @@ public:
   PicLatchRegister *m_latc;
 
 
-  ADCON0_16    adcon0;
-  ADCON1       adcon1;
-  ADRES        adresl;
-  ADRES        adresh;
+  sfr_register adresl;
+  sfr_register adresh;
   INTCON_16    intcon;
   INTCON2      intcon2;
   INTCON3      intcon3;
@@ -112,8 +111,8 @@ public:
   CCPCON       ccp2con;
   CCPRL        ccpr2l;
   CCPRH        ccpr2h;
-  TMR3L        tmr3l;
-  TMR3H        tmr3h;
+  TMRL         tmr3l;
+  TMRH         tmr3h;
   T3CON        t3con;
   PIR_SET_2    pir_set_def;
 
@@ -130,7 +129,7 @@ public:
   Indirect_Addressing  ind1;
   Indirect_Addressing  ind2;
   USART_MODULE         usart;
-  Stack16              stack16;
+  //Stack16              stack16;
   TBL_MODULE           tbl;
   TMR2_MODULE          tmr2_module;
   TMR3_MODULE          tmr3_module;
@@ -141,10 +140,9 @@ public:
   virtual void create_symbols();
 
   void interrupt();
-  virtual void por();
   virtual void create();// {return;};
-  virtual PROCESSOR_TYPE isa(){return _16BIT_PROCESSOR_;};
-  virtual PROCESSOR_TYPE base_isa(){return _16BIT_PROCESSOR_;};
+  virtual PROCESSOR_TYPE isa(){return _PIC17_PROCESSOR_;};
+  virtual PROCESSOR_TYPE base_isa(){return _PIC17_PROCESSOR_;};
   virtual instruction * disasm (unsigned int address, unsigned int inst)
     {
       return disasm16(this, address, inst);
@@ -152,8 +150,6 @@ public:
   virtual void create_sfr_map();
   virtual void create_config_memory();
 
-  virtual void create_stack() {stack = &stack16;};
-
   // Return the portion of pclath that is used during branching instructions
   virtual unsigned int get_pclath_branching_jump()
   {
@@ -174,16 +170,27 @@ public:
   // -- the derived classes must define their parameters appropriately.
 
   virtual unsigned int register_memory_size () const { return 0x1000;};
+  virtual unsigned int last_actual_register () const { return 0x0F7F;};
   //virtual void set_out_of_range_pm(unsigned int address, unsigned int value);
 
   virtual void create_iopin_map();
 
-  virtual int  map_pm_address2index(int address) {return address/2;};
-  virtual int  map_pm_index2address(int index) {return index*2;};
+  virtual int  map_pm_address2index(int address) const {return address/2;};
+  virtual int  map_pm_index2address(int index) const {return index*2;};
   virtual unsigned int get_program_memory_at_address(unsigned int address);
   virtual unsigned int get_config_word(unsigned int address);
   virtual bool set_config_word(unsigned int address, unsigned int cfg_word);
   virtual unsigned int configMemorySize() { return CONFIG7H-CONFIG1L+1; }
+  virtual void enter_sleep();
+  virtual void exit_sleep();
+/*
+  virtual void osc_mode(unsigned int value);
+  void set_osc_pin_Number(unsigned int i, unsigned int val) 
+	{osc_pin_Number[i] = val;};
+  unsigned char get_osc_pin_Number(unsigned int i) 
+	{return(osc_pin_Number[i]);};
+*/
+
 
 
   static pic_processor *construct();
@@ -195,8 +202,95 @@ public:
   void setCurrentDisasmAddress(unsigned a) { m_current_disasm_address =a; }
 protected:
   unsigned int m_current_disasm_address;  // Used only when .hex/.cod files are loaded
+//  unsigned char osc_pin_Number[2];
 };
 
+class _16bit_compat_adc : public _16bit_processor 
+{
+public:
+
+  ADCON0       *adcon0;
+  ADCON1       *adcon1;
+
+  _16bit_compat_adc(const char *_name=0, const char *desc=0);
+  virtual void create_symbols();
+  virtual void create();
+  virtual void create_sfr_map();
+  virtual void a2d_compat();
+};
+class _16bit_v2_adc : public _16bit_processor 
+{
+public:
+
+  ADCON0_V2       *adcon0;
+  ADCON1_V2       *adcon1;
+  ADCON2_V2       *adcon2;
+
+  _16bit_v2_adc(const char *_name=0, const char *desc=0);
+  virtual void create(int nChannels);
+};
 #define cpu16 ( (_16bit_processor *)cpu)
 
+#define FOSC0   (1<<0)
+#define FOSC1   (1<<1)
+#define FOSC2   (1<<2)
+// FOSC3 may not be used
+#define FOSC3   (1<<3)
+#define OSCEN   (1<<5)
+
+//------------------------------------------------------------------------
+// Config1H - default 3 bits FOSC
+
+class Config1H : public ConfigWord 
+{
+
+#define CONFIG1H_default (OSCEN | FOSC2 | FOSC1 | FOSC0)
+public:
+  Config1H(_16bit_processor *pCpu, unsigned int addr)
+    : ConfigWord("CONFIG1H", CONFIG1H_default, "Oscillator configuration", pCpu, addr)
+  {
+	set(CONFIG1H_default);
+  }
+
+  virtual void set(gint64 v)
+  {
+    Integer::set(v);
+
+    if (m_pCpu)
+    {
+	m_pCpu->osc_mode(v & ( FOSC2 | FOSC1 | FOSC0));
+    }
+  }
+
+  virtual string toString();
+
+};
+
+//------------------------------------------------------------------------
+// Config1H -  4 bits FOSC
+
+class Config1H_4bits : public ConfigWord 
+{
+
+public:
+  Config1H_4bits(_16bit_processor *pCpu, unsigned int addr, unsigned int def_val)
+    : ConfigWord("CONFIG1H", def_val, "Oscillator configuration", pCpu, addr)
+  {
+	set(def_val);
+  }
+
+  virtual void set(gint64 v)
+  {
+    Integer::set(v);
+
+    if (m_pCpu)
+    {
+	m_pCpu->osc_mode(v & ( FOSC3 | FOSC2 | FOSC1 | FOSC0));
+    }
+  }
+
+  virtual string toString();
+
+};
+
 #endif
