@@ -35,6 +35,7 @@ class InvalidRegister;   // Forward reference
 #include "uart.h"
 #include "a2dconverter.h"
 #include "ssp.h"
+#include "rcon.h"
 #define _16BIT_REGISTER_MASK   0xfff
 
 class _16bit_processor;
@@ -53,6 +54,7 @@ class PORTB;
 class BSR : public sfr_register
 {
 public:
+  BSR(Processor *, const char *pName, const char *pDesc=0);
 
   unsigned int register_page_bits;
 
@@ -82,87 +84,102 @@ class Indirect_Addressing;   // Forward reference
 
 class FSRL : public sfr_register
 {
- public:
-  Indirect_Addressing  *iam;  // The IA module to which FSRL belongs
-
+public:
+  FSRL(Processor *, const char *pName, const char *pDesc, Indirect_Addressing *pIAM);
   void put(unsigned int new_value);
   void put_value(unsigned int new_value);
-      
+
+protected:
+  Indirect_Addressing  *iam;
 };
 
 class FSRH : public sfr_register
 {
  public:
-  Indirect_Addressing  *iam;  // The IA module to which FSRH belongs
+  FSRH(Processor *, const char *pName, const char *pDesc, Indirect_Addressing *pIAM);
 
   void put(unsigned int new_value);
   void put_value(unsigned int new_value);
       
+protected:
+  Indirect_Addressing  *iam;
 };
 
 class INDF16 : public sfr_register
 {
  public:
-  Indirect_Addressing  *iam;  // The IA module to which INDF belongs
+  INDF16(Processor *, const char *pName, const char *pDesc, Indirect_Addressing *pIAM);
 
   void put(unsigned int new_value);
   void put_value(unsigned int new_value);
   unsigned int get();
   unsigned int get_value();
       
+protected:
+  Indirect_Addressing  *iam;
 };
 
 class PREINC : public sfr_register
 {
- public:
-  Indirect_Addressing  *iam;  // The IA module to which PREINC belongs
+public:
+  PREINC(Processor *, const char *pName, const char *pDesc, Indirect_Addressing *pIAM);
 
   void put(unsigned int new_value);
   void put_value(unsigned int new_value);
   unsigned int get();
   unsigned int get_value();
       
+protected:
+  Indirect_Addressing  *iam;
 };
 
 class POSTINC : public sfr_register
 {
- public:
-  Indirect_Addressing  *iam;  // The IA module to which POSTINC belongs
+public:
+  POSTINC(Processor *, const char *pName, const char *pDesc, Indirect_Addressing *pIAM);
 
   void put(unsigned int new_value);
   void put_value(unsigned int new_value);
   unsigned int get();
   unsigned int get_value();
       
+protected:
+  Indirect_Addressing  *iam;
 };
 
 class POSTDEC : public sfr_register
 {
- public:
-  Indirect_Addressing  *iam;  // The IA module to which POSTDEC belongs
+public:
+  POSTDEC(Processor *, const char *pName, const char *pDesc, Indirect_Addressing *pIAM);
 
   void put(unsigned int new_value);
   void put_value(unsigned int new_value);
   unsigned int get();
   unsigned int get_value();
       
+protected:
+  Indirect_Addressing  *iam;
 };
 
 class PLUSW : public sfr_register
 {
- public:
-  Indirect_Addressing  *iam;  // The IA module to which PLUSW belongs
+public:
+  PLUSW(Processor *, const char *pName, const char *pDesc, Indirect_Addressing *pIAM);
 
   void put(unsigned int new_value);
   void put_value(unsigned int new_value);
   unsigned int get();
   unsigned int get_value();
       
+protected:
+  Indirect_Addressing  *iam;
 };
 
 class Indirect_Addressing
 {
 public:
+  Indirect_Addressing(_16bit_processor *cpu, const string &n);
+
   _16bit_processor *cpu;
 
   unsigned int fsr_value;     // 16bit concatenation of fsrl and fsrh
@@ -190,7 +207,7 @@ public:
   POSTDEC postdec;
   PLUSW   plusw;
 
-  void init(_16bit_processor *new_cpu);
+  //void init(_16bit_processor *new_cpu);
   void put(unsigned int new_value);
   unsigned int get();
   unsigned int get_value();
@@ -264,7 +281,7 @@ public:
   virtual unsigned int get();
   virtual unsigned int get_value();
 
-  PCL16();
+  PCL16(Processor *, const char *pName, const char *pDesc=0);
 };
 
 //---------------------------------------------------------
@@ -284,7 +301,7 @@ public:
   virtual unsigned int get_value();
   //virtual unsigned int get_next();
 
-  Program_Counter16();
+  Program_Counter16(Processor *pCpu);
 };
 
 
@@ -296,19 +313,18 @@ class Stack16;
 
 class STKPTR : public sfr_register
 {
- public:
-  Stack16 *stack;
+public:
+  STKPTR(Processor *, const char *pName, const char *pDesc=0);
 
-  void put(unsigned int new_value);
+  Stack16 *stack;
   void put_value(unsigned int new_value);
-  unsigned int get();
-  unsigned int get_value();
-    
 };
 
 class TOSL : public sfr_register
 {
- public:
+public:
+  TOSL(Processor *, const char *pName, const char *pDesc=0);
+
   Stack16 *stack;
 
   void put(unsigned int new_value);
@@ -320,7 +336,9 @@ class TOSL : public sfr_register
 
 class TOSH : public sfr_register
 {
- public:
+public:
+  TOSH(Processor *, const char *pName, const char *pDesc=0);
+
   Stack16 *stack;
 
   void put(unsigned int new_value);
@@ -332,7 +350,9 @@ class TOSH : public sfr_register
 
 class TOSU : public sfr_register
 {
- public:
+public:
+  TOSU(Processor *, const char *pName, const char *pDesc=0);
+
   Stack16 *stack;
 
   void put(unsigned int new_value);
@@ -351,7 +371,7 @@ public:
   TOSH   tosh;
   TOSU   tosu;
 
-  Stack16();
+  Stack16(Processor *);
   virtual void push(unsigned int);
   virtual unsigned int pop();
   virtual void reset();
@@ -363,37 +383,22 @@ public:
 
 class TMR0_16;
 
-//---------------------------------------------------------
-class RCON :  public sfr_register
-{
-public:
-
-enum
-{
-  BOR  = 1<<0,
-  POR  = 1<<1,
-  PD   = 1<<2,
-  TO   = 1<<3,
-  RI   = 1<<4,
-  LWRT = 1<<6,
-  IPEN = 1<<7
-};
-};
 
 //---------------------------------------------------------
 class CPUSTA :  public sfr_register
 {
 public:
 
-enum
-{
-  BOR      = 1<<0,
-  POR      = 1<<1,
-  PD       = 1<<2,
-  TO       = 1<<3,
-  GLINTD   = 1<<4,
-  STKAV    = 1<<5,
-};
+  enum
+  {
+    BOR      = 1<<0,
+    POR      = 1<<1,
+    PD       = 1<<2,
+    TO       = 1<<3,
+    GLINTD   = 1<<4,
+    STKAV    = 1<<5,
+  };
+  CPUSTA(Processor *, const char *pName, const char *pDesc=0);
 };
 
 
@@ -401,15 +406,16 @@ enum
 // T0CON - Timer 0 control register
 class T0CON : public OPTION_REG
 {
- public:
+public:
 
   enum {
     T08BIT = 1<<6,
     TMR0ON = 1<<7
   };
 
-  T0CON();
+  T0CON(Processor *, const char *pName, const char *pDesc=0);
   void put(unsigned int new_value);
+  void initialize();
 };
 
 //---------------------------------------------------------
@@ -423,7 +429,9 @@ class T0CON : public OPTION_REG
 
 class TMR0H : public sfr_register
 {
- public:
+public:
+
+  TMR0H(Processor *, const char *pName, const char *pDesc=0);
 
   void put(unsigned int new_value);
   void put_value(unsigned int new_value);
@@ -436,7 +444,7 @@ class TMR0_16 : public TMR0
 {
 public:
 
-  TMR0_16();
+  TMR0_16(Processor *, const char *pName, const char *pDesc=0);
 
   T0CON  *t0con;
   INTCON *intcon;
@@ -456,10 +464,13 @@ public:
   virtual bool get_t0cs();
   virtual void initialize();
   virtual void start(int new_value,int sync=0);
+  virtual void sleep();
+  virtual void wake();
 };
 
 
 //---------------------------------------------------------
+/*
 class TMR3H : public TMRH
 {
 public:
@@ -471,21 +482,21 @@ class TMR3L : public TMRL
 public:
 
 };
-
+*/
 class T3CON : public T1CON
 {
 public:
-enum
-{
-  T3CCP1 = 1<<3,
-  T3CCP2 = 1<<6,
-};
+  enum
+  {
+    T3CCP1 = 1<<3,
+    T3CCP2 = 1<<6,
+  };
 
   CCPRL *ccpr1l;
   CCPRL *ccpr2l;
   TMRL  *tmr1l; 
 
-
+  T3CON(Processor *pCpu, const char *pName, const char *pDesc=0);
   virtual void put(unsigned int new_value);
 
 };
@@ -511,19 +522,6 @@ public:
 
 };
 
-//-------------------------------------------------------------------
-
-class ADCON0_16 : public ADCON0
-{
-public:
-  ADCON0_16();
-  virtual void set_interrupt();
-
-  virtual void setPir(PIR *);
-private:
-  PIR *m_pPir;
-};
-
 //-------------------------------------------------------------------
 #if 0
 
@@ -575,7 +573,9 @@ class TABPTRU : public sfr_register
 
 class TBL_MODULE
 {
- public:
+public:
+  TBL_MODULE(_16bit_processor *pCpu);
+
   unsigned int state;
   unsigned int internal_latch;
 
@@ -596,8 +596,7 @@ class TBL_MODULE
   void decrement();
   void read();
   void write();
-  void initialize(_16bit_processor *);
-
+  //void initialize(_16bit_processor *);
 };
 
 
@@ -612,7 +611,7 @@ class TBL_MODULE
 
 class LVDCON : public  sfr_register
 {
- public:
+public:
   unsigned int valid_bits;
 
   enum {
@@ -624,10 +623,7 @@ class LVDCON : public  sfr_register
     IRVST = 1<<5,
   };
 
-  LVDCON() {
-    valid_bits = 0x3f;;
-  }
+  LVDCON(Processor *, const char *pName, const char *pDesc=0);
 };
 
-
 #endif
