@@ -36,6 +36,7 @@ Boston, MA 02111-1307, USA.  */
 class CMSignalSource;
 class VRSignalSource;
 class CMCON;
+class TMRL;
 
   enum compare_inputs
    {
@@ -72,11 +73,12 @@ class VRCON : public sfr_register
       VREN = 1<<7
     };
 
-  VRCON(void);
+  VRCON(Processor *pCpu, const char *pName, const char *pDesc);
+  ~VRCON();
 
   virtual void put(unsigned int new_value);
   virtual void setIOpin(PinModule *);
-  virtual double get_Vref(void) { return(vr_Vref); };
+  virtual double get_Vref() { return(vr_Vref); };
                                                                                 
 protected:
   PinModule 		*vr_PinModule;
@@ -102,6 +104,25 @@ class CM_stimulus : public stimulus
      virtual void   set_nodeVoltage(double v);
 
 };
+class CMCON1 : public sfr_register
+{
+ public:
+  enum CMCON1_bits
+  {
+	CMSYNC = 1<<0,
+	T1GSS  = 1<<1
+  };
+
+  virtual void put(unsigned int);
+  void set_tmrl(TMRL *arg) { m_tmrl = arg; }
+  CMCON1(Processor *pCpu, const char *pName, const char *pDesc);
+  ~CMCON1();
+
+ private:
+  TMRL *m_tmrl;
+
+};
+
 class CMCON : public sfr_register
 {
  public:
@@ -132,16 +153,19 @@ class CMCON : public sfr_register
 
 
 
-  CMCON(void);
+  void set_tmrl(TMRL *arg) { m_tmrl = arg; }
+  CMCON(Processor *pCpu, const char *pName, const char *pDesc);
+  ~CMCON();
 
 protected:
   PinModule *cm_input[4];
   PinModule *cm_output[2];
-  const char *cm_input_pin[4];
-  const char *cm_output_pin[2];
+  char *cm_input_pin[4];
+  char *cm_output_pin[2];
   CMSignalSource *cm_source[2];
   unsigned int m_CMval[2];
   PIR_SET *pir_set;
+  TMRL *m_tmrl;
   CM_stimulus *cm_stimulus[4];
 
   static const int cMaxConfigurations=8;
@@ -151,20 +175,18 @@ protected:
 
 };
 
-class COMPARATOR_MODULE
+class ComparatorModule
 {
  public:
 
-  CMCON *cmcon;
-  VRCON vrcon;
-
-  void initialize( PIR_SET *pir_set, PinModule *pin_vr0, PinModule *pin_cm0, 
-	PinModule *pin_cm1, PinModule *pin_cm2,
-	PinModule *pin_cm3, PinModule *pin_cm4);
-
+  ComparatorModule(Processor *);
   void initialize( PIR_SET *pir_set, PinModule *pin_vr0, PinModule *pin_cm0, 
 	PinModule *pin_cm1, PinModule *pin_cm2,
 	PinModule *pin_cm3, PinModule *pin_cm4, PinModule *pin_cm5);
+  //protected:
+  CMCON cmcon;
+  CMCON1 cmcon1;
+  VRCON vrcon;
 
 };
 #endif // __COMPARATOR_H__
