@@ -280,10 +280,10 @@ instruction * dsPicProcessor::disasm (unsigned int address, unsigned int inst)
   pi = 0;
   for(int i =0; i<NUM_OP_DSPIC; i++)
     if((op_dsPic[i].inst_mask & inst) == op_dsPic[i].opcode)
-      pi = op_dsPic[i].inst_constructor(this, inst,address);
+      pi = op_dsPic[i].inst_constructor(this, inst, address);
 
   if(pi == 0)
-    pi = invalid_instruction::construct(this, inst);
+    pi = invalid_instruction::construct(this, inst, address);
 
   return (pi);
 
@@ -513,7 +513,7 @@ namespace dspic_instructions
 
   void MultiWordBranch::runtime_initialize()
   {
-    if(cpu_dsPic->program_memory[PMindex+1] != &bad_instruction) {
+    if(cpu_dsPic->program_memory[PMindex+1] != &cpu_dsPic->bad_instruction) {
   
       word2_opcode = cpu_dsPic->program_memory[PMindex+1]->get_opcode();
 
@@ -532,7 +532,7 @@ namespace dspic_instructions
       runtime_initialize();
 
     snprintf(return_str,len,"%s\t0x%05x",
-	     gpsimValue::name().c_str(),
+	     gpsimObject::name().c_str(),
 	     destination_index<<1);
 
     return(return_str);
@@ -686,8 +686,8 @@ namespace dspic_instructions
       ((resRV.data & 0xffff  )  ? 0  : eZ) |
       ((resRV.data & 0x10000 )  ? eC : 0) |
       (((resRV.data ^ baseRV.data ^ srcRV.data)&0x10) ? eDC : 0) |
-      (((resRV.data & ~baseRV.data & ~srcRV.data | 
-	 ~resRV.data & baseRV.data & srcRV.data) & 0x8000) ? eOV : 0) |
+      ((((resRV.data & ~baseRV.data & ~srcRV.data) | 
+	 (~resRV.data & baseRV.data & srcRV.data)) & 0x8000) ? eOV : 0) |
       ((resRV.data & 0x8000 )  ? eN : 0);
 
     cpu_dsPic->m_status.putFlags(flags, eC|eZ|eOV|eN|eDC, 0);
@@ -1321,8 +1321,8 @@ namespace dspic_instructions
       ((resRV.data & 0xffff  )  ? 0  : eZ) |
       ((resRV.data & 0x10000 )  ? eC : 0) |
       (((resRV.data ^ baseRV.data ^ srcRV.data)&0x10) ? eDC : 0) |
-      (((resRV.data & ~baseRV.data & ~srcRV.data | 
-	 ~resRV.data & baseRV.data & srcRV.data) & 0x8000) ? eOV : 0) |
+      ((((resRV.data & ~baseRV.data & ~srcRV.data) | 
+	 (~resRV.data & baseRV.data & srcRV.data)) & 0x8000) ? eOV : 0) |
       ((resRV.data & 0x8000 )  ? eN : 0);
 
     cpu_dsPic->m_status.putFlags(flags, eC|eZ|eOV|eN|eDC, 0);
