@@ -23,6 +23,10 @@ Boston, MA 02111-1307, USA.  */
 #include <typeinfo>
 #include <stdio.h>
 #include <ctype.h>
+#include <cstring>
+
+#include "../config.h"
+
 #include "gpsim_object.h"
 
 //========================================================================
@@ -41,15 +45,17 @@ gpsimObject::gpsimObject(const char *_name, const char *desc)
 
 gpsimObject::~gpsimObject()
 {
+  name_str.clear();
+  cpDescription = 0;
 }
 
 // The 'type' of any Viewable object is equivalent to the class name.
 string gpsimObject::showType()
 {
   const char* name;
-  
+
   name = typeid(*this).name();
-  
+
   /* Unfortunately, the class name string returned by typeid() is
    * implementation-specific.  If a particular compiler produces
    * ugly output, this is your chance to clean it up. */
@@ -75,9 +81,9 @@ string gpsimObject::showType()
     }
     name = buf;
   }
-  else	// just nn name
+  else  // just nn name
   {
-      while (isdigit(*name)) 
+      while (isdigit(*name))
         name++;
   }
 #elif defined _MSC_VER
@@ -123,6 +129,12 @@ char *gpsimObject::name(char *return_str, int len)
   return return_str;
 }
 
+/// TEMPORARY -- remove after gpsimValue and Value have been merged.
+unsigned int gpsimObject::get_value()
+{
+  return 0;
+}
+
 char *gpsimObject::toString(char *return_str, int len)
 {
   if(return_str)
@@ -145,7 +157,13 @@ string &gpsimObject::name(void) const
 
 string gpsimObject::toString()
 {
-  return showType();
+  //return showType();
+
+  char buff[64];
+  snprintf(buff,sizeof(buff), " = 0x%x",get_value());
+  string s = name() + string(buff);
+  return s;
+
 }
 
 // If derived classes don't redefine set_break and clear_break then
