@@ -45,7 +45,7 @@ Cycle_Counter cycles;
 // create an instance of inline get_cycles() method by taking its address
 Cycle_Counter &(*dummy_cycles)(void) = get_cycles;
 
-StopWatch stop_watch;
+StopWatch *stop_watch;
 
 
 //--------------------------------------------------
@@ -171,9 +171,13 @@ bool Cycle_Counter::set_break(guint64 future_cycle, TriggerObject *f, unsigned i
 
 #ifdef __DEBUG_CYCLE_COUNTER__
   cout << "Cycle_Counter::set_break  cycle = 0x" << hex<<future_cycle;
+  cout << " now=" << value;
 
     if(f)
-      cout << " has callback\n";
+    {
+      cout << " has callback ";
+	f->callback_print();
+    }
     else
       cout << " does not have callback\n";
 #endif
@@ -232,6 +236,11 @@ bool Cycle_Counter::set_break(guint64 future_cycle, TriggerObject *f, unsigned i
 	f->CallBackID = ++CallBackID_Sequence;
 
 #ifdef __DEBUG_CYCLE_COUNTER__
+      cout << "set_break l1->next=" << hex << l1->next << " ";
+      if (f)
+	f->callback_print();
+      else
+	cout << endl;
       cout << "cycle break " << future_cycle << " bpn " << bpn << '\n';
       if(f)
 	cout << "call back sequence number = "<< f->CallBackID <<'\n';
@@ -665,6 +674,7 @@ void Cycle_Counter::clear_current_break(TriggerObject *f)
   if(value == break_on_this && (!f || (f && f==active.next->f))) {
 
 #ifdef __DEBUG_CYCLE_COUNTER__
+    cout << "Cycle_Counter::clear_current_break ";
     cout << "current cycle " << hex << setw(16) << setfill('0') << value << endl;
     cout << "clearing current cycle break " << hex << setw(16) << setfill('0') << break_on_this;
     if(active.next->f)
@@ -770,16 +780,15 @@ class StopWatchValue : public Integer
 private:
   StopWatch *sw;
 public:
-  StopWatchValue(StopWatch *_sw) : Integer(0), sw(_sw) 
+  StopWatchValue(StopWatch *_sw) 
+    : Integer("stopwatch",0,
+              " A timer for monitoring and controlling the simulation.\n"
+              " The units are in simulation cycles.\n"
+              "  stopwatch.rollover - specifies rollover value.\n"
+              "  stopwatch.direction - specifies count direction.\n"
+              "  stopwatch.enable - enables counting if true.\n"), 
+      sw(_sw) 
   {
-    m_bClearableSymbol = false;
-    new_name("stopwatch");
-    set_description(" A timer for monitoring and controlling the simulation.\n"
-		    " The units are in simulation cycles.\n"
-		    "  stopwatch.rollover - specifies rollover value.\n"
-		    "  stopwatch.direction - specifies count direction.\n"
-		    "  stopwatch.enable - enables counting if true.\n"
-		    );
   }
   virtual void set(Value *v)
   {
@@ -791,7 +800,9 @@ public:
     i = (sw) ? sw->get() : 0;
     Integer::set(i);
   }
-  virtual int set_break(ObjectBreakTypes bt=eBreakAny, Expression *expr=0)
+  virtual int set_break(ObjectBreakTypes bt=eBreakAny, 
+			ObjectActionTypes at=eActionHalt, 
+			Expression *expr=0)
   {
     if(sw) sw->set_break(true);
     return -1;  // FIXME
@@ -808,11 +819,12 @@ class StopWatchRollover : public Integer
 private:
   StopWatch *sw;
 public:
-  StopWatchRollover(StopWatch *_sw) : Integer(1000000), sw(_sw) 
+  StopWatchRollover(StopWatch *_sw)
+    : Integer("stopwatch.rollover",
+              1000000,
+              " specifies the stop watch roll over time."), 
+      sw(_sw) 
   {
-    m_bClearableSymbol = false;
-    new_name("stopwatch.rollover");
-    set_description(" specifies the stop watch roll over time.");
   }
   virtual void set(Value *v)
   {
@@ -826,11 +838,11 @@ class StopWatchEnable : public Boolean
 private:
   StopWatch *sw;
 public:
-  StopWatchEnable(StopWatch *_sw) : Boolean(true) , sw(_sw)
+  StopWatchEnable(StopWatch *_sw)
+    : Boolean("stopwatch.enable",
+              true," If true, the stop watch is enabled."),
+      sw(_sw)
   {
-    m_bClearableSymbol = false;
-    new_name("stopwatch.enable");
-    set_description(" If true, the stop watch is enabled.");
   }
   virtual void set(Value *v)
   {
@@ -845,11 +857,12 @@ class StopWatchDirection : public Boolean
 private:
   StopWatch *sw;
 public:
-  StopWatchDirection(StopWatch *_sw) : Boolean(true) , sw(_sw)
+  StopWatchDirection(StopWatch *_sw) 
+    : Boolean("stopwatch.direction",
+              true,
+              " If true, the stop watch counts up otherwise down."),
+      sw(_sw)
   {
-    m_bClearableSymbol = false;
-    new_name("stopwatch.direction");
-    set_description(" If true, the stop watch counts up otherwise down.");
   }
   virtual void set(Value *v)
   {
@@ -872,7 +885,6 @@ StopWatch::StopWatch()
 {
 
   offset      = 0;
-  initialized = false;
   value       = new StopWatchValue(this);
   rollover    = new StopWatchRollover(this);
   enable      = new StopWatchEnable(this);
@@ -881,19 +893,20 @@ StopWatch::StopWatch()
   if(!value || !rollover || !enable || !direction)
     throw Error("StopWatch");
 
+  globalSymbolTable().addSymbol(value);
+  globalSymbolTable().addSymbol(rollover);
+  globalSymbolTable().addSymbol(enable);
+  globalSymbolTable().addSymbol(direction);
+
+  update();
 }
 
-void StopWatch::init()
+StopWatch::~StopWatch()
 {
-  if(!initialized) {
-    get_symbol_table().add(value);
-    get_symbol_table().add(rollover);
-    get_symbol_table().add(enable);
-    get_symbol_table().add(direction);
-
-    update();
-    initialized = true;
-  }
+  globalSymbolTable().deleteSymbol(value->name());
+  globalSymbolTable().deleteSymbol(rollover->name());
+  globalSymbolTable().deleteSymbol(enable->name());
+  globalSymbolTable().deleteSymbol(direction->name());
 }
 
 //----------------------------------------
@@ -905,7 +918,7 @@ guint64 StopWatch::get(void)
 {
 
   if(enable->getVal()) {
-    gint64 v = (cycles.value - offset) % rollover->getVal();
+    gint64 v = (cycles.get() - offset) % rollover->getVal();
     if(!direction->getVal())
       v = rollover->getVal() - v;
     return v;
@@ -947,7 +960,7 @@ void StopWatch::set_direction(bool b)
   direction->set(b);
 
   offset = 
-    cycles.value - 
+    cycles.get() - 
     ((rollover->getVal() - value->getVal()) % rollover->getVal());
 
 
@@ -974,9 +987,9 @@ void StopWatch::update()
 {
   if(enable->getVal()) {
     if(direction->getVal())
-      offset = cycles.value - value->getVal();
+      offset = cycles.get() - value->getVal();
     else
-      offset = cycles.value - (rollover->getVal() - value->getVal());
+      offset = cycles.get() - (rollover->getVal() - value->getVal());
 
     if(break_cycle)
       set_break(true);
@@ -995,9 +1008,9 @@ void StopWatch::set_break(bool b)
   guint64 old_break_cycle = break_cycle;
 
   if(direction->getVal())
-    break_cycle = cycles.value + rollover->getVal()  - get();
+    break_cycle = cycles.get() + rollover->getVal()  - get();
   else
-    break_cycle = cycles.value + get();
+    break_cycle = cycles.get() + get();
 
   if(old_break_cycle == break_cycle)
     return;
@@ -1011,7 +1024,7 @@ void StopWatch::set_break(bool b)
 
 void StopWatch::callback()
 {
-  break_cycle = cycles.value + rollover->getVal();
+  break_cycle = cycles.get() + rollover->getVal();
   cycles.set_break(break_cycle,this);
   cout << " stopwatch break\n";
   get_bp().halt();
