@@ -2,17 +2,17 @@
    Copyright (C) 2002 Ralf Forsberg
 
 This is based on the program icdprog 0.3 made by Geir Thomassen
-   
+
 gpsim is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.
-   
+
 gpsim is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
-   
+
 You should have received a copy of the GNU General Public License
 along with gpasm; see the file COPYING.  If not, write to
 the Free Software Foundation, 59 Temple Place - Suite 330,
@@ -22,17 +22,17 @@ Boston, MA 02111-1307, USA.  */
 /*
 
   FIXME - the design of this code needs to be revisited. Instead
-          of making the icd a separate entity that "controls" a 
-	  a processor, it makes MUCH more sense to derive the Icd
-	  class from the Processor class and let it effectively
-	  intercept and re-direct calls to the processor being
-	  debugged. This way, the gui and cli (and even external
-	  regression testing scripts) can treat the Icd just
-	  like it's another processor - no special circumstances
-	  are required.
+          of making the icd a separate entity that "controls" a
+          a processor, it makes MUCH more sense to derive the Icd
+          class from the Processor class and let it effectively
+          intercept and re-direct calls to the processor being
+          debugged. This way, the gui and cli (and even external
+          regression testing scripts) can treat the Icd just
+          like it's another processor - no special circumstances
+          are required.
 
 */
-   
+
 #include <assert.h>
 #include <stdio.h>
 #include <unistd.h>
@@ -40,6 +40,7 @@ Boston, MA 02111-1307, USA.  */
 #include <iomanip>
 #include <string>
 #include <list>
+#include <cstdio>
 #include <sys/ioctl.h>
 #include <fcntl.h>
 #include <termios.h>
@@ -80,7 +81,7 @@ public:
   Register *replaced;
   int is_stale;
 
-  icd_Register();
+  icd_Register(Processor *);
 
   virtual REGISTER_TYPES isa(void) {return replaced->isa();};
   virtual string &name(void)
@@ -88,7 +89,7 @@ public:
     if(replaced)
       return replaced->name();
     else
-      return gpsimValue::name();
+      return gpsimObject::name();
   };
 
   virtual void put_value(unsigned int new_value);
@@ -103,7 +104,7 @@ public:
   Status_register *replaced;
   int is_stale;
 
-  icd_StatusReg();
+  icd_StatusReg(Processor *);
 
   virtual REGISTER_TYPES isa(void) {return replaced->isa();};
   virtual string &name(void)
@@ -111,7 +112,7 @@ public:
     if(replaced)
       return replaced->name();
     else
-      return gpsimValue::name();
+      return gpsimObject::name();
   }
 
   virtual void put_value(unsigned int new_value);
@@ -123,10 +124,10 @@ public:
 class icd_WREG : public WREG
 {
 public:
-  WREG *replaced;  
+  WREG *replaced;
   int is_stale;
 
-  icd_WREG();
+  icd_WREG(Processor *);
 
   virtual REGISTER_TYPES isa(void) {return replaced->isa();};
   virtual string &name(void)
@@ -134,7 +135,7 @@ public:
     if(replaced)
       return replaced->name();
     else
-      return gpsimValue::name();
+      return gpsimObject::name();
   }
 
   virtual void put_value(unsigned int new_value);
@@ -149,7 +150,7 @@ public:
   PCLATH *replaced;
   int is_stale;
 
-  icd_PCLATH();
+  icd_PCLATH(Processor *);
 
   virtual REGISTER_TYPES isa(void) {return replaced->isa();};
   virtual string &name(void)
@@ -157,7 +158,7 @@ public:
     if(replaced)
       return replaced->name();
     else
-      return gpsimValue::name();
+      return gpsimObject::name();
   }
 
   virtual void put_value(unsigned int new_value);
@@ -168,10 +169,10 @@ public:
 class icd_FSR : public FSR
 {
 public:
-  FSR *replaced;   
+  FSR *replaced;
   int is_stale;
 
-  icd_FSR();
+  icd_FSR(Processor *);
 
   virtual REGISTER_TYPES isa(void) {return replaced->isa();};
   virtual string &name(void)
@@ -179,7 +180,7 @@ public:
     if(replaced)
       return replaced->name();
     else
-      return gpsimValue::name();
+      return gpsimObject::name();
   }
 
   virtual void put_value(unsigned int new_value);
@@ -194,7 +195,7 @@ public:
     Program_Counter *replaced;
     int is_stale;
 
-    icd_PC();
+    icd_PC(Processor *);
 
     virtual void put_value(unsigned int new_value);
     virtual unsigned int get_value(void);
@@ -223,8 +224,8 @@ static void dtr_set()
     if(icd_fd<0) return;
 
     if(ioctl(icd_fd, TIOCMBIS, &flag)) {
-	perror("ioctl");
-	exit(-1);
+        perror("ioctl");
+        throw new FatalError("ioctl");
     }
 }
 
@@ -235,8 +236,8 @@ static void dtr_clear()
     if(icd_fd<0) return;
 
     if(ioctl(icd_fd, TIOCMBIC, &flag)) {
-	perror("ioctl");
-	exit(-1);
+        perror("ioctl");
+        throw new FatalError("ioctl");
     }
 }
 
@@ -247,8 +248,8 @@ static void rts_set()
     if(icd_fd<0) return;
 
     if(ioctl(icd_fd, TIOCMBIS, &flag)) {
-	perror("ioctl");
-	exit(-1);
+        perror("ioctl");
+        throw new FatalError("ioctl");
     }
 }
 
@@ -259,8 +260,8 @@ static void rts_clear()
     if(icd_fd<0) return;
 
     if(ioctl(icd_fd, TIOCMBIC, &flag)) {
-	perror("ioctl");
-	exit(-1);
+        perror("ioctl");
+        throw new FatalError("ioctl");
     }
 }
 
@@ -281,7 +282,7 @@ static int icd_write(const char *s)
 
   write(icd_fd,s,  strlen(s));   /* Error checking ... */
 
-  return 1;  
+  return 1;
 }
 
 static int icd_read(unsigned char *p, int len)
@@ -295,13 +296,13 @@ static int icd_read(unsigned char *p, int len)
     rts_set();
 
     if(n_read != 1) {
-	cout << "Error in number of bytes read \n";
-	cout << "len="<<len<<endl;
-	return 0;
+        cout << "Error in number of bytes read \n";
+        cout << "len="<<len<<endl;
+        return 0;
     }
 
     if(len>1)
-	return n_read+icd_read(p+1,len-1);
+        return n_read+icd_read(p+1,len-1);
 
     return n_read;
 }
@@ -324,13 +325,13 @@ static int icd_cmd(const char *cmd, ...)
 
     if(!icd_read(resp,2))
     {
-	icd_sync();
-	icd_write(command);
-	if(!icd_read(resp,2))
-	{
-	    cout << "Command "<<command<<" failed"<<endl;
-	    return -1;
-	}
+        icd_sync();
+        icd_write(command);
+        if(!icd_read(resp,2))
+        {
+            cout << "Command "<<command<<" failed"<<endl;
+            return -1;
+        }
     }
 
     return (resp[0] << 8 ) |  resp[1];
@@ -345,12 +346,12 @@ static int icd_sync(void)
 
     while(tries>0)
     {
-	tries--;
+        tries--;
 
-	if(icd_cmd("$$6307\r")==1)
-	    return 1;
-	icd_write("$");
-	icd_read(buf,0x42);
+        if(icd_cmd("$$6307\r")==1)
+            return 1;
+        icd_write("$");
+        icd_read(buf,0x42);
     }
 
     puts("***************** DID NOT SYNC!");
@@ -366,17 +367,17 @@ static int icd_baudrate_init()
     if(icd_fd<0) return 0;
 
     while(tries) {
-	write(icd_fd,"U",1);
+        write(icd_fd,"U",1);
 
-	if(read(icd_fd,&ch,1) > 0) {
-	    rts_clear();
-	    udelay(10);
-	    rts_set();
-	    if(ch=='u') {
-		return 1;
-	    }
-	}
-	tries--;
+        if(read(icd_fd,&ch,1) > 0) {
+            rts_clear();
+            udelay(10);
+            rts_set();
+            if(ch=='u') {
+                return 1;
+            }
+        }
+        tries--;
     }
 
     return 0;
@@ -394,35 +395,35 @@ char *icd_target(void)
     rev = type & 0x1F;
 
     if(dev_id == 0x3FFF) {
-	sprintf(return_string,"no target");
+        sprintf(return_string,"no target");
     } else {
-	switch(type) {
-	case 0x68:
-	    sprintf(return_string,"16F870 rev %d",rev);
-	    break;
-	case 0x69:
-	    sprintf(return_string,"16F871 rev %d",rev);
-	    break;
-	case 0x47:
-	    sprintf(return_string,"16F872 rev %d",rev);
-	    break;
-	case 0x4B:
-	    sprintf(return_string,"16F873 rev %d",rev);
-	    break;
-	case 0x49:
-	    sprintf(return_string,"16F874 rev %d",rev);
-	    break;
-	case 0x4F:
-	    sprintf(return_string,"16F876 rev %d",rev);
-	    break;
-	case 0x4D:
-	    sprintf(return_string,"16F877 rev %d",rev);
-	    break;
+        switch(type) {
+        case 0x68:
+            sprintf(return_string,"16F870 rev %d",rev);
+            break;
+        case 0x69:
+            sprintf(return_string,"16F871 rev %d",rev);
+            break;
+        case 0x47:
+            sprintf(return_string,"16F872 rev %d",rev);
+            break;
+        case 0x4B:
+            sprintf(return_string,"16F873 rev %d",rev);
+            break;
+        case 0x49:
+            sprintf(return_string,"16F874 rev %d",rev);
+            break;
+        case 0x4F:
+            sprintf(return_string,"16F876 rev %d",rev);
+            break;
+        case 0x4D:
+            sprintf(return_string,"16F877 rev %d",rev);
+            break;
 
-	default:
-	    sprintf(return_string,"Unknown, device id = %02X",dev_id);
-	    break;
-	}
+        default:
+            sprintf(return_string,"Unknown, device id = %02X",dev_id);
+            break;
+        }
     }
     return return_string;
 }
@@ -432,8 +433,8 @@ struct termios oldtio, newtio;
 void put_dumb_register(Register **frp, int address)
 {
     Register *fr = *frp;
-    icd_Register *ir = new icd_Register;
-    ir->set_cpu(fr->get_cpu());
+    icd_Register *ir = new icd_Register(fr->get_cpu());
+    //ir->set_cpu(fr->get_cpu());
     *frp = ir;
     ir->replaced = fr;
     ir->address = address;
@@ -441,8 +442,8 @@ void put_dumb_register(Register **frp, int address)
 void put_dumb_status_register(Status_register **frp)
 {
     Status_register *fr = *frp;
-    icd_StatusReg *ir = new icd_StatusReg;
-    ir->set_cpu(fr->get_cpu());
+    icd_StatusReg *ir = new icd_StatusReg(fr->get_cpu());
+    //ir->set_cpu(fr->get_cpu());
     *frp = ir;
     ir->replaced = fr;
     ir->address = fr->address;
@@ -450,8 +451,7 @@ void put_dumb_status_register(Status_register **frp)
 void put_dumb_pc_register(Program_Counter **frp)
 {
     Program_Counter *fr = *frp;
-    icd_PC *ir = new icd_PC;
-    ir->set_cpu(fr->get_cpu());
+    icd_PC *ir = new icd_PC(fr->get_cpu());
     ir->memory_size_mask = fr->memory_size_mask;
     *frp = ir;
     ir->replaced = fr;
@@ -459,24 +459,24 @@ void put_dumb_pc_register(Program_Counter **frp)
 void put_dumb_pclath_register(PCLATH **frp)
 {
     PCLATH *fr = *frp;
-    icd_PCLATH *ir = new icd_PCLATH;
-    ir->set_cpu(fr->get_cpu());
+    icd_PCLATH *ir = new icd_PCLATH(fr->get_cpu());
+    //ir->set_cpu(fr->get_cpu());
     *frp = ir;
     ir->replaced = fr;
 }
 void put_dumb_w_register(WREG **frp)
 {
     WREG *fr = *frp;
-    icd_WREG *ir = new icd_WREG;
-    ir->set_cpu(fr->get_cpu());
+    icd_WREG *ir = new icd_WREG(fr->get_cpu());
+    //ir->set_cpu(fr->get_cpu());
     *frp = ir;
     ir->replaced = fr;
 }
 void put_dumb_fsr_register(FSR **frp)
 {
     FSR *fr = *frp;
-    icd_FSR *ir = new icd_FSR;
-    ir->set_cpu(fr->get_cpu());
+    icd_FSR *ir = new icd_FSR(fr->get_cpu());
+    //ir->set_cpu(fr->get_cpu());
     *frp = ir;
     ir->replaced = fr;
 }
@@ -489,7 +489,7 @@ static void create_dumb_register_file(void)
 
     for(unsigned int i=0;i<cpu->register_memory_size();i++)
     {
-	put_dumb_register(&cpu->registers[i], i);
+        put_dumb_register(&cpu->registers[i], i);
     }
     put_dumb_status_register(&cpu->status);
     put_dumb_pc_register(&cpu->pc);
@@ -499,19 +499,19 @@ static void create_dumb_register_file(void)
 }
 
 
-int icd_connect(char *port)
+int icd_connect(const char *port)
 {
   pic_processor *pic=dynamic_cast<pic_processor *>(active_cpu);
 
     if(!pic)
     {
-	cout << "You have to load the .cod file (or .hex and processor)" << endl;
-	return 0;
+        cout << "You have to load the .cod file (or .hex and processor)" << endl;
+        return 0;
     }
 
     if((icd_fd=open(port, O_NOCTTY | O_RDWR | O_SYNC)) == -1) {
-	perror("Error opening device:");
-	return 0;
+        perror("Error opening device:");
+        return 0;
     }
 
     tcgetattr(icd_fd, &oldtio);
@@ -533,8 +533,8 @@ int icd_connect(char *port)
     rts_set();
 
     if(!icd_baudrate_init()) {
-	fprintf(stderr,"Can't initialize the ICD\n");
-	return 0;
+        fprintf(stderr,"Can't initialize the ICD\n");
+        return 0;
     }
 
     create_dumb_register_file();
@@ -546,19 +546,19 @@ int icd_connect(char *port)
      this. The program works ok without this though ..*/
 
 
-    //	printf("ICD ver %s, target %s\n",icd_version(),icd_target());
-    //	printf("Vdd %.2f, Vpp %.2f\n", icd_vdd(), icd_vpp());
+    //  printf("ICD ver %s, target %s\n",icd_version(),icd_target());
+    //  printf("Vdd %.2f, Vpp %.2f\n", icd_vdd(), icd_vpp());
 
     if(icd_has_debug_module())
     {
-	if(verbose)
-	    cout << "Debug module present"<<endl;
+        if(verbose)
+            cout << "Debug module present"<<endl;
     }
     else
     {
-	cout << "Debug module not present. Enabling..."<<flush;
-	icd_cmd("$$7008\r"); // Enable debug module
-	cout << "Done." << endl;
+        cout << "Debug module not present. Enabling..."<<flush;
+        icd_cmd("$$7008\r"); // Enable debug module
+        cout << "Done." << endl;
     }
 
     icd_reset();
@@ -572,7 +572,7 @@ int icd_has_debug_module(void)
 
     icd_cmd("$$700A\r");
     if(icd_cmd("$$6307\r")==1)
-	return 1;
+        return 1;
     return 0;
 }
 
@@ -598,9 +598,9 @@ static void make_stale(void)
 
     for(unsigned int i=0;i<cpu->register_memory_size();i++)
     {
-	icd_Register *ir = dynamic_cast<icd_Register*>(cpu->registers[i]);
-	assert(ir!=0);
-	ir->is_stale=1;
+        icd_Register *ir = dynamic_cast<icd_Register*>(cpu->registers[i]);
+        assert(ir!=0);
+        ir->is_stale=1;
     }
 
     icd_WREG *iw = dynamic_cast<icd_WREG*>(cpu->W);
@@ -648,7 +648,7 @@ int icd_detected(void)
     if(icd_fd<0) return 0;
 
     if(use_icd)
-	return 1;
+        return 1;
     return 0;
 }
 
@@ -708,9 +708,9 @@ int icd_run(void)
 
     if(icd_cmd("$$700F\r")!=1)
     {
-	icd_sync();
-	if(icd_cmd("$$700F\r")!=1)
-	    cout << "fjsdk" << endl;
+        icd_sync();
+        if(icd_cmd("$$700F\r")!=1)
+            cout << "fjsdk" << endl;
     }
     return 1;
 }
@@ -720,7 +720,7 @@ int icd_stopped(void)
     if(icd_fd<0) return 0;
 
     if(icd_cmd("$$701E\r")!=1)
-	return 1;
+        return 1;
     return 0;
 }
 
@@ -743,7 +743,7 @@ int icd_set_break(int address)
 
     if(icd_cmd("$$%04X\r",address)!=address)
     {
-	puts("DEBUG: Set breakpoint failed?");
+        puts("DEBUG: Set breakpoint failed?");
         return 0;
     }
 
@@ -769,74 +769,74 @@ void icd_set_bulk(int flag)
 // Get the value of the specified file memory address
 /*int icd_read_file(int address)
 {
-	unsigned char buf[8];
-	int offset = address - address%8;
+        unsigned char buf[8];
+        int offset = address - address%8;
         if(icd_fd<0) return 0;
 
-	int value;
+        int value;
 
-	cout << "this is deprecated" << endl;
-	
-	icd_cmd("$$%04X\r",0x7800+offset);
-	icd_cmd("$$7C08\r");
+        cout << "this is deprecated" << endl;
+
+        icd_cmd("$$%04X\r",0x7800+offset);
+        icd_cmd("$$7C08\r");
+
+        icd_write("$$7D08\r");
+        icd_read(buf,8);
+
+        value = buf[address%8];
 
-	icd_write("$$7D08\r");
-	icd_read(buf,8);
-	
-	value = buf[address%8];
-	
   pic_processor *pic=dynamic_cast<pic_processor *>(active_cpu);
   if(!pic)
     return;
 
-		    //if(gpsim_register_is_valid(1,REGISTER_RAM,address) &&
-		    //   !gpsim_register_is_alias(1,REGISTER_RAM,address))
-		    {
-			switch(address)
-			{
-			case 2:
-			case 3:
-			case 4:
-			case 10:
+                    //if(gpsim_register_is_valid(1,REGISTER_RAM,address) &&
+                    //   !gpsim_register_is_alias(1,REGISTER_RAM,address))
+                    {
+                        switch(address)
+                        {
+                        case 2:
+                        case 3:
+                        case 4:
+                        case 10:
                             break;
-			default:
-			    pic->registers[address]->put_value(value);
-			cout << "Read file address " << address << "=" << value << endl;
+                        default:
+                            pic->registers[address]->put_value(value);
+                        cout << "Read file address " << address << "=" << value << endl;
                             break;
-			}
-		    }
-	
-	return 1;
+                        }
+                    }
+
+        return 1;
 }
 
 int icd_write_file(int address, int data)
 {
         if(icd_fd<0) return 0;
 
-	
 
-	printf("Write file address 0x%04X with data 0x%02X\n",address,data);
-	return 1;
+
+        printf("Write file address 0x%04X with data 0x%02X\n",address,data);
+        return 1;
 }
 
 int icd_read_eeprom(int address)
 {
         if(icd_fd<0) return 0;
 
-	
 
-	printf("Read eeprom address 0x%04X\n",address);
-	return 1;
+
+        printf("Read eeprom address 0x%04X\n",address);
+        return 1;
 }
 
 int write_eeprom(int address, int data)
 {
         if(icd_fd<0) return 0;
 
-	
 
-	printf("Write eeprom address 0x%04X with data 0x%02X\n",address,data);
-	return 1;
+
+        printf("Write eeprom address 0x%04X with data 0x%02X\n",address,data);
+        return 1;
 }
 
 int icd_get_state()
@@ -845,75 +845,76 @@ int icd_get_state()
 
     if(icd_fd<0) return 0;
 
-	
 
-	cout << "Get state" << endl;
-	pc=icd_cmd("$$701F\r");
-	status=icd_cmd("$$7016\r")&0x00ff;
-	w=icd_cmd("$$7017\r")&0x00ff;
-	pclath=icd_cmd("$$7018\r")&0x00ff;
-	fsr=icd_cmd("$$7019\r")&0x00ff;
 
-	pic_processor *pic=dynamic_cast<pic_processor *>(active_cpu);
-	if(!pic)
-	  return;
+        cout << "Get state" << endl;
+        pc=icd_cmd("$$701F\r");
+        status=icd_cmd("$$7016\r")&0x00ff;
+        w=icd_cmd("$$7017\r")&0x00ff;
+        pclath=icd_cmd("$$7018\r")&0x00ff;
+        fsr=icd_cmd("$$7019\r")&0x00ff;
 
-	pic->pc->put_value(pc);
-	pic->status->put_value(status);
-	pic->W->put_value(w);
-	pic->pclath->put_value(pclath);
+        pic_processor *pic=dynamic_cast<pic_processor *>(active_cpu);
+        if(!pic)
+          return;
+
+        pic->pc->put_value(pc);
+        pic->status->put_value(status);
+        pic->W->put_value(w);
+        pic->pclath->put_value(pclath);
         pic->fsr->put_value(fsr);
 
-	return 1;
+        return 1;
 }
  */
 // Get all of file memory
 /*int icd_get_file()
 {
-	unsigned char buf[64];
-	
+        unsigned char buf[64];
+
         if(icd_fd<0) return 0;
 
-	
 
-	cout << "Get file" << endl;
 
-	pic_processor *pic=dynamic_cast<pic_processor *>(active_cpu);
-	if(!pic)
-	  return;
-	
-	for(int i=0;i<pic->register_memory_size()/0x40;i++)
-	{
-	    if(icd_cmd("$$%04X\r",0x7A00+i)!=i)
-		puts("EEEEEEEEEEEEEEEEEEEEE");
+        cout << "Get file" << endl;
 
-		icd_write("$$7D40\r");
-		icd_read(buf,64);
-		for(int j=0;j<64;j++)
-		{
-		    if(gpsim_register_is_valid(1,REGISTER_RAM,i*0x40+j) &&
-		       !gpsim_register_is_alias(1,REGISTER_RAM,i*0x40+j))
-		    {
-			switch(i*0x40+j)
-			{
-			case 2:
-			case 3:
-			case 4:
-			case 10:
+        pic_processor *pic=dynamic_cast<pic_processor *>(active_cpu);
+        if(!pic)
+          return;
+
+        for(int i=0;i<pic->register_memory_size()/0x40;i++)
+        {
+            if(icd_cmd("$$%04X\r",0x7A00+i)!=i)
+                puts("EEEEEEEEEEEEEEEEEEEEE");
+
+                icd_write("$$7D40\r");
+                icd_read(buf,64);
+                for(int j=0;j<64;j++)
+                {
+                    if(gpsim_register_is_valid(1,REGISTER_RAM,i*0x40+j) &&
+                       !gpsim_register_is_alias(1,REGISTER_RAM,i*0x40+j))
+                    {
+                        switch(i*0x40+j)
+                        {
+                        case 2:
+                        case 3:
+                        case 4:
+                        case 10:
                             break;
-			default:
-			    pic->registers[i*0x40+j]->put_value(buf[j]);
+                        default:
+                            pic->registers[i*0x40+j]->put_value(buf[j]);
                             break;
-			}
-		    }
-		}
-//	}
-	
-	return 1;
+                        }
+                    }
+                }
+//      }
+
+        return 1;
 }
 */
 
-icd_Register::icd_Register()
+icd_Register::icd_Register(Processor *pCpu)
+  : Register(pCpu,"","")
 {
     replaced=0;
     value.put(0x42);
@@ -937,129 +938,130 @@ unsigned int icd_Register::get(void)
 {
     if(is_stale)
     {
-	switch(address)
-	{
-	case 2:
-	    value.put(icd_cmd("$$701F\r"));
-	    cpu_pic->pcl->value.put(value.get() & 0xff);
-	    cpu_pic->pclath->value.put(value.get() >> 8);
+        switch(address)
+        {
+        case 2:
+            value.put(icd_cmd("$$701F\r"));
+            cpu_pic->pcl->value.put(value.get() & 0xff);
+            cpu_pic->pclath->value.put(value.get() >> 8);
 
-	    is_stale=0;
-	    break;
-	case 3:
-	    value.put(icd_cmd("$$7016\r")&0x00ff);
-	    is_stale=0;
-	    replaced->update();
-	    break;
-	case 4:
-	    value.put(icd_cmd("$$7019\r")&0x00ff);
-	    is_stale=0;
-	    replaced->update();
-	    break;
-	case 10:
-	    value.put(icd_cmd("$$701F\r"));
-	    cpu_pic->pcl->value.put(value.get() & 0xff);
-	    cpu_pic->pclath->value.put(value.get() >> 8);
+            is_stale=0;
+            break;
+        case 3:
+            value.put(icd_cmd("$$7016\r")&0x00ff);
+            is_stale=0;
+            replaced->update();
+            break;
+        case 4:
+            value.put(icd_cmd("$$7019\r")&0x00ff);
+            is_stale=0;
+            replaced->update();
+            break;
+        case 10:
+            value.put(icd_cmd("$$701F\r"));
+            cpu_pic->pcl->value.put(value.get() & 0xff);
+            cpu_pic->pclath->value.put(value.get() >> 8);
 
-	    is_stale=0;
-	    break;
-	default:
-	    {
-		if(bulk_flag==0)
-		{
-		    unsigned char buf[8];
-		    int offset = address - address%8;
-		    icd_cmd("$$%04X\r",0x7800+offset);
-		    icd_cmd("$$7C08\r");
-		    icd_write("$$7D08\r");
-		    icd_read(buf,8);
-		    for(int i=0;i<8;i++)
-		    {
-			switch(offset+i)
-			{
-			case 2:
-			case 3:
-			case 4:
-			case 10:
-			    break;
-			default:
-			    icd_Register *ifr = static_cast<icd_Register*>(get_cpu()->registers[offset+i]);
-			    assert(ifr!=0);
-			    ifr->value.put(buf[i]);
-			    ifr->is_stale=0;
-			    break;
+            is_stale=0;
+            break;
+        default:
+            {
+                if(bulk_flag==0)
+                {
+                    unsigned char buf[8];
+                    int offset = address - address%8;
+                    icd_cmd("$$%04X\r",0x7800+offset);
+                    icd_cmd("$$7C08\r");
+                    icd_write("$$7D08\r");
+                    icd_read(buf,8);
+                    for(int i=0;i<8;i++)
+                    {
+                        switch(offset+i)
+                        {
+                        case 2:
+                        case 3:
+                        case 4:
+                        case 10:
+                            break;
+                        default:
+                            icd_Register *ifr = static_cast<icd_Register*>(get_cpu()->registers[offset+i]);
+                            assert(ifr!=0);
+                            ifr->value.put(buf[i]);
+                            ifr->is_stale=0;
+                            break;
 
-			}
-		    }
-		    for(int i=0;i<0x8;i++)
-		    {
-			switch(offset+i)
-			{
-			case 2:
-			case 3:
-			case 4:
-			case 10:
-			    break;
-			default:
-			    icd_Register *ifr = static_cast<icd_Register*>(get_cpu()->registers[offset+i]);
-			    assert(ifr!=0);
-			    ifr->replaced->update();
-			    break;
-			}
-		    }
-		}
-		else
-		{
-		    unsigned char buf[64];
-		    int offset=address-address%0x40;
-		    assert(offset>=0);
-		    if(icd_cmd("$$%04X\r",0x7A00+offset/0x40)!=offset/0x40)
-			puts("DDDDDDDDDDDDDDDDDDD");
-		    icd_write("$$7D40\r");
-		    int n_read = icd_read(buf,0x40);
-		    for(unsigned int i=0;i<0x40;i++)
-		    {
-			switch(offset+i)
-			{
-			case 2:
-			case 3:
-			case 4:
-			case 10:
-			    break;
-			default:
-			    icd_Register *ifr = static_cast<icd_Register*>(get_cpu()->registers[offset+i]);
-			    assert(ifr!=0);
-			    ifr->value.put(buf[i]);
-			    ifr->is_stale=0;
-			    break;
+                        }
+                    }
+                    for(int i=0;i<0x8;i++)
+                    {
+                        switch(offset+i)
+                        {
+                        case 2:
+                        case 3:
+                        case 4:
+                        case 10:
+                            break;
+                        default:
+                            icd_Register *ifr = static_cast<icd_Register*>(get_cpu()->registers[offset+i]);
+                            assert(ifr!=0);
+                            ifr->replaced->update();
+                            break;
+                        }
+                    }
+                }
+                else
+                {
+                    unsigned char buf[64];
+                    int offset=address-address%0x40;
+                    assert(offset>=0);
+                    if(icd_cmd("$$%04X\r",0x7A00+offset/0x40)!=offset/0x40)
+                        puts("DDDDDDDDDDDDDDDDDDD");
+                    icd_write("$$7D40\r");
+                    icd_read(buf,0x40);
+                    for(unsigned int i=0;i<0x40;i++)
+                    {
+                        switch(offset+i)
+                        {
+                        case 2:
+                        case 3:
+                        case 4:
+                        case 10:
+                            break;
+                        default:
+                            icd_Register *ifr = static_cast<icd_Register*>(get_cpu()->registers[offset+i]);
+                            assert(ifr!=0);
+                            ifr->value.put(buf[i]);
+                            ifr->is_stale=0;
+                            break;
 
-			}
-		    }
-		    for(int i=0;i<0x40;i++)
-		    {
-			switch(offset+i)
-			{
-			case 2:
-			case 3:
-			case 4:
-			case 10:
-			    break;
-			default:
-			    icd_Register *ifr = static_cast<icd_Register*>(get_cpu()->registers[offset+i]);
-			    assert(ifr!=0);
-			    ifr->replaced->update();
-			    break;
-			}
-		    }
-		}
-	    }
-	    break;
-	}
+                        }
+                    }
+                    for(int i=0;i<0x40;i++)
+                    {
+                        switch(offset+i)
+                        {
+                        case 2:
+                        case 3:
+                        case 4:
+                        case 10:
+                            break;
+                        default:
+                            icd_Register *ifr = static_cast<icd_Register*>(get_cpu()->registers[offset+i]);
+                            assert(ifr!=0);
+                            ifr->replaced->update();
+                            break;
+                        }
+                    }
+                }
+            }
+            break;
+        }
     }
     return(value.get());
 }
 
-icd_WREG::icd_WREG()
+icd_WREG::icd_WREG(Processor *pCpu)
+  : WREG(pCpu,"","")
 {
     replaced=0;
     value.put(0x42);
@@ -1083,14 +1085,15 @@ unsigned int icd_WREG::get(void)
 {
     if(is_stale)
     {
-	value.put(icd_cmd("$$7017\r")&0x00ff);
-	is_stale=0;
-	replaced->update();
+        value.put(icd_cmd("$$7017\r")&0x00ff);
+        is_stale=0;
+        replaced->update();
     }
     return(value.get());
 }
 
-icd_StatusReg::icd_StatusReg()
+icd_StatusReg::icd_StatusReg(Processor *pCpu)
+  : Status_register(pCpu,"","")
 {
     replaced=0;
     value.put(0x42);
@@ -1116,15 +1119,16 @@ unsigned int icd_StatusReg::get(void)
 {
     if(is_stale)
     {
-	value.put(icd_cmd("$$7016\r")&0x00ff);
-	is_stale=0;
-	replaced->update();
+        value.put(icd_cmd("$$7016\r")&0x00ff);
+        is_stale=0;
+        replaced->update();
     }
     return(value.get());
 }
 
 
-icd_FSR::icd_FSR()
+icd_FSR::icd_FSR(Processor *pCpu)
+  : FSR(pCpu,"","")
 {
     replaced=0;
     value.put(0x42);
@@ -1148,13 +1152,14 @@ unsigned int icd_FSR::get_value(void)
 
     if(is_stale)
     {
-	value.put(icd_cmd("$$7019\r")&0x00ff);
-	is_stale=0;
-	replaced->update();
+        value.put(icd_cmd("$$7019\r")&0x00ff);
+        is_stale=0;
+        replaced->update();
     }
     return(value.get());
 }
-icd_PCLATH::icd_PCLATH()
+icd_PCLATH::icd_PCLATH(Processor *pCpu)
+  : PCLATH(pCpu,"","")
 {
     replaced=0;
     value.put(0x42);
@@ -1180,13 +1185,14 @@ unsigned int icd_PCLATH::get_value(void)
 
     if(is_stale)
     {
-	value.put((icd_cmd("$$701F\r")&0xff00)>>8);
-	is_stale=0;
-	replaced->update();
+        value.put((icd_cmd("$$701F\r")&0xff00)>>8);
+        is_stale=0;
+        replaced->update();
     }
     return(value.get());
 }
-icd_PC::icd_PC()
+icd_PC::icd_PC(Processor *pCpu)
+  : Program_Counter("pc", "ICD Program Counter", pCpu)
 {
     replaced=0;
     value=0x42;
@@ -1203,11 +1209,11 @@ unsigned int icd_PC::get_value(void)
 
     if(is_stale)
     {
-	value = icd_cmd("$$701F\r");
-	cpu_pic->pcl->value.put(value & 0xff);
-	cpu_pic->pclath->value.put(value >> 8);
+        value = icd_cmd("$$701F\r");
+        cpu_pic->pcl->value.put(value & 0xff);
+        cpu_pic->pclath->value.put(value >> 8);
 
-	is_stale=0;
+        is_stale=0;
     }
     return(value);
 }
