@@ -41,41 +41,24 @@ Boston, MA 02111-1307, USA.  */
 //--------------------------------------------------
 // member functions for the INTCON base class
 //--------------------------------------------------
-INTCON::INTCON(void)
-  : interrupt_trace(0)
+INTCON::INTCON(Processor *pCpu, const char *pName, const char *pDesc)
+  : sfr_register(pCpu, pName, pDesc),
+    interrupt_trace(0)
 {
-  new_name("intcon");
 }
 
-void INTCON::set_T0IF(void)
+void INTCON::set_T0IF()
 {
   Dprintf((" INTCON::%s\n",__FUNCTION__));
-  trace.raw(write_trace.get() | value.get());
 
-  value.put(value.get() | T0IF);
-
-  if (value.get() & (GIE | T0IE))
-  {
-    trace.interrupt();
-  }
-}
-
-void INTCON::set_cpu(Processor *new_cpu)
-{
-  cpu = new_cpu;
-
-  pic_processor *pcpu = dynamic_cast<pic_processor *>(cpu);
-
-  assert(pcpu);
+  put(value.get() | T0IF);
 
 }
 
 void INTCON::put(unsigned int new_value)
 {
   Dprintf((" INTCON::%s\n",__FUNCTION__));
-
   trace.raw(write_trace.get() | value.get());
-
   value.put(new_value);
 
   // Now let's see if there's a pending interrupt
@@ -89,35 +72,78 @@ void INTCON::put(unsigned int new_value)
   // note: bit6 is not handled here because it is processor
   // dependent (e.g. EEIE for x84 and ADIE for x7x).
 
-  if(value.get() & GIE)
-    {
-
-      if( (((value.get()>>3)&value.get()) & (T0IF | INTF | RBIF)) )
-	{
-	  trace.interrupt();
-	  bp.set_interrupt();
-	}
-      else if(value.get() & XXIE)
-	{
-	  if(check_peripheral_interrupt())
-	    {
-	      trace.interrupt();
-	      bp.set_interrupt();
-	    }
-	}
-    }
+  if(value.get() & GIE  &&
+     ( ((value.get()>>3)&value.get() & (T0IF | INTF | RBIF)) ||
+       ( value.get() & XXIE && check_peripheral_interrupt() )))
+      cpu_pic->BP_set_interrupt();
 }
 
-void INTCON::peripheral_interrupt(void)
+void INTCON::peripheral_interrupt ( bool hi_pri )
 {
   Dprintf((" INTCON::%s\n",__FUNCTION__));
 
+  if ( hi_pri )
+      cout << "Dodgy call to 14-bit INTCON::peripheral_interrupt with priority set\n";
+
   if(  (value.get() & (GIE | XXIE)) == (GIE | XXIE) )
-    bp.set_interrupt();
+    cpu_pic->BP_set_interrupt();
 }
 
+//----------------------------------------------------------------------
+//----------------------------------------------------------------------
+INTCON2::INTCON2(Processor *pCpu, const char *pName, const char *pDesc)
+  : sfr_register(pCpu, pName, pDesc), m_bsRBPU(0)
+{}
 
-bool INTCON_14_PIR::check_peripheral_interrupt()
+bool INTCON2::assignBitSink(unsigned int bitPosition, BitSink *pBS)
+{
+  if (bitPosition == 7) 
+    m_bsRBPU = pBS;
+  return true;
+}
+
+bool INTCON2::releaseBitSink(unsigned int bitPosition, BitSink *)
+{
+  if (bitPosition == 7) 
+    m_bsRBPU = 0;
+  return true;
+}
+
+//----------------------------------------------------------------------
+//----------------------------------------------------------------------
+void INTCON2::put_value(unsigned int new_value)
+{
+  unsigned int old_value = value.get();
+  value.put(new_value);
+
+  if ((old_value ^ new_value) & RBPU  && m_bsRBPU)
+    m_bsRBPU->setSink((new_value & RBPU) != 0);
+
+
+}
+
+//----------------------------------------------------------------------
+//----------------------------------------------------------------------
+void INTCON2::put(unsigned int new_value)
+{
+  trace.raw(write_trace.get() | value.get());
+  put_value(new_value);
+}
+
+//----------------------------------------------------------------------
+//----------------------------------------------------------------------
+INTCON3::INTCON3(Processor *pCpu, const char *pName, const char *pDesc)
+  : sfr_register(pCpu, pName, pDesc)
+{}
+
+//----------------------------------------------------------------------
+//----------------------------------------------------------------------
+INTCON_14_PIR::INTCON_14_PIR(Processor *pCpu, const char *pName, const char *pDesc)
+  : INTCON(pCpu, pName, pDesc),
+    pir_set(0)
+{}
+
+int INTCON_14_PIR::check_peripheral_interrupt()
 {
   assert(pir_set != 0);
 
@@ -132,16 +158,56 @@ bool INTCON_14_PIR::check_peripheral_interrupt()
 // intcon for the 16-bit processor cores.
 //
 //----------------------------------------------------------------------
-INTCON_16::INTCON_16()
-  : interrupt_vector(0), rcon(0), intcon2(0)
+INTCON_16::INTCON_16(Processor *pCpu, const char *pName, const char *pDesc)
+  : INTCON(pCpu, pName, pDesc),
+    interrupt_vector(0), rcon(0), intcon2(0), pir_set(0)
 {
-
 }
 
 
+void INTCON_16::peripheral_interrupt ( bool hi_pri )
+{
+  Dprintf((" INTCON_16::%s\n",__FUNCTION__));
+  assert(rcon != 0);
+
+  if(rcon->value.get() & RCON::IPEN)
+  {
+      cout << "peripheral interrupt, priority " << hi_pri << "\n";
+    if ( hi_pri )
+    {
+      if ( value.get() & GIEH )
+      {
+        set_interrupt_vector(INTERRUPT_VECTOR_HI);
+        cpu_pic->BP_set_interrupt();
+      }
+    }
+    else
+    {
+      if ( ( value.get() & (GIEH|GIEL) ) == (GIEH|GIEL) )
+      {
+        set_interrupt_vector(INTERRUPT_VECTOR_LO);
+        cpu_pic->BP_set_interrupt();
+      }
+    }
+  }
+  else
+  {
+    if(  (value.get() & (GIE | XXIE)) == (GIE | XXIE) )
+      cpu_pic->BP_set_interrupt();
+  }
+}
+
+int INTCON_16::check_peripheral_interrupt()
+{
+  assert(pir_set != 0);
+
+  Dprintf((" INTCON_16::%s\n",__FUNCTION__));
+  return (pir_set->interrupt_status());     // Not quite right, but...
+  // was return 0; but that was blatantly broken
+}
 
 //----------------------------------------------------------------------
-// void INTCON_16::clear_gies(void)
+// void INTCON_16::clear_gies()
 //
 //  This routine clears the global interrupt enable bit(s). If priority
 // interrupts are used (IPEN in RCON is set) then the appropriate gie
@@ -151,26 +217,28 @@ INTCON_16::INTCON_16()
 //
 //----------------------------------------------------------------------
 
-void INTCON_16::clear_gies(void)
+void INTCON_16::clear_gies()
 {
 
   assert(cpu != 0);
 
-  if(haveHighPriorityInterrupt())
-    put(get() & ~GIEH);
+  if ( !(rcon->value.get() & RCON::IPEN) )
+    put(value.get() & ~GIE);
+  else if ( isHighPriorityInterrupt() )
+    put(value.get() & ~GIEH);
   else
-    put(get() & ~GIEL);
+    put(value.get() & ~GIEL);
 
 
 }
 
 
 //----------------------------------------------------------------------
-// void INTCON_16::clear_gies(void)
+// void INTCON_16::clear_gies()
 //
 //----------------------------------------------------------------------
 
-void INTCON_16::set_gies(void)
+void INTCON_16::set_gies()
 {
 
   assert(rcon != 0);
@@ -246,8 +314,10 @@ void INTCON_16::put(unsigned int new_value)
   if(rcon->value.get() & RCON::IPEN)
     {
       unsigned int i1;
+      int i2;
 
       // Use interrupt priorities
+      // %%%FIXME%%% ***BUG*** - does not attempt to look for peripheral interrupts
 
       if( 0 == (value.get() & GIEH))
 	return;    // Interrupts are disabled
@@ -265,13 +335,13 @@ void INTCON_16::put(unsigned int new_value)
       // associated priority bit!
 
       i1 =  ( (value.get()>>3)&value.get()) & (T0IF | INTF | RBIF);
+      i2 =  check_peripheral_interrupt();
 
-      if(i1 & ( (intcon2->value.get() & (T0IF | RBIF)) | INTF))
+      if ( ( i1 & ( (intcon2->value.get() & (T0IF | RBIF)) | INTF) )
+        || ( i2 & 2 ) )
 	{
-	  //cout << " selecting high priority vector\n";
 	  set_interrupt_vector(INTERRUPT_VECTOR_HI);
-	  trace.interrupt();
-	  bp.set_interrupt();
+          cpu_pic->BP_set_interrupt();
 	  return;
 	}
 
@@ -280,12 +350,13 @@ void INTCON_16::put(unsigned int new_value)
       // interrupts pending. So let's check for the low priority
       // ones.
 
-      if(i1 & ( (~intcon2->value.get() & (T0IF | RBIF)) | INTF))
+      if ( ( (i1 & (~intcon2->value.get() & (T0IF | RBIF)))
+          || (i2 & 1) )  
+        && (value.get() & GIEL) )
 	{
 	  //cout << " selecting low priority vector\n";
 	  set_interrupt_vector(INTERRUPT_VECTOR_LO);
-	  trace.interrupt();
-	  bp.set_interrupt();
+          cpu_pic->BP_set_interrupt();
 	  return;
 	}
 
@@ -300,17 +371,11 @@ void INTCON_16::put(unsigned int new_value)
       if(value.get() & GIE) 
 	{
 	  if( ( (value.get()>>3)&value.get()) & (T0IF | INTF | RBIF) )
-	    {
-	      trace.interrupt();
-	      bp.set_interrupt();
-	    }
+            cpu_pic->BP_set_interrupt();
 	  else if(value.get() & XXIE)
 	    {
 	      if(check_peripheral_interrupt())
-		{
-		  trace.interrupt();
-		  bp.set_interrupt();
-		}
+                cpu_pic->BP_set_interrupt();
 	    }
 
 	}
