@@ -43,12 +43,23 @@ Boston, MA 02111-1307, USA.  */
 //--------------------------------------------------
 // 
 //--------------------------------------------------
+SignalControl::~SignalControl()
+{
+}
 PeripheralSignalSource::PeripheralSignalSource(PinModule *_pin)
   : m_pin(_pin), m_cState('?')
 {
   assert(m_pin);
 }
 
+PeripheralSignalSource::~PeripheralSignalSource()
+{
+}
+
+void PeripheralSignalSource::release()
+{
+}
+
 // getState is called when the PinModule is attempting to
 // update the output state for the I/O Pin.
 
@@ -110,6 +121,17 @@ public:
     : m_register(_reg), m_bitMask(1<<bitPosition)
   {
   }
+
+  ~SignalSource()
+  {
+  }
+  virtual void release()
+  {
+    if (verbose)
+        cout << "Deleting SignalSource\n";
+    delete this;
+  }
+
   char getState()
   {
     // return m_register ? 
@@ -128,6 +150,7 @@ private:
 };
 
 
+//------------------------------------------------------------------------
 PortSink::PortSink(PortRegister *portReg, unsigned int iobit)
   : m_PortRegister(portReg), m_iobit(iobit)
 {
@@ -140,19 +163,26 @@ void PortSink::setSinkState(char cNewSinkState)
 
   m_PortRegister->setbit(m_iobit,cNewSinkState);
 }
+void PortSink::release()
+{
+  //cout << "PortSink::release() ;" << this << endl;
+  delete this;
+}
 //------------------------------------------------------------------------
-PortRegister::PortRegister(unsigned int numIopins, unsigned int _mask)
-  : sfr_register(),
+PortRegister::PortRegister(Module *pCpu, const char *pName, const char *pDesc,
+                           unsigned int numIopins, unsigned int _mask)
+  : sfr_register(pCpu, pName, pDesc),
     PortModule(numIopins),
     mEnableMask(_mask),  
     drivingValue(0), rvDrivenValue(0,0)
 
 {
-
+  bit_mask = numIopins-1; // ugh.
 }
 
 void PortRegister::setEnableMask(unsigned int newEnableMask)
 {
+  mOutputMask = newEnableMask;
   //unsigned int maskDiff = getEnableMask() ^ newEnableMask;
   unsigned int oldEnableMask = getEnableMask();
 
@@ -242,6 +272,10 @@ void PortRegister::setbit(unsigned int bit_number, char new3State)
       rvDrivenValue.init |= (1<<bit_number);
 
     value = rvDrivenValue;
+  } else {
+    Dprintf(("PortRegister::::setbit() %s INVALID BIT bit=%d mask=0x%x\n",
+	     name().c_str(), bit_number, bit_mask));
+
   }
 
 }
@@ -251,11 +285,11 @@ unsigned int PortRegister::get()
   trace.raw(read_trace.get()  | rvDrivenValue.data);
   trace.raw(read_trace.geti() | rvDrivenValue.init);
 
-  return rvDrivenValue.data;
+  return mOutputMask & rvDrivenValue.data;
 }
 unsigned int PortRegister::get_value()
 {
-  return rvDrivenValue.data;
+  return mOutputMask & rvDrivenValue.data;
 }
 void PortRegister::putDrive(unsigned int new_value)
 {
@@ -276,15 +310,16 @@ PortModule::PortModule(unsigned int numIopins)
   iopins = new PinModule *[mNumIopins];
   for (unsigned int i=0; i<mNumIopins; i++)
     iopins[i] = &AnInvalidPinModule;
-  //iopins[i] = new PinModule(this,i);
-
 }
+
 PortModule::~PortModule()
 {
   for (unsigned int i=0; i<mNumIopins; i++)
-    delete iopins[i];
-
-  delete iopins;
+    if (iopins[i] != &AnInvalidPinModule) {
+      //cout << __FUNCTION__ << " deleting pin:"<<i<< ':' <<iopins[i] <<endl;
+      delete iopins[i];
+    }
+  delete [] iopins;
 }
 
 PinModule &PortModule::operator [] (unsigned int iPinNumber)
@@ -310,8 +345,10 @@ PinModule * PortModule::getIOpins(unsigned int iPinNumber)
 void PortModule::updatePort()
 {
   for (unsigned int i=0; i<mNumIopins; i++) 
-    if (iopins[i])
+  {
+    if (iopins[i] != &AnInvalidPinModule)
       iopins[i]->updatePinModule();
+  }
 }
 void PortModule::updateUI()
 {
@@ -324,6 +361,13 @@ void PortModule::updatePin(unsigned int iPinNumber)
     iopins[iPinNumber]->updatePinModule();
 }
 
+void PortModule::updatePins(unsigned int iPinBitMask)
+{
+  for (unsigned int i=0,j=1; i<mNumIopins; i++, j<<=1)
+    if (j&iPinBitMask && iopins[i])
+      iopins[i]->updatePinModule();
+}
+
 SignalSink *PortModule::addSink(SignalSink *new_sink, unsigned int iPinNumber)
 {
   if (iPinNumber < mNumIopins)
@@ -337,7 +381,6 @@ IOPIN *PortModule::addPin(IOPIN *new_pin, unsigned int iPinNumber)
     // If there is not a PinModule for this pin, then add one.
     if (iopins[iPinNumber] == &AnInvalidPinModule)
       iopins[iPinNumber] = new PinModule(this,iPinNumber);
-
     iopins[iPinNumber]->setPin(new_pin);
   }
   return new_pin;
@@ -369,6 +412,8 @@ PinModule::PinModule()
     m_defaultPullupControl(0), m_activePullupControl(0),
     m_pin(0), m_port(0), m_pinNumber(0)
 {
+    for(int i = 0; i < ANALOG_TABLE_SIZE; i++)
+	m_analog_reg[i] = NULL;
 
 }
 
@@ -378,11 +423,35 @@ PinModule::PinModule(PortModule *_port, unsigned int _pinNumber, IOPIN *_pin)
     m_cLastSourceState('?'), m_cLastPullupControlState('?'),
     m_defaultSource(0), m_activeSource(0),
     m_defaultControl(0), m_activeControl(0),
-    m_defaultPullupControl(0), m_activePullupControl(0),
+    m_defaultPullupControl(0), m_activePullupControl(0), 
     m_pin(_pin), m_port(_port), m_pinNumber(_pinNumber),
     m_bForcedUpdate(false)
 {
   setPin(m_pin);
+  for(int i = 0; i < 3; i++)
+    m_analog_reg[i] = NULL;
+}
+
+PinModule::~PinModule()
+{
+  if (m_activeSource && (m_activeSource != m_defaultSource)) {
+    //cout << __FUNCTION__ << " deleting active source:"<<m_activeSource<<endl;
+    //cout << "    state:" <<m_activeSource->getState() <<endl;
+    //m_activeSource->release();
+  }
+  if (m_defaultSource)
+    m_defaultSource->release();
+
+  if (m_activeControl && (m_activeControl != m_defaultControl))
+    m_activeControl->release();
+  if (m_defaultControl)
+    m_defaultControl->release();
+
+  if (m_activePullupControl && (m_activePullupControl != m_defaultPullupControl))
+    m_activePullupControl->release();
+  if (m_defaultPullupControl)
+    m_defaultPullupControl->release();
+
 }
 
 void PinModule::setPin(IOPIN *new_pin)
@@ -413,11 +482,14 @@ void PinModule::updatePinModule()
 	    m_cLastControlState,m_cLastSourceState,m_cLastPullupControlState));
 
   char cCurrentControlState = getControlState();
+  unsigned int old_dir = m_pin->get_direction();
+  unsigned int new_dir = (cCurrentControlState=='1') ? IOPIN::DIR_INPUT : 
+		IOPIN::DIR_OUTPUT;
 
-  if (cCurrentControlState != m_cLastControlState) {
+
+  if (new_dir != old_dir) {
     m_cLastControlState = cCurrentControlState;
-    m_pin->update_direction((cCurrentControlState=='1') ? IOPIN::DIR_INPUT : IOPIN::DIR_OUTPUT,
-			    false);
+    m_pin->update_direction(new_dir, false);
     bStateChange = true;
   }
 
@@ -458,7 +530,7 @@ void PinModule::setDefaultControl(SignalControl *newDefaultControl)
     setControl(m_defaultControl);
   }
   else
-	delete newDefaultControl;
+    delete newDefaultControl;   //// YIKES!!! -- wouldn't it be better to return an error code?
 }
 void PinModule::setControl(SignalControl *newControl)
 {
@@ -535,6 +607,75 @@ void PinModule::updateUI()
   m_port->updateUI();
 }
 
+//	AnalogReq is called by modules such as ADC and Comparator
+//	to set or release a pin to/from analog mode. When a pin is in
+//	analog mode the TRIS register is still active and output pins
+//	are still driven high or low, but reads of the port register 
+//	return 0 for the pin. When a pin mode is changes the breadboard
+//	name of the pin is changed to newname.
+//
+//	A table of each calling module is kept as a module may 
+//	request analog mode after another has. The pin is put in
+//	analog mode when the first module requests it (up=true) 
+//	and is taken out of analog mode when all modules have 
+//	requested analog mode to be released (up=false);
+//	
+void PinModule::AnalogReq(Register * reg, bool analog, const char *newname)
+{
+    int i, index;
+    unsigned int total_cnt = 0;
+
+    if (!m_port) return;
+
+    // is the calling register in the table and what is the current
+    // count of modules requesting analog mode    
+
+    for(i=0, index=-1; i < ANALOG_TABLE_SIZE && m_analog_reg[i]; i++)
+    {
+	if (m_analog_reg[i] == reg)
+	    index = i;
+        if (m_analog_active[i])
+	    total_cnt++;
+    }
+
+    // Register is not in table so add it.
+    //
+    if (index < 0)
+    {
+	assert(i < ANALOG_TABLE_SIZE); // table not large enough
+	index = i;
+	m_analog_reg[index] = reg;
+	m_analog_active[index] = false;
+    }
+
+    if (analog)	// Set pin to analog mode request
+    {
+	m_analog_active[index] = true;
+	if (total_cnt == 0)
+	{
+	    unsigned int mask = m_port->getOutputMask();
+ 	    mask &= ~(1 << getPinNumber());
+	    m_port->setOutputMask(mask);
+	    Dprintf(("PinModule::UpAnalogCnt up %s  newname=%s mask=%x\n", getPin().name().c_str(), newname, mask));
+	    getPin().newGUIname(newname);
+	    getPin().set_is_analog(true);
+ 	}
+    }
+    else if (!analog && m_analog_active[index])  // release register request 
+						 // for analog pin
+    {
+	m_analog_active[index] = false;
+ 	if (total_cnt == 1)
+        {
+	    unsigned int mask = m_port->getOutputMask();
+	    mask |= (1 << getPinNumber());
+	    Dprintf(("PinModule::UpAnalogCnt down %s  newname=%s mask=%x\n", getPin().name().c_str(), newname, mask));
+	    m_port->setOutputMask(mask);
+	    getPin().newGUIname(newname);
+	    getPin().set_is_analog(false);
+	}
+    }
+}
 
 
 // The IOPORT class is deprecated.
