@@ -108,24 +108,16 @@ class PinModule;
 /// via SignalControl. The control is usually used to control the I/O pin 
 /// direction (i.e. whether it's an input or output...), drive value, 
 /// pullup state, etc.
-
+/// When a Pin Module is through with the SignalControl, it will call
+/// the release() method. This is primarily used to delete the SignalControl
+/// objects.
 
 class SignalControl
 {
 public:
+  virtual ~SignalControl();  //// fixme
   virtual char getState()=0;
-};
-
-///------------------------------------------------------------
-/// SinkRecipient
-/// Interface class that redirects a driven I/O pin change
-/// to the appropriate peripheral register that wants the
-/// change.
-
-class SinkRecipient
-{
-public:
-  virtual void setState(const char)=0;
+  virtual void release()=0;
 };
 
 ///------------------------------------------------------------
@@ -136,6 +128,8 @@ class PeripheralSignalSource : public SignalControl
 {
 public:
   PeripheralSignalSource(PinModule *_pin);
+  virtual ~PeripheralSignalSource();
+  virtual void release();
 
   /// getState - called by the PinModule to determine the source state
   virtual char getState();
@@ -172,6 +166,10 @@ public:
 
   virtual void updatePin(unsigned int iPinNumber);
 
+  /// updatePins -- Update several I/O pins
+
+  virtual void updatePins(unsigned int iPinBitMask);
+
   /// updateUI -- convey pin state info to a User Interface (e.g. the gui).
 
   virtual void   updateUI();
@@ -219,8 +217,15 @@ public:
 
   PinModule * getIOpins(unsigned int pin_number);
 
+  // set/get OutputMask which controls bits returned on I/O
+  // port register get() call. Used to return 0 for  analog pins
+  virtual void setOutputMask (unsigned int OutputMask) 
+	{ mOutputMask = OutputMask;}
+  virtual unsigned int getOutputMask () 
+	{ return(mOutputMask);}
 protected:
   unsigned int mNumIopins;
+  unsigned int  mOutputMask;
 
 private:
 
@@ -239,16 +244,17 @@ private:
 /// The parent class 'PinMonitor', allows the PinModule to be 
 /// registered with the I/O pin. In other words, when the I/O pin 
 /// changes state, the PinModule will be notified.
+#define ANALOG_TABLE_SIZE 3
 
 class PinModule : public PinMonitor
 {
 public:
   PinModule();
   PinModule(PortModule *, unsigned int _pinNumber, IOPIN *new_pin=0);
-  virtual ~PinModule() {}
+  virtual ~PinModule();
 
   /// updatePinModule -- The low level I/O pin state is resolved here 
-  /// by examined the direction and state of the I/O pin. 
+  /// by examining the direction and state of the I/O pin. 
 
   void updatePinModule();
 
@@ -259,6 +265,7 @@ public:
   void refreshPinOnUpdate(bool bForcedUpdate);
 
   void setPin(IOPIN *);
+  void clrPin() { m_pin = NULL; } 
   void setDefaultSource(SignalControl *);
   void setSource(SignalControl *);
   void setDefaultControl(SignalControl *);
@@ -270,6 +277,7 @@ public:
   char getSourceState();
   char getPullupControlState();
   unsigned int getPinNumber() { return m_pinNumber;}
+  void AnalogReq(Register *reg, bool analog, const char *newName);
 
   IOPIN &getPin() { return *m_pin;}
 
@@ -296,6 +304,8 @@ private:
   PortModule   *m_port;
   unsigned int  m_pinNumber;
   bool          m_bForcedUpdate;
+  Register     *m_analog_reg[ANALOG_TABLE_SIZE];	
+  bool		m_analog_active[ANALOG_TABLE_SIZE];
 };
 
 
@@ -304,7 +314,8 @@ private:
 class PortRegister : public sfr_register, public PortModule
 {
 public:
-  PortRegister(unsigned int numIopins, unsigned int enableMask);
+  PortRegister(Module *pCpu, const char *pName, const char *pDesc,
+               unsigned int numIopins, unsigned int enableMask);
 
   virtual void put(unsigned int new_value);
   virtual void put_value(unsigned int new_value);
@@ -331,7 +342,13 @@ class PortSink : public SignalSink
 {
 public:
   PortSink(PortRegister *portReg, unsigned int iobit);
+
+  virtual ~PortSink()
+  {
+  }
+
   virtual void setSinkState(char);
+  virtual void release();
 private:
   PortRegister *m_PortRegister;
   unsigned int  m_iobit;
