@@ -1,10 +1,12 @@
 #include <iostream>
+#include <cstdio>
 
 #include "operator.h"
 #include "errors.h"
 #include "symbol.h"
 #include "processor.h"
 
+#include <typeinfo>
 
 static bool isFloat(Value *v)
 {
@@ -763,6 +765,7 @@ Value* OpAddressOf::applyOp(Value* operand)
 {
   Value* rVal=0;
 
+  /*
   register_symbol *pReg =dynamic_cast<register_symbol*>(operand);
   if (pReg) {
     rVal = new Integer(pReg->getAddress());
@@ -776,6 +779,9 @@ Value* OpAddressOf::applyOp(Value* operand)
       }
     }
   }
+  */
+  Register *pReg = dynamic_cast<Register*>(operand);
+  rVal = pReg ? new Integer(pReg->getAddress()) : 0;
   if(rVal==0) {
     throw new TypeMismatch(showOp(), operand->showType());
   }
