@@ -62,7 +62,12 @@ using namespace std;
 #define FOLDERDELIMITERALTERNATIVE '/'
 #define PATHDELIMITER ";"
 #else
+#ifdef __APPLE__
+#define MODULE_EXT ".dylib"
+#else
 #define MODULE_EXT ".so"
+#endif
+#define MODULE_VER ".0"
 #define FOLDERDELIMITER '/'
 #define FOLDERDELIMITERALTERNATIVE '\\'
 #define PATHDELIMITER ":"
@@ -106,9 +111,9 @@ bool LIBGPSIM_EXPORT IsFileExtension(const char *pszFile, const char *pFileExt)
 {
   string s(pszFile);
 
-  int i = s.rfind(pFileExt) ;
+  string::size_type i = s.rfind('.') ;
   
-  return i>=0;
+  return (i != string::npos) && (s.substr(i+1)==pFileExt);
 
 }
 
@@ -250,9 +255,6 @@ static void * sLoad(const char *library_name)
 
 void FixupLibraryName(string &sPath) {
   translatePath(sPath);
-  if (STRICMP(&sPath[sPath.size() - (sizeof(MODULE_EXT)-1)], MODULE_EXT) != 0) {
-    sPath.append(MODULE_EXT);
-  }
 }
 
 void GetFileName(string &sPath, string &sName) {
@@ -317,34 +319,56 @@ void * load_library(const char *library_name, const char **pszError)
   FixupLibraryName(sPath);
   asDllSearchPath.AddPathFromFilePath(sPath, sFile);
 
-  // First, see if we can load the library from where ever the 
-  // system thinks libraries are located.
-  if( (handle = sLoad(sPath.c_str())) != 0)
-    return handle;
 
-  *pszError = get_error_message();
-  unsigned long uError = get_error(*pszError);
+  // loop twice: first time take the file name as is,
+  // the second time append the os-dependent library extension.
+  for (int i=0; i<2; i++) {
 
-  if (uError == OS_E_FILENOTFOUND) {
-    // Failed to find the library in the system paths, so try to load
-    // from one of our paths.
+    // First, see if we can load the library from where ever the 
+    // system thinks libraries are located.
+    if( (handle = sLoad(sPath.c_str())) != 0)
+      return handle;
 
-    free_error_message(*pszError);
+    *pszError = get_error_message();
+    unsigned long uError = get_error(*pszError);
 
-    CFileSearchPath::iterator itSearchPath;
-    for (itSearchPath = asDllSearchPath.begin();
-        itSearchPath != asDllSearchPath.end();
-        itSearchPath++) {
-      sPath = *itSearchPath + sFile;
+    if (uError == OS_E_FILENOTFOUND) {
+      // Failed to find the library in the system paths, so try to load
+      // from one of our paths.
 
-      handle = sLoad(sPath.c_str());
+      free_error_message(*pszError);
 
-      if (NULL != handle) {
-        return handle;
+      CFileSearchPath::iterator itSearchPath;
+      for (itSearchPath = asDllSearchPath.begin();
+	   itSearchPath != asDllSearchPath.end();
+	   itSearchPath++) {
+	sPath = *itSearchPath + sFile;
+
+	handle = sLoad(sPath.c_str());
+
+	if (NULL != handle) {
+	  return handle;
+	}
+	*pszError = get_error_message();
       }
-      *pszError = get_error_message();
     }
+    // Append the module extension and try again.
+    string::size_type nPos = sFile.find(MODULE_EXT,0);
+    if( nPos == string::npos)
+    {
+        sFile.append(MODULE_EXT);
+    }
+#ifndef _WIN32
+    else if( sFile.find(MODULE_VER, nPos) == string::npos)
+    {
+	i--;
+	sFile.append(MODULE_VER);
+    }
+#endif 
+    sPath = sFile;
   }
+
+  // Should there be a free?
   if (*pszError)
       printf("Failed loading %s: %s\n",
             sPath.c_str(), *pszError);
