@@ -42,10 +42,26 @@ Boston, MA 02111-1307, USA.  */
 #include "symbol.h"
 
 P16F62x::P16F62x(const char *_name, const char *desc)
-  : P16X6X_processor(_name,desc)
+  : P16X6X_processor(_name,desc),
+    usart(this),
+    comparator(this)
 {
 }
 
+P16F62x::~P16F62x()
+{
+
+  delete_file_registers(0xc0, 0xef);
+  delete_file_registers(0x120,0x14f);
+
+  delete_sfr_register(usart.txreg);
+  delete_sfr_register(usart.rcreg);
+
+  delete eeprom;
+  eeprom = 0;
+
+}
+
 void P16F62x::create_iopin_map()
 {
   package = new Package(18);
@@ -78,18 +94,14 @@ void P16F62x::create_iopin_map()
 
 void P16F62x::create_sfr_map()
 {
- 
+
 
   add_file_registers(0xc0, 0xef, 0);   // 0xa0 - 0xbf are created in the P16X6X_processor class
   add_file_registers(0x120,0x14f,0);
   alias_file_registers(0x70,0x7f,0x80);
   alias_file_registers(0x70,0x7f,0x100);
   alias_file_registers(0x70,0x7f,0x180);
-
-
-  add_sfr_register(indf,   0x180);
-  add_sfr_register(indf,   0x100);
-
+  alias_file_registers(0x0,0x0,0x180);
   alias_file_registers(0x01,0x04,0x100);
   alias_file_registers(0x81,0x84,0x100);
 
@@ -98,8 +110,8 @@ void P16F62x::create_sfr_map()
 
   add_sfr_register(m_portb, 0x06);
   add_sfr_register(m_trisb, 0x86, RegisterValue(0xff,0));
-  add_sfr_register(m_portb, 0x106);
-  add_sfr_register(m_trisb, 0x186, RegisterValue(0xff,0));
+  alias_file_registers(0x06,0x06,0x100);
+  alias_file_registers(0x86,0x86,0x100);
 
 
   add_sfr_register(get_eeprom()->get_reg_eedata(),  0x9a);
@@ -107,16 +119,15 @@ void P16F62x::create_sfr_map()
   add_sfr_register(get_eeprom()->get_reg_eecon1(),  0x9c, RegisterValue(0,0));
   add_sfr_register(get_eeprom()->get_reg_eecon2(),  0x9d);
 
-  add_sfr_register(pclath, 0x18a, RegisterValue(0,0));
-  add_sfr_register(pclath, 0x10a, RegisterValue(0,0));
+  // PCLATH
+  alias_file_registers(0x0a,0x0a,0x180);
 
-  add_sfr_register(&intcon_reg, 0x18b, RegisterValue(0,0));
-  add_sfr_register(&intcon_reg, 0x10b, RegisterValue(0,0));
-  add_sfr_register(&intcon_reg, 0x08b, RegisterValue(0,0));
   add_sfr_register(&intcon_reg, 0x00b, RegisterValue(0,0));
+  alias_file_registers(0x0b,0x0b,0x180);
 
   usart.initialize(get_pir_set(),&(*m_portb)[2], &(*m_portb)[1],
-		   new _TXREG(&usart), new _RCREG(&usart));
+                   new _TXREG(this,"txreg", "USART Transmit Register", &usart),
+                   new _RCREG(this,"rcreg", "USART Receiver Register", &usart));
 
   add_sfr_register(&usart.rcsta, 0x18, RegisterValue(0,0),"rcsta");
   add_sfr_register(&usart.txsta, 0x98, RegisterValue(2,0),"txsta");
@@ -128,35 +139,35 @@ void P16F62x::create_sfr_map()
   intcon_reg.set_pir_set(get_pir_set());
 
   // Link the comparator and voltage ref to porta
-  comparator.initialize(get_pir_set(), &(*m_porta)[2], &(*m_porta)[0], 
-	&(*m_porta)[1], &(*m_porta)[2], &(*m_porta)[3], &(*m_porta)[3],
-	&(*m_porta)[4]);
+  comparator.initialize(get_pir_set(), &(*m_porta)[2], &(*m_porta)[0],
+        &(*m_porta)[1], &(*m_porta)[2], &(*m_porta)[3], &(*m_porta)[3],
+        &(*m_porta)[4]);
 
-  comparator.cmcon->set_configuration(1, 0, AN0, AN3, AN0, AN3, ZERO);
-  comparator.cmcon->set_configuration(2, 0, AN1, AN2, AN1, AN2, ZERO);
-  comparator.cmcon->set_configuration(1, 1, AN0, AN2, AN3, AN2, NO_OUT);
-  comparator.cmcon->set_configuration(2, 1, AN1, AN2, AN1, AN2, NO_OUT);
-  comparator.cmcon->set_configuration(1, 2, AN0, VREF, AN3, VREF, NO_OUT);
-  comparator.cmcon->set_configuration(2, 2, AN1, VREF, AN2, VREF, NO_OUT);
-  comparator.cmcon->set_configuration(1, 3, AN0, AN2, AN0, AN2, NO_OUT);
-  comparator.cmcon->set_configuration(2, 3, AN1, AN2, AN1, AN3, NO_OUT);
-  comparator.cmcon->set_configuration(1, 4, AN0, AN3, AN0, AN3, NO_OUT);
-  comparator.cmcon->set_configuration(2, 4, AN1, AN2, AN1, AN2, NO_OUT);
-  comparator.cmcon->set_configuration(1, 5, NO_IN, NO_IN, NO_IN, NO_IN, ZERO);
-  comparator.cmcon->set_configuration(2, 5, AN1, AN2, AN1, AN2, NO_OUT);
-  comparator.cmcon->set_configuration(1, 6, AN0, AN2, AN0, AN2, OUT0);
-  comparator.cmcon->set_configuration(2, 6, AN1, AN2, AN1, AN2, OUT1);
-  comparator.cmcon->set_configuration(1, 7, NO_IN, NO_IN, NO_IN, NO_IN, ZERO);
-  comparator.cmcon->set_configuration(2, 7, NO_IN, NO_IN, NO_IN, NO_IN, ZERO);
-  
+  comparator.cmcon.set_configuration(1, 0, AN0, AN3, AN0, AN3, ZERO);
+  comparator.cmcon.set_configuration(2, 0, AN1, AN2, AN1, AN2, ZERO);
+  comparator.cmcon.set_configuration(1, 1, AN0, AN2, AN3, AN2, NO_OUT);
+  comparator.cmcon.set_configuration(2, 1, AN1, AN2, AN1, AN2, NO_OUT);
+  comparator.cmcon.set_configuration(1, 2, AN0, VREF, AN3, VREF, NO_OUT);
+  comparator.cmcon.set_configuration(2, 2, AN1, VREF, AN2, VREF, NO_OUT);
+  comparator.cmcon.set_configuration(1, 3, AN0, AN2, AN0, AN2, NO_OUT);
+  comparator.cmcon.set_configuration(2, 3, AN1, AN2, AN1, AN3, NO_OUT);
+  comparator.cmcon.set_configuration(1, 4, AN0, AN3, AN0, AN3, NO_OUT);
+  comparator.cmcon.set_configuration(2, 4, AN1, AN2, AN1, AN2, NO_OUT);
+  comparator.cmcon.set_configuration(1, 5, NO_IN, NO_IN, NO_IN, NO_IN, ZERO);
+  comparator.cmcon.set_configuration(2, 5, AN1, AN2, AN1, AN2, NO_OUT);
+  comparator.cmcon.set_configuration(1, 6, AN0, AN2, AN0, AN2, OUT0);
+  comparator.cmcon.set_configuration(2, 6, AN1, AN2, AN1, AN2, OUT1);
+  comparator.cmcon.set_configuration(1, 7, NO_IN, NO_IN, NO_IN, NO_IN, ZERO);
+  comparator.cmcon.set_configuration(2, 7, NO_IN, NO_IN, NO_IN, NO_IN, ZERO);
 
-  add_sfr_register(comparator.cmcon, 0x1f, RegisterValue(0,0),"cmcon");
+
+  add_sfr_register(&comparator.cmcon, 0x1f, RegisterValue(0,0),"cmcon");
   add_sfr_register(&comparator.vrcon, 0x9f, RegisterValue(0,0),"vrcon");
 
-  comparator.cmcon->put(0);
+  comparator.cmcon.put(0);
 
-  // Link ccp1 and portb
-  //1((PORTB_62x*)portb)->ccp1con = &ccp1con;
+  // Link ccp1 onto portb
+  ccp1con.setIOpin(&((*m_portb)[3]));
 }
 
 void P16F62x::create_symbols()
@@ -196,20 +207,21 @@ bool P16F62x::set_config_word(unsigned int address, unsigned int cfg_word)
 
     unsigned int valid_pins = m_porta->getEnableMask();
 
+    set_int_osc(false);
     // Careful these bits not adjacent
     switch(cfg_word & (CFG_FOSC0 | CFG_FOSC1 | CFG_FOSC2)) {
 
     case 0:  // LP oscillator: low power crystal is on RA6 and RA7
     case 1:     // XT oscillator: crystal/resonator is on RA6 and RA7
     case 2:     // HS oscillator: crystal/resonator is on RA6 and RA7
-	(m_porta->getPin(6))->newGUIname("OSC2");
-	(m_porta->getPin(7))->newGUIname("OSC1");
-	break;
+        (m_porta->getPin(6))->newGUIname("OSC2");
+        (m_porta->getPin(7))->newGUIname("OSC1");
+        break;
 
-    case 0x13:  // ER oscillator: RA6 is CLKOUT, resistor (?) on RA7 
-	(m_porta->getPin(6))->newGUIname("CLKOUT");
-	(m_porta->getPin(7))->newGUIname("OSC1");
-	break;
+    case 0x13:  // ER oscillator: RA6 is CLKOUT, resistor (?) on RA7
+        (m_porta->getPin(6))->newGUIname("CLKOUT");
+        (m_porta->getPin(7))->newGUIname("OSC1");
+        break;
 
     case 3:     // EC:  RA6 is an I/O, RA7 is a CLKIN
     case 0x12:  // ER oscillator: RA6 is an I/O, RA7 is a CLKIN
@@ -218,27 +230,35 @@ bool P16F62x::set_config_word(unsigned int address, unsigned int cfg_word)
         break;
 
     case 0x10:  // INTRC: Internal Oscillator, RA6 and RA7 are I/O's
+	set_int_osc(true);
+        (m_porta->getPin(6))->newGUIname("porta6");
+        (m_porta->getPin(7))->newGUIname("porta7");
         valid_pins |= 0xc0;
         break;
 
     case 0x11:  // INTRC: Internal Oscillator, RA7 is an I/O, RA6 is CLKOUT
+	set_int_osc(true);
+	(m_porta->getPin(6))->newGUIname("CLKOUT");
+        (m_porta->getPin(7))->newGUIname("porta7");
+
         valid_pins = (valid_pins & 0xbf)|0x80;
         break;
 
     }
 
-    // If the /MCLRE bit is set then RA5 is the MCLR pin, otherwise it's 
+    // If the /MCLRE bit is set then RA5 is the MCLR pin, otherwise it's
     // a general purpose I/O pin.
 
-    if (! (cfg_word & CFG_MCLRE)) {   
-      valid_pins |= ( 1<< 5); 		// porta5 IO port
+    if (! (cfg_word & CFG_MCLRE)) {
+      unassignMCLRPin();
+      valid_pins |= ( 1<< 5);           // porta5 IO port
     }
     else
     {
-	(m_porta->getPin(5))->newGUIname("MCLR");
+	assignMCLRPin(4); 	// pin 4
     }
 
-    //cout << " porta valid_iopins " << porta->valid_iopins << 
+    //cout << " porta valid_iopins " << porta->valid_iopins <<
     //   "  tris valid io " << trisa.valid_iopins << '\n';
 
     if (valid_pins != m_porta->getEnableMask()) // enable new pins for IO
@@ -261,8 +281,7 @@ void  P16F62x::create(int ram_top, unsigned int eeprom_size)
 
   _14bit_processor::create();
 
-  e = new EEPROM_PIR(pir1);
-  e->set_cpu(this);
+  e = new EEPROM_PIR(this,pir1);
   e->initialize(eeprom_size);
   //e->set_pir_set(get_pir_set());
   e->set_intcon(&intcon_reg);
@@ -284,7 +303,7 @@ void  P16F62x::create(int ram_top, unsigned int eeprom_size)
 
 //========================================================================
 //
-// Pic 16F627 
+// Pic 16F627
 //
 
 Processor * P16F627::construct(const char *name)
@@ -295,8 +314,6 @@ Processor * P16F627::construct(const char *name)
   p->P16F62x::create(0x2f, 128);
   p->create_invalid_registers ();
   p->create_symbols();
-  symbol_table.add_module(p,p->name().c_str());
-
   return p;
 
 }
@@ -311,7 +328,7 @@ P16F627::P16F627(const char *_name, const char *desc)
 
 //========================================================================
 //
-// Pic 16F628 
+// Pic 16F628
 //
 
 Processor * P16F628::construct(const char *name)
@@ -322,8 +339,6 @@ Processor * P16F628::construct(const char *name)
   p->P16F62x::create(0x2f, 128);
   p->create_invalid_registers ();
   p->create_symbols();
-  symbol_table.add_module(p,p->name().c_str());
-
   return p;
 
 }
@@ -336,10 +351,16 @@ P16F628::P16F628(const char *_name, const char *desc)
 
 }
 
+P16F628::~P16F628()
+{
+  if (verbose)
+      cout << "'628 destructor\n";
+}
+
 
 //========================================================================
 //
-// Pic 16F648 
+// Pic 16F648
 //
 
 Processor * P16F648::construct(const char *name)
@@ -351,8 +372,6 @@ Processor * P16F648::construct(const char *name)
   p->create_sfr_map();
   p->create_invalid_registers ();
   p->create_symbols();
-  symbol_table.add_module(p,p->name().c_str());
-
   return p;
 
 }
@@ -367,6 +386,6 @@ P16F648::P16F648(const char *_name, const char *desc)
 
 void P16F648::create_sfr_map()
 {
- 
+
   add_file_registers(0x150,0x16f,0);
 }
