@@ -62,10 +62,11 @@ void P16F871::create_sfr_map()
   add_sfr_register(&pie2,   0x8d, RegisterValue(0,0));
 
   // Parent classes just set PIR1
-  pir_set_2_def.set_pir2(&pir2_2_reg);
+  pir_set_2_def.set_pir2(pir2_2_reg);
 
   usart.initialize(get_pir_set(),&(*m_portc)[6], &(*m_portc)[7],
-		   new _TXREG(&usart), new _RCREG(&usart));
+		   new _TXREG(this,"txreg", "USART Transmit Register", &usart), 
+                   new _RCREG(this,"rcreg", "USART Receiver Register", &usart));
 
   add_sfr_register(&usart.rcsta, 0x18, RegisterValue(0,0),"rcsta");
   add_sfr_register(&usart.txsta, 0x98, RegisterValue(2,0),"txsta");
@@ -80,8 +81,7 @@ void P16F871::create_sfr_map()
     pir2->set_pie(&pie2);
   }
 
-  pie2.pir    = get_pir2();
-  pie2.new_name("pie2");
+  pie2.setPir(get_pir2());
 
   add_sfr_register(get_eeprom()->get_reg_eedata(),  0x10c);
   add_sfr_register(get_eeprom()->get_reg_eecon1(),  0x18c, RegisterValue(0,0));
@@ -96,45 +96,42 @@ void P16F871::create_sfr_map()
   add_sfr_register(get_eeprom()->get_reg_eeadrh(),  0x10f);
 
 
-  alias_file_registers(0x80,0x80,0x80);
-  alias_file_registers(0x01,0x01,0x100);
-  alias_file_registers(0x82,0x84,0x80);
+  alias_file_registers(0x70,0x7f,0x80);
+  alias_file_registers(0x70,0x7f,0x100);
+  alias_file_registers(0x70,0x7f,0x180);
+
+  alias_file_registers(0x00,0x04,0x100);
+  alias_file_registers(0x80,0x84,0x100);
+
   alias_file_registers(0x06,0x06,0x100);
-  alias_file_registers(0x8a,0x8b,0x80);
-  alias_file_registers(0x100,0x100,0x80);
-  alias_file_registers(0x81,0x81,0x100);
-  alias_file_registers(0x102,0x104,0x80);
   alias_file_registers(0x86,0x86,0x100);
-  alias_file_registers(0x10a,0x10b,0x80);
+
+  alias_file_registers(0x0a,0x0b,0x080);
+  alias_file_registers(0x0a,0x0b,0x100);
+  alias_file_registers(0x0a,0x0b,0x180);
 
 
   alias_file_registers(0x20,0x7f,0x100);
   alias_file_registers(0xa0,0xbf,0x100);
-  alias_file_registers(0x70,0x7f,0x80);
-  alias_file_registers(0xf0,0xff,0x100);
 
 
-  // The rest of the A/D definition in 16C74
+
   add_sfr_register(&adcon0, 0x1f, RegisterValue(0,0));
   add_sfr_register(&adcon1, 0x9f, RegisterValue(0,0));
-
   add_sfr_register(&adres,  0x1e, RegisterValue(0,0));
   add_sfr_register(&adresl, 0x9e, RegisterValue(0,0));
 
+
+
   //1adcon0.analog_port = porta;
   //1adcon0.analog_port2 = porte;
 
   adcon0.setAdres(&adres);
   adcon0.setAdcon1(&adcon1);
   adcon0.setIntcon(&intcon_reg);
-  adcon0.pir_set = &pir_set_2_def;
+  adcon0.setPir(pir1);
   adcon0.setChannel_Mask(7);
 
-  adcon0.new_name("adcon0");
-  adcon1.new_name("adcon1");
-  adres.new_name("adresh");
-  adresl.new_name("adresl");
-
   adcon0.setAdresLow(&adresl);
 
   adcon0.setA2DBits(10);
@@ -196,10 +193,8 @@ void P16F871::create()
   P16C64::create();
 
   EEPROM_WIDE *e;
-  e = new EEPROM_WIDE(pir2);
-  e->set_cpu(this);
+  e = new EEPROM_WIDE(this,pir2);
   e->initialize(64);
-  //e->set_pir_set(get_pir_set());
   e->set_intcon(&intcon_reg);
   set_eeprom_wide(e);
 
@@ -224,9 +219,6 @@ Processor * P16F871::construct(const char *name)
   p->create();
   p->create_invalid_registers ();
   p->create_symbols();
-
-  symbol_table.add_module(p,p->name().c_str());
-
   return p;
 
 }
@@ -244,13 +236,18 @@ void P16F871::create_symbols()
 //========================================================================
 P16F871::P16F871(const char *_name, const char *desc)
   : P16C64(_name,desc) ,
-    pir2_2_reg(&intcon_reg,&pie2)
+    adcon0(this,"adcon0", "A2D Control 0"),
+    adcon1(this,"adcon1", "A2D Control 1"),
+    adres(this,"adres", "A2D Result"),
+    adresl(this,"adresl", "A2D Result Low"),
+    usart(this)
 {
   if(verbose)
     cout << "f871 constructor, type = " << isa() << '\n';
 
-  pir2 = &pir2_2_reg;
-
+  //pir2 = &pir2_2_reg;
+  pir2_2_reg = new PIR2v2(this,"pir2","Peripheral Interrupt Register",&intcon_reg,&pie2);
+  pir2 = pir2_2_reg;
 }
 
 
@@ -310,9 +307,6 @@ void P16F873::create_sfr_map()
 
   // The rest of the A/D definition in 16C73
   add_sfr_register(&adresl,  0x9e, RegisterValue(0,0));
-  adres.new_name("adresh");
-  adresl.new_name("adresl");
-
   adcon0.setAdresLow(&adresl);
   adcon0.setA2DBits(10);
   adcon1.setValidCfgBits(ADCON1::PCFG0 | ADCON1::PCFG1 | 
@@ -374,10 +368,8 @@ void P16F873::create()
   P16C73::create();
 
   EEPROM_WIDE *e;
-  e = new EEPROM_WIDE(pir2);
-  e->set_cpu(this);
+  e = new EEPROM_WIDE(this,pir2);
   e->initialize(128);
-  //e->set_pir_set(get_pir_set());
   e->set_intcon(&intcon_reg);
   set_eeprom_wide(e);
 
@@ -404,9 +396,6 @@ Processor * P16F873::construct(const char *name)
   p->create();
   p->create_invalid_registers ();
   p->create_symbols();
-  symbol_table.add_module(p,p->name().c_str());
-
-
   return p;
 
 }
@@ -422,7 +411,8 @@ void P16F873::create_symbols()
 }
 
 P16F873::P16F873(const char *_name, const char *desc)
-  : P16C73(_name,desc)
+  : P16C73(_name,desc),
+    adresl(this,"adresl", "A2D Result Low")
 {
   if(verbose)
     cout << "f873 constructor, type = " << isa() << '\n';
@@ -453,25 +443,25 @@ void P16F873A::create_sfr_map()
         &(*m_porta)[1], &(*m_porta)[2], 
 	&(*m_porta)[3], &(*m_porta)[4], &(*m_porta)[5]);
 
-  comparator.cmcon->set_configuration(1, 0, AN0, AN3, AN0, AN3, ZERO);
-  comparator.cmcon->set_configuration(2, 0, AN1, AN2, AN1, AN2, ZERO);
-  comparator.cmcon->set_configuration(1, 1, AN0, AN3, AN0, AN3, OUT0);
-  comparator.cmcon->set_configuration(2, 1, NO_IN, NO_IN, NO_IN, NO_IN, ZERO);
-  comparator.cmcon->set_configuration(1, 2, AN0, AN3, AN0, AN3, NO_OUT);
-  comparator.cmcon->set_configuration(2, 2, AN1, AN2, AN1, AN2, NO_OUT);
-  comparator.cmcon->set_configuration(1, 3, AN0, AN3, AN0, AN3, OUT0);
-  comparator.cmcon->set_configuration(2, 3, AN1, AN2, AN1, AN2, OUT1);
-  comparator.cmcon->set_configuration(1, 4, AN0, AN3, AN0, AN3, NO_OUT);
-  comparator.cmcon->set_configuration(2, 4, AN1, AN3, AN1, AN3, NO_OUT);
-  comparator.cmcon->set_configuration(1, 5, AN0, AN3, AN0, AN3, OUT0);
-  comparator.cmcon->set_configuration(2, 5, AN1, AN3, AN1, AN3, OUT1);
-  comparator.cmcon->set_configuration(1, 6, AN0, VREF, AN3, VREF, NO_OUT);
-  comparator.cmcon->set_configuration(2, 6, AN1, VREF, AN2, VREF, NO_OUT);
-  comparator.cmcon->set_configuration(1, 7, NO_IN, NO_IN, NO_IN, NO_IN, ZERO);
-  comparator.cmcon->set_configuration(2, 7, NO_IN, NO_IN, NO_IN, NO_IN, ZERO);
+  comparator.cmcon.set_configuration(1, 0, AN0, AN3, AN0, AN3, ZERO);
+  comparator.cmcon.set_configuration(2, 0, AN1, AN2, AN1, AN2, ZERO);
+  comparator.cmcon.set_configuration(1, 1, AN0, AN3, AN0, AN3, OUT0);
+  comparator.cmcon.set_configuration(2, 1, NO_IN, NO_IN, NO_IN, NO_IN, ZERO);
+  comparator.cmcon.set_configuration(1, 2, AN0, AN3, AN0, AN3, NO_OUT);
+  comparator.cmcon.set_configuration(2, 2, AN1, AN2, AN1, AN2, NO_OUT);
+  comparator.cmcon.set_configuration(1, 3, AN0, AN3, AN0, AN3, OUT0);
+  comparator.cmcon.set_configuration(2, 3, AN1, AN2, AN1, AN2, OUT1);
+  comparator.cmcon.set_configuration(1, 4, AN0, AN3, AN0, AN3, NO_OUT);
+  comparator.cmcon.set_configuration(2, 4, AN1, AN3, AN1, AN3, NO_OUT);
+  comparator.cmcon.set_configuration(1, 5, AN0, AN3, AN0, AN3, OUT0);
+  comparator.cmcon.set_configuration(2, 5, AN1, AN3, AN1, AN3, OUT1);
+  comparator.cmcon.set_configuration(1, 6, AN0, VREF, AN3, VREF, NO_OUT);
+  comparator.cmcon.set_configuration(2, 6, AN1, VREF, AN2, VREF, NO_OUT);
+  comparator.cmcon.set_configuration(1, 7, NO_IN, NO_IN, NO_IN, NO_IN, ZERO);
+  comparator.cmcon.set_configuration(2, 7, NO_IN, NO_IN, NO_IN, NO_IN, ZERO);
   
 
-  add_sfr_register(comparator.cmcon, 0x9c, RegisterValue(7,0),"cmcon");
+  add_sfr_register(&comparator.cmcon, 0x9c, RegisterValue(7,0),"cmcon");
   add_sfr_register(&comparator.vrcon, 0x9d, RegisterValue(0,0),"vrcon");
 
 }
@@ -486,13 +476,12 @@ Processor * P16F873A::construct(const char *name)
   p->create();
   p->create_invalid_registers ();
   p->create_symbols();
-  symbol_table.add_module(p,p->name().c_str());
-
   return p;
 
 }
 P16F873A::P16F873A(const char *_name, const char *desc)
-  : P16F873(_name,desc)
+  : P16F873(_name,desc),
+    comparator(this)
 {
   if(verbose)
     cout << "f873A constructor, type = " << isa() << '\n';
@@ -512,8 +501,6 @@ Processor * P16F876::construct(const char *name)
   p->create();
   p->create_invalid_registers ();
   p->create_symbols();
-  symbol_table.add_module(p,p->name().c_str());
-
   return p;
 
 }
@@ -572,8 +559,6 @@ Processor * P16F876A::construct(const char *name)
   p->create();
   p->create_invalid_registers ();
   p->create_symbols();
-  symbol_table.add_module(p,p->name().c_str());
-
   return p;
 
 }
@@ -607,7 +592,8 @@ void P16F876A::create()
 }
 
 P16F876A::P16F876A(const char *_name, const char *desc)
-  : P16F873A(_name,desc)
+  : P16F873A(_name,desc),
+    comparator(this)
 {
   if(verbose)
     cout << "f876A constructor, type = " << isa() << '\n';
@@ -663,9 +649,6 @@ void P16F874::create_sfr_map()
 
   // The rest of the A/D definition in 16C74
   add_sfr_register(&adresl,  0x9e, RegisterValue(0,0));
-  adres.new_name("adresh");
-  adresl.new_name("adresl");
-
   adcon0.setA2DBits(10);
   adcon0.setAdresLow(&adresl);
 
@@ -726,10 +709,9 @@ void P16F874::create()
   P16C74::create();
 
   EEPROM_WIDE *e;
-  e = new EEPROM_WIDE(pir2);
-  e->set_cpu(this);
+  e = new EEPROM_WIDE(this,pir2);
   e->initialize(128);
-  //e->set_pir_set(get_pir_set());
+
   e->set_intcon(&intcon_reg);
   set_eeprom_wide(e);
 
@@ -755,8 +737,6 @@ Processor * P16F874::construct(const char *name)
   p->create();
   p->create_invalid_registers ();
   p->create_symbols();
-  symbol_table.add_module(p,p->name().c_str());
-
   return p;
 
 }
@@ -771,7 +751,9 @@ void P16F874::create_symbols()
 }
 
 P16F874::P16F874(const char *_name, const char *desc)
-  : P16C74(_name,desc)
+  : P16C74(_name,desc),
+    comparator(this),
+    adresl(this,"adresl", "A2D Result Low")
 {
   if(verbose)
     cout << "f874 constructor, type = " << isa() << '\n';
@@ -803,24 +785,24 @@ void P16F874A::create_sfr_map()
         &(*m_porta)[1], &(*m_porta)[2], 
 	&(*m_porta)[3], &(*m_porta)[4], &(*m_porta)[5]);
 
-  comparator.cmcon->set_configuration(1, 0, AN0, AN3, AN0, AN3, ZERO);
-  comparator.cmcon->set_configuration(2, 0, AN1, AN2, AN1, AN2, ZERO);
-  comparator.cmcon->set_configuration(1, 1, AN0, AN3, AN0, AN3, OUT0);
-  comparator.cmcon->set_configuration(2, 1, NO_IN, NO_IN, NO_IN, NO_IN, ZERO);
-  comparator.cmcon->set_configuration(1, 2, AN0, AN3, AN0, AN3, NO_OUT);
-  comparator.cmcon->set_configuration(2, 2, AN1, AN2, AN1, AN2, NO_OUT);
-  comparator.cmcon->set_configuration(1, 3, AN0, AN3, AN0, AN3, OUT0);
-  comparator.cmcon->set_configuration(2, 3, AN1, AN2, AN1, AN2, OUT1);
-  comparator.cmcon->set_configuration(1, 4, AN0, AN3, AN0, AN3, NO_OUT);
-  comparator.cmcon->set_configuration(2, 4, AN1, AN3, AN1, AN3, NO_OUT);
-  comparator.cmcon->set_configuration(1, 5, AN0, AN3, AN0, AN3, OUT0);
-  comparator.cmcon->set_configuration(2, 5, AN1, AN3, AN1, AN3, OUT1);
-  comparator.cmcon->set_configuration(1, 6, AN0, VREF, AN3, VREF, NO_OUT);
-  comparator.cmcon->set_configuration(2, 6, AN1, VREF, AN2, VREF, NO_OUT);
-  comparator.cmcon->set_configuration(1, 7, NO_IN, NO_IN, NO_IN, NO_IN, ZERO);
-  comparator.cmcon->set_configuration(2, 7, NO_IN, NO_IN, NO_IN, NO_IN, ZERO);
+  comparator.cmcon.set_configuration(1, 0, AN0, AN3, AN0, AN3, ZERO);
+  comparator.cmcon.set_configuration(2, 0, AN1, AN2, AN1, AN2, ZERO);
+  comparator.cmcon.set_configuration(1, 1, AN0, AN3, AN0, AN3, OUT0);
+  comparator.cmcon.set_configuration(2, 1, NO_IN, NO_IN, NO_IN, NO_IN, ZERO);
+  comparator.cmcon.set_configuration(1, 2, AN0, AN3, AN0, AN3, NO_OUT);
+  comparator.cmcon.set_configuration(2, 2, AN1, AN2, AN1, AN2, NO_OUT);
+  comparator.cmcon.set_configuration(1, 3, AN0, AN3, AN0, AN3, OUT0);
+  comparator.cmcon.set_configuration(2, 3, AN1, AN2, AN1, AN2, OUT1);
+  comparator.cmcon.set_configuration(1, 4, AN0, AN3, AN0, AN3, NO_OUT);
+  comparator.cmcon.set_configuration(2, 4, AN1, AN3, AN1, AN3, NO_OUT);
+  comparator.cmcon.set_configuration(1, 5, AN0, AN3, AN0, AN3, OUT0);
+  comparator.cmcon.set_configuration(2, 5, AN1, AN3, AN1, AN3, OUT1);
+  comparator.cmcon.set_configuration(1, 6, AN0, VREF, AN3, VREF, NO_OUT);
+  comparator.cmcon.set_configuration(2, 6, AN1, VREF, AN2, VREF, NO_OUT);
+  comparator.cmcon.set_configuration(1, 7, NO_IN, NO_IN, NO_IN, NO_IN, ZERO);
+  comparator.cmcon.set_configuration(2, 7, NO_IN, NO_IN, NO_IN, NO_IN, ZERO);
 
-  add_sfr_register(comparator.cmcon, 0x9c, RegisterValue(7,0),"cmcon");
+  add_sfr_register(&comparator.cmcon, 0x9c, RegisterValue(7,0),"cmcon");
   add_sfr_register(&comparator.vrcon, 0x9d, RegisterValue(0,0),"vrcon");
 }
 
@@ -849,8 +831,6 @@ Processor * P16F874A::construct(const char *name)
   p->create();
   p->create_invalid_registers ();
   p->create_symbols();
-  symbol_table.add_module(p,p->name().c_str());
-
   return p;
 
 }
@@ -865,7 +845,8 @@ void P16F874A::create_symbols()
 }
 
 P16F874A::P16F874A(const char *_name, const char *desc)
-  : P16F874(_name,desc)
+  : P16F874(_name,desc),
+    comparator(this)
 {
   if(verbose)
     cout << "f874A constructor, type = " << isa() << '\n';
@@ -913,8 +894,6 @@ Processor * P16F877::construct(const char *name)
   p->create();
   p->create_invalid_registers ();
   p->create_symbols();
-  symbol_table.add_module(p,p->name().c_str());
-
   return p;
 
 }
@@ -976,8 +955,6 @@ Processor * P16F877A::construct(const char *name)
   p->create();
   p->create_invalid_registers ();
   p->create_symbols();
-  symbol_table.add_module(p,p->name().c_str());
-
   return p;
 
 }
@@ -992,7 +969,8 @@ void P16F877A::create_symbols()
 }
 
 P16F877A::P16F877A(const char *_name, const char *desc)
-  : P16F874A(_name,desc)
+  : P16F874A(_name,desc),
+    comparator(this)
 {
   if(verbose)
     cout << "f877A constructor, type = " << isa() << '\n';
