@@ -28,12 +28,17 @@ Boston, MA 02111-1307, USA.  */
 #include "pir.h"
 #include "ssp.h"
 #include "psp.h"
+#include "eeprom.h"
+#include "comparator.h"
+#include "a2dconverter.h"
+
 
 class P16C61 : public P16X8X
 {
 public:
 
   P16C61(const char *_name=0, const char *desc=0);
+  virtual ~P16C61();
 
   virtual PROCESSOR_TYPE isa(){return _P16C61_;};
   virtual unsigned int program_memory_size() const { return 0x400; };
@@ -45,7 +50,7 @@ public:
 
 
 //
-//   -- Define a class to contain most of the registers/peripherals 
+//   -- Define a class to contain most of the registers/peripherals
 //      of a 16x6x device (where the second `x' is >= 3
 //
 
@@ -100,6 +105,11 @@ class P16C62 : public  P16X6X_processor
   public:
 
   P16C62(const char *_name=0, const char *desc=0);
+
+  virtual ~P16C62()
+  {
+  }
+
   static Processor *construct(const char *name);
 
   TMR2_MODULE tmr2_module;
@@ -128,6 +138,8 @@ class P16C63 : public  P16C62
 
 
   P16C63(const char *_name=0, const char *desc=0);
+  virtual ~P16C63();
+
   static Processor *construct(const char *name);
   void create();
   void create_sfr_map();
@@ -142,9 +154,9 @@ class P16C64 : public  P16X6X_processor
   // better to just tell C++ to redefine pir1 and pir2 and PIR1v2 and
   // PIR2v2, but C++ only supports covariance in member function return
   // values.
-  PIR1v2 pir1_2_reg;
+  PIR1v2 *pir1_2_reg;
   PIR_SET_2 pir_set_2_def;
-  virtual PIR *get_pir1() { return (&pir1_2_reg); }
+  virtual PIR *get_pir1() { return (pir1_2_reg); }
   virtual PIR_SET *get_pir_set() { return (&pir_set_2_def); }
 
 
@@ -154,7 +166,7 @@ class P16C64 : public  P16X6X_processor
 
   PicPortRegister  *m_porte;
   PicPSP_TrisRegister  *m_trise;
-  PSP		    psp;
+  PSP               psp;
 
   P16C64(const char *_name=0, const char *desc=0);
   virtual ~P16C64();
@@ -188,10 +200,88 @@ class P16C65 : public  P16C64
 
 
   P16C65(const char *_name=0, const char *desc=0);
+  virtual ~P16C65();
   static Processor *construct(const char *name);
   void create();
   void create_sfr_map();
 };
 
 
+class P16F630 :  public _14bit_processor
+{
+public:
+
+  P16F630(const char *_name=0, const char *desc=0);
+  virtual ~P16F630();
+
+  T1CON   t1con;
+  PIR    *pir1;
+  PIE     pie1;
+  TMRL    tmr1l;
+  TMRH    tmr1h;
+  OSCCAL  osccal;
+
+  EEPROM_WIDE *e;
+  PIR1v3 *pir1_3_reg;
+
+  INTCON_14_PIR    intcon_reg;
+  ComparatorModule comparator;
+  PIR_SET_1    pir_set_def;
+  WPU              *m_wpu;
+  IOC              *m_ioc;
+
+
+  virtual PIR *get_pir2() { return (NULL); }
+  virtual PIR *get_pir1() { return (pir1); }
+  virtual PIR_SET *get_pir_set() { return (&pir_set_def); }
+
+
+
+  PicPortGRegister  *m_porta;
+  PicTrisRegister  *m_trisa;
+
+  PicPortRegister *m_portc;
+  PicTrisRegister  *m_trisc;
+
+  virtual PROCESSOR_TYPE isa(){return _P16F630_;}
+  static Processor *construct(const char *name);
+  void create(int);
+  virtual void create_symbols();
+  virtual void create_sfr_map();
+  virtual void create_iopin_map();
+  virtual void option_new_bits_6_7(unsigned int bits);
+  virtual bool hasSSP() {return false;}
+
+
+  virtual unsigned int program_memory_size() const { return 0x400; };
+  virtual unsigned int register_memory_size () const { return 0x100; }
+
+  virtual void set_eeprom_wide(EEPROM_WIDE *ep) { eeprom = ep; }
+  virtual void create_config_memory();
+  virtual bool set_config_word(unsigned int address, unsigned int cfg_word);
+
+
+
+
+
+
+};
+class P16F676 :  public P16F630
+{
+public:
+  ANSEL  ansel;
+  ADCON0_12F adcon0;
+  ADCON1_16F adcon1;
+  sfr_register  adresh;
+  sfr_register  adresl;
+
+
+  P16F676(const char *_name=0, const char *desc=0);
+  virtual ~P16F676();
+
+  virtual PROCESSOR_TYPE isa(){return _P16F676_;}
+  static Processor *construct(const char *name);
+  virtual void create(int);
+  virtual void create_sfr_map();
+};
 #endif
