@@ -56,7 +56,7 @@ public:
   {
   }
 
-  virtual bool interrupt_status()
+  virtual int interrupt_status()
   {
     return m_adcon0->getADIF();
   }
@@ -67,7 +67,11 @@ private:
 //------------------------------------------------------------------------
 //
 P16C71::P16C71(const char *_name, const char *desc)
-  : P16C61(_name, desc)
+  : P16C61(_name, desc),
+    adcon0(this,"adcon0", "A2D Control 0"),
+    adcon1(this,"adcon1", "A2D Control 1"),
+    adres(this,"adres", "A2D Result")
+
 {
   if(verbose)
     cout << "c71 constructor, type = " << isa() << '\n';
@@ -85,12 +89,11 @@ void P16C71::create_sfr_map()
   add_sfr_register(&adcon0, 0x08, RegisterValue(0,0));
   add_sfr_register(&adcon1, 0x88, RegisterValue(0,0));
 
-  add_sfr_register(&adres,  0x89, RegisterValue(0,0));
+  //add_sfr_register(&adres,  0x89, RegisterValue(0,0));
   add_sfr_register(&adres,  0x09, RegisterValue(0,0));
+  alias_file_registers(0x70,0x7f,0x80);
 
 
-  adres.new_name("adres");
-
   adcon1.setValidCfgBits(ADCON1::PCFG0 | ADCON1::PCFG1,0);
   adcon1.setNumberOfChannels(4);
   adcon1.setIOPin(0, &(*m_porta)[0]);
@@ -142,7 +145,7 @@ Processor * P16C71::construct(const char *name)
   p->create_invalid_registers ();
   p->create_symbols();
 
-  symbol_table.add_module(p,p->name().c_str());
+  globalSymbolTable().addModule(p);
 
   return p;
 
@@ -194,9 +197,6 @@ void P16C712::create_sfr_map()
   adcon1.setVrefHiConfiguration(3, 3);
   adcon1.setVrefHiConfiguration(5, 3);
 
-  adcon0.new_name("adcon0");
-  adcon1.new_name("adcon1");
-  adres.new_name("adres");
 }
 
 
@@ -225,7 +225,7 @@ Processor * P16C712::construct(const char *name)
   p->create_invalid_registers ();
   p->create_symbols();
 
-  symbol_table.add_module(p,p->name().c_str());
+  globalSymbolTable().addModule(p);
 
   return p;
 
@@ -233,11 +233,13 @@ Processor * P16C712::construct(const char *name)
 
 
 P16C712::P16C712(const char *_name, const char *desc)
-  : P16C62(_name, desc)
+  : P16C62(_name, desc),
+    adcon0(this,"adcon0", "A2D Control 0"),
+    adcon1(this,"adcon1", "A2D Control 1"),
+    adres(this,"adres", "A2D Result")
 {
   if(verbose)
     cout << "c712 constructor, type = " << isa() << '\n';
-
 }
 
 
@@ -255,7 +257,7 @@ Processor * P16C716::construct(const char *name)
   p->create_invalid_registers ();
   p->create_symbols();
 
-  symbol_table.add_module(p,p->name().c_str());
+  globalSymbolTable().addModule(p);
 
   return p;
 
@@ -280,8 +282,9 @@ void P16C72::create_sfr_map()
     cout << "creating c72 registers \n";
 
   // Parent classes just set PIR version 1
-  pir_set_2_def.set_pir1(&pir1_2_reg);
-  pir_set_2_def.set_pir2(&pir2_2_reg);
+
+  pir_set_2_def.set_pir1(pir1_2_reg);
+  pir_set_2_def.set_pir2(pir2_2_reg);
 
   add_sfr_register(&adcon0, 0x1f, RegisterValue(0,0));
   add_sfr_register(&adcon1, 0x9f, RegisterValue(0,0));
@@ -292,8 +295,7 @@ void P16C72::create_sfr_map()
   adcon0.setAdresLow(0);
   adcon0.setAdcon1(&adcon1);
   adcon0.setIntcon(&intcon_reg);
-  // adcon0.pir_set = get_pir_set();
-  adcon0.pir_set = &pir_set_2_def;
+  adcon0.setPir(pir1_2_reg);
   adcon0.setChannel_Mask(7); // even though there are only 5 inputs...
   adcon0.setA2DBits(8);
 
@@ -318,10 +320,6 @@ void P16C72::create_sfr_map()
   adcon1.setVrefHiConfiguration(3, 3);
   adcon1.setVrefHiConfiguration(5, 3);
 
-  adcon0.new_name("adcon0");
-  adcon1.new_name("adcon1");
-  adres.new_name("adres");
-
   // Link the A/D converter to the Capture Compare Module
   ccp2con.setADCON(&adcon0);
 }
@@ -358,7 +356,7 @@ Processor * P16C72::construct(const char *name)
   p->create_symbols();
 
 
-  symbol_table.add_module(p,p->name().c_str());
+  globalSymbolTable().addModule(p);
 
   return p;
 
@@ -367,13 +365,17 @@ Processor * P16C72::construct(const char *name)
 
 P16C72::P16C72(const char *_name, const char *desc)
   : P16C62(_name, desc),
-    pir1_2_reg(&intcon_reg,&pie1), pir2_2_reg(&intcon_reg,&pie2)
+    adcon0(this,"adcon0", "A2D Control 0"),
+    adcon1(this,"adcon1", "A2D Control 1"),
+    adres(this,"adres", "A2D Result")
 {
   if(verbose)
     cout << "c72 constructor, type = " << isa() << '\n';
 
-  pir1 = &pir1_2_reg;
-  pir2 = &pir2_2_reg;
+  pir1_2_reg = new PIR1v2(this,"pir1","Peripheral Interrupt Register",&intcon_reg,&pie1);
+  pir2_2_reg = new PIR2v2(this,"pir2","Peripheral Interrupt Register",&intcon_reg,&pie2);
+  pir1 = pir1_2_reg;
+  pir2 = pir2_2_reg;
 
 }
 
@@ -387,22 +389,19 @@ void P16C73::create_sfr_map()
     cout << "creating c73 registers \n";
 
   // Parent classes just set PIR version 1
-  pir_set_2_def.set_pir1(&pir1_2_reg);
-  pir_set_2_def.set_pir2(&pir2_2_reg);
+  pir_set_2_def.set_pir1(pir1_2_reg);
+  pir_set_2_def.set_pir2(pir2_2_reg);
 
   add_sfr_register(&adcon0, 0x1f, RegisterValue(0,0));
   add_sfr_register(&adcon1, 0x9f, RegisterValue(0,0));
 
   add_sfr_register(&adres,  0x1e, RegisterValue(0,0));
 
-  //1adcon0.analog_port = porta;
-  //2adcon0.analog_port2 = 0;
   adcon0.setAdres(&adres);
   adcon0.setAdresLow(0);
   adcon0.setAdcon1(&adcon1);
   adcon0.setIntcon(&intcon_reg);
-  // adcon0.pir_set = get_pir_set();
-  adcon0.pir_set = &pir_set_2_def;
+  adcon0.setPir(pir1_2_reg);
   adcon0.setChannel_Mask(7); // even though there are only 5 inputs...
   adcon0.setA2DBits(8);
 
@@ -429,11 +428,6 @@ void P16C73::create_sfr_map()
   adcon1.setVrefHiConfiguration(3, 3);
   adcon1.setVrefHiConfiguration(5, 3);
 
-
-  adcon0.new_name("adcon0");
-  adcon1.new_name("adcon1");
-  adres.new_name("adres");
-
   // Link the A/D converter to the Capture Compare Module
   ccp2con.setADCON(&adcon0);
 }
@@ -469,7 +463,7 @@ Processor * P16C73::construct(const char *name)
   p->create_invalid_registers ();
   p->create_symbols();
 
-  symbol_table.add_module(p,p->name().c_str());
+  globalSymbolTable().addModule(p);
 
   return p;
 
@@ -478,13 +472,18 @@ Processor * P16C73::construct(const char *name)
 
 P16C73::P16C73(const char *_name, const char *desc)
   : P16C63(_name, desc),
-    pir1_2_reg(&intcon_reg,&pie1), pir2_2_reg(&intcon_reg,&pie2)
+    adcon0(this,"adcon0", "A2D Control 0"),
+    adcon1(this,"adcon1", "A2D Control 1"),
+    adres(this,"adres", "A2D Result")
 {
   if(verbose)
     cout << "c73 constructor, type = " << isa() << '\n';
 
-  pir1 = &pir1_2_reg;
-  pir2 = &pir2_2_reg;
+  pir1_2_reg = new PIR1v2(this,"pir1","Peripheral Interrupt Register",&intcon_reg,&pie1);
+  pir2_2_reg = new PIR2v2(this,"pir2","Peripheral Interrupt Register",&intcon_reg,&pie2);
+
+  pir1 = pir1_2_reg;
+  pir2 = pir2_2_reg;
 
 
 }
@@ -535,8 +534,6 @@ void P16F73::create()
 
   P16C73::create();
 
-  pm_rd.set_cpu(this);
-
   status->rp_mask = 0x60;  // rp0 and rp1 are valid.
   indf->base_address_mask1 = 0x80; // used for indirect accesses above 0x100
   indf->base_address_mask2 = 0x1ff; // used for indirect accesses above 0x100
@@ -557,7 +554,7 @@ Processor * P16F73::construct(const char *name)
   p->create_invalid_registers ();
   p->create_symbols();
 
-  symbol_table.add_module(p,p->name().c_str());
+  globalSymbolTable().addModule(p);
 
   return p;
 
@@ -565,7 +562,8 @@ Processor * P16F73::construct(const char *name)
 
 
 P16F73::P16F73(const char *_name, const char *desc)
-  : P16C73(_name, desc)
+  : P16C73(_name, desc),
+    pm_rd(this)
 {
   if(verbose)
     cout << "f73 constructor, type = " << isa() << '\n';
@@ -584,8 +582,8 @@ void P16C74::create_sfr_map()
     cout << "creating c74 registers \n";
 
   // Parent classes just set PIR version 1
-  pir_set_2_def.set_pir1(&pir1_2_reg);
-  pir_set_2_def.set_pir2(&pir2_2_reg);
+  pir_set_2_def.set_pir1(pir1_2_reg);
+  pir_set_2_def.set_pir2(pir2_2_reg);
 
   add_sfr_register(&adcon0, 0x1f, RegisterValue(0,0));
   add_sfr_register(&adcon1, 0x9f, RegisterValue(0,0));
@@ -599,7 +597,7 @@ void P16C74::create_sfr_map()
   adcon0.setAdresLow(0);
   adcon0.setAdcon1(&adcon1);
   adcon0.setIntcon(&intcon_reg);
-  adcon0.pir_set = &pir_set_2_def;
+  adcon0.setPir(pir1_2_reg);
   adcon0.setChannel_Mask(7);
   adcon0.setA2DBits(8);
 
@@ -630,11 +628,6 @@ void P16C74::create_sfr_map()
   adcon1.setVrefHiConfiguration(3, 3);
   adcon1.setVrefHiConfiguration(5, 3);
 
-
-  adcon0.new_name("adcon0");
-  adcon1.new_name("adcon1");
-  adres.new_name("adres");
-
   // Link the A/D converter to the Capture Compare Module
   ccp2con.setADCON(&adcon0);
 }
@@ -670,7 +663,7 @@ Processor * P16C74::construct(const char *name)
   p->create_invalid_registers ();
   p->create_symbols();
 
-  symbol_table.add_module(p,p->name().c_str());
+  globalSymbolTable().addModule(p);
 
   return p;
 
@@ -679,13 +672,18 @@ Processor * P16C74::construct(const char *name)
 
 P16C74::P16C74(const char *_name, const char *desc)
   : P16C65(_name, desc) ,
-    pir1_2_reg(&intcon_reg,&pie1), pir2_2_reg(&intcon_reg,&pie2)
+    adcon0(this,"adcon0", "A2D Control 0"),
+    adcon1(this,"adcon1", "A2D Control 1"),
+    adres(this,"adres", "A2D Result")
 {
   if(verbose)
     cout << "c74 constructor, type = " << isa() << '\n';
 
-  pir1 = &pir1_2_reg;
-  pir2 = &pir2_2_reg;
+  pir1_2_reg = new PIR1v2(this,"pir1","Peripheral Interrupt Register",&intcon_reg,&pie1);
+  pir2_2_reg = new PIR2v2(this,"pir2","Peripheral Interrupt Register",&intcon_reg,&pie2);
+
+  pir1 = pir1_2_reg;
+  pir2 = pir2_2_reg;
 
 }
 
@@ -735,8 +733,6 @@ void P16F74::create()
 
   P16C74::create();
 
-  pm_rd.set_cpu(this);
-
   status->rp_mask = 0x60;  // rp0 and rp1 are valid.
   indf->base_address_mask1 = 0x80; // used for indirect accesses above 0x100
   indf->base_address_mask2 = 0x1ff; // used for indirect accesses above 0x100
@@ -757,7 +753,7 @@ Processor * P16F74::construct(const char *name)
   p->create_invalid_registers ();
   p->create_symbols();
 
-  symbol_table.add_module(p,p->name().c_str());
+  globalSymbolTable().addModule(p);
 
   return p;
 
@@ -765,11 +761,11 @@ Processor * P16F74::construct(const char *name)
 
 
 P16F74::P16F74(const char *_name, const char *desc)
-  : P16C74(_name, desc)
+  : P16C74(_name, desc),
+    pm_rd(this)
 {
   if(verbose)
     cout << "f74 constructor, type = " << isa() << '\n';
-
 }
 
 
