@@ -35,7 +35,7 @@ class P16C71 :  public P16C61
 
   ADCON0 adcon0;
   ADCON1 adcon1;
-  ADRES  adres;
+  sfr_register  adres;
 
   virtual PROCESSOR_TYPE isa(){return _P16C71_;};
   virtual void create_symbols();
@@ -60,7 +60,7 @@ class P16C712 :  public P16C62
 
   ADCON0 adcon0;
   ADCON1 adcon1;
-  ADRES  adres;
+  sfr_register  adres;
 
   virtual PROCESSOR_TYPE isa(){return _P16C712_;};
 
@@ -95,18 +95,18 @@ class P16C72 : public P16C62
   // better to just tell C++ to redefine pir1 and pir2 and PIR1v2 and
   // PIR2v2, but C++ only supports covariance in member function return
   // values.
-  PIR1v2 pir1_2_reg;
-  PIR2v2 pir2_2_reg;
+  PIR1v2 *pir1_2_reg;
+  PIR2v2 *pir2_2_reg;
   PIR_SET_2 pir_set_2_def;
-  ADCON0_withccp adcon0;
+  ADCON0 adcon0;
   ADCON1 adcon1;
-  ADRES  adres;
+  sfr_register  adres;
 
   virtual PROCESSOR_TYPE isa(){return _P16C72_;};
   virtual void create_symbols();
   void create_sfr_map();
-  virtual PIR *get_pir1() { return (&pir1_2_reg); }
-  virtual PIR *get_pir2() { return (&pir2_2_reg); }
+  virtual PIR *get_pir1() { return (pir1_2_reg); }
+  virtual PIR *get_pir2() { return (pir2_2_reg); }
   virtual PIR_SET *get_pir_set() { return (&pir_set_2_def); }
 
 
@@ -124,18 +124,18 @@ class P16C73 : public P16C63
   // better to just tell C++ to redefine pir1 and pir2 and PIR1v2 and
   // PIR2v2, but C++ only supports covariance in member function return
   // values.
-  PIR1v2 pir1_2_reg;
-  PIR2v2 pir2_2_reg;
+  PIR1v2 *pir1_2_reg;
+  PIR2v2 *pir2_2_reg;
   PIR_SET_2 pir_set_2_def;
-  ADCON0_withccp adcon0;
+  ADCON0 adcon0;
   ADCON1 adcon1;
-  ADRES  adres;
+  sfr_register  adres;
 
   virtual PROCESSOR_TYPE isa(){return _P16C73_;};
   virtual void create_symbols();
   void create_sfr_map();
-  virtual PIR *get_pir1() { return (&pir1_2_reg); }
-  virtual PIR *get_pir2() { return (&pir2_2_reg); }
+  virtual PIR *get_pir1() { return (pir1_2_reg); }
+  virtual PIR *get_pir2() { return (pir2_2_reg); }
   virtual PIR_SET *get_pir_set() { return (&pir_set_2_def); }
 
   P16C73(const char *_name=0, const char *desc=0);
@@ -170,18 +170,18 @@ class P16C74 : public P16C65 // Not a typo, a 'c74 is more like a 'c65 then a 'c
   // better to just tell C++ to redefine pir1 and pir2 and PIR1v2 and
   // PIR2v2, but C++ only supports covariance in member function return
   // values.
-  PIR1v2 pir1_2_reg;
-  PIR2v2 pir2_2_reg;
+  PIR1v2 *pir1_2_reg;
+  PIR2v2 *pir2_2_reg;
   PIR_SET_2 pir_set_2_def;
-  ADCON0_withccp adcon0;
+  ADCON0 adcon0;
   ADCON1 adcon1;
-  ADRES  adres;
+  sfr_register  adres;
 
   virtual PROCESSOR_TYPE isa(){return _P16C74_;};
   virtual void create_symbols();
   void create_sfr_map();
-  virtual PIR *get_pir1() { return (&pir1_2_reg); }
-  virtual PIR *get_pir2() { return (&pir2_2_reg); }
+  virtual PIR *get_pir1() { return (pir1_2_reg); }
+  virtual PIR *get_pir2() { return (pir2_2_reg); }
   virtual PIR_SET *get_pir_set() { return (&pir_set_2_def); }
 
   virtual unsigned int program_memory_size() const { return 0x1000; };
