@@ -48,6 +48,10 @@ Boston, MA 02111-1307, USA.  */
 P16X8X::P16X8X(const char *_name, const char *desc)
   : Pic14Bit(_name,desc)
 {
+  if(config_modes)
+    config_modes->valid_bits = ConfigMode::CM_FOSC0 | ConfigMode::CM_FOSC1 | 
+      ConfigMode::CM_FOSC1x | ConfigMode::CM_WDTE | ConfigMode::CM_PWRTE;
+
 }
 
 void P16X8X::create_sfr_map()
@@ -99,6 +103,7 @@ void P16X8X::create_iopin_map()
   package->assign_pin(14, 0);
   package->assign_pin(15, 0);
   package->assign_pin(16, 0);
+
 }
 
 
@@ -111,8 +116,7 @@ void  P16X8X::create(int ram_top)
 
   _14bit_processor::create();
 
-  e = new EEPROM;
-  e->set_cpu(this);
+  e = new EEPROM(this);
   e->initialize(EEPROM_SIZE);
 
   e->set_intcon(&intcon_reg);
@@ -134,11 +138,9 @@ Processor * P16C84::construct(const char *name)
 
   P16C84 *p = new P16C84(name);
 
-  p->P16X8X::create(0x2f);
+  p->create(0x2f);
   p->create_invalid_registers ();
   p->create_symbols();
-  symbol_table.add_module(p,p->name().c_str());
-
   return p;
 
 }
@@ -148,6 +150,11 @@ P16C84::P16C84(const char *_name, const char *desc)
 {
 }
 
+void  P16C84::create(int ram_top)
+{
+  P16X8X::create(0x2f);
+  createMCLRPin(4);
+}
 
 
 //========================================================================
@@ -161,11 +168,9 @@ Processor * P16F84::construct(const char *name)
 
   P16F84 *p = new P16F84(name);
 
-  p->P16X8X::create(0x4f);
+  p->create(0x4f);
   p->create_invalid_registers ();
   p->create_symbols();
-  symbol_table.add_module(p,p->name().c_str());
-
   return p;
 
 }
@@ -175,6 +180,13 @@ P16F84::P16F84(const char *_name, const char *desc)
 {
 }
 
+void  P16F84::create(int ram_top)
+{
+  P16X8X::create(0x4f);
+
+  createMCLRPin(4);
+}
+
 //========================================================================
 //
 // Pic 16F83
@@ -190,15 +202,19 @@ Processor * P16F83::construct(const char *name)
 
   P16F83 *p = new P16F83(name);;
 
-  p->P16X8X::create(0x2f);
+  p->create(0x2f);
   p->create_invalid_registers ();
   p->create_symbols();
-  symbol_table.add_module(p,p->name().c_str());
-
   return p;
 
 }
 
+void  P16F83::create(int ram_top)
+{
+  P16X8X::create(0x2f);
+  createMCLRPin(4);
+}
+
 //========================================================================
 P16CR83::P16CR83(const char *_name, const char *desc)
   : P16F83(_name,desc)
