@@ -194,13 +194,12 @@ Processor * P17C7xx::construct(const char *name)
   p->pic_processor::create_symbols();
 
   p->new_name("p17c7xx");
-  symbol_table.add_module(p,p->name().c_str());
-
   return p;
 
 }
 
 P17C7xx::P17C7xx()
+  : cpusta(this,"cpusta","")
 {
   if(verbose)
     cout << "17c7xx constructor, type = " << isa() << '\n';
@@ -229,15 +228,17 @@ void  P17C7xx::create(int ram_top)
   pic_processor::create();
 
   fast_stack.init(this);
+  /*
   ind0.init(this);
   ind1.init(this);
   ind2.init(this);
+  */
   tmr0l.initialize();
   intcon.initialize();
 
   
   //usart.initialize(this);
-  tbl.initialize(this);
+  //tbl.initialize(this);
   //tmr0l.start(0);
 
   //  create_iopin_map();
@@ -281,8 +282,6 @@ Processor * P17C75x::construct(const char *name)
   p->pic_processor::create_symbols();
 
   p->new_name("p17c75x");
-  symbol_table.add_module(p,p->name().c_str());
-
   return p;
 
 }
@@ -492,8 +491,6 @@ Processor * P17C756::construct(const char *name)
   p->pic_processor::create_symbols();
 
   p->new_name("p17c756");
-  symbol_table.add_module(p,p->name().c_str());
-
   return p;
 
 }
@@ -551,8 +548,6 @@ Processor * P17C756A::construct(const char *name)
   p->pic_processor::create_symbols();
 
   p->new_name("p17c756a");
-  symbol_table.add_module(p,p->name().c_str());
-
   return p;
 
 }
@@ -613,8 +608,6 @@ Processor * P17C752::construct(const char *name)
   p->pic_processor::create_symbols();
 
   p->new_name("p17c752");
-  symbol_table.add_module(p,p->name().c_str());
-
   return p;
 
 }
@@ -671,8 +664,6 @@ Processor * P17C762::construct(const char *name)
   p->pic_processor::create_symbols();
 
   p->new_name("p17c762");
-  symbol_table.add_module(p,p->name().c_str());
-
   return p;
 
 }
@@ -729,8 +720,6 @@ Processor * P17C766::construct(const char *name)
   p->pic_processor::create_symbols();
 
   p->new_name("p17c766");
-  symbol_table.add_module(p,p->name().c_str());
-
   return p;
 
 }
