@@ -24,12 +24,16 @@ Boston, MA 02111-1307, USA.  */
 #include "16bit-processors.h"
 #include "eeprom.h"
 #include "psp.h"
+#include "comparator.h"
 
 class PicPortRegister;
 class PicTrisRegister;
 class PicLatchRegister;
+class ADCON0_V2;
+class ADCON1_V2;
+class ADCON2_V2;
 
-class P18C2x2 : public _16bit_processor
+class P18C2x2 : public _16bit_compat_adc
 {
  public:
 
@@ -38,6 +42,7 @@ class P18C2x2 : public _16bit_processor
   void create();
 
   virtual PROCESSOR_TYPE isa(){return _P18Cxx2_;};
+  virtual PROCESSOR_TYPE base_isa(){return _PIC18_PROCESSOR_;};
   virtual void create_symbols();
 
   virtual unsigned int program_memory_size() const { return 0x400; };
@@ -55,6 +60,7 @@ class P18C242 : public P18C2x2
   void create();
 
   virtual unsigned int program_memory_size() const { return 0x2000; };
+  virtual unsigned int last_actual_register () const { return 0x02FF;};
 
 };
 
@@ -68,6 +74,7 @@ class P18C252 : public P18C242
   void create();
 
   virtual unsigned int program_memory_size() const { return 0x4000; };
+  virtual unsigned int last_actual_register () const { return 0x05FF;};
 
 
 };
@@ -76,7 +83,8 @@ class P18C252 : public P18C242
  *  class definitions for the 18C4x2 family
  */
 
-class P18C4x2 : public _16bit_processor
+//class P18C4x2 : public _16bit_processor
+class P18C4x2 : public _16bit_compat_adc
 {
  public:
 
@@ -96,6 +104,7 @@ class P18C4x2 : public _16bit_processor
   void create();
 
   virtual PROCESSOR_TYPE isa(){return _P18Cxx2_;};
+  virtual PROCESSOR_TYPE base_isa(){return _PIC18_PROCESSOR_;};
   virtual void create_symbols();
 
   virtual unsigned int program_memory_size() const { return 0x400; };
@@ -114,6 +123,7 @@ class P18C442 : public P18C4x2
   static Processor *construct(const char *name);
   void create();
   virtual unsigned int program_memory_size() const { return 0x2000; };
+  virtual unsigned int last_actual_register () const { return 0x02FF;};
 
 };
 
@@ -126,6 +136,7 @@ class P18C452 : public P18C442
   static Processor *construct(const char *name);
   void create();
   virtual unsigned int program_memory_size() const { return 0x4000; };
+  virtual unsigned int last_actual_register () const { return 0x05FF;};
 
 };
 
@@ -215,21 +226,50 @@ class P18F452 : public P18F442
 
 };
 
-class P18Fxx20 : public _16bit_processor
+class P18F2455 : public P18F242
+{
+ public:
+  virtual PROCESSOR_TYPE isa(){return _P18F2455_;};
+  P18F2455(const char *_name=0, const char *desc=0);
+  static Processor *construct(const char *name);
+  void create();
+
+  virtual unsigned int program_memory_size() const { return 0x3000; };
+  virtual unsigned int last_actual_register () const { return 0x07FF;};
+
+};
+
+class P18F4455 : public P18F442
+{
+ public:
+  virtual PROCESSOR_TYPE isa(){return _P18F4455_;};
+  P18F4455(const char *_name=0, const char *desc=0);
+  static Processor *construct(const char *name);
+  void create();
+
+  virtual unsigned int program_memory_size() const { return 0x3000; };
+  virtual unsigned int last_actual_register () const { return 0x07FF;};
+
+};
+
+class P18Fxx20 : public _16bit_v2_adc
 {
 public:
   P18Fxx20(const char *_name=0, const char *desc=0);
+  virtual PROCESSOR_TYPE base_isa(){return _PIC18_PROCESSOR_;};
 };
 
 class P18F1220 : public P18Fxx20
 {
  public:
+  OSCTUNE      osctune;
   virtual PROCESSOR_TYPE isa(){return _P18F1220_;};
   P18F1220(const char *_name=0, const char *desc=0);
   static Processor *construct(const char *name);
   void create();
   virtual void create_iopin_map();
   virtual unsigned int program_memory_size() const { return 0x1000; };
+  virtual void osc_mode(unsigned int value);
 
 };
 
@@ -246,4 +286,93 @@ class P18F1320 : public P18F1220
 
 };
 
+
+
+class P18F2x21 : public _16bit_v2_adc
+{
+ public:
+
+  PicPortRegister  *m_porte;
+  PicPSP_TrisRegister  *m_trise;
+  PicLatchRegister *m_late;
+
+  OSCTUNE      osctune;
+  ComparatorModule comparator;
+
+  P18F2x21(const char *_name=0, const char *desc=0);
+
+  void create();
+
+  virtual PROCESSOR_TYPE isa(){return _P18Cxx2_;};
+  virtual PROCESSOR_TYPE base_isa(){return _PIC18_PROCESSOR_;};
+  virtual void create_symbols();
+
+  virtual unsigned int program_memory_size() const { return 0x400; };
+
+// Setting the correct register memory size breaks things
+//  virtual unsigned int register_memory_size () const { return 0x200;};
+  virtual unsigned int last_actual_register () const { return 0x01FF;};
+
+  virtual void create_iopin_map();
+  virtual void create_sfr_map();
+
+
+  virtual void set_out_of_range_pm(unsigned int address, unsigned int value);
+  virtual void set_eeprom(EEPROM *ep) {
+    // Use set_eeprom_pir as the 18Fxxx devices use an EEPROM with PIR
+   assert(0);
+  }
+  virtual void set_eeprom_pir(EEPROM_PIR *ep) { eeprom = ep; }
+  virtual EEPROM_PIR *get_eeprom() { return ((EEPROM_PIR *)eeprom); }
+};
+
+class P18F2321 : public P18F2x21
+{
+ public:
+  virtual PROCESSOR_TYPE isa(){return _P18F2321_;};
+  P18F2321(const char *_name=0, const char *desc=0);
+  static Processor *construct(const char *name);
+  void create();
+
+  virtual unsigned int program_memory_size() const { return 0x1000; };
+  virtual void osc_mode(unsigned int value);
+
+};
+
+
+class P18F4x21 : public P18F2x21
+{
+ public:
+
+  PicPSP_PortRegister  *m_portd;
+  PicTrisRegister  *m_trisd;
+  PicLatchRegister *m_latd;
+
+  PSP               psp;
+
+  P18F4x21(const char *_name=0, const char *desc=0);
+
+  void create();
+
+  virtual void create_symbols();
+
+  virtual void create_iopin_map();
+  virtual void create_sfr_map();
+
+};
+
+class P18F4321 : public P18F4x21
+{
+ public:
+  virtual PROCESSOR_TYPE isa(){return _P18F4321_;};
+  P18F4321(const char *_name=0, const char *desc=0);
+  static Processor *construct(const char *name);
+  void create();
+
+  virtual unsigned int program_memory_size() const { return 0x1000; };
+  virtual void osc_mode(unsigned int value);
+
+};
+
+
 #endif
