@@ -42,6 +42,13 @@ Package::Package(unsigned int _number_of_pins)
   create_pkg(_number_of_pins);
 }
 
+Package::~Package()
+{
+  if (pins)
+    destroy_pin(0); // delete all of the pins
+  delete [] pins;
+  delete [] m_pinGeometry;
+}
 void Package::create_pkg(unsigned int _number_of_pins)
 {
   if(number_of_pins)
@@ -153,25 +160,47 @@ PinGeometry *Package::getPinGeometry(unsigned int pin_number)
 
 //-------------------------------------------------------------------
 //-------------------------------------------------------------------
-void Package::assign_pin(unsigned int pin_number, IOPIN *pin)
+void Package::assign_pin(unsigned int pin_number, IOPIN *pin, bool warn)
 {
 
   switch(pin_existance(pin_number)) {
 
   case E_PIN_EXISTS:
-    if(pins[pin_number-1])
+    if(pins[pin_number-1] && warn)
       cout << "warning: Package::assign_pin. Pin number " << pin_number << " already exists.\n";
 
   case E_NO_PIN:
     pins[pin_number-1] = pin;
 
-    // Tell the I/O pin its number
-    //pin->new_pin_number(pin_number);
+    if ((bool)verbose && pin)
+      cout << "assigned pin " << pin->name() << " to package pin number " << dec << pin_number<<endl;
+
     break;
 
   }
 
 
+}
+
+//-------------------------------------------------------------------
+void Package::destroy_pin(unsigned int pin_number, IOPIN *pin)
+{
+  if (pin_number) {
+
+    if(pin_number <= number_of_pins) {
+      IOPIN *pPin = pins[pin_number-1];
+      if (pPin)
+          delete pPin;
+      pins[pin_number-1] = 0;
+    }
+
+  } else {
+    // Delete all pins
+    for (pin_number=1; pin_number <= number_of_pins; pin_number++)
+      destroy_pin(pin_number);
+    number_of_pins = 0;
+  }
+
 }
 void Package::create_iopin_map(void)
 {
@@ -220,3 +249,18 @@ void PinGeometry::convertToNew()
     m_bShowPinname = true;
   }
 }
+
+//------------------------------------------------------------------------
+void Package::showPins()
+{
+  unsigned int pin_number;
+
+  for (pin_number=0; pin_number < number_of_pins; pin_number++) {
+
+    IOPIN *pPin = pins[pin_number];
+    cout << " pin #"<<dec<<pin_number << " ptr "<<pPin<<endl;
+    if (pPin)
+      cout << "pin name:" << pPin->name() << endl;
+  }
+
+}
