@@ -79,16 +79,12 @@ public:
   unsigned int number_of_pins;
 
 
-  IOPIN **pins;  /* An array containing all of the package's pins. The index
-		  * into the array is the package's pin #. If pins[i] is NULL
-		  * then there's gpsim does not provide any resources for 
-		  * simulating the pin.
-		  */
-
   Package(void);
   Package(unsigned int number_of_pins);
+  virtual ~Package();
 
-  void assign_pin(unsigned int pin_number, IOPIN *pin);
+  void assign_pin(unsigned int pin_number, IOPIN *pin, bool warn=true);
+  void destroy_pin(unsigned int pin_number, IOPIN *pin=0);
   void create_pkg(unsigned int _number_of_pins);
 
   unsigned int isa(void){return _NO_PACKAGE_;};
@@ -105,12 +101,21 @@ public:
   void setPinGeometry(unsigned int pin_number, float x, float y, int orientation, bool bShowName);
   PinGeometry *getPinGeometry(unsigned int pin_number);
 
+  // Debug
+  void showPins();
+
 protected:
   inline bool bIsValidPinNumber(unsigned int pin_number)
   {
     return (pin_number > 0) && (pin_number<=number_of_pins);
   }
 
+  IOPIN **pins;  /* An array containing all of the package's pins. The index
+		  * into the array is the package's pin #. If pins[i] is NULL
+		  * then there's gpsim does not provide any resources for 
+		  * simulating the pin.
+		  */
+
   // pin_position is used by the breadboard to position the pin
   // Its value can be in the range from 0.0000 to 3.9999.
   // 0.0 is upmost left position. 0.9999 is lowest left.
