@@ -30,21 +30,81 @@ Boston, MA 02111-1307, USA.  */
 #include "symbol.h"
 #include "xref.h"
 
+//------------------------------------------------------------------------
+AddressSymbol::AddressSymbol(Processor *pCpu, const char *_name, unsigned int _val)
+  :  Integer(_val)
+
+{
+  new_name(_name);
+  cpu = pCpu;
+}
+
+string AddressSymbol::toString()
+{
+  char buf[256];
+  int i = (int)getVal();
+  snprintf(buf,sizeof(buf), " at address %d = 0x%X",i,i);
+
+  return string(buf);
+}
+
+Value* AddressSymbol::evaluate()
+{
+  return copy();
+}
+int AddressSymbol::set_break(ObjectBreakTypes bt, ObjectActionTypes at,
+                             Expression *pExpr)
+{
+  if (bt == gpsimObject::eBreakExecute && cpu)
+    return get_bp().set_execution_break((Processor*)cpu,getVal(),pExpr);
+
+  return -1;
+}
+
+LineNumberSymbol::LineNumberSymbol(Processor *pCpu,
+                                   const char *_name,
+                                   unsigned int _val)
+  :  AddressSymbol(pCpu,_name,_val)
+{
+  if(!_name) {
+    char buf[64];
+    snprintf(buf,sizeof(buf), "line_%04x",_val);
+    new_name(buf);
+  }
+
+}
+
+
+
 
 instruction::instruction(Processor *pProcessor,
-			 unsigned int uOpCode, 
-			 unsigned int uAddrOfInstr)
-  : gpsimValue(pProcessor),
+                         unsigned int uOpCode,
+                         unsigned int uAddrOfInstr)
+  : Value("","",pProcessor),
     m_bIsModified(false),
     cycle_count(0),
-    opcode(uOpCode), 
+    opcode(uOpCode),
     m_uAddrOfInstr(uAddrOfInstr),
+    pLineSymbol(0),
     file_id(-1),
     hll_file_id(-1),
     src_line(-1),
     lst_line(-1),
     hll_src_line(-1)
 {
+  if (cpu) {
+    pLineSymbol = new LineNumberSymbol((Processor*)cpu,0, m_uAddrOfInstr);
+    if (!cpu->addSymbol(pLineSymbol)) {
+      delete pLineSymbol;
+      pLineSymbol = 0;
+    }
+  }
+}
+instruction::~instruction()
+{
+  // cout << __FUNCTION__<<endl;
+  if (cpu)
+    cpu->deleteSymbol(pLineSymbol);
 }
 
 void instruction::decode(Processor *new_cpu, unsigned int new_opcode)
@@ -53,8 +113,31 @@ void instruction::decode(Processor *new_cpu, unsigned int new_opcode)
   opcode = new_opcode;
 }
 
-void invalid_instruction::execute(void)  
-{ 
+int instruction::set_break(ObjectBreakTypes bt, ObjectActionTypes at, Expression *pExpr)
+{
+  if (bt == gpsimObject::eBreakExecute)
+    return get_bp().set_execution_break(get_cpu(),m_uAddrOfInstr,pExpr);
+
+  return -1;
+}
+
+void instruction::addLabel(string &rLabel)
+{
+  if (cpu) {
+    addName(rLabel);
+    cpu->addSymbol(this, &rLabel);
+  }
+}
+
+//------------------------------------------------------------------------
+invalid_instruction::invalid_instruction(Processor *new_cpu,unsigned int new_opcode, unsigned int address)
+  : instruction(new_cpu,new_opcode,address)
+{
+  new_name("INVALID");
+}
+
+void invalid_instruction::execute()
+{
   //cout << "*** INVALID INSTRUCTION ***\n";
 #ifdef __DEBUG_VERBOSE__
   debug();
@@ -66,37 +149,18 @@ void invalid_instruction::execute(void)
 
 };
 
-invalid_instruction::invalid_instruction(Processor *new_cpu,unsigned int new_opcode)
-  : instruction(new_cpu,new_opcode,0)
+void invalid_instruction::addLabel(string &rLabel)
 {
-  new_name("INVALID");
+  cout << "*** WARNING: adding label '"<<rLabel << "' to an invalid instruction\n";
 }
 
-invalid_instruction::invalid_instruction(Processor *new_cpu,
-                                         unsigned int address, 
-                                         unsigned int new_opcode)
-  : instruction(new_cpu,new_opcode,address)
-{
-  new_name("INVALID");
-}
 
-// Instantiate an invalid instruction
-invalid_instruction bad_instruction;
-
-
-//------------------------------------------------------------------------
-void instruction::add_line_number_symbol(int address)
-{
-
-  symbol_table.add_line_number(address);
-
-}
 //------------------------------------------------------------------------
 // update_line_number(int file, int sline, int lline, int hllfile, int hllsline)
-// 
+//
 // map the instruction to the source file that generated it.
 // If a line number is less than 0, then this means that the
-// existing mapping should remain unchanged. This allows this 
+// existing mapping should remain unchanged. This allows this
 // method to be called multiple times; once for each type of
 // mapping.
 
@@ -142,23 +206,26 @@ AliasedInstruction::AliasedInstruction()
 {
 }
 
-AliasedInstruction::AliasedInstruction(Processor *pProcessor, 
-		     unsigned int uOpCode, 
-		     unsigned int uAddrOfInstr)
-  : instruction(pProcessor, uOpCode, uAddrOfInstr), 
+AliasedInstruction::AliasedInstruction(Processor *pProcessor,
+                     unsigned int uOpCode,
+                     unsigned int uAddrOfInstr)
+  : instruction(pProcessor, uOpCode, uAddrOfInstr),
     m_replaced(0)
 {
 }
 
+AliasedInstruction::~AliasedInstruction()
+{
+}
+
 void AliasedInstruction::setReplaced(instruction *_replaced)
 {
-  if (!m_replaced)
-    m_replaced = _replaced;
+  m_replaced = _replaced;
 }
 
 instruction * AliasedInstruction::getReplaced()
 {
-  return m_replaced ? m_replaced : &bad_instruction;
+  return m_replaced ? m_replaced : &dynamic_cast<Processor *>(cpu)->bad_instruction;
 }
 
 void AliasedInstruction::execute()
@@ -193,6 +260,12 @@ int AliasedInstruction::get_hll_src_line()
 {
   return getReplaced()->get_hll_src_line();
 }
+void AliasedInstruction::update_line_number(int file, int sline, int lline, int hllfile, int hllsline)
+{
+  if (m_replaced)
+    m_replaced->update_line_number(file, sline, lline, hllfile, hllsline);
+}
+
 int AliasedInstruction::get_lst_line()
 {
   return getReplaced()->get_lst_line();
@@ -217,7 +290,7 @@ char *AliasedInstruction::name(char *cPtr,int len)
 {
   return getReplaced()->name(cPtr,len);
 }
-void AliasedInstruction::update(void)
+void AliasedInstruction::update()
 {
   getReplaced()->update();
 }
@@ -236,9 +309,9 @@ bool AliasedInstruction::isBase()
 
 
 //------------------------------------------------------------------------
-Literal_op::Literal_op(Processor *pProcessor, 
-		       unsigned int uOpCode, 
-		       unsigned int uAddrOfInstr)
+Literal_op::Literal_op(Processor *pProcessor,
+                       unsigned int uOpCode,
+                       unsigned int uAddrOfInstr)
   : instruction(pProcessor, uOpCode, uAddrOfInstr),
     L(uOpCode&0xff)
 {
@@ -247,7 +320,7 @@ char *Literal_op::name(char *return_str,int len)
 {
 
   snprintf(return_str,len,"%s\t0x%02x",
-	   gpsimValue::name().c_str(),L);
+           gpsimObject::name().c_str(),L);
 
   return(return_str);
 }
@@ -259,9 +332,9 @@ void Literal_op::decode(Processor *new_cpu, unsigned int new_opcode)
   L = opcode & 0xff;
 }
 
-Bit_op::Bit_op(Processor *pProcessor, 
-		       unsigned int uOpCode, 
-		       unsigned int uAddrOfInstr)
+Bit_op::Bit_op(Processor *pProcessor,
+                       unsigned int uOpCode,
+                       unsigned int uAddrOfInstr)
   : instruction(pProcessor, uOpCode, uAddrOfInstr),
     mask(0),register_address(0),
     access(false),
@@ -276,44 +349,18 @@ void Bit_op::decode(Processor *new_cpu, unsigned int new_opcode)
   cpu = new_cpu;
   switch(cpu_pic->base_isa())
     {
-    case _16BIT_PROCESSOR_:
-      switch(cpu_pic->isa()) {
-        case  _P18Cxx2_:
-	case  _P18C2x2_:
-	case  _P18C242_:
-	case  _P18C252_:
-	case  _P18C442_:
-	case  _P18C452_:
-	case  _P18F242_:
-	case  _P18F252_:
-	case  _P18F442_:
-	case  _P18F248_:
-	case  _P18F448_:
-	case  _P18F452_:
-	case  _P18F1220_:
-	case  _P18F1320_:
-          mask = 1 << ((opcode >> 9) & 7);
-          register_address = opcode & REG_MASK_16BIT;
-		  access = (opcode & ACCESS_MASK_16BIT) ? true : false;
-          if((!access) && (opcode & 0x80))
-	    register_address |= 0xf00;
+    case _PIC17_PROCESSOR_:
+      mask = 1 << ((opcode >> 8) & 7);
+      register_address = opcode & REG_MASK_16BIT;
+      access = 0;
+      break;
 
-          break;
-	case _P17C7xx_:
-	case _P17C75x_:
-	case _P17C756_:
-	case _P17C756A_:
-	case _P17C762_:
-	case _P17C766_:
-          mask = 1 << ((opcode >> 8) & 7);
-          register_address = opcode & REG_MASK_16BIT;
-          access = 0;
-	  break;
-
-	default:
-          cout << "ERROR: (Bit_op) the processor is not defined\n";
-	  break;
-      }
+    case _PIC18_PROCESSOR_:
+      mask = 1 << ((opcode >> 9) & 7);
+      register_address = opcode & REG_MASK_16BIT;
+      access = (opcode & ACCESS_MASK_16BIT) ? true : false;
+      if((!access) && (opcode & 0x80))
+        register_address |= 0xf00;
       break;
 
     case _14BIT_PROCESSOR_:
@@ -345,13 +392,15 @@ char * Bit_op::name(char *return_str,int len)
 
   switch(cpu_pic->base_isa())
     {
-    case _16BIT_PROCESSOR_:
+    case _PIC17_PROCESSOR_:
+        cout << "Bit_op::name %%% FIX ME %%% treating 17x as 18x\n";
+    case _PIC18_PROCESSOR_:
       bit = ((opcode >> 9) & 7);
       snprintf(return_str,len,"%s\t%s,%d,%c",
-	       gpsimValue::name().c_str(),
-	       reg->name().c_str(), 
-	       bit,
-	       access ? '1' : '0');
+               gpsimObject::name().c_str(),
+               reg->name().c_str(),
+               bit,
+               access ? '1' : '0');
 
       return(return_str);
       break;
@@ -369,18 +418,18 @@ char * Bit_op::name(char *return_str,int len)
 
 
   snprintf(return_str,len,"%s\t%s,%d",
-	   gpsimValue::name().c_str(),
-	   reg->name().c_str(),
-	   bit);
+           gpsimObject::name().c_str(),
+           reg->name().c_str(),
+           bit);
 
   return(return_str);
 }
 
 
 //----------------------------------------------------------------
-Register_op::Register_op(Processor *pProcessor, 
-		       unsigned int uOpCode, 
-		       unsigned int uAddrOfInstr)
+Register_op::Register_op(Processor *pProcessor,
+                       unsigned int uOpCode,
+                       unsigned int uAddrOfInstr)
   : instruction(pProcessor, uOpCode, uAddrOfInstr),
     register_address(0),
     destination(false), access(false)
@@ -399,17 +448,17 @@ char * Register_op::name(char *return_str,int len)
 
   source = get_cpu()->registers[register_address];
 
-  if(cpu_pic->base_isa() != _16BIT_PROCESSOR_)
+  if ( cpu_pic->base_isa() != _PIC18_PROCESSOR_ )
     snprintf(return_str,len,"%s\t%s,%c",
-	     gpsimValue::name().c_str(),
-	     source->name().c_str(),
-	     destination ? 'f' : 'w');
+             gpsimObject::name().c_str(),
+             source->name().c_str(),
+             destination ? 'f' : 'w');
   else
     snprintf(return_str,len,"%s\t%s,%c,%c",
-	     gpsimValue::name().c_str(),
-	     source->name().c_str(), 
-	     destination ? 'f' : 'w',
-	     access ? '1' : '0');
+             gpsimObject::name().c_str(),
+             source->name().c_str(),
+             destination ? 'f' : 'w',
+             access ? '1' : '0');
 
   return(return_str);
 }
@@ -424,30 +473,32 @@ char * Register_op::name(char *return_str,int len)
 // is the lower 5 bits while in the 14-bit core it's the lower 7.
 
 void  Register_op::decode(Processor *new_cpu, unsigned int new_opcode)
-{ 
+{
   opcode = new_opcode;
   cpu = new_cpu;
 
   switch(cpu_pic->base_isa())
     {
-    case _16BIT_PROCESSOR_:
-		destination = (opcode & DESTINATION_MASK_16BIT) ? true : false;
-		access = (opcode & ACCESS_MASK_16BIT) ? true : false;
+    case _PIC17_PROCESSOR_:
+        cout << "Register_op::decode %%% FIXME %%% - PIC17 core is not the same as PIC18\n";
+    case _PIC18_PROCESSOR_:
+      destination = (opcode & DESTINATION_MASK_16BIT) ? true : false;
+      access = (opcode & ACCESS_MASK_16BIT) ? true : false;
       register_address = opcode & REG_MASK_16BIT;
       if((!access) && (opcode & 0x80))
-	register_address |= 0xf00;
+        register_address |= 0xf00;
 
       break;
 
     case _14BIT_PROCESSOR_:
       register_address = opcode & REG_MASK_14BIT;
-	  destination = (opcode & DESTINATION_MASK_14BIT) ? true : false;
+      destination = (opcode & DESTINATION_MASK_14BIT) ? true : false;
       access = 1;
       break;
 
     case _12BIT_PROCESSOR_:
       register_address = opcode & REG_MASK_12BIT;
-	  destination = (opcode & DESTINATION_MASK_12BIT) ? true : false;
+      destination = (opcode & DESTINATION_MASK_12BIT) ? true : false;
       access = 1;
       break;
 
@@ -463,8 +514,8 @@ Register * Register_op::source = 0;
 
 //--------------------------------------------------
 
-ADDWF::ADDWF (Processor *new_cpu, unsigned int new_opcode)
-  : Register_op(new_cpu, new_opcode, 0)
+ADDWF::ADDWF (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Register_op(new_cpu, new_opcode, address)
 {
 
   decode(new_cpu, new_opcode);
@@ -472,7 +523,7 @@ ADDWF::ADDWF (Processor *new_cpu, unsigned int new_opcode)
 
 }
 
-void ADDWF::execute(void)
+void ADDWF::execute()
 {
   unsigned int new_value,src_value,w_value;
 
@@ -498,14 +549,14 @@ void ADDWF::execute(void)
 
 //--------------------------------------------------
 
-ANDLW::ANDLW (Processor *new_cpu, unsigned int new_opcode)
-  : Literal_op(new_cpu, new_opcode, 0)
+ANDLW::ANDLW (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Literal_op(new_cpu, new_opcode, address)
 {
   decode(new_cpu, new_opcode);
   new_name("andlw");
 }
 
-void ANDLW::execute(void)
+void ANDLW::execute()
 {
   unsigned int new_value;
 
@@ -520,8 +571,8 @@ void ANDLW::execute(void)
 
 //--------------------------------------------------
 
-ANDWF::ANDWF (Processor *new_cpu, unsigned int new_opcode)
-  : Register_op(new_cpu, new_opcode, 0)
+ANDWF::ANDWF (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Register_op(new_cpu, new_opcode, address)
 {
 
   decode(new_cpu, new_opcode);
@@ -529,7 +580,7 @@ ANDWF::ANDWF (Processor *new_cpu, unsigned int new_opcode)
 
 }
 
-void ANDWF::execute(void)
+void ANDWF::execute()
 {
   unsigned int new_value;
 
@@ -554,15 +605,15 @@ void ANDWF::execute(void)
 
 //--------------------------------------------------
 
-BCF::BCF (Processor *new_cpu, unsigned int new_opcode)
-  : Bit_op(new_cpu, new_opcode, 0)
+BCF::BCF (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Bit_op(new_cpu, new_opcode, address)
 {
   decode(new_cpu, new_opcode);
   mask ^= 0xff;        // decode initializes the mask to 1<<bit
   new_name("bcf");
 }
 
-void BCF::execute(void)
+void BCF::execute()
 {
 
   if(!access)
@@ -570,7 +621,7 @@ void BCF::execute(void)
   else
     reg = cpu_pic->register_bank[register_address];
 
-  reg->put(reg->get() & mask);
+  reg->put(reg->value.get() & mask);
 
   cpu_pic->pc->increment();
 
@@ -578,14 +629,14 @@ void BCF::execute(void)
 
 //--------------------------------------------------
 
-BSF::BSF (Processor *new_cpu, unsigned int new_opcode)
-  : Bit_op(new_cpu, new_opcode, 0)
+BSF::BSF (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Bit_op(new_cpu, new_opcode, address)
 {
   decode(new_cpu, new_opcode);
   new_name("bsf");
 }
 
-void BSF::execute(void)
+void BSF::execute()
 {
 
   if(!access)
@@ -593,7 +644,7 @@ void BSF::execute(void)
   else
     reg = cpu_pic->register_bank[register_address];
 
-  reg->put(reg->get() | mask);
+  reg->put(reg->value.get() | mask);
 
 
   cpu_pic->pc->increment();
@@ -602,14 +653,14 @@ void BSF::execute(void)
 
 //--------------------------------------------------
 
-BTFSC::BTFSC (Processor *new_cpu, unsigned int new_opcode)
-  : Bit_op(new_cpu, new_opcode, 0)
+BTFSC::BTFSC (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Bit_op(new_cpu, new_opcode, address)
 {
   decode(new_cpu, new_opcode);
   new_name("btfsc");
 }
 
-void BTFSC::execute(void)
+void BTFSC::execute()
 {
 
   if(!access)
@@ -621,21 +672,21 @@ void BTFSC::execute(void)
 
   if(!result)
     cpu_pic->pc->skip();                  // Skip next instruction
-
-  cpu_pic->pc->increment();
+  else
+    cpu_pic->pc->increment();
 
 }
 
 //--------------------------------------------------
 
-BTFSS::BTFSS (Processor *new_cpu, unsigned int new_opcode)
-  : Bit_op(new_cpu, new_opcode, 0)
+BTFSS::BTFSS (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Bit_op(new_cpu, new_opcode, address)
 {
   decode(new_cpu, new_opcode);
   new_name("btfss");
 }
 
-void BTFSS::execute(void)
+void BTFSS::execute()
 {
 
   if(!access)
@@ -647,14 +698,14 @@ void BTFSS::execute(void)
 
   if(result)
     cpu_pic->pc->skip();                  // Skip next instruction
-
-  cpu_pic->pc->increment();
+  else
+    cpu_pic->pc->increment();
 
 }
 
 //--------------------------------------------------
-CALL::CALL (Processor *new_cpu, unsigned int new_opcode)
-  : instruction(new_cpu, new_opcode, 0)
+CALL::CALL (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : instruction(new_cpu, new_opcode, address)
 {
 
   switch(cpu_pic->base_isa())
@@ -673,7 +724,7 @@ CALL::CALL (Processor *new_cpu, unsigned int new_opcode)
   new_name("call");
 }
 
-void CALL::execute(void)
+void CALL::execute()
 {
 
   cpu_pic->stack->push(cpu_pic->pc->get_next());
@@ -686,23 +737,23 @@ char * CALL::name(char *return_str,int len)
 {
 
   snprintf(return_str,len,"%s\t0x%04x",
-	   gpsimValue::name().c_str(),
-	   destination);
+           gpsimObject::name().c_str(),
+           destination);
 
   return(return_str);
 }
 
 
 //--------------------------------------------------
-CLRF::CLRF (Processor *new_cpu, unsigned int new_opcode)
-  : Register_op(new_cpu, new_opcode, 0)
+CLRF::CLRF (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Register_op(new_cpu, new_opcode, address)
 {
   decode(new_cpu, new_opcode);
 
   new_name("clrf");
 }
 
-void CLRF::execute(void)
+void CLRF::execute()
 {
 
   if(!access)
@@ -719,8 +770,8 @@ char * CLRF::name(char *return_str,int len)
 {
 
   snprintf(return_str,len,"%s\t%s",
-	   gpsimValue::name().c_str(),
-	   get_cpu()->registers[register_address]->name().c_str());
+           gpsimObject::name().c_str(),
+           get_cpu()->registers[register_address]->name().c_str());
 
   return(return_str);
 }
@@ -728,14 +779,14 @@ char * CLRF::name(char *return_str,int len)
 
 //--------------------------------------------------
 
-CLRW::CLRW (Processor *new_cpu, unsigned int new_opcode)
-  : instruction(new_cpu, new_opcode, 0)
+CLRW::CLRW (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : instruction(new_cpu, new_opcode, address)
 {
   decode(new_cpu, new_opcode);
   new_name("clrw");
 }
 
-void CLRW::execute(void)
+void CLRW::execute()
 {
 
   cpu_pic->W->put(0);
@@ -747,14 +798,14 @@ void CLRW::execute(void)
 
 //--------------------------------------------------
 
-CLRWDT::CLRWDT (Processor *new_cpu, unsigned int new_opcode)
-  : instruction(new_cpu, new_opcode, 0)
+CLRWDT::CLRWDT (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : instruction(new_cpu, new_opcode, address)
 {
   decode(new_cpu, new_opcode);
   new_name("clrwdt");
 }
 
-void CLRWDT::execute(void)
+void CLRWDT::execute()
 {
 
 
@@ -768,14 +819,14 @@ void CLRWDT::execute(void)
 
 //--------------------------------------------------
 
-COMF::COMF (Processor *new_cpu, unsigned int new_opcode)
-  : Register_op(new_cpu, new_opcode, 0)
+COMF::COMF (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Register_op(new_cpu, new_opcode, address)
 {
   decode(new_cpu, new_opcode);
   new_name("comf");
 }
 
-void COMF::execute(void)
+void COMF::execute()
 {
   unsigned int new_value;
 
@@ -799,14 +850,14 @@ void COMF::execute(void)
 
 //--------------------------------------------------
 
-DECF::DECF (Processor *new_cpu, unsigned int new_opcode)
-  : Register_op(new_cpu, new_opcode, 0)
+DECF::DECF (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Register_op(new_cpu, new_opcode, address)
 {
   decode(new_cpu, new_opcode);
   new_name("decf");
 }
 
-void DECF::execute(void)
+void DECF::execute()
 {
   unsigned int new_value;
 
@@ -830,14 +881,14 @@ void DECF::execute(void)
 
 //--------------------------------------------------
 
-DECFSZ::DECFSZ (Processor *new_cpu, unsigned int new_opcode)
-  : Register_op(new_cpu, new_opcode, 0)
+DECFSZ::DECFSZ (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Register_op(new_cpu, new_opcode, address)
 {
   decode(new_cpu, new_opcode);
   new_name("decfsz");
 }
 
-void DECFSZ::execute(void)
+void DECFSZ::execute()
 {
   unsigned int new_value;
 
@@ -855,14 +906,14 @@ void DECFSZ::execute(void)
 
   if(0==new_value)
     cpu_pic->pc->skip();                  // Skip next instruction
-
-  cpu_pic->pc->increment();
+  else
+    cpu_pic->pc->increment();
 
 }
 
 //--------------------------------------------------
-GOTO::GOTO (Processor *new_cpu, unsigned int new_opcode)
-  : instruction(new_cpu, new_opcode, 0)
+GOTO::GOTO (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : instruction(new_cpu, new_opcode, address)
 {
 
   switch(cpu_pic->base_isa())
@@ -881,7 +932,7 @@ GOTO::GOTO (Processor *new_cpu, unsigned int new_opcode)
   new_name("goto");
 }
 
-void GOTO::execute(void)
+void GOTO::execute()
 {
 
   cpu_pic->pc->jump(cpu_pic->get_pclath_branching_jump() | destination);
@@ -892,7 +943,7 @@ char * GOTO::name(char *return_str,int len)
 {
 
   snprintf(return_str,len,"%s\t0x%04x",
-	  gpsimValue::name().c_str(),destination);
+          gpsimObject::name().c_str(),destination);
 
   return(return_str);
 }
@@ -900,14 +951,14 @@ char * GOTO::name(char *return_str,int len)
 
 //--------------------------------------------------
 
-INCF::INCF (Processor *new_cpu, unsigned int new_opcode)
-  : Register_op(new_cpu, new_opcode, 0)
+INCF::INCF (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Register_op(new_cpu, new_opcode, address)
 {
   decode(new_cpu, new_opcode);
   new_name("incf");
 }
 
-void INCF::execute(void)
+void INCF::execute()
 {
   unsigned int new_value;
 
@@ -934,14 +985,14 @@ void INCF::execute(void)
 
 //--------------------------------------------------
 
-INCFSZ::INCFSZ (Processor *new_cpu, unsigned int new_opcode)
-  : Register_op(new_cpu, new_opcode, 0)
+INCFSZ::INCFSZ (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Register_op(new_cpu, new_opcode, address)
 {
   decode(new_cpu, new_opcode);
   new_name("incfsz");
 }
 
-void INCFSZ::execute(void)
+void INCFSZ::execute()
 {
   unsigned int new_value;
 
@@ -959,21 +1010,21 @@ void INCFSZ::execute(void)
 
   if(0==new_value)
     cpu_pic->pc->skip();                  // Skip next instruction
-
-  cpu_pic->pc->increment();
+  else
+    cpu_pic->pc->increment();
 
 }
 
 //--------------------------------------------------
 
-IORLW::IORLW (Processor *new_cpu, unsigned int new_opcode)
-  : Literal_op(new_cpu, new_opcode, 0)
+IORLW::IORLW (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Literal_op(new_cpu, new_opcode, address)
 {
   decode(new_cpu, new_opcode);
   new_name("iorlw");
 }
 
-void IORLW::execute(void)
+void IORLW::execute()
 {
   unsigned int new_value;
 
@@ -988,14 +1039,14 @@ void IORLW::execute(void)
 
 //--------------------------------------------------
 
-IORWF::IORWF (Processor *new_cpu, unsigned int new_opcode)
-  : Register_op(new_cpu, new_opcode, 0)
+IORWF::IORWF (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Register_op(new_cpu, new_opcode, address)
 {
   decode(new_cpu, new_opcode);
   new_name("iorwf");
 }
 
-void IORWF::execute(void)
+void IORWF::execute()
 {
   unsigned int new_value;
 
@@ -1019,14 +1070,14 @@ void IORWF::execute(void)
 
 //--------------------------------------------------
 
-MOVLW::MOVLW (Processor *new_cpu, unsigned int new_opcode)
-  : Literal_op(new_cpu, new_opcode, 0)
+MOVLW::MOVLW (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Literal_op(new_cpu, new_opcode, address)
 {
   decode(new_cpu, new_opcode);
   new_name("movlw");
 }
 
-void MOVLW::execute(void)
+void MOVLW::execute()
 {
 
   cpu_pic->W->put(L);
@@ -1037,14 +1088,14 @@ void MOVLW::execute(void)
 
 //--------------------------------------------------
 
-MOVF::MOVF (Processor *new_cpu, unsigned int new_opcode)
-  : Register_op(new_cpu, new_opcode, 0)
+MOVF::MOVF (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Register_op(new_cpu, new_opcode, address)
 {
   decode(new_cpu, new_opcode);
   new_name("movf");
 }
 
-void MOVF::execute(void)
+void MOVF::execute()
 {
   unsigned int source_value;
 
@@ -1070,7 +1121,7 @@ void MOVF::execute(void)
 }
 
 
-void MOVF::debug(void)
+void MOVF::debug()
 {
 
    cout << "MOVF:  ";
@@ -1078,14 +1129,14 @@ void MOVF::debug(void)
 }
 
 //--------------------------------------------------
-MOVWF::MOVWF (Processor *new_cpu, unsigned int new_opcode)
-  : Register_op(new_cpu, new_opcode, 0)
+MOVWF::MOVWF (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Register_op(new_cpu, new_opcode, address)
 {
   decode(new_cpu, new_opcode);
   new_name("movwf");
 }
 
-void MOVWF::execute(void)
+void MOVWF::execute()
 {
 
   if(!access)
@@ -1100,8 +1151,8 @@ char * MOVWF::name(char *return_str, int len)
 {
 
   snprintf(return_str,len,"%s\t%s",
-	   gpsimValue::name().c_str(),
-	   get_cpu()->registers[register_address]->name().c_str());
+           gpsimObject::name().c_str(),
+           get_cpu()->registers[register_address]->name().c_str());
 
   return(return_str);
 }
@@ -1109,8 +1160,8 @@ char * MOVWF::name(char *return_str, int len)
 
 //--------------------------------------------------
 
-NOP::NOP (Processor *new_cpu, unsigned int new_opcode)
-  : instruction(new_cpu,new_opcode,0)
+NOP::NOP (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : instruction(new_cpu,new_opcode,address)
 {
 
   decode(new_cpu,new_opcode);
@@ -1129,24 +1180,24 @@ NOP::NOP (Processor *new_cpu, unsigned int new_opcode)
   */
 }
 
-void NOP::execute(void)
+void NOP::execute()
 {
   cpu_pic->pc->increment();
 }
 
 //--------------------------------------------------
 
-OPTION::OPTION (Processor *new_cpu, unsigned int new_opcode)
-  : instruction(new_cpu,new_opcode,0)
+OPTION::OPTION (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : instruction(new_cpu,new_opcode,address)
 {
   decode(new_cpu, new_opcode);
   new_name("option");
 }
 
-void OPTION::execute(void)
+void OPTION::execute()
 {
 
-  cpu_pic->option_reg.put(cpu_pic->W->get());
+  cpu_pic->put_option_reg(cpu_pic->W->get());
 
   cpu_pic->pc->increment();
 
@@ -1154,14 +1205,14 @@ void OPTION::execute(void)
 
 //--------------------------------------------------
 
-RETLW::RETLW (Processor *new_cpu, unsigned int new_opcode)
-  : Literal_op(new_cpu, new_opcode, 0)
+RETLW::RETLW (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Literal_op(new_cpu, new_opcode, address)
 {
   decode(new_cpu, new_opcode);
   new_name("retlw");
 }
 
-void RETLW::execute(void)
+void RETLW::execute()
 {
 
   cpu_pic->W->put(L);
@@ -1172,14 +1223,14 @@ void RETLW::execute(void)
 
 //--------------------------------------------------
 
-RLF::RLF (Processor *new_cpu, unsigned int new_opcode)
-  : Register_op(new_cpu, new_opcode, 0)
+RLF::RLF (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Register_op(new_cpu, new_opcode, address)
 {
   decode(new_cpu, new_opcode);
   new_name("rlf");
 }
 
-void RLF::execute(void)
+void RLF::execute()
 {
   unsigned int new_value;
 
@@ -1203,14 +1254,14 @@ void RLF::execute(void)
 
 //--------------------------------------------------
 
-RRF::RRF (Processor *new_cpu, unsigned int new_opcode)
-  : Register_op(new_cpu, new_opcode, 0)
+RRF::RRF (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Register_op(new_cpu, new_opcode, address)
 {
   decode(new_cpu, new_opcode);
   new_name("rrf");
 }
 
-void RRF::execute(void)
+void RRF::execute()
 {
   unsigned int new_value,old_value;
 
@@ -1235,30 +1286,28 @@ void RRF::execute(void)
 
 //--------------------------------------------------
 
-SLEEP::SLEEP (Processor *new_cpu, unsigned int new_opcode)
-  : instruction(new_cpu,new_opcode,0)
+SLEEP::SLEEP (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : instruction(new_cpu,new_opcode,address)
 {
   decode(new_cpu, new_opcode);
   new_name("sleep");
 }
 
-void SLEEP::execute(void)
+void SLEEP::execute()
 {
-
   cpu_pic->enter_sleep();
-  bp.set_sleep();
 }
 
 //--------------------------------------------------
 
-SUBWF::SUBWF (Processor *new_cpu, unsigned int new_opcode)
-  : Register_op(new_cpu, new_opcode, 0)
+SUBWF::SUBWF (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Register_op(new_cpu, new_opcode, address)
 {
   decode(new_cpu, new_opcode);
   new_name("subwf");
 }
 
-void SUBWF::execute(void)
+void SUBWF::execute()
 {
   unsigned int new_value,src_value,w_value;
 
@@ -1285,14 +1334,14 @@ void SUBWF::execute(void)
 
 //--------------------------------------------------
 
-SWAPF::SWAPF (Processor *new_cpu, unsigned int new_opcode)
-  : Register_op(new_cpu, new_opcode, 0)
+SWAPF::SWAPF (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Register_op(new_cpu, new_opcode, address)
 {
   decode(new_cpu, new_opcode);
   new_name("swapf");
 }
 
-void SWAPF::execute(void)
+void SWAPF::execute()
 {
   unsigned int src_value;
 
@@ -1314,8 +1363,8 @@ void SWAPF::execute(void)
 
 
 //--------------------------------------------------
-TRIS::TRIS (Processor *new_cpu, unsigned int new_opcode)
-  : Register_op(new_cpu, new_opcode, 0)
+TRIS::TRIS (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Register_op(new_cpu, new_opcode, address)
 {
   decode(new_cpu, new_opcode);
 
@@ -1330,10 +1379,10 @@ TRIS::TRIS (Processor *new_cpu, unsigned int new_opcode)
     {
       cout << "Warning: TRIS address '" << register_address << "' is  out of range\n";
 
-	// set the address to a 'bad value' that's
-	// easy to detect at run time:
-	register_address = 0;
-      
+        // set the address to a 'bad value' that's
+        // easy to detect at run time:
+        register_address = 0;
+
     }
   else {
      if(cpu_pic->base_isa() == _14BIT_PROCESSOR_)
@@ -1342,15 +1391,15 @@ TRIS::TRIS (Processor *new_cpu, unsigned int new_opcode)
   new_name("tris");
 }
 
-void TRIS::execute(void)
+void TRIS::execute()
 {
   if(register_address)
     {
       // Execute the instruction only if the register is valid.
       if(cpu_pic->base_isa() == _14BIT_PROCESSOR_)
-	cpu_pic->registers[register_address]->put(cpu_pic->W->get());
+        cpu_pic->registers[register_address]->put(cpu_pic->W->get());
       else
-	cpu_pic->tris_instruction(register_address);
+        cpu_pic->tris_instruction(register_address);
     }
 
   cpu_pic->pc->increment();
@@ -1360,8 +1409,8 @@ char * TRIS::name(char *return_str,int len)
 {
 
   snprintf(return_str,len,"%s\t%s",
-	   gpsimValue::name().c_str(),
-	   cpu_pic->registers[register_address]->name().c_str());
+           gpsimObject::name().c_str(),
+           cpu_pic->registers[register_address]->name().c_str());
 
   return(return_str);
 }
@@ -1369,14 +1418,14 @@ char * TRIS::name(char *return_str,int len)
 
 //--------------------------------------------------
 
-XORLW::XORLW (Processor *new_cpu, unsigned int new_opcode)
-  : Literal_op(new_cpu, new_opcode, 0)
+XORLW::XORLW (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Literal_op(new_cpu, new_opcode, address)
 {
   decode(new_cpu, new_opcode);
   new_name("xorlw");
 }
 
-void XORLW::execute(void)
+void XORLW::execute()
 {
   unsigned int new_value;
 
@@ -1391,14 +1440,14 @@ void XORLW::execute(void)
 
 //--------------------------------------------------
 
-XORWF::XORWF (Processor *new_cpu, unsigned int new_opcode)
-  : Register_op(new_cpu, new_opcode, 0)
+XORWF::XORWF (Processor *new_cpu, unsigned int new_opcode, unsigned int address)
+  : Register_op(new_cpu, new_opcode, address)
 {
   decode(new_cpu, new_opcode);
   new_name("xorwf");
 }
 
-void XORWF::execute(void)
+void XORWF::execute()
 {
   unsigned int new_value;
 
