@@ -28,6 +28,7 @@ Boston, MA 02111-1307, USA.  */
 #include "processor.h"
 #include "program_files.h"
 
+#include "pic-registers.h"
 #include "14bit-registers.h"
 #include "trigger.h"
 
@@ -37,17 +38,30 @@ class Register;
 class sfr_register;
 class pic_register;
 class ConfigMemory;
+class ResetTraceType;
+class InterruptTraceType;
 
 enum PROCESSOR_TYPE
 {
   _PIC_PROCESSOR_,
   _14BIT_PROCESSOR_,
   _12BIT_PROCESSOR_,
-  _16BIT_PROCESSOR_,
+  _PIC17_PROCESSOR_,
+  _PIC18_PROCESSOR_,
   _P10F200_,
   _P10F202_,
+  _P10F204_,
+  _P10F206_,
+  _P10F220_,
+  _P10F222_,
   _P12C508_,
   _P12C509_,
+  _P12F508_,
+  _P12F509_,
+  _P12F510_,
+  _P12F629_,
+  _P12F675_,
+  _P12F683_,
   _P16C84_,
   _P16CR83_,
   _P16CR84_,
@@ -67,7 +81,9 @@ enum PROCESSOR_TYPE
   _P16CR62_,
   _P16F627_,
   _P16F628_,
+  _P16F630_,
   _P16F648_,
+  _P16F676_,
   _P16C63_,
   _P16C64_,
   _P16C64A_,
@@ -81,6 +97,8 @@ enum PROCESSOR_TYPE
   _P16F74_,
   _P16F87_,
   _P16F88_,
+  _P16F818_,
+  _P16F819_,
   _P16F871_,
   _P16F873_,
   _P16F873A_,
@@ -110,10 +128,14 @@ enum PROCESSOR_TYPE
   _P18F442_,
   _P18F452_,
   _P18F1220_,
-  _P18F1320_
+  _P18F1320_,
+  _P18F2455_,
+  _P18F4455_,
+  _P18F2321_,
+  _P18F4321_,
 };
 
-// Configuration modes.
+// Configuration modes.  DELETE THIS...
 //  The configuration mode bits are the config word bits remapped.
 //  The remapping removes processor dependent bit definitions.
 class ConfigMode {
@@ -131,16 +153,21 @@ class ConfigMode {
     CM_CPD =   1<<7,
     CM_MCLRE = 1<<8,    // MCLR enable
 
-    CM_FOSC1x = 1<<9,   // Hack for internal oscillators
+    CM_FOSC1x = 1<<31,   // Hack for internal oscillators
   };
 
   int config_mode;
   int valid_bits;
-  ConfigMode() { 
-    config_mode = 0xffff; 
+
+  ConfigMode() {
+    config_mode = 0xffff;
     valid_bits = CM_FOSC0 | CM_FOSC1 | CM_WDTE;
   };
 
+  virtual ~ConfigMode()
+  {
+  }
+
   virtual void set_config_mode(int new_value) { config_mode = new_value & valid_bits;};
   virtual void set_valid_bits(int new_value) { valid_bits = new_value;};
   void set_fosc0(){config_mode |= CM_FOSC0;};
@@ -150,6 +177,11 @@ class ConfigMode {
   void clear_fosc1(){config_mode &= ~CM_FOSC1;};
   bool get_fosc1(){return (0 != (config_mode & CM_FOSC1));};
   bool get_fosc1x(){return (0 != (config_mode & CM_FOSC1x));};
+  void set_fosc01(int v)
+  {
+    config_mode = (config_mode & ~(CM_FOSC0 | CM_FOSC1)) |
+      (v & (CM_FOSC0 | CM_FOSC1));
+  }
 
   void set_cp0()  {config_mode |= CM_CP0;  valid_bits |= CM_CP0;};
   void clear_cp0(){config_mode &= ~CM_CP0; valid_bits |= CM_CP0;};
@@ -160,14 +192,15 @@ class ConfigMode {
 
   void enable_wdt()  {config_mode |= CM_WDTE;};
   void disable_wdt() {config_mode &= ~CM_WDTE;};
+  void set_wdte(bool b) { config_mode = b ? (config_mode | CM_WDTE) : (config_mode & ~CM_WDTE); }
   bool get_wdt()     {return (0 != (config_mode & CM_WDTE));};
 
-  void enable_mclre()  {config_mode |= CM_MCLRE;};
-  void disable_mclre() {config_mode &= ~CM_MCLRE;};
+  void set_mclre(bool b) { config_mode = b ? (config_mode | CM_MCLRE) : (config_mode & ~CM_MCLRE); }
   bool get_mclre()     {return (0 != (config_mode & CM_MCLRE));};
 
   void enable_pwrte()   {config_mode |= CM_PWRTE;  valid_bits |= CM_PWRTE;};
   void disable_pwrte()  {config_mode &= ~CM_PWRTE; valid_bits |= CM_PWRTE;};
+  void set_pwrte(bool b) { config_mode = b ? (config_mode | CM_PWRTE) : (config_mode & ~CM_PWRTE); }
   bool get_pwrte()      {return (0 != (config_mode & CM_PWRTE));};
   bool is_valid_pwrte() {return (0 != (valid_bits & CM_PWRTE));};
 
@@ -180,18 +213,19 @@ class ConfigMode {
 // Watch Dog Timer
 //
 
-class WDT : public TriggerObject
+class WDT : public TriggerObject, public gpsimObject
 {
 public:
   WDT(pic_processor *, double _timeout);
   void put(unsigned int new_value);
   virtual void initialize(bool enable);
+  virtual void swdten(bool enable);
   void set_timeout(double);
-  virtual void reset();
+  virtual void set_prescale(unsigned int);
+  virtual void set_postscale(unsigned int);
+  virtual void reset(RESET_TYPE r);
   void clear();
   virtual void callback();
-  virtual void start_sleep();
-  virtual void new_prescale();
   virtual void update();
   virtual void callback_print();
   void set_breakpoint(unsigned int bpn);
@@ -201,15 +235,16 @@ protected:
   pic_processor *cpu;           // The cpu to which this wdt belongs.
 
   unsigned int
-    value,
     breakpoint,
-    prescale;
+    prescale,
+    postscale;
   guint64
     future_cycle;
 
   double timeout;   // When no prescaler is assigned
   bool   wdte;
   bool   warned;
+  bool   cfgw_enable;  // Enabled from Configureation word
 
 
 };
@@ -252,33 +287,34 @@ public:
   unsigned int config_word;      // as read from hex or cod file
   ConfigMode   *config_modes;    // processor dependent configuration bits.
 
-  unsigned int pll_factor;       // 2^pll_factor is the speed boost the PLL adds 
+  unsigned int pll_factor;       // 2^pll_factor is the speed boost the PLL adds
                                  // to the instruction execution rate.
 
   WDT          wdt;
 
   INDF         *indf;
   FSR          *fsr;
-  Stack         *stack;
+  Stack        *stack;
 
   Status_register *status;
-  WREG          *W;
-  OPTION_REG   option_reg;
+  WREG         *W;
   PCL          *pcl;
   PCLATH       *pclath;
-
+  PCHelper     *m_PCHelper;
   TMR0         tmr0;
   int          num_of_gprs;
 
   EEPROM      *eeprom;       // set to NULL for PIC's that don't have a data EEPROM
 
   bool LoadProgramFile(const char *pFilename,
-		       FILE *pFile,
-		       const char *pProcessorName
-		       );
+                       FILE *pFile,
+                       const char *pProcessorName
+                       );
 
   void add_sfr_register(Register *reg, unsigned int addr,
-			RegisterValue por_value=RegisterValue(0,0),const char *new_name=0);
+                        RegisterValue por_value=RegisterValue(0,0),const char *new_name=0);
+  void delete_sfr_register(Register *pReg);
+  void remove_sfr_register(Register *pReg);
 
   void init_program_memory(unsigned int memory_size);
   void build_program_memory(int *memory,int minaddr, int maxaddr);
@@ -287,23 +323,33 @@ public:
   virtual void create_config_memory() = 0;
   virtual void tris_instruction(unsigned int tris_register) {return;};
   virtual void create_symbols();
-  virtual void create_stack() {stack = new Stack;};
   virtual void run(bool refresh=true);
   virtual void finish();
 
   void sleep();
   virtual void enter_sleep();
-  void step(unsigned int steps,bool refresh=true);
-  void step_over(bool refresh=true);
+  virtual void exit_sleep();
+  bool is_sleeping();
+  virtual void step(unsigned int steps,bool refresh=true);
+  virtual void step_over(bool refresh=true);
+  virtual void step_cycle();
 
   virtual void step_one(bool refresh=true) {
-    program_memory[pc->value]->execute();
+    if (pc->value >= 0 && pc->value < program_memory_size())
+        program_memory[pc->value]->execute();
+    else
+    {
+        cout << "Program counter not valid " << hex << pc->value << endl;
+        get_bp().halt();
+    }
   }
 
   // Take a snap shot of the internal state.
   virtual void save_state();
 
   virtual void interrupt() { return; };
+  //// TEMPORARY - consolidate the various bp.set_interrupt() calls to one function:
+  void BP_set_interrupt();
   void pm_write();
 
   virtual bool set_config_word(unsigned int address, unsigned int cfg_word);
@@ -312,7 +358,6 @@ public:
   virtual ConfigMode *create_ConfigMode() { return new ConfigMode; };
   virtual void reset(RESET_TYPE r);
 
-  virtual void por();
   virtual void create();
 
   virtual PROCESSOR_TYPE isa(){return _PIC_PROCESSOR_;};
@@ -324,15 +369,64 @@ public:
   virtual unsigned int get_pclath_branching_modpcl()=0;
 
   virtual void option_new_bits_6_7(unsigned int)=0;
+  virtual void put_option_reg(unsigned int) {}
 
   virtual void set_eeprom(EEPROM *e);
   virtual EEPROM *get_eeprom() { return (eeprom); }
+  virtual void createMCLRPin(int pkgPinNumber);
+  virtual void assignMCLRPin(int pkgPinNumber);
+  virtual void unassignMCLRPin();
+  virtual void osc_mode(unsigned int );
+
+  // Activity States reflect what the processor is currently doing
+  // (The breakpoint class formally implemented this functionality).
+  enum eProcessorActivityStates {
+    ePAActive,      // Normal state
+    ePAIdle,        // Processor is held in reset
+    ePASleeping,    // Processor is sleeping
+    ePAInterrupt,   // do we need this?
+    ePAPMWrite      // Processor is busy performing a program memory write
+  };
+  eProcessorActivityStates getActivityState() { return m_ActivityState; }
 
   pic_processor(const char *_name=0, const char *desc=0);
   virtual ~pic_processor();
 
+  void set_osc_pin_Number(unsigned int i, unsigned int val, PinModule *pm)
+        {osc_pin_Number[i] = val; m_osc_Monitor[i] = pm;}
+  unsigned char get_osc_pin_Number(unsigned int i)
+        {return osc_pin_Number[i];}
+  PinModule * get_osc_PinMonitor(unsigned int i)
+	{ return m_osc_Monitor[i]; }
+
+  void set_clk_pin(unsigned int pkg_Pin_Number,
+                                     PinModule *PinMod,
+                                     const char * name,
+                                     bool in);
+  void clr_clk_pin(unsigned int pkg_Pin_Number, PinModule *PinMod );
+
+  virtual void set_int_osc(bool val){ internal_osc = val;}
+  virtual bool get_int_osc(){ return internal_osc; }
+
+
+
+
 protected:
-  ConfigMemory **m_configMemory;
+  ConfigMemory *m_configMemory;
+  eProcessorActivityStates m_ActivityState;
+  ResetTraceType *m_pResetTT;
+  InterruptTraceType *m_pInterruptTT;
+  // Most midrange PIC's have a dedicated MCLR pin.
+  // For the ones that don't, m_MCLR will be null.
+  IOPIN *m_MCLR;
+  IOPIN *m_MCLR_Save;
+  int   m_MCLR_pin;
+  PinMonitor *m_MCLRMonitor;
+  string m_mclr_pin_name;
+  unsigned char osc_pin_Number[2];
+  PinModule *m_osc_Monitor[2];
+  bool internal_osc;
+
 };
 
 
@@ -344,17 +438,43 @@ protected:
 // The configuration memory is only a tiny portion of the overall processor
 // program memory space (only 1-word on the mid range devices). So, explicit
 // attributes are created for each memory configuration word. Since the meaning
-// of configuration memory varies from processor to processor, it is up to 
+// of configuration memory varies from processor to processor, it is up to
 // each process to derive from this class.
 
-class ConfigMemory : public Integer
+
+class ConfigWord : public Integer
 {
 public:
-  ConfigMemory(const char *_name, unsigned int default_val, const char *desc,
-	       pic_processor *pCpu, unsigned int addr);
+  ConfigWord(const char *_name, unsigned int default_val, const char *desc,
+             pic_processor *pCpu, unsigned int addr);
+  unsigned int ConfigWordAdd() { return m_addr; }
 protected:
   pic_processor *m_pCpu;
   unsigned int m_addr;
 };
 
+class ConfigMemory
+{
+public:
+  ConfigMemory(pic_processor *pCpu, unsigned int nWords);
+  ~ConfigMemory();
+  int addConfigWord(unsigned int addr, ConfigWord *);
+  ConfigWord *getConfigWord(unsigned int addr);
+protected:
+  pic_processor *m_pCpu;
+  ConfigWord **m_ConfigWords;
+  unsigned int m_nConfigWords;
+};
+
+/*
+class ConfigMemory : public Integer
+{
+public:
+  ConfigMemory(const char *_name, unsigned int default_val, const char *desc,
+               pic_processor *pCpu, unsigned int addr);
+protected:
+  pic_processor *m_pCpu;
+  unsigned int m_addr;
+};
+*/
 #endif
