@@ -29,6 +29,15 @@ Boston, MA 02111-1307, USA.  */
 #include "interface.h"
 #include "pic-registers.h"
 
+#include "clock_phase.h"
+
+//#define DEBUG
+#if defined(DEBUG)
+#define Dprintf(arg) {printf("0x%06X %s() ",cycles.get(),__FUNCTION__); printf arg; }
+#else
+#define Dprintf(arg) {}
+#endif
+
 //------------------------------------------------------------------------
 // member functions for the Program_Counter base class
 //------------------------------------------------------------------------
@@ -36,7 +45,8 @@ Boston, MA 02111-1307, USA.  */
 
 //--------------------------------------------------
 
-Program_Counter::Program_Counter(void)
+Program_Counter::Program_Counter(const char *name, const char *desc, Module *pM)
+  : Value(name,desc,pM)
 {
   if(verbose)
     cout << "pc constructor\n";
@@ -47,19 +57,30 @@ Program_Counter::Program_Counter(void)
   pclath_mask = 0x1800;    // valid pclath bits for branching in 14-bit cores 
   instruction_phase = 0;
 
-  _xref.assign_data(this);
+  xref().assign_data(this);
 
   trace_state = 0;
   trace_increment = 0;
   trace_branch = 0;
   trace_skip = 0;
   trace_other = 0;
-  new_name("pc");
+}
+
+
+
+Program_Counter::~Program_Counter()
+{
+  if (cpu)
+    cpu->removeSymbol(this);
+  delete m_pPCTraceType;
 }
 
 //--------------------------------------------------
-void Program_Counter::set_trace_command(unsigned int new_command)
+void Program_Counter::set_trace_command()
 {
+  m_pPCTraceType = new PCTraceType(get_cpu(),1);
+  unsigned int new_command = trace.allocateTraceType(m_pPCTraceType);
+
   trace_increment = new_command | (0<<16);
   trace_branch    = new_command | (1<<16);
   trace_skip      = new_command | (2<<16);
@@ -69,8 +90,9 @@ void Program_Counter::set_trace_command(unsigned int new_command)
 // increment - update the program counter. All non-branching instructions pass through here.
 //   
 
-void Program_Counter::increment(void)
+void Program_Counter::increment()
 {
+  Dprintf(("PC=0x%x\n",value));
 
   // Trace the value of the program counter before it gets changed.
   trace.raw(trace_increment | value);
@@ -82,7 +104,12 @@ void Program_Counter::increment(void)
   // counter).
 
   cpu_pic->pcl->value.put(value & 0xff);
+
+#ifdef CLOCK_EXPERIMENTS
+  mCurrentPhase->setNextPhase(mExecute1Cycle);
+#else
   get_cycles().increment();
+#endif
 }
 
 //--------------------------------------------------
@@ -90,38 +117,104 @@ void Program_Counter::increment(void)
 // in the trace buffer as a 'skip' instead of a 'pc update'.
 //   
 
-void Program_Counter::skip(void)
+void Program_Counter::skip()
 {
+  Dprintf(("PC=0x%x\n",value));
 
   // Trace the value of the program counter before it gets changed.
   trace.raw(trace_skip | value);
 
+
+#ifdef CLOCK_EXPERIMENTS
+  mExecute2ndHalf->firstHalf((value + 2) & memory_size_mask);
+#else
+  //value = (value + 1) & memory_size_mask;
+  //cpu_pic->pcl->value.put(value & 0xff);
+  //get_cycles().increment();
+
   value = (value + 1) & memory_size_mask;
-
-  // Update pcl. Note that we don't want to pcl.put() because that 
-  // will trigger a break point if there's one set on pcl. (A read/write
-  // break point on pcl should not be triggered by advancing the program
-  // counter).
-
-  cpu_pic->pcl->value.put( value & 0xff);
+  cpu_pic->pcl->value.put(value & 0xff);
   get_cycles().increment();
+
+  trace.raw(trace_increment | value);
+
+  value = (value + 1) & memory_size_mask;
+  cpu_pic->pcl->value.put(value & 0xff);
+  get_cycles().increment();
+#endif
+
 }
 
 //--------------------------------------------------
 // start_skip - The next instruction is going to be skipped
 //   
 
-void Program_Counter::start_skip(void)
+void Program_Counter::start_skip()
 {
 
 }
 
+//========================================================================
+#if defined(CLOCK_EXPERIMENTS)
+
+phaseExecute2ndHalf::phaseExecute2ndHalf(Processor *pcpu)
+  : ProcessorPhase(pcpu), m_uiPC(0)
+{
+}
+phaseExecute2ndHalf::~phaseExecute2ndHalf()
+{
+}
+
+ClockPhase *phaseExecute2ndHalf::firstHalf(unsigned int uiPC)
+{
+  Dprintf(("first half of 2 cycle instruction new PC=0x%x\n",uiPC));
+  ((pic_processor *)m_pcpu)->pc->value = uiPC;
+  ((pic_processor *)m_pcpu)->pcl->value.put(uiPC&0xff);
+  mCurrentPhase->setNextPhase(this);
+  return this;
+}
+
+ClockPhase *phaseExecute2ndHalf::advance()
+{
+  Dprintf(("second half of 2 cycle instruction\n"));
+  mCurrentPhase->setNextPhase(mExecute1Cycle);
+  get_cycles().increment();
+  return m_pNextPhase;
+}
+
+
+phaseExecuteInterrupt::phaseExecuteInterrupt(Processor *pcpu)
+  : ProcessorPhase(pcpu), m_uiPC(0)
+{
+}
+phaseExecuteInterrupt::~phaseExecuteInterrupt()
+{
+}
+ClockPhase *phaseExecuteInterrupt::firstHalf(unsigned int uiPC)
+{
+  Dprintf(("first half of Interrupt new PC=0x%x\n",uiPC));
+  ((pic_processor *)m_pcpu)->pc->value = uiPC;
+  ((pic_processor *)m_pcpu)->pcl->value.put(uiPC&0xff);
+  mCurrentPhase->setNextPhase(this);
+  return this;
+}
+
+ClockPhase *phaseExecuteInterrupt::advance()
+{
+  Dprintf(("second half of Interrupt\n"));
+  mCurrentPhase->setNextPhase(mExecute1Cycle);
+  get_cycles().increment();
+  return m_pNextPhase;
+}
+#endif // defined(CLOCK_EXPERIMENTS)
+
 //--------------------------------------------------
 // jump - update the program counter. All branching instructions except computed gotos
 //        and returns go through here.
 
 void Program_Counter::jump(unsigned int new_address)
 {
+  Dprintf(("PC=0x%x new 0x%x\n",value,new_address));
 
   // Trace the value of the program counter before it gets changed.
   trace.raw(trace_branch | value);
@@ -129,14 +222,17 @@ void Program_Counter::jump(unsigned int new_address)
   // Use the new_address and the cached pclath (or page select bits for 12 bit cores)
   // to generate the destination address:
 
-  value = new_address & memory_size_mask;
 
   // see Update pcl comment in Program_Counter::increment()
-
-  cpu_pic->pcl->value.put(value & 0xff);
   
+#ifdef CLOCK_EXPERIMENTS
+  mExecute2ndHalf->firstHalf(new_address & memory_size_mask);
+#else
+  value = new_address & memory_size_mask;
+  cpu_pic->pcl->value.put(value & 0xff);
   get_cycles().increment();
   get_cycles().increment();
+#endif
 
 }
 
@@ -146,6 +242,14 @@ void Program_Counter::jump(unsigned int new_address)
 
 void Program_Counter::interrupt(unsigned int new_address)
 {
+  Dprintf(("PC=0x%x 0x%x\n",value,new_address));
+
+#ifdef CLOCK_EXPERIMENTS
+  // Trace the value of the program counter before it gets changed.
+  trace.raw(trace_branch | value);
+
+  mExecuteInterrupt->firstHalf(new_address & memory_size_mask);
+#else
 
   // Trace the value of the program counter before it gets changed.
   trace.raw(trace_branch | value);
@@ -159,6 +263,7 @@ void Program_Counter::interrupt(unsigned int new_address)
   
   get_cycles().increment();
   get_cycles().increment();
+#endif
 
 }
 
@@ -170,6 +275,8 @@ void Program_Counter::interrupt(unsigned int new_address)
 void Program_Counter::computed_goto(unsigned int new_address)
 {
 
+  Dprintf(("PC=0x%x new=0x%x\n",value,new_address));
+
   // Trace the value of the program counter before it gets changed.
   trace.raw(trace_other | value);
 
@@ -178,8 +285,6 @@ void Program_Counter::computed_goto(unsigned int new_address)
 
   value = (new_address | cpu_pic->get_pclath_branching_modpcl() ) & memory_size_mask;
 
-  //trace.cycle_increment();
-
   // see Update pcl comment in Program_Counter::increment()
 
   cpu_pic->pcl->value.put(value & 0xff);
@@ -187,32 +292,48 @@ void Program_Counter::computed_goto(unsigned int new_address)
   // The instruction modifying the PCL will also increment the program counter.
   // So, pre-compensate the increment with a decrement:
   value--;
+
+  // The computed goto is a 2-cycle operation. The first cycle occurs within 
+  // the instruction (i.e. via the ::increment() method). The second cycle occurs
+  // here:
+
+#ifdef CLOCK_EXPERIMENTS
+  mCurrentPhase = mExecute1Cycle;
+#else
   get_cycles().increment();
+#endif
 }
 
 //--------------------------------------------------
 // new_address - write a new value to the program counter. All returns pass through here.
 //
 
-void Program_Counter::new_address(unsigned int new_value)
+void Program_Counter::new_address(unsigned int new_address)
 {
+  Dprintf(("PC=0x%x new 0x%x\n",value, new_address&0xffff));
+
   // Trace the value of the program counter before it gets changed.
   trace.raw(trace_branch | value);
 
-  value = new_value & memory_size_mask;
+#ifdef CLOCK_EXPERIMENTS
+  mExecute2ndHalf->firstHalf(new_address & memory_size_mask);
+#else
+  value = new_address & memory_size_mask;
 
   // see Update pcl comment in Program_Counter::increment()
 
   cpu_pic->pcl->value.put(value & 0xff);
+
   get_cycles().increment();
   get_cycles().increment();
+#endif
 }
 
 //--------------------------------------------------
 // get_next - get the next address that is just pass the current one
 //            (used by 'call' to obtain the return address)
 
-unsigned int Program_Counter::get_next(void)
+unsigned int Program_Counter::get_next()
 {
 
   return( (value + cpu_pic->program_memory[value]->instruction_size()) & memory_size_mask);
@@ -228,6 +349,7 @@ void Program_Counter::put_value(unsigned int new_value)
 {
   // FIXME 
 #define PCLATH_MASK              0x1f
+  Dprintf(("PC=0x%x new 0x%x\n",value, new_value&0xffff));
 
   trace.raw(trace_other | value);
 
@@ -240,10 +362,13 @@ void Program_Counter::put_value(unsigned int new_value)
   update();
 }
 
-void Program_Counter::reset(void)
+void Program_Counter::reset()
 { 
   //trace.program_counter(value);  //FIXME
   value = reset_address;
+#ifdef CLOCK_EXPERIMENTS
+  mExecute2ndHalf->firstHalf(reset_address & memory_size_mask);
+#endif
 }
 
 
@@ -252,22 +377,80 @@ void Program_Counter::reset(void)
 // Helper registers
 //
  
-PCHelper::PCHelper(ProgramMemoryAccess *new_pma)
+PCHelper::PCHelper(Processor *pCpu,ProgramMemoryAccess *new_pma)
+  : Register(pCpu, "PC", "Program Counter"),
+    pma(new_pma)
 {
-  pma = new_pma;
-  new_name("PC");
+  assert(pma);
 }
 
 void PCHelper::put_value(unsigned int new_value)
 {
-  if(pma)
-    pma->set_PC(new_value);
+  //  if(pma)
+  pma->set_PC(new_value);
 }
 
-unsigned int PCHelper::get_value(void)
+unsigned int PCHelper::get_value()
 {
- if(pma)
-    return pma->get_PC();
+  // if(pma)
+  return pma->get_PC();
 
- return 0;
+  //return 0;
+}
+
+
+//--------------------------------------------------
+// member functions for the OPTION base class
+//--------------------------------------------------
+OPTION_REG::OPTION_REG(Processor *pCpu, const char *pName, const char *pDesc)
+  : sfr_register(pCpu, pName, pDesc)
+{
+  por_value = RegisterValue(0xff,0);
+  wdtr_value = RegisterValue(0xff,0);
+  // The chip reset will place the proper value here.
+  value = RegisterValue(0,0);  // por_value;
+}
+
+// make sure intial por_value does it's stuff
+void OPTION_REG::initialize()
+{
+    cpu_pic->tmr0.new_prescale();
+    cpu_pic->wdt.set_postscale( (value.get() & PSA) ? (value.get() & ( PS2 | PS1 | PS0 )) : 0);
+    cpu_pic->option_new_bits_6_7(value.get() & (T0CS | BIT6 | BIT7));
+}
+
+void OPTION_REG::put(unsigned int new_value)
+{
+
+  //FIXME trace OPTION for 12bit processors is broken.
+  trace.raw(write_trace.get() | value.get());
+
+  unsigned int old_value = value.get();
+  value.put(new_value);
+
+  // First, check the tmr0 clock source bit to see if we are  changing from
+  // internal to external (or vice versa) clocks.
+  //if( (value ^ old_value) & T0CS)
+  //    cpu_pic->tmr0.new_clock_source();
+
+  // %%%FIX ME%%% - can changing the state of TOSE cause the timer to
+  // increment if tmr0 is being clocked by an external clock?
+
+  // Now check the rest of the tmr0 bits.
+  if( (value.get() ^ old_value) & (T0CS | T0SE | PSA | PS2 | PS1 | PS0))
+    cpu_pic->tmr0.new_prescale();
+
+  if( (value.get() ^ old_value) & (PSA | PS2 | PS1 | PS0))
+    cpu_pic->wdt.set_postscale( (value.get() & PSA) ? (value.get() & ( PS2 | PS1 | PS0 )) : 0);
+
+  if( (value.get() ^ old_value) & (T0CS | BIT6 | BIT7))
+    cpu_pic->option_new_bits_6_7(value.get() & (T0CS | BIT6 | BIT7));
+
+
+}
+
+
+void OPTION_REG::reset(RESET_TYPE r)
+{
+  putRV(por_value);
 }
