@@ -41,9 +41,9 @@ class PCHelper : public Register
 {
 public:
 
-  PCHelper(ProgramMemoryAccess *);
+  PCHelper(Processor *pCpu, ProgramMemoryAccess *);
   virtual void put_value(unsigned int new_value);
-  virtual unsigned int get_value(void);
+  virtual unsigned int get_value();
   virtual unsigned int register_size () const
   { 
     return 2;
@@ -52,4 +52,55 @@ public:
   ProgramMemoryAccess *pma;
 };
 
+//---------------------------------------------------------
+// OPTION_REG - 
+
+class OPTION_REG : public sfr_register
+{
+public:
+
+enum
+  {
+    PS0    = 1<<0,
+    PS1    = 1<<1,
+    PS2    = 1<<2,
+    PSA    = 1<<3,
+    T0SE   = 1<<4,
+    T0CS   = 1<<5,
+    BIT6   = 1<<6,
+    BIT7   = 1<<7
+  };
+
+  unsigned int prescale;
+
+
+  OPTION_REG(Processor *pCpu, const char *pName, const char *pDesc=0);
+
+  inline unsigned int get_prescale()
+  {
+      return value.get() & (PS0 | PS1 | PS2);
+  }
+
+  inline unsigned int get_psa()
+  {
+      return value.get() & PSA;
+  }
+
+  inline unsigned int get_t0cs()
+  {
+      return value.get() & T0CS;
+  }
+
+  inline unsigned int get_t0se()
+  {
+      return value.get() & T0SE;
+  }
+
+  virtual void put(unsigned int new_value);
+  virtual void reset(RESET_TYPE r);
+  virtual void initialize();
+
+};
+
+
 #endif
