@@ -39,6 +39,7 @@ class ProgramFileType;
 class FileContext;
 class FileContextList;
 class ProgramMemoryCollection;
+class CPU_Freq;
 
 //---------------------------------------------------------
 /// MemoryAccess - A base class designed to support
@@ -50,9 +51,9 @@ class MemoryAccess :  public TriggerObject, public gpsimObject
 public:
 
   MemoryAccess(Processor *new_cpu);
+  ~MemoryAccess();
 
   virtual Processor *get_cpu(void);
-  virtual void set_cpu(Processor *p);
 
   list<Register *> SpecialRegisters;
 
@@ -80,7 +81,8 @@ class ProgramMemoryAccess :  public MemoryAccess
   };
 
 
-  ProgramMemoryAccess(Processor *new_cpu=0);
+  ProgramMemoryAccess(Processor *new_cpu);
+  ~ProgramMemoryAccess();
 
   virtual void putToAddress(unsigned int addr, instruction *new_instruction);
   virtual void putToIndex(unsigned int uIndex, instruction *new_instruction);
@@ -144,31 +146,31 @@ class ProgramMemoryAccess :  public MemoryAccess
   // A couple of functions for manipulating  breakpoints
   virtual unsigned int  set_break_at_address(unsigned int address);
   virtual unsigned int  set_notify_at_address(unsigned int address,
-					      TriggerObject *cb);
+                                              TriggerObject *cb);
   virtual unsigned int  set_profile_start_at_address(unsigned int address,
-					    TriggerObject *cb);
+                                            TriggerObject *cb);
   virtual unsigned int  set_profile_stop_at_address(unsigned int address,
-					   TriggerObject *cb);
+                                           TriggerObject *cb);
   virtual int clear_break_at_address(unsigned int address,
-				     enum instruction::INSTRUCTION_TYPES type);
+                                     enum instruction::INSTRUCTION_TYPES type);
   virtual int clear_break_at_address(unsigned int address,
     instruction * pInstruction);
   virtual int clear_notify_at_address(unsigned int address);
   virtual int clear_profile_start_at_address(unsigned int address);
   virtual int clear_profile_stop_at_address(unsigned int address);
   virtual int address_has_break(unsigned int address,
-				enum instruction::INSTRUCTION_TYPES type=instruction::BREAKPOINT_INSTRUCTION);
+                                enum instruction::INSTRUCTION_TYPES type=instruction::BREAKPOINT_INSTRUCTION);
   virtual int address_has_notify(unsigned int address);
   virtual int address_has_profile_start(unsigned int address);
   virtual int address_has_profile_stop(unsigned int address);
   virtual instruction *find_instruction(unsigned int address,
-					enum instruction::INSTRUCTION_TYPES type);
+                                        enum instruction::INSTRUCTION_TYPES type);
   virtual void toggle_break_at_address(unsigned int address);
   virtual void set_break_at_line(unsigned int file_id, unsigned int src_line);
   virtual void clear_break_at_line(unsigned int file_id, 
-				   unsigned int src_line);
+                                   unsigned int src_line);
   virtual void toggle_break_at_line(unsigned int file_id, 
-				    unsigned int src_line);
+                                    unsigned int src_line);
 
   void set_hll_mode(unsigned int);
   enum HLL_MODES get_hll_mode(void) { return hll_mode;}
@@ -204,7 +206,7 @@ class RegisterMemoryAccess : public MemoryAccess
 {
  public:
   
-  RegisterMemoryAccess(Processor *new_cpu=0);
+  RegisterMemoryAccess(Processor *pCpu);
   virtual ~RegisterMemoryAccess();
   virtual Register *get_register(unsigned int address);
   unsigned int get_size(void) { return nRegisters; }
@@ -212,9 +214,10 @@ class RegisterMemoryAccess : public MemoryAccess
 
   // The insertRegister and removeRegister methods are used primarily
   // to set and clear breakpoints.
-  bool insertRegister(int address, Register *);
-  bool removeRegister(int address, Register *);
+  bool insertRegister(unsigned int address, Register *);
+  bool removeRegister(unsigned int address, Register *);
   bool hasBreak(unsigned int address);
+  void reset(RESET_TYPE r);
 
   Register &operator [] (unsigned int address);
 
@@ -338,8 +341,8 @@ public:
   /// Load the source code for this processor. The pProcessorName
   /// is an optional name that a user can assign to the processor.
   virtual bool LoadProgramFile(const char *hex_file, 
-			       FILE *pFile, 
-			       const char *pProcessorName) = 0;
+                               FILE *pFile, 
+                               const char *pProcessorName) = 0;
   /// The source files for this processor.
   FileContextList files;
 
@@ -395,7 +398,7 @@ public:
   /// (for those processors that support interrupts), the processor is
   /// in a different 'state' then when the interrupt flag is cleared.
 
-  list<ProgramMemoryAccess *> pma_context;
+  std::list<ProgramMemoryAccess *> pma_context;
 
   /// Tracing
   /// The readTT and writeTT are TraceType objects for tracing
@@ -407,22 +410,27 @@ public:
   TraceType *readTT, *writeTT;
   map <unsigned int, TraceType *> mTrace;
 
-  virtual void set(const char *cP,int len=0);
-  virtual void get(char *, int len);
+  // Processor's 'bad_instruction' object
+  invalid_instruction bad_instruction;
+
+  // --- TSD removed 01JAN07 These don't appear to be used anywhere
+  //virtual void set(const char *cP,int len=0);
+  //virtual void get(char *, int len);
 
   //
   // Creation and manipulation of registers
   //
 
-  void create_invalid_registers (void);
+  void create_invalid_registers ();
+  void delete_invalid_registers ();
   void add_file_registers(unsigned int start_address, 
-			  unsigned int end_address, 
-			  unsigned int alias_offset);
+                          unsigned int end_address, 
+                          unsigned int alias_offset);
   void delete_file_registers(unsigned int start_address, 
-			     unsigned int end_address);
+                             unsigned int end_address, bool bRemoveWithoutDelete=false);
   void alias_file_registers(unsigned int start_address, 
-			    unsigned int end_address, 
-			    unsigned int alias_offset);
+                            unsigned int end_address, 
+                            unsigned int alias_offset);
   virtual int  map_rm_address2index(int address) {return address;};
   virtual int  map_rm_index2address(int index) {return index;};
   virtual void init_register_memory(unsigned int memory_size);
@@ -441,19 +449,22 @@ public:
   virtual void init_program_memory_at_index(unsigned int address,
     unsigned int value);
   virtual void init_program_memory_at_index(unsigned int address, 
-					    const unsigned char *, int nBytes);
+                                            const unsigned char *, int nBytes);
   virtual unsigned int program_memory_size(void) const {return 0;};
+  virtual unsigned int program_address_limit(void) const {
+      return map_pm_index2address(program_memory_size());
+  };
   virtual unsigned int get_program_memory_at_address(unsigned int address);
   void build_program_memory(unsigned int *memory,
-			    unsigned int minaddr, 
-			    unsigned int maxaddr);
+                            unsigned int minaddr, 
+                            unsigned int maxaddr);
 
-  virtual int  map_pm_address2index(int address) {return address;};
-  virtual int  map_pm_index2address(int index) {return index;};
+  virtual int  map_pm_address2index(int address) const {return address;};
+  virtual int  map_pm_index2address(int index) const {return index;};
   virtual void set_out_of_range_pm(unsigned int address, unsigned int value);
   guint64 cycles_used(unsigned int address);
   virtual bool         IsAddressInRange(unsigned int address) {
-    return address < program_memory_size();
+    return address < program_address_limit();
   }
 
   // opcode_size - number of bytes for an opcode.
@@ -465,16 +476,17 @@ public:
   // First the source files:
 
   void attach_src_line(unsigned int address,
-		       unsigned int file_id,
-		       unsigned int sline,
-		       unsigned int lst_line);
+                       unsigned int file_id,
+                       unsigned int sline,
+                       unsigned int lst_line);
   void read_src_files(void);
 
 
   virtual void dump_registers(void);
   virtual instruction * disasm ( unsigned int address,unsigned int inst)=0;
 
-  virtual void initializeAttributes();
+  //virtual void initializeAttributes();
+
   //
   // Processor State 
   //
@@ -490,24 +502,25 @@ public:
   // Execution control
   //
 
-  virtual void run(bool refresh=true);
+  virtual void run(bool refresh=true) = 0;
   virtual void run_to_address(unsigned int destination);
-  virtual void finish(void);
+  virtual void finish(void) = 0;
 
   virtual void sleep(void) {};
-  virtual void step(unsigned int steps,bool refresh=true);
+  virtual void step(unsigned int steps,bool refresh=true) = 0;
   virtual void step_over(bool refresh=true);
   virtual void step_one(bool refresh=true) = 0;
+  virtual void step_cycle() = 0;
   virtual void interrupt(void) = 0 ;
 
   // Simulation modes
 
   /// setWarnMode - when true, gpsim will issue warnings whenever
-  /// suspicious is occuring.
+  /// something suspicious is occuring.
   virtual void setWarnMode(bool);
   virtual bool getWarnMode() { return bWarnMode; }
 
-  /// setSafeMode - when true, model will model the 'official'
+  /// setSafeMode - when true, gpsim will model the 'official'
   /// behavior of the chip. When false, the simulator behaves the same
   /// as the hardware.
   virtual void setSafeMode(bool);
@@ -552,6 +565,8 @@ public:
     return clocks_per_inst;
   }
 
+  void update_cps(void);
+
   virtual double get_OSCperiod();
 
   virtual double get_InstPeriod()
@@ -560,11 +575,11 @@ public:
   }
 
   virtual void disassemble (signed int start_address, 
-			    signed int end_address);
+                            signed int end_address);
   virtual void list(unsigned int file_id, 
-		    unsigned int pcval, 
-		    int start_line, 
-		    int end_line);
+                    unsigned int pcval, 
+                    int start_line, 
+                    int end_line);
 
   // Configuration control
 
@@ -577,8 +592,7 @@ public:
   // Processor reset
   // 
   
-  virtual void reset(RESET_TYPE r) {};
-  virtual void por(void) = 0;            // por = Power On Reset
+  virtual void reset(RESET_TYPE r) = 0;
 
   virtual double get_Vdd() { return Vdd; }
   virtual void set_Vdd(double v) { Vdd = v; }
@@ -602,7 +616,7 @@ public:
 
 private:
 
-  Float *mFrequency;
+  CPU_Freq *mFrequency;
 
   // Simulation modes
   bool bSafeMode;
@@ -611,6 +625,11 @@ private:
   bool bBreakOnReset;
   Boolean *m_pbBreakOnInvalidRegisterRead;
   Boolean *m_pbBreakOnInvalidRegisterWrite;
+  Boolean *m_pWarnMode;
+  Boolean *m_pSafeMode;
+  Boolean *m_pUnknownMode;
+  Boolean *m_pBreakOnReset;
+
 };
 
 
@@ -661,10 +680,14 @@ public:
   //
   ProcessorConstructor(
        tCpuContructor    _cpu_constructor,
-			 const char *name1, 
-			 const char *name2, 
-			 const char *name3=0,
-			 const char *name4=0);
+                         const char *name1, 
+                         const char *name2, 
+                         const char *name3=0,
+                         const char *name4=0);
+
+  virtual ~ProcessorConstructor()
+  {
+  }
 
   static ProcessorConstructorList * processor_list;
   static ProcessorConstructorList * GetList();
