@@ -225,46 +225,41 @@ char * RegisterValue::toBitStr(char *s, int len, unsigned int BitPos,
 // Member functions for the file_register base class
 //--------------------------------------------------
 //
-
-Register::Register(void)
-  : gpsimValue(0), alias_mask(0)
+Register::Register(Module *_cpu, const char *pName, const char *pDesc)
+  : Value(pName,pDesc,_cpu),
+    address(0xffffffff),
+    alias_mask(0)
 {
-  new_name("file_register");
 
   // For now, initialize the register with valid data and set that data equal to 0.
   // Eventually, the initial value will be marked as 'uninitialized.
 
   value = RegisterValue(0,0);
   por_value = RegisterValue(0,0);
-  //putRV_notrace(por_value);   /// Why is this here? 
-  _xref.assign_data(this);
+
+  xref().assign_data(this);
   read_access_count=0;
   write_access_count=0;
   bit_mask = 7;
 
 }
-//ugh duplication in constructors...
-Register::Register(Processor *_cpu)
-  : gpsimValue(_cpu)
+Register::~Register()
 {
-
-  new_name("file_register");
-
-  // For now, initialize the register with valid data and set that data equal to 0.
-  // Eventually, the initial value will be marked as 'uninitialized.
-
-  value = RegisterValue(0,0);
-  por_value = RegisterValue(0,0);
-  //putRV_notrace(por_value);
-  _xref.assign_data(this);
-  read_access_count=0;
-  write_access_count=0;
-  bit_mask = 7;
-
+  if (cpu) {
+    //cout << "Removing register from ST:" << name_str <<  " addr "<< this << endl;
+    cpu->removeSymbol(this);
+  }
 }
-Register::~Register(void)
-{
 
+//------------------------------------------------------------
+int Register::set_break(ObjectBreakTypes bt, ObjectActionTypes at, Expression *expr)
+{
+  return get_bp().set_break(bt, at, this, expr);
+}
+
+int Register::clear_break()
+{
+  return -1;
 }
 
 //------------------------------------------------------------
@@ -280,7 +275,7 @@ Register::~Register(void)
 //  on the type of break point, this get() may
 //  or may not get called).
 
-unsigned int Register::get(void)
+unsigned int Register::get()
 {
   trace.raw(read_trace.get() | value.get());
   return(value.get());
@@ -379,41 +374,9 @@ void Register::put_value(unsigned int new_value)
 /// New accessor functions
 //////////////////////////////////////////////////////////////
 
-//------------------------------------------------------------
-
-///  SimulatedGet()
-///  Return the contents of the file register.
-///  (note - breakpoints on file register reads
-///  are not checked here. Instead, a breakpoint
-///  object replaces those instances of file 
-///  registers for which we wish to monitor.
-///  So a file_register::SimulatedGet call will invoke
-///  the breakpoint::get member function. Depending
-///  on the type of break point, this SimulatedGet() may
-///  or may not get called).
-
-unsigned int Register::SimulatedGet(void)
-{
-  // for now call the old API
-  return get();
-}
-
-//------------------------------------------------------------
-///  SimulatedSet()
-///  Update the contents of the register.
-///  See the comment above in Register::SimulatedGet()
-///  with respect to break points
-//
-
-void Register::SimulatedSet(unsigned int new_value)
-{
-  // for now call the old API
-  put(new_value);
-}
-
 unsigned int Register::register_size () const
 { 
-  Processor *pProc = gpsimValue::get_cpu();
+  Processor *pProc = Value::get_cpu();
   return pProc == 0 ? 1 : pProc->register_size();
 }
 
@@ -448,19 +411,76 @@ char * Register::toBitStr(char *s, int len)
   return getRV_notrace().toBitStr(s,len,bits);
 }
 
+//-----------------------------------------------------------
 
-//--------------------------------------------------
-//--------------------------------------------------
-//--------------------------------------------------
-sfr_register::sfr_register() 
-  : Register(), wdtr_value(0,0)
-{}
-
-sfr_register::sfr_register(Processor *_cpu)
-  : Register(_cpu), wdtr_value(0,0)
+void Register::new_name(const char *s)
 {
+
+  if(s) {
+    string str(s);
+    new_name(str);
+  }
 }
 
+void Register::new_name(string &new_name)
+{
+  if (name_str != new_name) {
+    if (name_str.empty()) {
+      name_str = new_name;
+      return;
+    }
+
+    if (cpu) {
+      addName(new_name);
+      cpu->addSymbol(this, &new_name);
+    }
+    
+  }
+}
+//------------------------------------------------------------------------
+// set -- assgin the value of some other object to this Register
+//
+// This is used (primarily) during Register stimuli processing. If 
+// a register stimulus is attached to this register, then it will
+// call ::set() and supply a Value pointer.
+
+void Register::set(Value * pVal)
+{
+  Register *pReg = dynamic_cast<Register *>(pVal);
+  if (pReg) {
+    putRV(pReg->getRV());
+    return;
+  }
+
+  if (pVal) {
+    put_value( (unsigned int)*pVal);
+  }
+}
+
+//------------------------------------------------------------------------
+// copy - create a new Value object that's a 'copy' of this object
+//
+// We really don't perform a true copy. Instead, an Integer object 
+// is created containing the same numeric value of this object. 
+// This code is called during expression parsing. *NOTE* this copied
+// object can be assigned a new value, however that value will not
+// propagate to the Register! 
+
+Value *Register::copy()
+{
+  return new ValueWrapper(this);
+}
+void Register::get(gint64 &i)
+{
+  i = get_value();
+}
+//--------------------------------------------------
+//--------------------------------------------------
+//--------------------------------------------------
+sfr_register::sfr_register(Module *pCpu, const char *pName, const char *pDesc) 
+  : Register(pCpu,pName,pDesc), wdtr_value(0,0)
+{}
+
 void sfr_register::reset(RESET_TYPE r)
 {
   switch (r) {
@@ -502,7 +522,7 @@ void InvalidRegister::put(unsigned int new_value)
   return;
 }
 
-unsigned int InvalidRegister::get(void)
+unsigned int InvalidRegister::get()
 {
   cout << "attempt read from invalid file register\n";
   if (address != AN_INVALID_ADDRESS)
@@ -517,21 +537,9 @@ unsigned int InvalidRegister::get(void)
 }
 
 
-InvalidRegister::InvalidRegister(unsigned int at_address)
-  : Register()
-{
-
-  char name_str[100];
-  sprintf (name_str, "INVREG_%X", at_address);
-  new_name(name_str);
-  address = at_address;
-}
-
-InvalidRegister::InvalidRegister(void)
-{
-  new_name("INVALID_REGISTER");
-  address = AN_INVALID_ADDRESS;
-}
+InvalidRegister::InvalidRegister(Processor *pCpu, const char *pName, const char *pDesc) 
+  : Register(pCpu,pName,pDesc)
+{}
 
 RegisterCollection::RegisterCollection (Processor   *pProcessor, 
 					const char  *pC_collection_name,
@@ -540,10 +548,16 @@ RegisterCollection::RegisterCollection (Processor   *pProcessor,
   IIndexedCollection(16), m_ReturnValue(0) 
 {
   m_pProcessor = pProcessor;
-  Value::new_name(pC_collection_name);
+  gpsimObject::new_name(pC_collection_name);
   m_ppRegisters = ppRegisters;
   m_uSize = uiSize;
-  get_symbol_table().add(this);
+  pProcessor->addSymbol(this);
+}
+
+RegisterCollection:: ~RegisterCollection()
+{
+  if (m_pProcessor)
+    m_pProcessor->removeSymbol(this);
 }
 
 unsigned int RegisterCollection::GetSize() {
@@ -557,7 +571,11 @@ Value &RegisterCollection::GetAt(unsigned int uIndex, Value *) {
   m_ReturnValue.set((int)m_ppRegisters[uIndex]->get_value());
   m_ReturnValue.setBitmask(m_pProcessor->register_mask());
   ostringstream sIndex;
-  sIndex << Value::name() << "[" << hex << m_szPrefix << uIndex << "]" << '\000';
+  if (m_pProcessor)
+      sIndex << m_pProcessor->name() << "." ;
+
+  sIndex << Value::name() << "[" 
+	  << hex << m_szPrefix << uIndex << "]" << '\000';
   m_ReturnValue.new_name(sIndex.str().c_str());
   return m_ReturnValue;
 }
