@@ -49,6 +49,7 @@ Boston, MA 02111-1307, USA.  */
 #include "sim_context.h"
 #include "breakpoints.h"
 #include "trace.h"
+#include "symbol.h"
 
 //================================================================================
 // Global Declarations
@@ -66,22 +67,39 @@ Boston, MA 02111-1307, USA.  */
 //
 //
 
-CSimulationContext::CSimulationContext() :
-  m_bEnableLoadSource(*new Boolean("EnableSourceLoad", true,
-    "Enables and disables loading of source code")) {
+CSimulationContext::CSimulationContext() 
+  :  m_bEnableLoadSource(*new Boolean("EnableSourceLoad", true,
+                                      "Enables and disables loading of source code")) 
+{
   active_cpu_id = 0;
   cpu_ids = 0;
-  m_bEnableLoadSource.setClearableSymbol(false);
   m_pbUserCanceled = NULL;
+  globalSymbolTable().addSymbol(&m_bEnableLoadSource);
 }
 
-void CSimulationContext::Initialize() {
-  get_symbol_table().add(&m_bEnableLoadSource);
+CSimulationContext::~CSimulationContext() 
+{
+  globalSymbolTable().deleteSymbol("EnableSourceLoad");
 }
 
-CSimulationContext *CSimulationContext::s_SimulationContext = new CSimulationContext();
+void CSimulationContext::Initialize()
+{
+}
 
-CSimulationContext *CSimulationContext::GetContext() {
+//------------------------------------------------------------
+// The static pointer 's_SimulationContext' in CSimulationContext
+// is initialized to 0 here, but initialized to something valid 
+// when GetContext is called.
+
+CSimulationContext *CSimulationContext::s_SimulationContext = 0;
+
+CSimulationContext *CSimulationContext::GetContext()
+{
+  // Declare static CSimulationContext and return a pointer to it.
+  // Note, we could 'new' an object, but there's no way to call the
+  // destructor in that case.
+  static CSimulationContext sContext;
+  s_SimulationContext = &sContext;
   return s_SimulationContext;
 }
 
@@ -115,7 +133,10 @@ Processor * CSimulationContext::SetProcessorByType(const char * processor_type,
   Processor *p;
   CProcessorList::iterator it = processor_list.findByType(string(processor_type));
   GetBreakpoints().clear_all(GetActiveCPU());
-  GetSymbolTable().Reinitialize();
+
+  cout << __FUNCTION__ << " FIXME \n";
+  // GetSymbolTable().Reinitialize();
+
   if(processor_list.end() == it) {
     p = add_processor(processor_type,processor_new_name);
   }
@@ -161,13 +182,13 @@ Processor * CSimulationContext::add_processor(ProcessorConstructor *pc,
 Processor * CSimulationContext::add_processor(Processor *p)
 {
     processor_list.insert(CProcessorList::value_type(p->name(), p));
-    p->initializeAttributes();
+    //p->initializeAttributes();
     active_cpu = p;
     //p->processor_id = 
     active_cpu_id = ++cpu_ids;
     if(verbose) {
       cout << p->name() << '\n';
-      cout << "Program Memory size " <<  p->program_memory_size() << '\n';
+      cout << "Program Memory size " <<  p->program_memory_size() << " words\n";
       cout << "Register Memory size " <<  p->register_memory_size() << '\n';
     }
 
@@ -257,21 +278,24 @@ void CSimulationContext::dump_processor_list(void)
 
 }
 
-void CSimulationContext::Clear() {
-  GetBreakpoints().clear_all(GetActiveCPU());
+void CSimulationContext::Clear() 
+{
+
   CProcessorList::iterator processor_iterator; 
   for (processor_iterator = processor_list.begin();
        processor_iterator != processor_list.end(); 
-       processor_iterator++) {
-      CProcessorList::value_type vt = *processor_iterator;
-      Processor *p = vt.second;
-      delete p;
-    }
-  GetSymbolTable().clear_all();
+       ++processor_iterator) {
+    CProcessorList::value_type vt = *processor_iterator;
+    Processor *p = vt.second;
+    GetBreakpoints().clear_all(p);
+    delete p;
+  }
   processor_list.clear();
 }
 
-void CSimulationContext::Reset(RESET_TYPE r) {
+void CSimulationContext::Reset(RESET_TYPE r)
+{
+  /*
   Symbol_Table &ST = get_symbol_table();
   Symbol_Table::module_symbol_iterator it;
   Symbol_Table::module_symbol_iterator itEnd = ST.endModuleSymbol();
@@ -281,6 +305,8 @@ void CSimulationContext::Reset(RESET_TYPE r) {
         m->reset(r);
       }
   }
+  */
+  cout << __FUNCTION__ << " FIXME \n";
 }
 
 void CSimulationContext::NotifyUserCanceled() {
@@ -297,9 +323,9 @@ void CSimulationContext::NotifyUserCanceled() {
   }
 }
 
-extern Symbol_Table symbol_table;  // There's only one instance of "the" symbol table
-Symbol_Table & CSimulationContext::GetSymbolTable() {
-  return symbol_table;
+SymbolTable & CSimulationContext::GetSymbolTable() 
+{
+  return gSymbolTable;
 }
 
 Breakpoints & CSimulationContext::GetBreakpoints() {
