@@ -36,6 +36,12 @@ Boston, MA 02111-1307, USA.  */
 // SSPSTAT - Synchronous Serial Port Status register.
 
 
+_SSPSTAT::_SSPSTAT(Processor *pCpu, SSP_MODULE *pSSP)
+  : sfr_register(pCpu, "sspstat","Synchronous Serial Port Status"),
+    m_sspmod(pSSP)
+{
+}
+
 /*
 	only CKE and SMP is writable
 */
@@ -61,14 +67,6 @@ void _SSPSTAT::put_value(unsigned int new_value)
   value.put(new_value);
 
 }
-/*
-	register pointer to SSPMOD
-*/
-void _SSPSTAT::setSSPMODULE(SSP_MODULE *sspmod)
-{
-  m_sspmod = sspmod;
-}
-
 class SDI_SignalSink : public SignalSink
 {
 public:
@@ -78,6 +76,7 @@ public:
     assert(_ssp_mod);
   }
   virtual ~SDI_SignalSink(){}
+  virtual void release(){}
 
   void setSinkState(char new3State)
   {
@@ -96,6 +95,7 @@ public:
     assert(_ssp_mod);
   }
   virtual ~SCL_SignalSink(){}
+  virtual void release(){}
 
   void setSinkState(char new3State)
   {
@@ -113,6 +113,7 @@ public:
     assert(_ssp_mod);
   }
   virtual ~SS_SignalSink(){}
+  virtual void release(){}
 
   void setSinkState(char new3State)
   {
@@ -127,20 +128,11 @@ private:
 // SSPCON - Synchronous Serial Port Control register.
 //-----------------------------------------------------------
 
-_SSPCON::_SSPCON()
+_SSPCON::_SSPCON(Processor *pCpu, SSP_MODULE *pSSP)
+  : sfr_register(pCpu, "sspcon","Synchronous Serial Port Control"),
+    m_sspmod(pSSP)
 {
 }
-/*
-	register pointer to SSPMOD
-*/
-void _SSPCON::setSSPMODULE(SSP_MODULE *sspmod)
-{
-  m_sspmod = sspmod;
-}
-
-/*
-	return true if an SPI mode is enabled
-*/
 bool _SSPCON::isSPIActive(unsigned int value)
 {
     if (value & SSPEN)
@@ -253,8 +245,9 @@ bool _SSPCON::isI2CSlave(unsigned int val)
 // SSPBUF - Synchronous Serial Port Control register.
 //-----------------------------------------------------------
 
-_SSPBUF::_SSPBUF()
-  :  m_sspmod(0), m_bIsFull(false)
+_SSPBUF::_SSPBUF(Processor *pCpu, SSP_MODULE *pSSP)
+  : sfr_register(pCpu, "sspbuf","Synchronous Serial Port Buffer"),
+    m_sspmod(pSSP), m_bIsFull(false)
 {
 }
 
@@ -265,6 +258,7 @@ void _SSPBUF::put(unsigned int new_value)
 {
   put_value(new_value);
   m_sspmod->newSSPBUF(value.get());
+  m_bIsFull = false;
 }
 
 /*
@@ -276,12 +270,12 @@ void _SSPBUF::put_value(unsigned int new_value)
   trace.raw(write_trace.get() | value.get());
   value.put(new_value & 0xff);
 }
-
+/*
 void _SSPBUF::setSSPMODULE( SSP_MODULE *_sspmod)
 {
   m_sspmod = _sspmod;
 }
-
+*/
 unsigned int _SSPBUF::get()
 {
   if( m_sspmod )
@@ -302,12 +296,12 @@ unsigned int _SSPBUF::get_value()
 //-----------------------------------------------------------
 // SSPADD - Synchronous Serial Port Address (for I2C)
 //-----------------------------------------------------------
-
-
-void _SSPADD::setSSPMODULE( SSP_MODULE *_sspmod)
+_SSPADD::_SSPADD(Processor *pCpu, SSP_MODULE *pSSP)
+  : sfr_register(pCpu, "sspadd","Synchronous Serial Port Address (I2C)"),
+    m_sspmod(pSSP)
 {
-  m_sspmod = _sspmod;
 }
+
 void _SSPADD::put(unsigned int new_value)
 {
   trace.raw(write_trace.get() | value.get());
@@ -339,7 +333,7 @@ void SPI::clock( bool ClockState )
   unsigned int sspstat_val = m_sspstat->value.get();
   unsigned int sspcon_val = m_sspcon->value.get();
 
-cout << "SPi clock " << ClockState << " m_state=" << m_state << endl;
+  //cout << "SPi clock " << ClockState << " m_state=" << m_state << endl;
 
   if( ClockState ) // rising edge
   {
@@ -450,14 +444,15 @@ void SPI::set_halfclock_break()
     break;
   }
   
-  get_cycles().set_break(get_cycles().value + clock_in_cycles, this);
+  get_cycles().set_break(get_cycles().get() + clock_in_cycles, this);
 }
 void SPI::callback()
 {
   if (!m_sspmod)
     return;
 
-  cout << "SPI callback m_state=" << m_state << endl;
+  if (verbose)
+    cout << "SPI callback m_state=" << m_state << endl;
 
   switch( m_state ) {
   case eIDLE:
@@ -491,8 +486,13 @@ void SPI::startSPI()
     bits_transfered = 0;
 }
 
-SSP_MODULE::SSP_MODULE()
-  : m_pirset(0),
+SSP_MODULE::SSP_MODULE(Processor *pCpu)
+  : sspbuf(pCpu,this),
+    sspcon(pCpu,this),
+    sspstat(pCpu,this),
+    sspcon2(pCpu,this),
+    sspadd(pCpu,this),
+    m_pirset(0),
     m_spi(0),
     m_i2c(0),
     m_sck(0),
@@ -508,14 +508,10 @@ SSP_MODULE::SSP_MODULE()
     m_SS_Sink(0),
     m_sink_set(false)
 {
-  sspbuf.setSSPMODULE(this);
-  sspcon.setSSPMODULE(this);
-  sspcon2.setSSPMODULE(this);
-  sspstat.setSSPMODULE(this);
-  sspadd.setSSPMODULE(this);
-
 }
-SSP_MODULE::~SSP_MODULE(){}
+SSP_MODULE::~SSP_MODULE()
+{
+}
 
 void SPI::newSSPBUF(unsigned int newTxByte)
 {
@@ -584,7 +580,7 @@ void SPI::stop_transfer()
       if (verbose)
           cout << "SPI: Stopping transfer. SSPBUF Overflow setting SSPOV." << endl;
         m_sspcon->setSSPOV();
-
+        m_sspmod->set_sspif();      // The real PIC sets sspif even with overflow
     } else {
       cout << "SPI: Stopping transfer. Cancel finish." << endl;
       // The transfer was canceled in some way
@@ -650,10 +646,10 @@ void I2C::callback()
 
     if (verbose & 2)
         cout << "I2C::callback i2c_state " << i2c_state << " phase=" << phase <<endl;
-    if (future_cycle != get_cycles().value)
+    if (future_cycle != get_cycles().get())
     {
 	cout << "I2C program error future_cycle=" << future_cycle << " now=" 
-		<< get_cycles().value << " i2c_state=" << i2c_state << endl;
+		<< get_cycles().get() << " i2c_state=" << i2c_state << endl;
     }
 
     future_cycle = 0;
@@ -760,7 +756,7 @@ void I2C::callback()
 		if (verbose & 2)
 		{
 		    cout << "I2C::callback CLK_TX_BYTE _ACK=" << n_ack <<
-		        " clock=" << get_cycles().value << endl;
+		        " clock=" << get_cycles().get() << endl;
 		}
 		if (n_ack)
 		    m_sspcon2->put_value(m_sspcon2->value.get() | _SSPCON2::ACKSTAT);
@@ -1096,7 +1092,7 @@ void I2C::setBRG()
 {
     if (future_cycle)
 	cout << "ERROR I2C::setBRG called with future_cycle=" << future_cycle << endl;
-      future_cycle = get_cycles().value + 
+      future_cycle = get_cycles().get() + 
 	  	((m_sspadd->value.get() &0x7f)/ 2) + 1;
       get_cycles().set_break(future_cycle, this);
 }
@@ -1188,7 +1184,7 @@ void I2C::sda(bool data_val)
 	    case CLK_START:
 		if (phase == 0)
 		{	
-		    guint64 fc = get_cycles().value + 
+		    guint64 fc = get_cycles().get() + 
 			((m_sspadd->value.get() &0x7f)/ 2) + 1;
 
 		    if (future_cycle)
@@ -1697,14 +1693,12 @@ bool SSP_MODULE::SaveSSPsr(unsigned int value)
 }
 //-----------------------------------------------------------
 //-------------------------------------------------------------------
-
-/*
-	register pointer to SSPMOD
-*/
-void _SSPCON2::setSSPMODULE(SSP_MODULE *sspmod)
+_SSPCON2::_SSPCON2(Processor *pCpu, SSP_MODULE *pSSP)
+  : sfr_register(pCpu, "sspcon2","Synchronous Serial Port Control"),
+    m_sspmod(pSSP)
 {
-  m_sspmod = sspmod;
 }
+
 /*
 	write to SSPCON2 without processing data
 */
@@ -1755,7 +1749,3 @@ void _SSPCON2::put(unsigned int new_value)
 	
 }
 
-_SSPCON2::_SSPCON2(void)
-{
-}
- 
