@@ -50,6 +50,11 @@ class TraceObject
 {
 public:
   TraceObject();
+
+  virtual ~TraceObject()
+  {
+  }
+
   virtual void print(FILE *)=0;
   virtual void print_frame(TraceFrame *,FILE *);
   virtual void getState(TraceFrame *);   // FIXME Is this even used?
@@ -59,6 +64,11 @@ class CycleTraceObject : public TraceObject
 {
 public:
   CycleTraceObject();
+
+  virtual ~CycleTraceObject()
+  {
+  }
+
   virtual void print(FILE *);
   virtual void print_frame(TraceFrame *,FILE *);
   virtual void getState(TraceFrame *);
@@ -68,6 +78,11 @@ class InvalidTraceObject : public TraceObject
 {
 public:
   InvalidTraceObject(int type);
+
+  virtual ~InvalidTraceObject()
+  {
+  }
+
   virtual void print(FILE *);
 protected:
   int mType;
@@ -82,6 +97,10 @@ public:
   {
   }
 
+  virtual ~ProcessorTraceObject()
+  {
+  }
+
   virtual void print(FILE *)=0;
 };
 
@@ -93,16 +112,20 @@ public:
   ModuleTraceType *pModuleTraceType;
   unsigned int mTracedData;
 
-  ModuleTraceObject(Module *_module, 
-		    ModuleTraceType *pmtt,
-		    unsigned int d)
-    : TraceObject() , 
+  ModuleTraceObject(Module *_module,
+                    ModuleTraceType *pmtt,
+                    unsigned int d)
+    : TraceObject() ,
       pModule(_module),
       pModuleTraceType(pmtt),
       mTracedData(d)
   {
   }
 
+  virtual ~ModuleTraceObject()
+  {
+  }
+
   virtual void print(FILE *);
 };
 
@@ -114,6 +137,11 @@ public:
   RegisterValue to;
 
   RegisterWriteTraceObject(Processor *_cpu, Register *_reg, RegisterValue trv);
+
+  virtual ~RegisterWriteTraceObject()
+  {
+  }
+
   virtual void print(FILE *);
   virtual void getState(TraceFrame *);
 };
@@ -122,6 +150,11 @@ class RegisterReadTraceObject : public RegisterWriteTraceObject
 {
 public:
   RegisterReadTraceObject(Processor *_cpu, Register *_reg, RegisterValue trv);
+
+  virtual ~RegisterReadTraceObject()
+  {
+  }
+
   virtual void print(FILE *);
   virtual void getState(TraceFrame *);
 };
@@ -132,15 +165,20 @@ public:
   unsigned int address;
 
   PCTraceObject(Processor *_cpu, unsigned int _address);
+
+  virtual ~PCTraceObject()
+  {
+  }
+
   virtual void print(FILE *);
   virtual void print_frame(TraceFrame *,FILE *);
 };
 
 //========================================================================
 // The TraceType class is the base class for the various trace types that
-// are supported by gpsim. In general, when a trace type is created, 
+// are supported by gpsim. In general, when a trace type is created,
 // a 32 bit identifier is created. The upper byte of this identifier is
-// the "type" of the trace and is dynamically allocated. The lower 24 
+// the "type" of the trace and is dynamically allocated. The lower 24
 // bits of this identifer are 0. When a TraceType is traced, the lower
 // 24 bits are filled with the information that is to be recorded in
 // the trace buffer. The whole 32 bits are then written to the trace buffer
@@ -151,14 +189,18 @@ class TraceType
 public:
 
 
-  TraceType(unsigned int nTraceEntries);
+  TraceType(unsigned int nTraceEntries, const char *desc);
+
+  virtual ~TraceType()
+  {
+  }
 
   void setType(unsigned int t) { mType = t;}
   // The actual type of the TraceType is an 8-bit field in the
   // upper 32-bits of an unsigned integer. A TraceType can have
   // more than one type, although the types are consecutive.
   unsigned int type(unsigned int iType=0)
-  { 
+  {
     return (iType<mSize) ? (mType + (iType<<24)) : mType;
   }
   unsigned int size() { return mSize; }
@@ -168,7 +210,7 @@ public:
   // and attempt to decode them. In addition, the index is
   // incremented by the number of trace entries this type
   // used in the trace buffer (note, that this may not equal
-  // to the allocated mSize). 
+  // to the allocated mSize).
 
   virtual TraceObject *decode(unsigned int tbi) = 0;
 
@@ -176,7 +218,7 @@ public:
   // will return the number of trace buffer entries that
   // were used to record this TraceType event. The default
   // is the size allocated for this type.
-  
+
   virtual int entriesUsed(Trace *,unsigned int tbi);
 
   virtual bool isFrameBoundary() { return false;}
@@ -191,11 +233,17 @@ public:
   }
   virtual int bitsTraced() { return 24; }
   virtual int dump_raw(Trace *,unsigned tbi, char *buf, int bufsize);
+
+  // Debugging - provide a way to see the TraceTypes that have been allocated.
+  virtual void showInfo();
+  const char *cpDescription();
 private:
-  unsigned int mType;		// The integer type is dynamically
-				// assigned by the Trace class.
-  unsigned int mSize;		// The number of positions this
-				// type occupies
+  unsigned int mType;           // The integer type is dynamically
+                                // assigned by the Trace class.
+  unsigned int mSize;           // The number of positions this
+                                // type occupies
+protected:
+  const char *mpDescription;    //
 
 };
 
@@ -203,6 +251,11 @@ class CycleTraceType : public TraceType
 {
 public:
   CycleTraceType(unsigned int nTraceEntries);
+
+  virtual ~CycleTraceType()
+  {
+  }
+
   virtual TraceObject *decode(unsigned int tbi);
   virtual bool isFrameBoundary();
   virtual int dump_raw(Trace *,unsigned tbi, char *buf, int bufsize);
@@ -213,11 +266,11 @@ class ModuleTraceType : public TraceType
 {
 public:
   Module *pModule;
-  const char *pDescription;
-  ModuleTraceType(Module *_pModule, 
-		  unsigned int nTraceEntries,
-		  const char *desc)
-    : TraceType(nTraceEntries), pModule(_pModule), pDescription(desc)
+  ModuleTraceType(Module *_pModule,
+                  unsigned int nTraceEntries,
+                  const char *desc);
+
+  virtual ~ModuleTraceType()
   {
   }
 
@@ -231,9 +284,11 @@ class ProcessorTraceType : public TraceType
 public:
   Processor *cpu;
 
-  ProcessorTraceType(Processor *_cpu, 
-		     unsigned int nTraceEntries)
-    : TraceType(nTraceEntries), cpu(_cpu)
+  ProcessorTraceType(Processor *_cpu,
+                     unsigned int nTraceEntries,
+                     const char *pDesc);
+
+  virtual ~ProcessorTraceType()
   {
   }
 
@@ -245,6 +300,11 @@ class PCTraceType : public ProcessorTraceType
 {
 public:
   PCTraceType(Processor *_cpu, unsigned int nTraceEntries);
+
+  virtual ~PCTraceType()
+  {
+  }
+
   virtual TraceObject *decode(unsigned int tbi);
   virtual bool isFrameBoundary() { return true; }
   virtual int dump_raw(Trace *,unsigned tbi, char *buf, int bufsize);
@@ -256,6 +316,10 @@ public:
 
   RegisterWriteTraceType(Processor *_cpu, unsigned int nTraceEntries);
 
+  virtual ~RegisterWriteTraceType()
+  {
+  }
+
   virtual TraceObject *decode(unsigned int tbi);
   virtual int dump_raw(Trace *,unsigned tbi, char *buf, int bufsize);
 };
@@ -266,10 +330,47 @@ public:
 
   RegisterReadTraceType(Processor *_cpu, unsigned int nTraceEntries);
 
+  virtual ~RegisterReadTraceType()
+  {
+  }
+
   virtual TraceObject *decode(unsigned int tbi);
   virtual int dump_raw(Trace *,unsigned tbi, char *buf, int bufsize);
 };
 
+// Trace Type for Resets
+
+class ResetTraceObject : public ProcessorTraceObject
+{
+public:
+  ResetTraceObject(Processor *_cpu, RESET_TYPE r);
+
+  virtual ~ResetTraceObject()
+  {
+  }
+
+  virtual void print(FILE *fp);
+protected:
+  RESET_TYPE m_reset;
+};
+
+class ResetTraceType : public ProcessorTraceType
+{
+public:
+  ResetTraceType(Processor *_cpu);
+
+  virtual ~ResetTraceType()
+  {
+  }
+
+  TraceObject *decode(unsigned int tbi);
+  void record(RESET_TYPE r);
+  int dump_raw(Trace *pTrace,unsigned int tbi, char *buf, int bufsize);
+
+  unsigned int m_uiTT;
+};
+
+
 //========================================================================
 // TraceFrame
 //
@@ -283,7 +384,7 @@ public:
   list <TraceObject *> traceObjects;
   guint64 cycle_time;
 
-  TraceFrame(); 
+  TraceFrame();
   virtual ~TraceFrame();
   virtual void add(TraceObject *to);
   virtual void print(FILE *);
@@ -299,7 +400,7 @@ public:
   FILE *log_file;
 
   void log();
-  void enable(char*);
+  void enable(const char*);
   void disable();
 
   TraceRawLog();
@@ -307,12 +408,12 @@ public:
 
 };
 //------------------------------------------------------------
-class traceValue : public gpsimValue
+class traceValue : public gpsimObject
 {
  public:
-  traceValue(void);
+  traceValue();
   virtual void put_value(unsigned int new_value) {};
-  virtual unsigned int get_value(void);
+  virtual unsigned int get_value();
 };
 //---------------------------------------------------------
 // Class for trace buffer
@@ -323,12 +424,7 @@ class Trace
 
   enum eTraceTypes {
     NOTHING            =  0x3fffffff,
-    BREAKPOINT         = (2<<24),
-    INTERRUPT          = (3<<24),
-    _RESET             = (4<<24),
-    WRITE_TRIS         = (5<<24),
-    OPCODE_WRITE       = (7<<24),
-    LAST_TRACE_TYPE    = (8<<24),
+    LAST_TRACE_TYPE    = (1<<24),
 
     TYPE_MASK          = (0xff<<24),
     CYCLE_COUNTER_LO   = (0x80<<24),
@@ -378,7 +474,7 @@ class Trace
     trace_buffer[trace_index] = ui;
     trace_index = (trace_index + 1) & TRACE_BUFFER_MASK;
   }
-
+  /*
   inline void opcode_write (unsigned int address, unsigned int opcode)
   {
     trace_buffer[trace_index] = OPCODE_WRITE | (address & 0xffffff);
@@ -386,7 +482,7 @@ class Trace
     trace_buffer[trace_index] = OPCODE_WRITE | (opcode & 0xffff);
     trace_index = (trace_index + 1) & TRACE_BUFFER_MASK;
   }
-
+  */
 
   inline void cycle_counter (guint64 cc)
   {
@@ -396,29 +492,6 @@ class Trace
     trace_buffer[trace_index] = (unsigned int)(CYCLE_COUNTER_HI | (cc>>32) | (cc & CYCLE_COUNTER_LO));
     trace_index = (trace_index + 1) & TRACE_BUFFER_MASK;
   }
-  inline void breakpoint (unsigned int bp)
-  {
-    trace_buffer[trace_index] = BREAKPOINT | bp;
-    trace_index = (trace_index + 1) & TRACE_BUFFER_MASK;
-  }
-
-  void interrupt (void)
-  {
-    trace_buffer[trace_index] = INTERRUPT;
-    trace_index = (trace_index + 1) & TRACE_BUFFER_MASK;
-  }
-  inline void reset (RESET_TYPE r)
-    {
-      trace_buffer[trace_index] = _RESET | ( (int) r);
-      trace_index = (trace_index + 1) & TRACE_BUFFER_MASK;
-    }
-
-  inline void write_TRIS (unsigned int value)
-  {
-    trace_buffer[trace_index] = WRITE_TRIS | value;
-    trace_index = (trace_index + 1) & TRACE_BUFFER_MASK;
-  }
-
   inline bool near_full(void) {
     return (trace_index > TRACE_BUFFER_NEAR_FULL);
   }
@@ -436,14 +509,14 @@ class Trace
     return index & TRACE_BUFFER_MASK;
   }
 
-  // inRange - returns true if the trace index i is between the 
-  // indices of low and high. 
+  // inRange - returns true if the trace index i is between the
+  // indices of low and high.
   // It's assumed that the range does not exceed half of the trace buffer
   bool inRange(unsigned int i, unsigned int low, unsigned int high)
   {
     i = tbi(i);
 
-    if( low < high) 
+    if( low < high)
       return (i >= low && i <= high);
     // Looks like the range straddles the roll over boundary.
     return (i >= low || i <= high);
@@ -471,7 +544,7 @@ class Trace
   int is_cycle_trace(unsigned int index, guint64 *cvt_cycle);
 
   // When logging is enabled, the entire trace buffer will be copied to a file.
-  void enableLogging(char *fname);
+  void enableLogging(const char *fname);
   void disableLogging();
 
   unsigned int allocateTraceType(TraceType *);
@@ -482,6 +555,8 @@ class Trace
   void deleteTraceFrame(void);
   void printTraceFrame(FILE *);
 
+  // Display information about allocated traces.
+  void showInfo();
 
 };
 
@@ -595,7 +670,7 @@ extern ProfileKeeper profile_keeper;
  *
  * Each small buffer is associated with a contiguous time span. The
  * start and end of this span is recorded so that one can quickly
- * ascertain if a certain time instant resideds in the buffer. 
+ * ascertain if a certain time instant resideds in the buffer.
  *
  * The binary tree is fairly standard. A single top node records three
  * numbers: the start time for the left child, the end time for the
@@ -617,9 +692,9 @@ extern ProfileKeeper profile_keeper;
  * before the buffer fills, then this buffer is closed and added to the
  * binary and a new buffer is started.
  *
- * Repeated events are not logged. E.g.. if two 1's are logged, the 
+ * Repeated events are not logged. E.g.. if two 1's are logged, the
  * second one is ignored.
- * 
+ *
  */
 
 class BoolEventBuffer : public TriggerObject
