@@ -45,7 +45,7 @@ class BreakTraceType : public TraceType
 public:
 
   BreakTraceType()
-    : TraceType(2)
+    : TraceType(2, "Break")
   {
   }
 
@@ -119,6 +119,10 @@ TriggerAction::TriggerAction()
 {
 }
 
+TriggerAction::~TriggerAction()
+{
+}
+
 bool TriggerAction::evaluate()
 {
   action();
@@ -182,6 +186,20 @@ TriggerObject::TriggerObject(TriggerAction *ta)
     set_action(&DefaultTrigger);
 }
 
+TriggerObject::~TriggerObject()
+{
+  /*
+  cout << "Trigger Object destructor\n";
+  if (m_PExpr)
+    cout << "deleting expression "<< m_PExpr->toString() << endl;
+  */
+  delete m_PExpr;
+
+  if (m_action != &DefaultTrigger)
+    delete m_action;
+
+}
+
 void TriggerObject::callback()
 {
   cout << "generic callback\n";
@@ -192,6 +210,10 @@ void TriggerObject::callback_print()
   cout << " has callback, ID =  " << CallBackID << '\n';
 }
 
+void  TriggerObject::clear_trigger()
+{
+}
+
 int TriggerObject::find_free()
 {
   bpn = bp.find_free();
