@@ -38,7 +38,7 @@ Boston, MA 02111-1307, USA.  */
 #endif
 
 //--------------------------------------------------
-// 
+//
 //--------------------------------------------------
 class TXSignalSource : public SignalControl
 {
@@ -48,10 +48,17 @@ public:
   {
     assert(m_txsta);
   }
+  ~TXSignalSource()
+  {
+  }
   char getState()
   {
     return m_txsta->getState();
   }
+  void release()
+  {
+    delete this;
+  }
 private:
   _TXSTA *m_txsta;
 };
@@ -73,43 +80,75 @@ public:
   {
     m_rcsta->setState(new3State);
   }
+  void release()
+  {
+    delete this;
+  }
 private:
   _RCSTA *m_rcsta;
 };
 
 //-----------------------------------------------------------
-_RCSTA::_RCSTA(USART_MODULE *pUSART)
-  : rcreg(0), spbrg(0), txsta(0), mUSART(pUSART), m_PinModule(0), m_sink(0), m_cRxState('?')
+_RCSTA::_RCSTA(Processor *pCpu, const char *pName, const char *pDesc, USART_MODULE *pUSART)
+  : sfr_register(pCpu, pName, pDesc), rcreg(0), spbrg(0),
+    txsta(0),
+    state(_RCSTA::RCSTA_DISABLED),
+    mUSART(pUSART),
+    m_PinModule(0), m_sink(0), m_cRxState('?')
 {
   assert(mUSART);
 }
 
+_RCSTA::~_RCSTA()
+{
+}
+
 //-----------------------------------------------------------
-_TXSTA::_TXSTA(USART_MODULE *pUSART)
-  : txreg(0), spbrg(0), mUSART(pUSART), m_PinModule(0),  m_source(0), m_cTxState('?')
+_TXSTA::_TXSTA(Processor *pCpu, const char *pName, const char *pDesc, USART_MODULE *pUSART)
+  : sfr_register(pCpu, pName, pDesc), txreg(0), spbrg(0),
+    mUSART(pUSART),
+    m_PinModule(0),
+    m_source(0), m_cTxState('?'), bInvertPin(0)
 {
   assert(mUSART);
 }
 
+_TXSTA::~_TXSTA()
+{
+}
+
 //-----------------------------------------------------------
-_RCREG::_RCREG(USART_MODULE *pUSART)
-  : mUSART(pUSART), m_rcsta(0)
+_RCREG::_RCREG(Processor *pCpu, const char *pName, const char *pDesc, USART_MODULE *pUSART)
+  : sfr_register(pCpu, pName, pDesc), mUSART(pUSART), m_rcsta(0)
 {
   assert(mUSART);
 }
 
-_TXREG::_TXREG(USART_MODULE *pUSART)
-  : m_txsta(0), mUSART(pUSART)
+_TXREG::_TXREG(Processor *pCpu, const char *pName, const char *pDesc, USART_MODULE *pUSART)
+  : sfr_register(pCpu, pName, pDesc), m_txsta(0), mUSART(pUSART)
 {
   assert(mUSART);
 }
 
 
-_SPBRG::_SPBRG()
-  : txsta(0), rcsta(0)
+_BAUDCON::_BAUDCON(Processor *pCpu, const char *pName, const char *pDesc)
+  : sfr_register(pCpu, pName, pDesc)
 {
 }
 
+_SPBRG::_SPBRG(Processor *pCpu, const char *pName, const char *pDesc)
+  : sfr_register(pCpu, pName, pDesc),
+    txsta(0), rcsta(0), brgh(0), baudcon(0)
+{
+}
+
+
+_SPBRGH::_SPBRGH(Processor *pCpu, const char *pName, const char *pDesc)
+  : sfr_register(pCpu, pName, pDesc), m_spbrg(0)
+{
+}
+
+
 //-----------------------------------------------------------
 // TXREG - USART Transmit Register
 
@@ -133,7 +172,10 @@ void _TXREG::put(unsigned int new_value)
       // If the transmit buffer is empty and the transmitter is enabled
       // then transmit this new data now...
 
-      m_txsta->start_transmitting();
+      if ( m_txsta->value.get() & _TXSTA::SENDB )   // %%%FIXME - better in start_transmitting?
+        m_txsta->transmit_break();
+      else
+        m_txsta->start_transmitting();
     }
 
 }
@@ -166,7 +208,7 @@ void _TXSTA::setIOpin(PinModule *newPinModule)
 void _TXSTA::putTXState(char newTXState)
 {
 
-  m_cTxState = newTXState;
+  m_cTxState = bInvertPin ? newTXState ^ 1 : newTXState;
 
   if (m_PinModule)
     m_PinModule->updatePinModule();
@@ -192,6 +234,9 @@ void _TXSTA::put(unsigned int new_value)
 
   trace.raw(write_trace.get() | value.get());
 
+  if ( ! mUSART->IsEUSART() )
+      new_value &= ~SENDB;      // send break only supported on EUSART
+
   // The TRMT bit is controlled entirely by hardware.
   // It is high if the TSR has any data.
 
@@ -213,12 +258,12 @@ void _TXSTA::put(unsigned int new_value)
     if(value.get() & TXEN) {
       Dprintf(("TXSTA - enabling transmitter\n"));
       if (m_PinModule)
-	m_PinModule->setSource(m_source);
+        m_PinModule->setSource(m_source);
       mUSART->emptyTX();
     } else {
       stop_transmitting();
       if (m_PinModule)
-	m_PinModule->setSource(0);
+        m_PinModule->setSource(0);
 
     }
   }
@@ -247,13 +292,13 @@ void _TXSTA::stop_transmitting()
   // is set. In other words, when the Transmitter is enabled
   // the txreg is emptied (and hence TXIF set). But what happens
   // when TXEN is cleared? Should we clear TXIF too?
-  // 
+  //
   // There is one sentence that says when the transmitter is
   // disabled that the whole transmitter is reset. So I interpret
   // this to mean that the TXIF gets cleared. I could be wrong
   // (and I don't have a way to test it on a real device).
-  // 
-  // Another interpretation is that TXIF retains it state 
+  //
+  // Another interpretation is that TXIF retains it state
   // through changing TXEN. However, when SPEN (serial port) is
   // set then the whole usart is reinitialized and TXIF will
   // get set.
@@ -268,7 +313,7 @@ void _TXSTA::start_transmitting()
 
   // Build the serial byte that's about to be transmitted.
   // I doubt the pic really does this way, but gpsim builds
-  // the entire bit stream including the start bit, 8 data 
+  // the entire bit stream including the start bit, 8 data
   // bits, optional 9th data bit and the stop, and places
   // this into the tsr register. But since the contents of
   // the tsr are inaccessible, I guess we'll never know.
@@ -287,10 +332,10 @@ void _TXSTA::start_transmitting()
   //  of a new character waits for the next callback. This delay maybe,
   //  in fact, the stop bit of the previous transmision,
   //
-  //  [Recall that the TRMT bit indicates when the tsr 
+  //  [Recall that the TRMT bit indicates when the tsr
   //  {transmit shift register} is empty. It's not tied to
   //  an interrupt pin, so the pic application software
-  //  most poll this bit. 
+  //  most poll this bit.
   //
   //  RRR Also The following is wrong:
   //  This bit is set after the STOP
@@ -308,11 +353,11 @@ void _TXSTA::start_transmitting()
   if(value.get() & TX9)
     {
       // Copy the stop bit and the 9th data bit to the tsr.
-      // (See note above for the reason why two stop bits 
+      // (See note above for the reason why two stop bits
       // are appended to the packet.)
 
       tsr |= ( (value.get() & TX9D) ? (3<<9) : (2<<9));
-      bit_count = 11;  // 1 start, 9 data, 1 stop 
+      bit_count = 11;  // 1 start, 9 data, 1 stop
     }
   else
     {
@@ -320,7 +365,7 @@ void _TXSTA::start_transmitting()
       // for the reason why two stop bits are appended to
       // the packet.)
       tsr |= (1<<9);
-      bit_count = 10;  // 1 start, 8 data, 1 stop 
+      bit_count = 10;  // 1 start, 8 data, 1 stop
     }
 
 
@@ -329,7 +374,7 @@ void _TXSTA::start_transmitting()
   if(cpu)
     get_cycles().set_break(spbrg->get_cpu_cycle(1), this);
 
-  // The TSR now has data, so clear the Transmit Shift 
+  // The TSR now has data, so clear the Transmit Shift
   // Register Status bit.
 
   trace.raw(write_trace.get() | value.get());
@@ -341,13 +386,42 @@ void _TXSTA::start_transmitting()
 
 }
 
+void _TXSTA::transmit_break()
+{
+  Dprintf(("starting a USART sync-break transmission\n"));
+
+  // A sync-break is 13 consecutive low bits and one stop bit. Use the
+  // standard transmit logic to achieve this
+
+  if(!txreg)
+    return;
+
+  tsr = 1<<13;
+
+  bit_count = 14;  // 13 break, 1 stop
+
+  // Set a callback breakpoint at the next SPBRG edge
+  if(cpu)
+    get_cycles().set_break(spbrg->get_cpu_cycle(1), this);
+
+  // The TSR now has data, so clear the Transmit Shift
+  // Register Status bit.
+
+  trace.raw(write_trace.get() | value.get());
+  value.put(value.get() & ~TRMT);
+
+  // Tell the TXREG that its data has been consumed.
+
+  mUSART->emptyTX();
+}
+
 void _TXSTA::transmit_a_bit()
 {
 
 
   if(bit_count) {
 
-    Dprintf(("Transmit bit #%x: bit val:%d time:0x%llx\n",bit_count, (tsr&1),get_cycles().value));
+    Dprintf(("Transmit bit #%x: bit val:%d time:0x%llx\n",bit_count, (tsr&1),get_cycles().get()));
 
     putTXState(tsr&1 ? '1' : '0');
 
@@ -361,18 +435,20 @@ void _TXSTA::transmit_a_bit()
 
 void _TXSTA::callback()
 {
-  Dprintf(("TXSTA callback - time:%llx\n",get_cycles().value));
+  Dprintf(("TXSTA callback - time:%llx\n",get_cycles().get()));
 
   transmit_a_bit();
 
   if(!bit_count) {
 
+    value.put(value.get() & ~SENDB);
+
     // tsr is empty.
     // If there is any more data in the TXREG, then move it to
     // the tsr and continue transmitting other wise set the TRMT bit
 
     // (See the note above about the 'extra' stop bit that was stuffed
-    // into the tsr register. 
+    // into the tsr register.
 
     if(mUSART->bIsTXempty())
       value.put(value.get() | TRMT);
@@ -399,7 +475,7 @@ void _TXSTA::callback_print()
 // Receiver portion of the USART
 //-----------------------------------------------------------
 //
-// First RCSTA -- Receiver Control and Status 
+// First RCSTA -- Receiver Control and Status
 // The RCSTA class controls the usart reception. The PIC usarts have
 // two modes: synchronous and asynchronous.
 // Asynchronous reception:
@@ -459,18 +535,22 @@ void _RCSTA::put(unsigned int new_value)
     if( (value.get() & (SPEN | CREN)) == (SPEN | CREN) ) {
 
       // The receiver is enabled. Now check to see if that just happened
-      
-      if(diff & (SPEN | CREN)) {
-    
-	// The serial port has just been enabled. 
-	start_receiving();
 
-	// If the rx line is low, then go ahead and start receiving now.
-	//if(uart_port && !uart_port->get_bit(rx_bit))
-	if (m_cRxState == '0' || m_cRxState == 'w')
-	  receive_start_bit();
+      if(diff & (SPEN | CREN)) {
+
+        // The serial port has just been enabled.
+        start_receiving();
+
+        // If the rx line is low, then go ahead and start receiving now.
+        //if(uart_port && !uart_port->get_bit(rx_bit))
+        if (m_cRxState == '0' || m_cRxState == 'w')
+          receive_start_bit();
       }
 
+      // Clear overrun error on CREN enabling
+      if ( diff & CREN )
+         value.put( value.get() & (~OERR) );
+
     } else {
       // The serial port is not enabled.
 
@@ -529,8 +609,8 @@ void _RCSTA::setState(char new_RxState)
 //
 // A new bit needs to be copied to the the Receive Shift Register.
 // If the receiver is receiving data, then this routine will copy
-// the incoming bit to the rsr. If this is the last bit, then a 
-// check will be made to see if we need to set up for the next 
+// the incoming bit to the rsr. If this is the last bit, then a
+// check will be made to see if we need to set up for the next
 // serial byte.
 // If this is not the last bit, then the receive state machine.
 
@@ -539,7 +619,7 @@ void _RCSTA::receive_a_bit(unsigned int bit)
 
   // If we're waiting for the start bit and this isn't it then
   // we don't need to look any further
-  Dprintf(("receive_a_bit state:%d bit:%d time:0x%llx\n",state,bit,get_cycles().value));
+  Dprintf(("receive_a_bit state:%d bit:%d time:0x%llx\n",state,bit,get_cycles().get()));
 
   if( state == RCSTA_MAYBE_START) {
     if (bit)
@@ -559,6 +639,9 @@ void _RCSTA::receive_a_bit(unsigned int bit)
       if((value.get() & RX9) == 0)
         rsr >>= 1;
 
+      // Clear any framing error
+       value.put(value.get() & (~FERR) );
+
       // copy the rsr to the fifo
       if(rcreg)
         rcreg->push( rsr & 0xff);
@@ -566,7 +649,12 @@ void _RCSTA::receive_a_bit(unsigned int bit)
       Dprintf(("_RCSTA::receive_a_bit received 0x%02X\n",rsr & 0xff));
 
     } else {
-      //not stop bit; discard the data and go back receiving
+      //no stop bit; framing error
+       value.put(value.get() | FERR);
+
+      // copy the rsr to the fifo
+      if(rcreg)
+        rcreg->push( rsr & 0xff);
     }
     // If we're continuously receiving, then set up for the next byte.
     // FIXME -- may want to set a half bit delay before re-starting...
@@ -618,12 +706,13 @@ void _RCSTA::overrun()
 
 void _RCSTA::set_callback_break(unsigned int spbrg_edge)
 {
-  unsigned int cpi = (cpu) ? p_cpu->get_ClockCycles_per_Instruction() : 4;
-  //  last_cycle = cycles.value;
+  unsigned int time_to_event;
 
   if(cpu && spbrg)
-    get_cycles().set_break(get_cycles().value + (spbrg->value.get() + 1) * spbrg_edge/cpi, this);
-
+  {
+    time_to_event = ( spbrg->get_cycles_per_tick() * spbrg_edge ) / TOTAL_SAMPLE_STATES;
+    get_cycles().set_break(get_cycles().get() + time_to_event, this);
+  }
 }
 void _RCSTA::receive_start_bit()
 {
@@ -633,7 +722,7 @@ void _RCSTA::receive_start_bit()
     Dprintf(("  but not enabled\n"));
     return;
   }
-  
+
   if(txsta && (txsta->value.get() & _TXSTA::BRGH))
     set_callback_break(BRGH_FIRST_MID_SAMPLE);
   else
@@ -689,9 +778,9 @@ void _RCSTA::callback()
     // If this wasn't the last bit then go ahead and set a break for the next bit.
     if(state==RCSTA_RECEIVING) {
       if(txsta && (txsta->value.get() & _TXSTA::BRGH))
-	set_callback_break(TOTAL_BRGH_STATES -(BRGH_THIRD_MID_SAMPLE - BRGH_FIRST_MID_SAMPLE));
+        set_callback_break(TOTAL_SAMPLE_STATES -(BRGH_THIRD_MID_SAMPLE - BRGH_FIRST_MID_SAMPLE));
       else
-	set_callback_break(TOTAL_BRGL_STATES -(BRGL_THIRD_MID_SAMPLE - BRGL_FIRST_MID_SAMPLE));
+        set_callback_break(TOTAL_SAMPLE_STATES -(BRGL_THIRD_MID_SAMPLE - BRGL_FIRST_MID_SAMPLE));
 
       sample_state = RCSTA_WAITING_MID1;
     }
@@ -723,7 +812,7 @@ void _RCREG::push(unsigned int new_value)
 
     if (m_rcsta)
       m_rcsta->overrun();
-      
+
     Dprintf(("receive overrun\n"));
 
   } else {
@@ -767,6 +856,9 @@ unsigned int _RCREG::get()
   return value.get();
 }
 
+
+
+
 //-----------------------------------------------------------
 // SPBRG - Serial Port Baud Rate Generator
 //
@@ -777,39 +869,53 @@ unsigned int _RCREG::get()
 
 void _SPBRG::get_next_cycle_break()
 {
-  unsigned int cpi = (cpu) ? p_cpu->get_ClockCycles_per_Instruction() : 4;
-
-
-  if(txsta && (txsta->value.get() & _TXSTA::SYNC))
-    {
-      // Synchronous mode
-      future_cycle = last_cycle + (value.get() + 1)*4/cpi;
-    }
-  else
-    {
-      // Asynchronous mode
-      if(txsta && (txsta->value.get() & _TXSTA::BRGH))
-      {
-	future_cycle = last_cycle + (value.get() + 1)*16/cpi;
-      }
-      else
-      {
-	future_cycle = last_cycle + (value.get() + 1)*64/cpi;
-      }
-    }
+  future_cycle = last_cycle + get_cycles_per_tick();
 
   if(cpu)
     get_cycles().set_break(future_cycle, this);
 
   //Dprintf(("SPBRG::callback next break at 0x%llx\n",future_cycle));
-  
+
+}
+
+unsigned int _SPBRG::get_cycles_per_tick()
+{
+    unsigned int cpi = (cpu) ? p_cpu->get_ClockCycles_per_Instruction() : 4;
+    unsigned int brgval, cpt;
+
+    if ( baudcon && baudcon->brg16() )
+    {
+        brgval =  ( brgh ? brgh->value.get() * 256 : 0 ) + value.get();
+        cpt = 4;    // hi-speed divisor in 16-bit mode is 4
+    }
+    else
+    {
+        brgval = value.get();
+        cpt = 16;   // hi-speed divisor in 8-bit mode is 16
+    }
+
+    if ( txsta && (txsta->value.get() & _TXSTA::SYNC) )
+    {
+      // Synchronous mode - divisor is always 4
+      cpt = 4;
+    }
+    else
+    {
+        // Asynchronous mode
+        if(txsta && !(txsta->value.get() & _TXSTA::BRGH))
+        {
+            cpt *= 4;   // lo-speed divisor is 4 times hi-speed
+        }
+    }
+
+    return ((brgval+1)*cpt)/cpi;
 }
 
 void _SPBRG::start()
 {
 
   if(cpu)
-    last_cycle = get_cycles().value;
+    last_cycle = get_cycles().get();
   start_cycle = last_cycle;
 
   get_next_cycle_break();
@@ -831,7 +937,7 @@ guint64 _SPBRG::get_last_cycle()
   // There's a chance that a SPBRG break point exists on the current
   // cpu cycle, but has not yet been serviced.
   if(cpu)
-    return( (get_cycles().value == future_cycle) ? future_cycle : last_cycle);
+    return( (get_cycles().get() == future_cycle) ? future_cycle : last_cycle);
   else
     return 0;
 }
@@ -852,53 +958,76 @@ guint64 _SPBRG::get_last_cycle()
 //     baud = cpu frequency / 64 / (spbrg.value + 1)
 //
 // What this routine will do is return the cpu cycle corresponding
-// to a (rising) edge of the spbrg clock. 
+// to a (rising) edge of the spbrg clock.
 
 guint64 _SPBRG::get_cpu_cycle(unsigned int edges_from_now)
 {
-
-  unsigned int cpi = (cpu) ? p_cpu->get_ClockCycles_per_Instruction() : 4;
-
   // There's a chance that a SPBRG break point exists on the current
-  // cpu cycle, but has not yet been serviced. 
-  guint64 cycle = (get_cycles().value == future_cycle) ? future_cycle : last_cycle;
-
-  if(txsta && (txsta->value.get() & _TXSTA::SYNC))
-    // Synchronous mode
-    return ( edges_from_now * (value.get() + 1)*4/cpi + cycle);
-
-  else  {
-      // Asynchronous mode
-    if(txsta && (txsta->value.get() & _TXSTA::BRGH))
-      return ( edges_from_now * (value.get() + 1)*16/cpi + cycle);
-    else
-      return ( edges_from_now * (value.get() + 1)*64/cpi + cycle);
-  }
+  // cpu cycle, but has not yet been serviced.
+  guint64 cycle = (get_cycles().get() == future_cycle) ? future_cycle : last_cycle;
 
+  return ( edges_from_now * get_cycles_per_tick() + cycle );
 }
+
 void _SPBRG::callback()
 {
 
-  last_cycle = get_cycles().value;
+  last_cycle = get_cycles().get();
 
   //Dprintf(("SPBRG rollover at cycle:0x%llx\n",last_cycle));
 
   if(rcsta && (rcsta->value.get() & _RCSTA::SPEN))
     {
 
-      // If the serial port is enabled, then set another 
+      // If the serial port is enabled, then set another
       // break point for the next clock edge.
       get_next_cycle_break();
 
     }
 }
 
+//-----------------------------------------------------------
+// TXSTA - Transmit Register Status and Control
+
+void _BAUDCON::put_value(unsigned int new_value)
+{
+
+  put(new_value);
+
+  update();
+
+}
+
+void _BAUDCON::put(unsigned int new_value)
+{
+
+  unsigned int old_value = value.get();
+
+  trace.raw(write_trace.get() | value.get());
+
+  // The RCIDL bit is controlled entirely by hardware.
+  new_value &= ~RCIDL;
+  if ( rcsta->rc_is_idle() ) new_value |= RCIDL;
+
+  value.put(new_value);
+
+  Dprintf((" BAUDCON value=0x%x\n",value.get()));
+
+
+  if( (old_value ^ value.get()) & TXCKP) {
+
+    // The TXCKP bit has changed states.
+    //
+    txsta->set_pin_pol ((value.get() & TXCKP) ? true : false);
+  }
+}
+
 //--------------------------------------------------
 // member functions for the USART
 //--------------------------------------------------
 void USART_MODULE::initialize(PIR_SET *_pir_set,
-			      PinModule *tx_pin, PinModule *rx_pin,
-			      _TXREG *_txreg, _RCREG *_rcreg)
+                              PinModule *tx_pin, PinModule *rx_pin,
+                              _TXREG *_txreg, _RCREG *_rcreg)
 {
   assert(_txreg && _rcreg);
 
@@ -959,8 +1088,34 @@ void USART_MODULE::clear_rcif()
 }
 
 //--------------------------------------------------
-USART_MODULE::USART_MODULE()
-  : txsta(this), rcsta(this)
+USART_MODULE::USART_MODULE(Processor *pCpu)
+  : txsta(pCpu,"txsta","USART Transmit Status",this),
+    rcsta(pCpu,"rcsta","USART Receive Status",this),
+    spbrg(pCpu,"spbrg","Serial Port Baud Rate Generator"),
+    spbrgh(pCpu,"spbrgh","Serial Port Baud Rate high byte"),
+    baudcon(pCpu,"baudcon","Serial Port Baud Rate Control"),
+    is_eusart(false)
 {
+    baudcon.txsta = &txsta;
+    baudcon.rcsta = &rcsta;
+}
+
+//--------------------------------------------------
+void USART_MODULE::set_eusart ( bool is_it )
+{
+    if ( is_it )
+    {
+        spbrgh.assign_spbrg ( &spbrg );
+        spbrg.baudcon = &baudcon;
+        spbrg.brgh = &spbrgh;
+        is_eusart = true;
+    }
+    else
+    {
+        spbrgh.assign_spbrg ( 0 );
+        spbrg.baudcon = 0;
+        spbrg.brgh = 0;
+        is_eusart = false;
+    }
 }
 
