@@ -22,6 +22,7 @@ Boston, MA 02111-1307, USA.  */
 #include <stdio.h>
 #include <iostream>
 #include <iomanip>
+#include <cstdio>
 
 #include "processor.h"
 
@@ -33,6 +34,8 @@ Boston, MA 02111-1307, USA.  */
 #include "../config.h"
 #include "cmd_gpsim.h"
 
+#include <typeinfo>
+
 char * TrimWhiteSpaceFromString(char * pBuffer) {
   size_t iPos = 0;
   char * pChar = pBuffer;
@@ -87,19 +90,53 @@ string &toupper(string & sStr) {
 
 //------------------------------------------------------------------------
 Value::Value()
-  : xref(0), m_bClearableSymbol(true)
+  : cpu(0), m_aka(0)
 {
 }
 
-Value::Value(const char *_name, const char *desc)
-  : gpsimObject(_name,desc), xref(0), m_bClearableSymbol(true)
+Value::Value(const char *_name, const char *desc, Module *pMod)
+  : gpsimObject(_name,desc), cpu(pMod),m_aka(0)
 {
-  
 }
 
 Value::~Value()
 {
-  delete xref;
+  // Remove references of this Value from the symbol table:
+  if (cpu) {
+
+    //cout << "Deleting value named:" << name_str <<  " addr "<< this << endl;
+    cpu->removeSymbol(name_str);
+
+    if (m_aka) {
+      //cout << "m_aka ==" << m_aka << endl;
+
+      list <string>::iterator it;
+      it = m_aka->begin();
+      while(it != m_aka->end()) {
+        string s(*it);
+        //cout << " Value label " << s << endl;
+        cpu->removeSymbol(s);
+        ++it;
+      }
+      m_aka->clear();
+      delete m_aka;
+    }
+
+  }
+}
+void Value::update()
+{
+  _xref._update();
+}
+
+void Value::add_xref(void *an_xref)
+{
+  _xref._add(an_xref);
+}
+
+void Value::remove_xref(void *an_xref)
+{
+  _xref.clear(an_xref);
 }
 
 void Value::set(const char *cP,int i)
@@ -130,9 +167,6 @@ void Value::set(Value *v)
   throw new Error(" cannot assign a Value to a " + showType());
 }
 
-// JRH - Note: If this function Value::set(Expression) is ever
-//       used anywhere besides yyparse() you may want to move
-//       the delete expr; statement into the calling code.
 void Value::set(Expression *expr)
 {
   try {
@@ -140,7 +174,8 @@ void Value::set(Expression *expr)
 
     if(!expr)
       throw new Error(" null expression ");
-
+    if (verbose)
+      cout << toString() << " is being assigned expression " << expr->toString() << endl;
     v = expr->evaluate();
     if(!v)
       throw new Error(" cannot evaluate expression ");
@@ -156,9 +191,6 @@ void Value::set(Expression *expr)
     delete err;
   }
 
-
-  delete expr;
-
 }
 
 void Value::set(Packet &pb)
@@ -169,7 +201,7 @@ void Value::set(Packet &pb)
 void Value::get(gint64 &i)
 {
   throw new Error(showType() +
-		  " cannot be converted to an integer ");
+                  " cannot be converted to an integer ");
 }
 
 void Value::get(int &i)
@@ -190,17 +222,17 @@ void Value::get(guint64 &i)
 void Value::get(bool &b)
 {
   throw new Error(showType() +
-		  " cannot be converted to a boolean");
+                  " cannot be converted to a boolean");
 }
 
 void Value::get(double &d)
 {
   throw new Error(showType() +
-		  " cannot be converted to a double ");
+                  " cannot be converted to a double ");
 }
 
 // get as a string - no error is thrown if the derived class
-// does not provide a method for converting to a string - 
+// does not provide a method for converting to a string -
 // instead we'll return a bogus value.
 
 void Value::get(char *buffer, int buf_size)
@@ -216,8 +248,8 @@ void Value::get(Packet &pb)
 
 bool Value::compare(ComparisonOperator *compOp, Value *rvalue)
 {
-  throw new Error(compOp->showOp() + 
-		  " comparison is not defined for " + showType());
+  throw new Error(compOp->showOp() +
+                  " comparison is not defined for " + showType());
 }
 
 Value *Value::copy()
@@ -226,75 +258,152 @@ Value *Value::copy()
 
 }
 
+/*
 void Value::set_xref(Value *v)
 {
   delete xref;
   xref = v;
 }
-void Value::setClearableSymbol(bool bClear) {
-  m_bClearableSymbol = bClear;
-}
-
-bool Value::isClearable() {
-  return m_bClearableSymbol;
-}
-
 Value *Value::get_xref()
 {
-
   return xref;
 }
+*/
 
-//------------------------------------------------------------------------
-// gpsimValue
-
-gpsimValue::gpsimValue(void)
-  : cpu(0)
-{
-}
-
-gpsimValue::gpsimValue(Module *_cpu)
-  : cpu(_cpu)
-{
-}
-
-gpsimValue::~gpsimValue(void)
-{
-}
-
-void gpsimValue::update(void)
-{
-  _xref._update();
-}
-
-void gpsimValue::add_xref(void *an_xref)
-{
-  _xref._add(an_xref);
-}
-
-void gpsimValue::remove_xref(void *an_xref)
-{
-  _xref.clear(an_xref);
-}
-
-string gpsimValue::toString()
-{
-  char buff[64];
-  snprintf(buff,sizeof(buff), " = 0x%x",get_value());
-  string s = name() + string(buff);
-  return s;
-}
-
-Processor *gpsimValue::get_cpu() const
+Processor *Value::get_cpu() const
 {
   return static_cast<Processor *>(cpu);
 }
 
-void gpsimValue::set_cpu(Processor *new_cpu)
+void Value::set_cpu(Processor *new_cpu)
 {
   cpu = new_cpu;
 }
+void Value::set_module(Module *new_cpu)
+{
+  cpu = new_cpu;
+}
+Module *Value::get_module()
+{
+  return cpu;
+}
 
+void Value::addName(string &r_sAliasedName)
+{
+  if (!m_aka)
+    m_aka = new list<string>();
+
+  //cout << "Adding name " << r_sAliasedName << " to "<< name() << endl;
+  m_aka->push_back(r_sAliasedName);
+}
+
+//------------------------------------------------------------------------
+ValueWrapper::ValueWrapper(Value *pCopy)
+  : m_pVal(pCopy)
+{
+}
+ValueWrapper::~ValueWrapper()
+{
+}
+unsigned int ValueWrapper::get_leftVal()
+{
+  return m_pVal->get_leftVal();
+}
+unsigned int ValueWrapper::get_rightVal()
+{
+  return m_pVal->get_rightVal();
+}
+void ValueWrapper::set(const char *cP,int len)
+{
+  m_pVal->set(cP,len);
+}
+void ValueWrapper::set(double d)
+{
+  m_pVal->set(d);
+}
+void ValueWrapper::set(gint64 i)
+{
+  m_pVal->set(i);
+}
+void ValueWrapper::set(int i)
+{
+  m_pVal->set(i);
+}
+void ValueWrapper::set(bool b)
+{
+  m_pVal->set(b);
+}
+void ValueWrapper::set(Value *v)
+{
+  m_pVal->set(v);
+}
+void ValueWrapper::set(Expression *e)
+{
+  m_pVal->set(e);
+}
+void ValueWrapper::set(Packet &p)
+{
+  m_pVal->set(p);
+}
+void ValueWrapper::get(bool &b)
+{
+  m_pVal->get(b);
+}
+void ValueWrapper::get(int &i)
+{
+  m_pVal->get(i);
+}
+void ValueWrapper::get(guint64 &i)
+{
+  m_pVal->get(i);
+}
+void ValueWrapper::get(gint64 &i)
+{
+  m_pVal->get(i);
+}
+void ValueWrapper::get(double &d)
+{
+  m_pVal->get(d);
+}
+void ValueWrapper::get(char *pC, int len)
+{
+  m_pVal->get(pC,len);
+}
+void ValueWrapper::get(Packet &p)
+{
+  m_pVal->get(p);
+}
+Value *ValueWrapper::copy()
+{
+  return m_pVal->copy();
+}
+void ValueWrapper::update()
+{
+  m_pVal->update();
+}
+Value *ValueWrapper::evaluate()
+{
+  return m_pVal->evaluate();
+}
+
+bool ValueWrapper::compare(ComparisonOperator *compOp, Value *rvalue)
+{
+  if(!compOp || !rvalue)
+    return false;
+
+  gint64 i,r;
+
+  m_pVal->get(i);
+  rvalue->get(r);
+
+  if(i < r)
+    return compOp->less();
+
+  if(i > r)
+    return compOp->greater();
+
+  return compOp->equal();
+}
 
 /*****************************************************************
  * The AbstractRange class.
@@ -316,12 +425,12 @@ string AbstractRange::toString()
   char buff[256];
 
   string str = "";
-  
+
   snprintf(buff,sizeof(buff),"%d:%d",left,right);
   return (string(buff));
 }
 
-string AbstractRange::toString(char* format)
+string AbstractRange::toString(const char* format)
 {
   char cvtBuf[1024];
 
@@ -359,8 +468,8 @@ AbstractRange* AbstractRange::typeCheck(Value* val, string valDesc)
 }
 bool AbstractRange::compare(ComparisonOperator *compOp, Value *rvalue)
 {
-  throw new Error(compOp->showOp() + 
-		  " comparison is not defined for " + showType());
+  throw new Error(compOp->showOp() +
+                  " comparison is not defined for " + showType());
 }
 
 Value *AbstractRange::copy()
@@ -404,7 +513,7 @@ bool Boolean::Parse(const char *pValue, bool &bValue) {
     return true;
   }
   else if(strncmp("false", pValue, sizeof("false")-1) == 0) {
-  	bValue = false;
+        bValue = false;
     return true;
   }
   return false;
@@ -420,7 +529,6 @@ Boolean * Boolean::NewObject(const char *_name, const char *pValue, const char *
 
 Boolean::~Boolean()
 {
-
 }
 
 string Boolean::toString()
@@ -456,7 +564,7 @@ char *Boolean::toBitStr(char *return_str, int len)
   return return_str;
 }
 
-string Boolean::toString(char* format)
+string Boolean::toString(const char* format)
 {
   char cvtBuf[1024];
   bool b;
@@ -478,7 +586,7 @@ Boolean* Boolean::typeCheck(Value* val, string valDesc)
 
 bool Boolean::compare(ComparisonOperator *compOp, Value *rvalue)
 {
-  
+
   Boolean *rv = typeCheck(rvalue,"");
 
   switch(compOp->isa()) {
@@ -512,13 +620,13 @@ void Boolean::get(bool &b)
 // the member value. The reason for this is so that derived
 // classes can capture the access.
 void Boolean::get(int &i)
-{ 
+{
   bool b;
   get(b);
-  i = b ? 1 : 0; 
+  i = b ? 1 : 0;
 }
 /*
-void Boolean::get(double &d) 
+void Boolean::get(double &d)
 {
   bool b;
   get(b);
@@ -533,7 +641,7 @@ void Boolean::get(char *buffer, int buf_size)
     get(b);
     if(b)
       strncpy(buffer,"true",buf_size);
-    else 
+    else
       strncpy(buffer,"false",buf_size);
   }
 
@@ -555,8 +663,8 @@ void Boolean::set(Value *v)
 void Boolean::set(bool v)
 {
   value = v;
-  if(get_xref())
-    get_xref()->set(v);
+  //if(get_xref())
+  //  get_xref()->set(v);
 }
 
 void Boolean::set(const char *buffer, int buf_size)
@@ -635,10 +743,10 @@ void Integer::setDefaultBitmask(gint64 bitmask) {
 }
 
 Value *Integer::copy()
-{ 
+{
   gint64 i;
   get(i);
-  return new Integer(i); 
+  return new Integer(i);
 }
 
 void Integer::set(double d)
@@ -650,8 +758,8 @@ void Integer::set(double d)
 void Integer::set(gint64 i)
 {
   value = i;
-  if(get_xref())
-    get_xref()->set(i);
+  //if(get_xref())
+  //  get_xref()->set(i);
 }
 void Integer::set(int i)
 {
@@ -701,14 +809,14 @@ bool Integer::Parse(const char *pValue, gint64 &iValue) {
       }
       else {
         // decimal or 0x integer
-        return sscanf(pValue, "%" PRINTF_INT64_MODIFIER "i", &iValue) == 1;
+        return sscanf(pValue, "%" PRINTF_GINT64_MODIFIER "i", &iValue) == 1;
       }
     }
     else if(*pValue == '$' && ::isxdigit(*(pValue+1))) {
       // hexidecimal integer
       char szHex[10] = "0x";
       strcat(&szHex[0], pValue + 1);
-      return sscanf(szHex, "%"  PRINTF_INT64_MODIFIER "i" , &iValue) == 1;
+      return sscanf(szHex, "%"  PRINTF_GINT64_MODIFIER "i" , &iValue) == 1;
     }
     return false;
 }
@@ -723,12 +831,12 @@ Integer * Integer::NewObject(const char *_name, const char *pValue, const char *
 
 
 void Integer::get(gint64 &i)
-{ 
+{
   i = value;
 }
 
 void Integer::get(double &d)
-{ 
+{
   gint64 i;
   get(i);
   d = (double)i;
@@ -773,7 +881,7 @@ int Integer::set_break(ObjectBreakTypes bt, ObjectActionTypes at, Expression *ex
       return get_bp().set_execution_break(pCpu, iProgAddress, expr);
     }
   }
-  
+
   return -1;
 }
 
@@ -786,7 +894,7 @@ string Integer::toString()
 }
 
 
-string Integer::toString(char* format)
+string Integer::toString(const char* format)
 {
   char cvtBuf[1024];
 
@@ -798,7 +906,7 @@ string Integer::toString(char* format)
 }
 
 
-string Integer::toString(char* format, gint64 value)
+string Integer::toString(const char* format, gint64 value)
 {
   char cvtBuf[1024];
 
@@ -811,7 +919,7 @@ string Integer::toString(gint64 value)
   char cvtBuf[1024];
   long long int v=value;
   snprintf(cvtBuf,sizeof(cvtBuf), "%" PRINTF_INT64_MODIFIER "d", v);
-  return (string(&cvtBuf[0]));  
+  return (string(&cvtBuf[0]));
 }
 
 char *Integer::toString(char *return_str, int len)
@@ -835,7 +943,7 @@ char *Integer::toBitStr(char *return_str, int len)
     int mask=1<<31;
     for( ; mask ; mask>>=1, j++)
       if(j<len)
-	return_str[j] = ( (i & mask) ? 1 : 0);
+        return_str[j] = ( (i & mask) ? 1 : 0);
 
     if(j<len)
       return_str[j]=0;
@@ -861,14 +969,14 @@ Integer* Integer::assertValid(Value* val, string valDesc, gint64 valMin)
 
   iVal = Integer::typeCheck(val, valDesc);
   iVal->get(i);
-  
+
   if (i < valMin) {
     throw new Error(valDesc +
-                    " must be greater than " + Integer::toString(valMin) + 
+                    " must be greater than " + Integer::toString(valMin) +
                     ", saw " + Integer::toString(i)
                     );
   }
-  
+
   return(iVal);
 }
 
@@ -876,24 +984,24 @@ Integer* Integer::assertValid(Value* val, string valDesc, gint64 valMin, gint64
 {
   Integer* iVal;
   gint64 i;
-  
+
   iVal = (Integer::typeCheck(val, valDesc));
 
   iVal->get(i);
-  
+
   if ((i < valMin) || (i>valMax)) {
     throw new Error(valDesc +
-                    " must be be in the range [" + Integer::toString(valMin) + ".." + 
+                    " must be be in the range [" + Integer::toString(valMin) + ".." +
                     Integer::toString(valMax) + "], saw " + Integer::toString(i)
                     );
   }
-  
+
   return(iVal);
 }
 
 bool Integer::compare(ComparisonOperator *compOp, Value *rvalue)
 {
-  
+
   Integer *rv = typeCheck(rvalue,"");
 
   gint64 i,r;
@@ -950,7 +1058,7 @@ Float::Float(const char *_name, double newValue,const char *_desc)
   value = newValue;
 }
 
-bool Float::Parse(const char *pValue, double &fValue) 
+bool Float::Parse(const char *pValue, double &fValue)
 {
   return pValue ? sscanf(pValue,"%lg",&fValue) == 1 : false;
 }
@@ -970,8 +1078,8 @@ Float::~Float()
 void Float::set(double d)
 {
   value = d;
-  if(get_xref())
-    get_xref()->set(d);
+  //if(get_xref())
+  //  get_xref()->set(d);
 }
 
 void Float::set(gint64 i)
@@ -982,9 +1090,20 @@ void Float::set(gint64 i)
 
 void Float::set(Value *v)
 {
+  /* typeCheck means cannot set integers - RRR
   Float *fv = typeCheck(v,string("set "));
   double d = fv->getVal();
   set(d);
+  */
+   double d;
+
+  if (typeid(*v) != typeid(Float) &&
+      typeid(*v) != typeid(Integer))
+  {
+    throw new TypeMismatch(string("set "), "Float", v->showType());
+  }
+   v->get(d);
+   set(d);
 }
 
 void Float::set(const char *buffer, int buf_size)
@@ -1009,13 +1128,13 @@ void Float::set(Packet &p)
 }
 
 void Float::get(gint64 &i)
-{ 
+{
   double d;
   get(d);
   i = (gint64)d;
 }
 void Float::get(double &d)
-{ 
+{
   d = value;
 }
 
@@ -1050,7 +1169,7 @@ string Float::toString()
 }
 
 
-string Float::toString(char* format)
+string Float::toString(const char* format)
 {
   char cvtBuf[1024];
 
@@ -1085,7 +1204,7 @@ Float* Float::typeCheck(Value* val, string valDesc)
 
 bool Float::compare(ComparisonOperator *compOp, Value *rvalue)
 {
-  
+
   Float *rv = typeCheck(rvalue,"");
 
   double d,r;
@@ -1119,6 +1238,18 @@ String::String(const char *newValue)
   else
     value = 0;
 }
+
+String::String(const char *newValue, size_t len)
+{
+  if (newValue) {
+    value = (char *)malloc(len + 1);
+    strncpy(value, newValue, len);
+    value[len] = '\0';
+  }
+  else
+    value = 0;
+}
+
 String::String(const char *_name, const char *newValue,const char *_desc)
   : Value(_name,_desc)
 {
@@ -1135,12 +1266,11 @@ String::~String()
     free(value);
 }
 
-
 string String::toString()
 {
   if(value)
     return string(value);
-  else 
+  else
     return string("");
 }
 
@@ -1196,42 +1326,6 @@ void String::get(Packet &p)
   p.EncodeString(value);
 }
 
-/*
-string String::toString(char* format)
-{
-  char cvtBuf[1024];
-
-  sprintf(cvtBuf, format, value.c_str());
-  return (string(&cvtBuf[0]));
-}
-*/
-/**
-String* String::typeCheck(Value* val, string valDesc)
-{
-  if (typeid(*val) != typeid(String)) {
-    throw new TypeMismatch(valDesc, "String", val->showType());
-  }
-
-  // This static cast is totally safe in light of our typecheck, above.
-  return((String*)(val));
-}
-*/
-
-/*
-bool String::compare(ComparisonOperator *compOp, Value *rvalue)
-{
-  
-  String *rv = typeCheck(rvalue,"");
-
-  if(value < rv->value)
-    return compOp->less();
-
-  if(value > rv->value)
-    return compOp->greater();
-
-  return compOp->equal();
-}
-*/
 const char *String::getVal()
 {
   return value;
@@ -1241,3 +1335,35 @@ Value *String::copy()
 {
   return new String(value);
 }
+
+//------------------------------------------------------------------------
+namespace gpsim {
+  Function::Function(const char *_name, const char *desc)
+    : gpsimObject(_name,desc)
+  {
+  }
+
+  Function::~Function()
+  {
+    cout << "Function destructor\n";
+  }
+
+  string Function::description()
+  {
+    if(cpDescription)
+      return string(cpDescription);
+    else
+      return string("no description");
+  }
+
+  string Function::toString()
+  {
+    return name();
+  }
+
+  void Function::call(ExprList_t *vargs)
+  {
+    cout << "calling " << name() << endl;
+  }
+
+}
