@@ -14,7 +14,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with gpasm; see the file COPYING.  If not, write to
+along with gpsim; see the file COPYING.  If not, write to
 the Free Software Foundation, 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.  */
 
@@ -24,18 +24,24 @@ Boston, MA 02111-1307, USA.  */
 //-------------------------------------------------------------------
 
 #include <list>
+using namespace std;
+
+class gpsimObject;
 
 class XrefObject {
-    unsigned int *data;
-    list<void*> xrefs;
-public:
-    XrefObject();
-    XrefObject(unsigned int *value);
-    ~XrefObject();
+ private:
+  gpsimObject *data;
+  list<void*> xrefs;
+ public:
+  XrefObject();
+  XrefObject(gpsimObject *value);
+  virtual ~XrefObject();
     
-    virtual void add(void *xref);
-    virtual void clear(void *xref);
-    virtual void update();
+  virtual void _add(void *xref);
+  virtual void clear(void *xref);
+  virtual void _update();
+  virtual int get_val(void);
+  virtual void assign_data(gpsimObject *);
 };
 
 #endif
